// Generated by CoffeeScript 1.12.2
(function() {
  var Application;

  window.GS = {
    dataCache: {}
  };

  window.gs = {};

  window.$MOBILE = false;

  Application = (function() {

    /**
    * This class provides general application functions.
    *
    * @namespace gs
    * @class Application
    * @static
    * @final
    * @memberof gs
    * @constructor
     */
    function Application() {

      /**
      * A callback-function called if the application will exit.
      * @property onExit
      * @type Function
      * @static
       */
      this.onExit = null;

      /**
      * A callback-function called if the application is ready.
      * @property onReady
      * @type Function
      * @static
       */
      this.onReady = null;
    }


    /**
    * Initializes the application. Needs to be called before anything else.
    *
    * @method initialize
    * @static
     */

    Application.prototype.initialize = function() {
      gs.Platform = {};
      if (typeof window !== void 0 && window.navigator) {
        gs.Platform.isEdge = window.navigator.userAgent.indexOf("Edge") !== -1;
        gs.Platform.isChrome = !gs.Platform.isEdge && window.navigator.userAgent.indexOf("Chrome") !== -1;
        gs.Platform.isSafari = !gs.Platform.isEdge && !gs.Platform.isChrome && window.navigator.userAgent.indexOf("Safari") !== -1;
        gs.Platform.isSafariMobile = !!window.navigator.userAgent.match(/iPad/i) || !!window.navigator.userAgent.match(/iPhone/i);
        gs.Platform.isCordova = typeof cordova !== "undefined";
        gs.Platform.isAndroid = gs.Platform.isCordova && cordova.platformId === "android";
        gs.Platform.isAtsumaru = typeof RPGAtsumaru !== "undefined";
      }
      if (gs.Platform.isCordova) {
        return document.addEventListener('deviceready', (function(_this) {
          return function() {
            return typeof _this.onReady === "function" ? _this.onReady() : void 0;
          };
        })(this), false);
      } else {
        return jQuery(document).ready((function(_this) {
          return function() {
            return typeof _this.onReady === "function" ? _this.onReady() : void 0;
          };
        })(this));
      }
    };


    /**
    * Quits the game. It depends on the platform. For example: It has no effect on most smartphones or tablets.
    *
    * @method exit
    * @static
     */

    Application.prototype.exit = function() {
      var gui;
      if ($PARAMS.platform.isSteamOS) {
        return window.close();
      } else if ($PARAMS.isOffline && (window.require != null)) {
        gui = require('nw.gui');
        return gui.App.quit();
      } else {
        if (this.onExit != null) {
          return this.onExit();
        }
      }
    };

    Application.prototype.getPlatformSpecificGraphicsFilePath = function(filePath, languageCode) {
      var ext, exts, filePathLC, path, resources;
      resources = GS.dataCache["RESOURCES"].items.resources;
      filePathLC = filePath + "_" + languageCode;
      ext = null;
      exts = null;
      path = null;
      if (resources[filePathLC]) {
        path = filePathLC;
        exts = resources[filePathLC] || ".png";
      } else {
        path = filePath;
        exts = resources[filePath] || ".png";
      }
      ext = exts;
      ext = ext === ".fnt" ? ".png" : ext;
      if (gs.Platform.isCordova && gs.Platform.isAndroid && $PARAMS.usePlayStoreExpansionFile) {
        return ("content://" + $PARAMS.gameIdentifier + "/main_expansion/resources/") + GS.DataPreparer.prepareFilePath(path + ext);
      } else {
        return "resources/" + GS.DataPreparer.prepareFilePath(path + ext);
      }
    };

    Application.prototype.getPlatformSpecificVideoFilePath = function(filePath, languageCode) {
      var ext, exts, filePathLC, path, resources;
      resources = GS.dataCache["RESOURCES"].items.resources;
      filePathLC = filePath + "_" + languageCode;
      ext = null;
      exts = null;
      path = null;
      if (resources[filePathLC]) {
        path = filePathLC;
        exts = resources[filePathLC] || [".webm"];
      } else {
        path = filePath;
        exts = resources[filePath] || [".webm"];
      }
      if (gs.Platform.isSafari || gs.Platform.isEdge || gs.Platform.isSafariMobile) {
        ext = exts.indexOf(".mp4") !== -1 ? ".mp4" : "";
        ext = !ext ? exts[0] : ext;
      } else {
        ext = exts.indexOf(".webm") !== -1 ? ".webm" : "";
        ext = !ext ? exts[0] || ".webm" : ".webm";
      }
      if (gs.Platform.isCordova && gs.Platform.isAndroid && $PARAMS.usePlayStoreExpansionFile) {
        return ("content://" + $PARAMS.gameIdentifier + "/main_expansion/resources/") + GS.DataPreparer.prepareFilePath(path + ext);
      } else {
        return "resources/" + GS.DataPreparer.prepareFilePath(path + ext);
      }
    };

    Application.prototype.getPlatformSpecificAudioFilePath = function(filePath, languageCode, noPrepare) {
      var ext, exts, filePathLC, path, resources;
      resources = GS.dataCache["RESOURCES"].items.resources;
      filePathLC = filePath + "_" + languageCode;
      ext = null;
      exts = null;
      path = null;
      if (resources[filePathLC]) {
        path = filePathLC;
        exts = resources[filePathLC] || [".wav"];
      } else {
        path = filePath;
        exts = resources[filePath] || [".wav"];
      }
      if (gs.Platform.isSafari || gs.Platform.isEdge || gs.Platform.isSafariMobile) {
        ext = exts.indexOf(".m4a") !== -1 ? ".m4a" : "";
        ext = !ext && exts.indexOf(".mp4") !== -1 ? ".mp4" : ext;
        ext = !ext && exts.indexOf(".mp3") !== -1 ? ".mp3" : ext;
        ext = !ext && exts.indexOf(".aac") !== -1 ? ".aac" : ext;
        ext = !ext ? exts[0] : ext;
      } else {
        ext = exts[0];
      }
      if (gs.Platform.isCordova && gs.Platform.isAndroid && $PARAMS.usePlayStoreExpansionFile) {
        return ("content://" + $PARAMS.gameIdentifier + "/main_expansion/resources/") + GS.DataPreparer.prepareFilePath(path + ext);
      } else {
        return "resources/" + GS.DataPreparer.prepareFilePath(path + ext);
      }
    };

    Application.prototype.loadData = function(filePath, cbSuccess, cbError, dataType, mimeType) {
      var request;
      if (gs.Platform.isSafariMobile && gs.Platform.isCordova) {
        return resolveLocalFileSystemURL(cordova.file.applicationDirectory + "/www/" + filePath, function(fe) {
          return fe.file(function(f) {
            var reader;
            reader = new FileReader();
            reader.onloadend = function(e) {
              return cbSuccess(this.result);
            };
            reader.onerror = function(e) {
              return cbError(e);
            };
            if (dataType === 1) {
              return reader.readAsDataURL(f);
            } else {
              return reader.readAsArrayBuffer(f);
            }
          });
        }, function(e) {
          return cbError(e);
        });
      } else {
        request = new XMLHttpRequest();
        request.open('GET', filePath, true);
        request.responseType = 'arraybuffer';
        request.onload = (function(_this) {
          return function() {
            return cbSuccess(request.response, mimeType);
          };
        })(this);
        request.onerror = (function(_this) {
          return function(e) {
            return cbError(e);
          };
        })(this);
        return request.send();
      }
    };

    return Application;

  })();

  if (!window.gs) {
    window.gs = {};
  }

  window.Application = new Application();

  gs.Application = window.Application;

  gs.CallBack = function(name, object, data) {
    var f, func;
    if (data != null) {
      func = object[name];
      f = function() {
        var args, i, j, ref;
        args = new Array(arguments.length + 1);
        for (i = j = 0, ref = arguments.length; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
          args[i] = arguments[i];
        }
        args[args.length - 1] = data;
        return func.apply(object, args);
      };
    } else {
      f = function() {
        return object[name].apply(object, arguments);
      };
    }
    f.$vnm_cb = true;
    f.funcName = name;
    f.object = object;
    f.data = data;
    return f;
  };

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var EventEmitter;

  EventEmitter = (function() {
    function EventEmitter() {
      this.handlerList = {};
    }

    EventEmitter.prototype.on = function(event, data, callback) {
      if (!this.handlerList[event]) {
        this.handlerList[event] = [];
      }
      return this.handlerList[event].push({
        once: false,
        data: data,
        callback: callback
      });
    };

    EventEmitter.prototype.once = function(event, data, callback) {
      if (!this.handlerList[event]) {
        this.handlerList[event] = [];
      }
      return this.handlerList[event].push({
        once: true,
        data: data,
        callback: callback
      });
    };

    EventEmitter.prototype.emit = function(event, params) {
      var deletions, handler, handlers, i, j, len, len1;
      handlers = this.handlerList[event];
      deletions = [];
      if (handlers) {
        for (i = 0, len = handlers.length; i < len; i++) {
          handler = handlers[i];
          handler.callback({
            sender: this,
            params: params,
            data: handler.data
          });
          if (handler.once) {
            deletions.push(handler);
          }
        }
      }
      for (j = 0, len1 = deletions.length; j < len1; j++) {
        handler = deletions[j];
        handlers.remove(handler);
      }
      return null;
    };

    return EventEmitter;

  })();

  gs.EventEmitter = EventEmitter;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var ObjectCodec, ObjectCodecContext;

  ObjectCodecContext = (function() {
    function ObjectCodecContext(decodedStore, encodedStore, tempStore) {
      this.mode = 0;
      this.decodedObjectStore = decodedStore != null ? decodedStore : [];
      this.encodedObjectStore = encodedStore != null ? encodedStore : [];
      this.tempObjectStore = tempStore != null ? tempStore : [];
      this.restoreEventReceivers = [];
    }

    return ObjectCodecContext;

  })();

  gs.ObjectCodecContext = ObjectCodecContext;

  ObjectCodec = (function() {
    function ObjectCodec() {}

    ObjectCodec.onRestore = function(object, context) {
      var decObjStore, i, j, k, objStore, ref, ref1, v;
      decObjStore = context.decodedObjectStore;
      objStore = context.tempObjectStore;
      for (k in object) {
        v = object[k];
        if (objStore.indexOf(v) === -1 && decObjStore.indexOf(v) !== -1 && (v != null)) {
          objStore.push(v);
          if (v instanceof Array) {
            for (i = j = 0, ref = v.length; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
              gs.ObjectCodec.onRestore(v[i], context);
              if ((((ref1 = v[i]) != null ? ref1.onDataBundleRestore : void 0) != null) && context.restoreEventReceivers.indexOf(v[i]) === -1) {
                context.restoreEventReceivers.push(v[i]);
                v[i].onDataBundleRestore(null, context);
              }
            }
          } else if (typeof v === "object") {
            gs.ObjectCodec.onRestore(v, context);
            if (v.onDataBundleRestore && context.restoreEventReceivers.indexOf(v) === -1) {
              context.restoreEventReceivers.push(v);
              if (typeof v.onDataBundleRestore === "function") {
                v.onDataBundleRestore(null, context);
              }
            }
          }
        }
      }
      return null;
    };

    ObjectCodec.decode = function(object, context) {
      var a, decObjStore, decObject, i, id, j, objClass, objStore, ref;
      if (object == null) {
        return object;
      }
      objStore = context.encodedObjectStore;
      decObjStore = context.decodedObjectStore;
      decObject = null;
      if (typeof object === "string") {
        if (object.indexOf("$$vnm_obj_ref") !== -1) {
          id = parseInt(object.split(" ")[1]);
          object = objStore[id];
          decObject = decObjStore[id];
        }
      }
      if ((object != null) && (decObject == null)) {
        if (object instanceof Array) {
          a = [];
          decObjStore[id] = a;
          decObject = a;
          for (i = j = 0, ref = object.length; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
            a[i] = gs.ObjectCodec.decode(object[i], context);
          }
        } else if (typeof object === "object") {
          if (object.$class === "Function") {
            if (object.$vnm_cb) {
              gs.ObjectCodec.decodeProperties(object, context);
              decObject = gs.CallBack(object.name, object.object, object.data);
              decObjStore[id] = decObject;
            } else {
              decObject = eval("(" + object.source + ")");
              decObjStore[id] = decObject;
            }
          } else {
            objClass = window[object.$ns || "gs"][object.$class] || window[object.$class] || Object;
            if (objClass.fromDataBundle) {
              decObject = objClass.fromDataBundle(object, context);
              decObjStore[id] = decObject;
            } else {
              decObject = typeof objClass === "function" ? new objClass : objClass;
              decObjStore[id] = decObject;
              object = gs.ObjectCodec.decodeProperties(object, context);
              delete object.$class;
              delete object.$ns;
              Object.mixin(decObject, object);
            }
          }
        } else {
          decObject = object;
        }
      }
      return decObject;
    };

    ObjectCodec.decodeProperties = function(object, context) {
      var k, r, v;
      r = object;
      for (k in object) {
        v = object[k];
        if (v != null) {
          if (typeof v === "string") {
            if (v.indexOf("$$vnm_obj_ref") !== -1) {
              r[k] = gs.ObjectCodec.decode(v, context);
            } else if (v !== "$$vnm_no_decode") {
              r[k] = v;
            }
          } else {
            r[k] = v;
          }
        }
      }
      return r;
    };

    ObjectCodec.encode = function(object, context, blackList) {
      var a, encObjStore, i, index, j, o, objStore, r, ref;
      objStore = context.decodedObjectStore;
      encObjStore = context.encodedObjectStore;
      r = null;
      if (object instanceof Array || object instanceof Float32Array) {
        objStore.push(object);
        a = [];
        for (i = j = 0, ref = object.length; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
          a[i] = gs.ObjectCodec.encode(object[i], context, blackList);
        }
        index = objStore.indexOf(object);
        encObjStore[index] = a;
        r = "$$vnm_obj_ref " + index;
      } else if (object instanceof Object) {
        objStore.push(object);
        o = null;
        if (typeof object === "function") {
          if (object.$vnm_cb) {
            o = gs.ObjectCodec.encodeProperties({
              name: object.funcName,
              object: object.object,
              data: object.data
            }, context);
            o.$vnm_cb = true;
            o.$class = "Function";
          } else {
            o = {
              source: object.toString()
            };
            o.$class = "Function";
            o.$ns = null;
          }
        } else if (object.constructor.toDataBundle) {
          o = object.constructor.toDataBundle(object, context);
          o.$class = object.constructor.$name || object.constructor.name;
          o.$ns = object.constructor.$namespace;
        } else {
          o = gs.ObjectCodec.encodeProperties(object, context, object.constructor.objectCodecBlackList);
          o.$class = object.constructor.$name || object.constructor.name;
          o.$ns = object.constructor.$namespace;
        }
        index = objStore.indexOf(object);
        r = "$$vnm_obj_ref " + index;
        encObjStore[index] = o;
      } else {
        r = object;
      }
      return r;
    };

    ObjectCodec.encodeProperties = function(object, context, blackList) {
      var encObjStore, index, k, objStore, r, v;
      objStore = context.decodedObjectStore;
      encObjStore = context.encodedObjectStore;
      r = {};
      for (k in object) {
        if (object.hasOwnProperty(k) && (!blackList || blackList.indexOf(k) === -1)) {
          v = object[k];
          index = objStore.indexOf(v);
          if (index !== -1) {
            r[k] = "$$vnm_obj_ref " + index;
          } else {
            r[k] = gs.ObjectCodec.encode(v, context, null);
          }
        }
      }
      return r;
    };

    return ObjectCodec;

  })();

  gs.ObjectCodec = ObjectCodec;

}).call(this);

// Generated by CoffeeScript 1.12.2

/**
* An enumeration of game storage locations.
*
* @namespace gs
* @class GameStorageLocations
* @memberof gs
* @constructor
* @static
* @final
 */

(function() {
  var GameStorage, GameStorageLocations,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  GameStorageLocations = (function() {
    function GameStorageLocations() {}

    GameStorageLocations.initialize = function() {

      /**
      * Stores game data in working directory which is next to the executable file in most cases.
      * @property WORKING_DIRECTORY
      * @type number
      * @static
      * @final
       */
      this.WORKING_DIRECTORY = 0;

      /**
      * Stores game data in local app-data directory depending on the operating system.
      * @property APP_DATA_DIRECTORY
      * @type number
      * @static
      * @final
       */
      this.APP_DATA_DIRECTORY = 1;

      /**
      * Stores game data in internal storage.
      * @property INTERNAL_STORAGE
      * @type number
      * @static
      * @final
       */
      return this.INTERNAL_STORAGE = 2;
    };

    return GameStorageLocations;

  })();

  GameStorageLocations.initialize();

  gs.GameStorageLocations = GameStorageLocations;

  GameStorage = (function(superClass) {
    extend(GameStorage, superClass);


    /**
    * The GameStorage helps to store different kind of game data permanentely like
    * a save-game or the game settings. The way how the data is stored depends on
    * the platform. For example: If the game runs in a web-browser the data is stored
    * the internal local storage of the web-browser but if the game runs locally the
    * data is stored on disk.
    *
    * All object are stored as JSON so it is imported that the object to store can
    * be serialized to JSON and doesn't contain any circular references. Except for
    * the methods gs.GameStorage.setData and gs.GameStorage.getData since that methods
    * dealing with a regular string.
    *
    * @namespace gs
    * @class GameStorage
    * @extends gs.EventEmitter
    * @memberof gs
    * @constructor
    * @static
     */

    function GameStorage() {
      GameStorage.__super__.constructor.apply(this, arguments);
      this.initializationTimer = null;
    }


    /**
    * Clears the game storage. All data will be lost.
    *
    * @method clear
    * @static
     */

    GameStorage.prototype.clear = function() {
      return localStorage.clear();
    };


    /**
    * <p>Initializes the game storage. Needs to be called once when the game starts. It initializes
    * the storage depending on platform. In case of cloud-storage, it will connect to the cloud-server
    * to get the storage data.</p>
    *
    * <p>This method is asynchronous and fires a "finishInitialize" event when it is done.</p>
    *
    * @method initialize
    * @static
     */

    GameStorage.prototype.initialize = function() {
      if (gs.Platform.isAtsumaru) {
        this.atsumaruCloudStorage = RPGAtsumaru.storage.getItems();
        if (this.atsumaruCloudStorage.isFulfilled()) {
          return this.emit("finishInitialize");
        } else if (this.atsumaruCloudStorage.isRejected()) {
          this.atsumaruCloudStorage = null;
          return this.emit("finishInitialize");
        } else {
          return this.initializationTimer = setInterval(((function(_this) {
            return function() {
              if (_this.atsumaruCloudStorage.isFulfilled()) {
                clearInterval(_this.initializationTimer);
                _this.atsumaruCloudStorage = _this.atsumaruCloudStorage.value();
                return _this.emit("finishInitialize");
              } else if (_this.atsumaruCloudStorage.isRejected()) {
                clearInterval(_this.initializationTimer);
                _this.atsumaruCloudStorage = null;
                return _this.emit("finishInitialize");
              }
            };
          })(this)), 10);
        }
      } else {
        return this.emit("finishInitialize");
      }
    };


    /**
    * The path where the data is stored. The result depends on the platform.
    * For example: For a web-game the result is an empty string but for a local
    * running game the result is the file-path to the configured storage-directory of the game.
    *
    * @method storagePath
    * @static
     */

    GameStorage.prototype.storagePath = function() {
      if ($PARAMS.isOffline && !$PARAMS.platform.isSteamOS && (window.nw != null)) {
        switch (RecordManager.system.gameDataPath) {
          case gs.GameStorageLocations.APP_DATA_DIRECTORY:
            return nw.App.dataPath;
          case gs.GameStorageLocations.WORKING_DIRECTORY:
            return "./";
          case gs.GameStorageLocations.INTERNAL_STORAGE:
            return "";
          default:
            return "./";
        }
      } else {
        return "";
      }
    };


    /**
    * Checks if data already exists for the specified id/key.
    *
    * @method exists
    * @return {boolean} If <b>true</b> then there is already existing data for the id/key. Otherwise <b>false</b>
    * @static
     */

    GameStorage.prototype.exists = function(id) {
      if ($PARAMS.isOffline && !$PARAMS.platform.isSteamOS && (window.nw != null)) {
        return require("fs").existsSync(this.storagePath() + "/" + id + ".vndata");
      } else if (gs.Platform.isAtsumaru && this.atsumaruCloudStorage) {
        id = $PARAMS.uid + "_" + id;
        return (this.atsumaruCloudStorage.filter(function(v) {
          return v.key === id;
        })).length > 0;
      } else {
        return localStorage.getItem($PARAMS.uid + "_" + id) !== null;
      }
    };


    /**
    * Deletes the data stored under the specified id/key.
    *
    * @method remove
    * @return {string} id - The id/key where the data should be deleted.
    * @static
     */

    GameStorage.prototype.remove = function(id) {
      var item;
      if ($PARAMS.isOffline && !$PARAMS.platform.isSteamOS && (window.nw != null)) {
        if (require("fs").existsSync(this.storagePath() + "/" + id + ".vndata")) {
          return require("fs").unlinkSync(this.storagePath() + "/" + id + ".vndata");
        }
      } else if (gs.Platform.isAtsumaru && this.atsumaruCloudStorage) {
        id = $PARAMS.uid + "_" + id;
        RPGAtsumaru.storage.removeItem(id);
        item = this.atsumaruCloudStorage.filter(function(item) {
          return item.key === id;
        })[0];
        if (item) {
          return this.atsumaruCloudStorage.splice(this.atsumaruCloudStorage.indexOf(item), 1);
        }
      } else {
        return localStorage.removeItem($PARAMS.uid + "_" + id);
      }
    };


    /**
    * Stores a string under the specified id/key.
    *
    * @method setData
    * @param {string} id - The id/key where the data should be stored.
    * @param {string} data - The string to store.
    * @static
     */

    GameStorage.prototype.setData = function(id, data) {
      var item;
      if ($PARAMS.isOffline && !$PARAMS.platform.isSteamOS && (window.nw != null)) {
        return require("fs").writeFileSync(this.storagePath() + "/" + id + ".vndata", data);
      } else if (gs.Platform.isAtsumaru && this.atsumaruCloudStorage) {
        id = $PARAMS.uid + "_" + id;
        item = (this.atsumaruCloudStorage.filter(function(item) {
          return item.key === id;
        }))[0];
        if (!item) {
          item = {
            key: id,
            value: null
          };
          this.atsumaruCloudStorage.push(item);
        }
        item.value = data;
        return RPGAtsumaru.storage.setItems([
          {
            key: id,
            value: data
          }
        ]);
      } else {
        return localStorage.setItem($PARAMS.uid + "_" + id, data);
      }
    };


    /**
    * Gets a stored string for the specified id/key.
    *
    * @method getData
    * @param {string} id - The id/key of the string/data to get.
    * @return {string} The string/data or <b>null</b> if no data exists for the specified id/key.
    * @static
     */

    GameStorage.prototype.getData = function(id) {
      var result;
      result = null;
      if ($PARAMS.isOffline && !$PARAMS.platform.isSteamOS && (window.nw != null)) {
        result = require("fs").readFileSync(this.storagePath() + "/" + id + ".vndata", "utf8");
      } else if (gs.Platform.isAtsumaru && this.atsumaruCloudStorage) {
        id = $PARAMS.uid + "_" + id;
        result = this.atsumaruCloudStorage.filter(function(item) {
          return item.key === id;
        })[0];
        result = result != null ? result.value : void 0;
      } else {
        result = localStorage.getItem($PARAMS.uid + "_" + id);
      }
      return result;
    };


    /**
    * Stores an object under the specified id/key. The object will be serialized to
    * JSON so its important that the object doesn't contain any circular references.
    *
    * @method setObject
    * @param {string} id - The id/key where the object should be stored.
    * @param {Object} object - The object to store.
    * @static
     */

    GameStorage.prototype.setObject = function(id, object) {
      var item;
      if ($PARAMS.isOffline && !$PARAMS.platform.isSteamOS && (window.nw != null)) {
        return require("fs").writeFileSync(this.storagePath() + "/" + id + ".vndata", JSON.stringify(object));
      } else if (gs.Platform.isAtsumaru && this.atsumaruCloudStorage) {
        id = $PARAMS.uid + "_" + id;
        item = (this.atsumaruCloudStorage.filter(function(item) {
          return item.key === id;
        }))[0];
        if (!item) {
          item = {
            key: id,
            value: null
          };
          this.atsumaruCloudStorage.push(item);
        }
        item.value = JSON.stringify(object);
        return RPGAtsumaru.storage.setItems([
          {
            key: id,
            value: item.value
          }
        ]);
      } else {
        return localStorage.setItem($PARAMS.uid + "_" + id, JSON.stringify(object));
      }
    };


    /**
    * Gets a stored object for the specified id/key.
    *
    * @method getObject
    * @param {string} id - The id/key of the object to get.
    * @return {Object} The object or <b>null</b> if no object exists for the specified id/key.
    * @static
     */

    GameStorage.prototype.getObject = function(id) {
      var item, result;
      item = null;
      result = null;
      if ($PARAMS.isOffline && !$PARAMS.platform.isSteamOS && (window.nw != null)) {
        if (require("fs").existsSync(this.storagePath() + "/" + id + ".vndata")) {
          item = require("fs").readFileSync(this.storagePath() + "/" + id + ".vndata", "utf8");
        }
      } else if (gs.Platform.isAtsumaru && this.atsumaruCloudStorage) {
        id = $PARAMS.uid + "_" + id;
        item = (this.atsumaruCloudStorage.filter(function(item) {
          return item.key === id;
        }))[0];
        item = item != null ? item.value : void 0;
      } else {
        item = localStorage.getItem($PARAMS.uid + "_" + id);
      }
      result = null;
      if (item != null) {
        result = JSON.parse(item);
      }
      return result;
    };

    return GameStorage;

  })(gs.EventEmitter);

  window.GameStorage = new GameStorage();

  gs.GameStorage = window.GameStorage;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var GameNotifier;

  GameNotifier = (function() {
    function GameNotifier() {}

    GameNotifier.postSceneChange = function(scene) {
      scene = scene.truncate(16);
      if (gs.Platform.isAtsumaru) {
        return RPGAtsumaru.comment.changeScene(scene);
      }
    };

    GameNotifier.postResetSceneChange = function(scene) {
      scene = scene.truncate(16);
      if (gs.Platform.isAtsumaru) {
        return RPGAtsumaru.comment.resetAndChangeScene(scene);
      }
    };

    GameNotifier.postContextChange = function(change) {
      if (gs.Platform.isAtsumaru) {
        return RPGAtsumaru.comment.pushContextFactor(change);
      }
    };

    GameNotifier.postMinorChange = function(minorChange) {
      if (gs.Platform.isAtsumaru) {
        return RPGAtsumaru.comment.pushMinorContext();
      }
    };

    return GameNotifier;

  })();

  gs.GameNotifier = GameNotifier;

}).call(this);

// Generated by CoffeeScript 1.12.2

/**
* This class is used to load Data files which are stored as JSON files in the data-folder of
* the project as well as raw text data.
*
* @namespace gs
* @class Data
* @memberof gs
* @constructor
 */

(function() {
  var Data;

  Data = (function() {
    function Data() {}

    Data._loadTest = function(uid, callback) {
      var onerror, request;
      request = new XMLHttpRequest();
      request.open('GET', "./data/" + uid + ".json", true);
      request.responseType = 'arraybuffer';
      onerror = (function(_this) {
        return function(e) {
          console.log("ERROR");
          console.log(e);
          return typeof callback === "function" ? callback(null, e) : void 0;
        };
      })(this);
      request.onload = (function(_this) {
        return function() {
          var buffer, reader;
          buffer = GS.DataPreparer.prepare(request.response);
          reader = new FileReader();
          reader.onload = function(e) {
            var dataObject;
            dataObject = JSON.parse(e.target.result);
            GS.dataCache[uid] = dataObject;
            return typeof callback === "function" ? callback(dataObject, null) : void 0;
          };
          reader.onerror = onerror;
          return reader.readAsText(new Blob([buffer]));
        };
      })(this);
      request.onerror = onerror;
      return request.send();
    };

    Data._loadRelease = function(uid, callback) {
      return gs.Application.loadData("data/" + uid + ".json.js", (function(_this) {
        return function(data) {
          var buffer, reader;
          buffer = GS.DataPreparer.prepare(data);
          reader = new FileReader();
          reader.onload = function(e) {
            var dataObject;
            eval(e.target.result);
            dataObject = GS.dataCache[uid];
            return typeof callback === "function" ? callback(dataObject, null) : void 0;
          };
          reader.onerror = function(e) {
            console.log("ERROR");
            console.log(e);
            return typeof callback === "function" ? callback(null, e) : void 0;
          };
          return reader.readAsText(new Blob([buffer]));
        };
      })(this), (function(_this) {
        return function(e) {
          console.log("ERROR");
          console.log(e);
          return typeof callback === "function" ? callback(null, e) : void 0;
        };
      })(this));
    };


    /**
    * Loads a data file asynchronously.
    * @method load
    * @param {string} uid The UID of the data-file to load.
    * @param {Function} callback A callback function called if the loading-process has been finish. It
    * has the following signature:
    * <pre><code>
    * // dataObject - A JavaScript Object which is the parsed JSON from the Data file or <b>null</b> if an error occurred.
    * // error - The error object or <b>null</b> if no error occurred.
    * function(dataObject, error) {}
    * </code></pre>
     */

    Data.load = function(uid, callback) {
      if ($PARAMS.testOffline != null) {
        return this._loadTest(uid, callback);
      } else {
        return this._loadRelease(uid, callback);
      }
    };


    /**
    * Loads a file as raw text asynchronously.
    * @method loadRaw
    * @param {string} filePath The path/url to the file to load.
    * @param {Function} callback A callback function called if the loading-process has been finish. It
    * has the following signature:
    * <pre><code>
    * // text - The content of the file as text or <b>null</b> if an error occurred.
    * // error - The error object or <b>null</b> if no error occurred.
    * function(dataObject, error) {}
    * </code></pre>
     */

    Data.loadRaw = function(filePath, callback) {
      return gs.Application.loadData(filePath, (function(_this) {
        return function(data) {
          var buffer, reader;
          buffer = GS.DataPreparer.prepare(data);
          reader = new FileReader();
          reader.onload = function(e) {
            return typeof callback === "function" ? callback(e.target.result, null) : void 0;
          };
          reader.onerror = function(e) {
            console.log("ERROR");
            console.log(e);
            return typeof callback === "function" ? callback(null, e) : void 0;
          };
          return reader.readAsText(new Blob([buffer]));
        };
      })(this), (function(_this) {
        return function(e) {
          console.log("ERROR");
          console.log(e);
          return typeof callback === "function" ? callback(null, e) : void 0;
        };
      })(this));
    };

    return Data;

  })();

  gs.Data = Data;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var BinaryData, Mp4Data, OggData,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  BinaryData = (function() {
    function BinaryData(data) {
      this.data = data;
    }

    BinaryData.prototype.readString = function(data, index, length) {
      var i, j, ref, result;
      result = "";
      for (i = j = 0, ref = length; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
        result += String.fromCharCode(data[index + i]);
      }
      return result;
    };

    BinaryData.prototype.readNum32LE = function(data, index) {
      return data[index + 3] * 0x1000000 + data[index + 2] * 0x10000 + data[index + 1] * 0x100 + data[index + 0];
    };

    BinaryData.prototype.readNum32BE = function(data, index) {
      return data[index + 0] * 0x1000000 + data[index + 1] * 0x10000 + data[index + 2] * 0x100 + data[index + 3];
    };

    return BinaryData;

  })();

  OggData = (function(superClass) {
    extend(OggData, superClass);

    function OggData(data) {
      OggData.__super__.constructor.call(this, data);
      this.loopStart = 0;
      this.loopLength = 0;
      this.sampleRate = 0;
      this.parse(data);
    }

    OggData.prototype.parse = function(data) {
      var ex, headerType, i, index, j, k, numSegments, ref, ref1, segments, vorbisHeaderFound;
      try {
        index = 0;
        while (index < data.length) {
          if (this.readString(data, index, 4) === "OggS") {
            index += 26;
            vorbisHeaderFound = false;
            numSegments = data[index++];
            segments = [];
            for (i = j = 0, ref = numSegments; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
              segments.push(data[index++]);
            }
            for (i = k = 0, ref1 = numSegments; 0 <= ref1 ? k < ref1 : k > ref1; i = 0 <= ref1 ? ++k : --k) {
              if (this.readString(data, index + 1, 4) === "vorb") {
                headerType = data[index];
                if (headerType === 1) {
                  this.sampleRate = this.readNum32LE(data, index + 12);
                } else if (headerType === 3) {
                  this.readLoopData(data, index, segments[i]);
                }
                vorbisHeaderFound = true;
              }
              index += segments[i];
            }
            if (!vorbisHeaderFound) {
              break;
            }
          } else {
            break;
          }
        }
      } catch (error) {
        ex = error;
        console.log("ERROR PARSING OGG META DATA");
        console.log(ex);
      }
      return null;
    };

    OggData.prototype.readLoopData = function(data, index, size) {
      var i, j, ref, ref1, results, text;
      results = [];
      for (i = j = ref = index, ref1 = index + size - 10; ref <= ref1 ? j < ref1 : j > ref1; i = ref <= ref1 ? ++j : --j) {
        if (this.readString(data, i, 4) === "LOOP") {
          text = "";
          while (data[i] > 0) {
            text += String.fromCharCode(data[i++]);
          }
          if (text.match(/LOOPSTART=([0-9]+)/)) {
            this.loopStart = parseInt(RegExp.$1);
          }
          if (text.match(/LOOPLENGTH=([0-9]+)/)) {
            results.push(this.loopLength = parseInt(RegExp.$1));
          } else {
            results.push(void 0);
          }
        } else {
          results.push(void 0);
        }
      }
      return results;
    };

    return OggData;

  })(BinaryData);

  Mp4Data = (function(superClass) {
    extend(Mp4Data, superClass);

    function Mp4Data(data) {
      Mp4Data.__super__.constructor.call(this, data);
      this.loopStart = 0;
      this.loopLength = 0;
      this.sampleRate = 0;
      this.parse(data);
    }

    Mp4Data.prototype.parse = function(data) {
      var ex, index, name, results, size;
      try {
        if (this.readString(data, 4, 4) === "ftyp") {
          index = 0;
          results = [];
          while (index < data.length) {
            size = this.readNum32BE(data, index);
            name = this.readString(data, index + 4, 4);
            if (name === "moov") {
              results.push(index += 8);
            } else {
              if (name === "mvhd") {
                this.sampleRate = this.readNum32BE(data, index + 20);
              }
              if (name === "udta" || name === "meta") {
                this.readLoopData(data, index, size);
              }
              index += size;
              if (size <= 1) {
                break;
              } else {
                results.push(void 0);
              }
            }
          }
          return results;
        }
      } catch (error) {
        ex = error;
        console.log("ERROR PARSING MP4/M4A META DATA");
        return console.log(ex);
      }
    };

    Mp4Data.prototype.readLoopData = function(data, index, size) {
      var i, j, name, ref, ref1, results, value;
      results = [];
      for (i = j = ref = index, ref1 = index + size - 10; ref <= ref1 ? j < ref1 : j > ref1; i = ref <= ref1 ? ++j : --j) {
        if (this.readString(data, i, 4) === "LOOP") {
          name = "";
          while (data[i] > 0) {
            name += String.fromCharCode(data[i++]);
          }
          if (name === "LOOPSTART" || name === "LOOPLENGTH") {
            value = "";
            i += 16;
            while (data[i] > 0) {
              value += String.fromCharCode(data[i++]);
            }
            if (name === "LOOPSTART") {
              results.push(this.loopStart = parseInt(value));
            } else {
              results.push(this.loopLength = parseInt(value));
            }
          } else {
            results.push(void 0);
          }
        } else {
          results.push(void 0);
        }
      }
      return results;
    };

    return Mp4Data;

  })(BinaryData);

  gs.BinaryData = BinaryData;

  gs.OggData = OggData;

  gs.Mp4Data = Mp4Data;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var Input, MouseInput;

  MouseInput = (function() {

    /**
    * This class gives access to mouse or touch input. Mouse-buttons or mouse-position is automatically
    * mapped to touch-input on touch-devices like smartphones. For example: A single tap counts as
    * a left mouse-click and a double-tap counts as a right mouse-click.
    *
    *
    * @namespace gs
    * @class MouseInput
    * @static
    * @final
    * @memberof gs
    * @constructor
     */
    function MouseInput() {
      this.LEFT = 1;
      this.MIDDLE = 2;
      this.RIGHT = 3;
      this.BUTTON_LEFT = 1;
      this.BUTTON_MIDDLE = 2;
      this.BUTTON_RIGHT = 3;
      this.DIRECTION_NONE = 0;
      this.DIRECTION_HORIZONTAL = 1;
      this.DIRECTION_VERTICAL = 2;
      this.DIRECTION_BOTH = 3;

      /**
      * Stores the state of the mouse buttons.
      * @property buttons
      * @static
      * @readOnly
      * @type Uint8Array
       */
      this.buttons = new Uint8Array(new ArrayBuffer(4));

      /**
      * The current state of the mouse-wheel. Its the wheel-delta.
      * @property wheel
      * @type number
      * @readOnly
      * @static
       */
      this.wheel = 0;

      /**
      * The x-coordinate of the mouse/touch pointer position.
      * @property x
      * @type number
      * @static
       */
      this.x = 0;

      /**
      * The y-coordinate of the mouse/touch pointer position.
      * @property y
      * @type number
      * @static
       */
      this.y = 0;

      /**
      * Indicates if the mouse-input system is initialized.
      * @property initialized
      * @type boolean
      * @static
      * @readOnly
       */
      this.initialized = false;
      this.hammer = null;
      this.screen = null;

      /**
      * Indicates if mouse-move should be supported for touch-devices.
      * @property mouseMoveOnTouchDevices
      * @default false
      * @type boolean
      * @static
       */
      this.mouseMoveOnTouchDevices = 0;

      /**
      * Indicates if the mouse/touch pointer moved.
      * @property moved
      * @type boolean
      * @static
      * @readOnly
       */
      this.moved = false;

      /**
      * Indicates if a mouse-button is currently down.
      * @property buttonDown
      * @type boolean
      * @static
      * @readOnly
       */
      this.buttonDown = false;

      /**
      * Indicates if a mouse-button was held down before and is now up.
      * @property buttonUp
      * @type boolean
      * @static
      * @readOnly
       */
      this.buttonUp = false;

      /**
      * Indicates if the mouse-wheel state has changed.
      * @property wheelChanged
      * @type boolean
      * @static
      * @readOnly
       */
      this.wheelChanged = false;
      this.lx = 0;
      this.ly = 0;
    }


    /**
    * Initializes the mouse/touch system. Needs to be called before any mouse/touch input can be
    * received.
    *
    * @method initialize
    * @static
     */

    MouseInput.prototype.initialize = function() {
      Graphics.canvas.addEventListener("scroll", (function(_this) {
        return function(e) {
          return e.preventDefault();
        };
      })(this));
      window.addEventListener("scroll", (function(_this) {
        return function(e) {
          return e.preventDefault();
        };
      })(this));
      window.addEventListener("touchmove", (function(_this) {
        return function(e) {
          return e.preventDefault();
        };
      })(this));
      window.addEventListener("touchstart", (function(_this) {
        return function(e) {
          return e.preventDefault();
        };
      })(this));
      window.addEventListener("touchend", (function(_this) {
        return function(e) {
          return e.preventDefault();
        };
      })(this));
      window.document.addEventListener("touchmove", (function(_this) {
        return function(e) {
          return e.preventDefault();
        };
      })(this));
      window.document.addEventListener("touchstart", (function(_this) {
        return function(e) {
          return e.preventDefault();
        };
      })(this));
      window.document.addEventListener("touchend", (function(_this) {
        return function(e) {
          return e.preventDefault();
        };
      })(this));
      jQuery(Graphics.canvas).bind("touchmove", (function(_this) {
        return function(e) {
          var dx, dy, x, y;
          e = e.originalEvent;
          e.preventDefault();
          if (e.touches.length === 2) {
            dy = e.touches[0].clientY - _this.ly;
            dx = e.touches[0].clientX - _this.lx;
            x = Math.max(0, e.touches[0].clientX - Graphics.x);
            y = Math.max(0, e.touches[0].clientY - Graphics.y);
            _this.x = Math.round(x * Graphics.scaleFactorX);
            _this.y = Math.round(y * Graphics.scaleFactorY);
            _this.buttonDown = false;
            _this.buttons[1] = 0;
            if (dy > 0) {
              _this.wheel = -1;
            }
            if (dy < 0) {
              _this.wheel = 1;
            }
            _this.wheelSpeed = dy / 0.05;
            _this.wheelChanged = true;
          } else {
            x = Math.max(0, e.touches[0].clientX - Graphics.x);
            y = Math.max(0, e.touches[0].clientY - Graphics.y);
            _this.x = Math.round(x * Graphics.scaleFactorX);
            _this.y = Math.round(y * Graphics.scaleFactorY);
            _this.buttons[1] = 1;
            _this.buttonDown = true;
          }
          _this.ly = e.touches[0].clientY;
          _this.lx = e.touches[0].clientX;
          return false;
        };
      })(this));
      jQuery(Graphics.canvas).bind("touchstart", (function(_this) {
        return function(e) {
          var x, y;
          e.preventDefault();
          e.stopImmediatePropagation();
          e.stopPropagation();
          e = e.originalEvent;
          _this.ly = e.touches[0].clientY;
          _this.lx = e.touches[0].clientX;
          if (e.touches.length === 1) {
            x = Math.max(0, e.touches[0].clientX - Graphics.x);
            y = Math.max(0, e.touches[0].clientY - Graphics.y);
            _this.x = Math.round(x * Graphics.scaleFactorX);
            _this.y = Math.round(y * Graphics.scaleFactorY);
            _this.buttons[1] = 1;
            _this.buttonDown = true;
          }
          if (e.touches.length === 2) {
            _this.buttons[3] = 1;
            _this.buttonDown = true;
          }
          return false;
        };
      })(this));
      jQuery(Graphics.canvas).bind("touchend", (function(_this) {
        return function(e) {
          var x, y;
          e.preventDefault();
          e.stopImmediatePropagation();
          e.stopPropagation();
          e = e.originalEvent;
          if (e.changedTouches.length === 1) {
            x = Math.max(0, e.changedTouches[0].clientX - Graphics.x);
            y = Math.max(0, e.changedTouches[0].clientY - Graphics.y);
            _this.x = Math.round(x * Graphics.scaleFactorX);
            _this.y = Math.round(y * Graphics.scaleFactorY);
            _this.buttons[1] = 2;
            _this.buttonUp = true;
          }
          if (e.changedTouches.length === 3) {
            _this.buttons[3] = 2;
            _this.buttonUp = true;
          }
          _this.lx = 0;
          _this.ly = 0;
          return false;
        };
      })(this));
      if (!jQuery.browser.mobile) {
        if ($PARAMS.platform.isSteamOS) {
          this.screen = require("screen");
        } else {
          jQuery(Graphics.canvas).mousemove((function(_this) {
            return function(e) {
              _this.x = e.offsetX;
              _this.y = e.offsetY;
              return _this.moved = true;
            };
          })(this));
        }
        jQuery(document).bind("contextmenu", (function(_this) {
          return function(e) {
            e.preventDefault();
            e.stopPropagation();
            e.stopImmediatePropagation();
            return false;
          };
        })(this));
        Graphics.canvas.addEventListener("mousewheel", (function(_this) {
          return function(e) {
            _this.updateMouseWheel(e);
            return false;
          };
        })(this));
        jQuery(Graphics.canvas).mousedown((function(_this) {
          return function(e) {
            _this.buttons[e.which] = 1;
            e.preventDefault();
            e.stopPropagation();
            e.stopImmediatePropagation();
            _this.buttonDown = true;
            return false;
          };
        })(this));
        jQuery(Graphics.canvas).mouseup((function(_this) {
          return function(e) {
            _this.buttons[e.which] = 2;
            e.preventDefault();
            e.stopPropagation();
            e.stopImmediatePropagation();
            _this.buttonUp = true;
            return false;
          };
        })(this));
      }
      return this.initialized = true;
    };

    MouseInput.prototype.updateMouseWheel = function(e) {
      this.wheel = e.wheelDelta >= 0 ? 1 : -1;
      this.wheelSpeed = e.wheelDelta;
      return this.wheelChanged = true;
    };

    MouseInput.prototype.isButton = function(code) {
      var result;
      result = true;
      switch (code) {
        case this.LEFT:
          result = true;
          break;
        case this.MIDDLE:
          result = true;
          break;
        case this.RIGHT:
          result = true;
          break;
        default:
          result = false;
      }
      return result;
    };


    /**
    * Checks if the specified mouse button is pressed.
    *
    * @method press
    * @param {number} button - The button to check.
    * @return {boolean} If <b>true</b> the button is pressed. Otherwise <b>false</b>
    * @static
     */

    MouseInput.prototype.press = function(button) {
      return this.buttons[button] === 1;
    };


    /**
    * Updates the mouse/touch input-system. Needs to be called once per frame.
    *
    * @method update
    * @static
     */

    MouseInput.prototype.update = function() {
      var i, j, point, rect, ref;
      for (i = j = 0, ref = this.buttons.length; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
        if (this.buttons[i] === 2) {
          this.buttons[i] = 0;
        }
      }
      this.wheel = 0;
      this.wheelSpeed = 0;
      this.wheelChanged = false;
      this.buttonDown = false;
      this.buttonUp = false;
      this.moved = false;
      if (this.screen) {
        point = this.screen.getCursorScreenPoint();
        rect = Graphics.canvas.getClientRects()[0];
        this.x = Math.max(point.x - rect.left, 0);
        this.y = Math.max(point.y - rect.top, 0);
        this.x = Math.min(Math.round(this.x * Graphics.scaleFactorX), Graphics.width);
        this.y = Math.min(Math.round(this.y * Graphics.scaleFactorY), Graphics.height);
        if (this.x !== this.lx || this.y !== this.ly) {
          this.lx = this.x;
          this.ly = this.y;
          this.moved = true;
        }
      }
      return null;
    };


    /**
    * Clears the state of the mouse-buttons.
    *
    * @method clear
    * @static
     */

    MouseInput.prototype.clear = function() {
      var i, j, ref, results;
      results = [];
      for (i = j = 0, ref = this.buttons.length; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
        results.push(this.buttons[i] = 0);
      }
      return results;
    };

    return MouseInput;

  })();

  Input = (function() {

    /**
    * Indicates if the invisible virtual touch-buttons are enabled. For more info
    * about virtual buttons, see help-file.
    * @property touchButtonsEnabled
    * @type boolean
    * @default false
    * @static
     */
    Object.defineProperty(Input.prototype, "touchButtonsEnabled", {
      get: function() {
        return this.touchButtonsEnabled_;
      },
      set: function(value) {
        this.touchButtonsEnabled_ = value;
        if (value) {
          jQuery(window).bind("touchstart", this._onTouchStart);
          return jQuery(window).bind("touchend", this._onTouchEnd);
        } else {
          jQuery(window).unbind("touchstart", this._onTouchStart);
          return jQuery(window).unbind("touchend", this._onTouchEnd);
        }
      }
    });


    /**
    * This class manages the input system. It needs to be initializes before any input
    * data can be received. It supports keyboard, mouse, touch and gamepad input data.
    *
    * The input system has a set of buttons called LEFT, UP, RIGHT, DOWN, A, B, C, X, Y, Z, L, R,
    * START and SELECT which are emulated and available on most platforms. They are mapped
    * to the different input devices like to the keys of a keyboard or the buttons on a gamepad.
    *
    * @namespace gs
    * @class Input
    * @static
    * @final
    * @memberof gs
    * @constructor
     */

    function Input() {
      this.LEFT = 37;
      this.UP = 38;
      this.RIGHT = 39;
      this.DOWN = 40;
      this.A = 16;
      this.B = 88;
      this.C = 13;
      this.X = 65;
      this.Y = 83;
      this.Z = 68;
      this.L = 81;
      this.R = 87;
      this.START = this.C;
      this.SELECT = this.Z;
      this.KEY_CANCEL = 3;
      this.KEY_HELP = 6;
      this.KEY_BACK_SPACE = 8;
      this.KEY_TAB = 9;
      this.KEY_CLEAR = 12;
      this.KEY_RETURN = 13;
      this.KEY_ENTER = 14;
      this.KEY_SHIFT = 16;
      this.KEY_CONTROL = 17;
      this.KEY_ALT = 18;
      this.KEY_PAUSE = 19;
      this.KEY_CAPS_LOCK = 20;
      this.KEY_ESCAPE = 27;
      this.KEY_SPACE = 32;
      this.KEY_PAGE_UP = 33;
      this.KEY_PAGE_DOWN = 34;
      this.KEY_END = 35;
      this.KEY_HOME = 36;
      this.KEY_LEFT = 37;
      this.KEY_UP = 38;
      this.KEY_RIGHT = 39;
      this.KEY_DOWN = 40;
      this.KEY_PRINTSCREEN = 44;
      this.KEY_INSERT = 45;
      this.KEY_DELETE = 46;
      this.KEY_0 = 48;
      this.KEY_1 = 49;
      this.KEY_2 = 50;
      this.KEY_3 = 51;
      this.KEY_4 = 52;
      this.KEY_5 = 53;
      this.KEY_6 = 54;
      this.KEY_7 = 55;
      this.KEY_8 = 56;
      this.KEY_9 = 57;
      this.KEY_SEMICOLON = 59;
      this.KEY_EQUALS = 61;
      this.KEY_A = 65;
      this.KEY_B = 66;
      this.KEY_C = 67;
      this.KEY_D = 68;
      this.KEY_E = 69;
      this.KEY_F = 70;
      this.KEY_G = 71;
      this.KEY_H = 72;
      this.KEY_I = 73;
      this.KEY_J = 74;
      this.KEY_K = 75;
      this.KEY_L = 76;
      this.KEY_M = 77;
      this.KEY_N = 78;
      this.KEY_O = 79;
      this.KEY_P = 80;
      this.KEY_Q = 81;
      this.KEY_R = 82;
      this.KEY_S = 83;
      this.KEY_T = 84;
      this.KEY_U = 85;
      this.KEY_V = 86;
      this.KEY_W = 87;
      this.KEY_X = 88;
      this.KEY_Y = 89;
      this.KEY_Z = 90;
      this.KEY_CONTEXT_MENU = 93;
      this.KEY_NUMPAD0 = 96;
      this.KEY_NUMPAD1 = 97;
      this.KEY_NUMPAD2 = 98;
      this.KEY_NUMPAD3 = 99;
      this.KEY_NUMPAD4 = 100;
      this.KEY_NUMPAD5 = 101;
      this.KEY_NUMPAD6 = 102;
      this.KEY_NUMPAD7 = 103;
      this.KEY_NUMPAD8 = 104;
      this.KEY_NUMPAD9 = 105;
      this.KEY_MULTIPLY = 106;
      this.KEY_ADD = 107;
      this.KEY_SEPARATOR = 108;
      this.KEY_SUBTRACT = 109;
      this.KEY_DECIMAL = 110;
      this.KEY_DIVIDE = 111;
      this.KEY_F1 = 112;
      this.KEY_F2 = 113;
      this.KEY_F3 = 114;
      this.KEY_F4 = 115;
      this.KEY_F5 = 116;
      this.KEY_F6 = 117;
      this.KEY_F7 = 118;
      this.KEY_F8 = 119;
      this.KEY_F9 = 120;
      this.KEY_F10 = 121;
      this.KEY_F11 = 122;
      this.KEY_F12 = 123;
      this.KEY_F13 = 124;
      this.KEY_F14 = 125;
      this.KEY_F15 = 126;
      this.KEY_F16 = 127;
      this.KEY_F17 = 128;
      this.KEY_F18 = 129;
      this.KEY_F19 = 130;
      this.KEY_F20 = 131;
      this.KEY_F21 = 132;
      this.KEY_F22 = 133;
      this.KEY_F23 = 134;
      this.KEY_F24 = 135;
      this.KEY_NUM_LOCK = 144;
      this.KEY_SCROLL_LOCK = 145;
      this.KEY_COMMA = 188;
      this.KEY_PERIOD = 190;
      this.KEY_SLASH = 191;
      this.KEY_BACK_QUOTE = 192;
      this.KEY_OPEN_BRACKET = 219;
      this.KEY_BACK_SLASH = 220;
      this.KEY_CLOSE_BRACKET = 221;
      this.KEY_QUOTE = 222;
      this.KEY_META = 224;

      /**
      * Indicates if a key is currently pressed.
      * @property keyDown
      * @type boolean
      * @static
      * @readOnly
       */
      this.keyDown = false;

      /**
      * Indicates if a key is currently released.
      * @property keyUp
      * @type boolean
      * @static
      * @readOnly
       */
      this.keyUp = false;

      /**
      * Stores the state of the keyboard.
      * @property keys
      * @type Uint8Array
      * @static
      * @readOnly
       */
      this.keys = new Uint8Array(new ArrayBuffer(256));
      this.repeatedKeys = new Uint8Array(new ArrayBuffer(256));
      this.triggeredKeys = new Uint8Array(new ArrayBuffer(256));
      this.pressTimes = new Uint32Array(new ArrayBuffer(256 * 4));
      this.gamepadButtons = [14, 15, 12, 13, 0, 1, 2, 3, 0];
      this.gamepadKeys = [this.LEFT, this.RIGHT, this.UP, this.DOWN, this.A, this.B, this.X, this.Y, this.C];
      this.gamepadKeyStates = [0, 0, 0, 0, 0, 0, 0, 0, 0];

      /**
      * Gives direct access to the mouse/touch.
      * @property Mouse
      * @type gs.MouseInput
      * @static
      * @readOnly
       */
      this.Mouse = new MouseInput();

      /**
      * Gives direct access to the gamepad
      * @property Gamepad
      * @type gs.GamePad
      * @static
      * @readOnly
       */
      this.Gamepad = null;
    }

    Input.prototype._onTouchStart = function(e) {
      var h3, hh, qw;
      e.preventDefault();
      if (!this.touchButtonsEnabled) {
        return;
      }
      this.tsx = e.originalEvent.touches[0].clientX;
      this.tsy = e.originalEvent.touches[0].clientY;
      qw = window.innerWidth / 4;
      hh = window.innerHeight / 2;
      h3 = Math.round(hh / 3);
      if (this.tsx > qw * 3 && this.tsy > hh) {
        return this.keys[this.B] = 1;
      } else if (this.tsx > qw * 3 && this.tsy < hh) {
        return this.keys[this.X] = 1;
      } else if (this.tsx > qw * 2 && this.tsy > hh) {
        return this.keys[this.A] = 1;
      } else if (this.tsx > qw * 2 && this.tsy < hh) {
        return this.keys[this.Y] = 1;
      } else if (this.tsx < qw * 1 && this.tsy > hh && this.tsy < hh + h3) {
        this.keys[this.UP] = 1;
        return this.upFrameCount = Graphics.frameCount;
      } else if (this.tsx < qw * 1 && this.tsy > hh + h3 * 2) {
        this.keys[this.DOWN] = 1;
        return this.downFrameCount = Graphics.frameCount;
      } else if (this.tsx < qw * 1 && this.tsy > hh + h3 && this.tsy < hh + h3 * 2 && this.tsx < qw / 2) {
        this.keys[this.LEFT] = 1;
        return this.leftFrameCount = Graphics.frameCount;
      } else if (this.tsx < qw * 1 && this.tsy > hh + h3 && this.tsy < hh + h3 * 2 && this.tsx > qw / 2) {
        this.keys[this.RIGHT] = 1;
        return this.rightFrameCount = Graphics.frameCount;
      }
    };

    Input.prototype._onTouchEnd = function(e) {
      var h3, hh, qw;
      e.preventDefault();
      if (!this.touchButtonsEnabled) {
        return;
      }
      qw = window.innerWidth / 4;
      hh = window.innerHeight / 2;
      h3 = Math.round(hh / 3);
      if (this.tsx > qw * 3 && this.tsy > hh) {
        this.keys[this.B] = 2;
        return this.pressTimes[this.B] = 0;
      } else if (this.tsx > qw * 3 && this.tsy < hh) {
        this.keys[this.X] = 2;
        return this.pressTimes[this.X] = 0;
      } else if (this.tsx > qw * 2 && this.tsy > hh) {
        this.keys[this.A] = 2;
        return this.pressTimes[this.A] = 0;
      } else if (this.tsx > qw * 2 && this.tsy < hh) {
        this.keys[this.Y] = 2;
        return this.pressTimes[this.Y] = 0;
      } else if (this.tsx < qw * 1 && this.tsy > hh && this.tsy < hh + h3) {
        this.keys[this.UP] = 2;
        return this.pressTimes[this.UP] = 0;
      } else if (this.tsx < qw * 1 && this.tsy > hh + h3 * 2) {
        this.keys[this.DOWN] = 2;
        return this.pressTimes[this.DOWN] = 0;
      } else if (this.tsx < qw * 1 && this.tsy > hh + h3 && this.tsy < hh + h3 * 2 && this.tsx < qw / 2) {
        this.keys[this.LEFT] = 2;
        return this.pressTimes[this.LEFT] = 0;
      } else if (this.tsx < qw * 1 && this.tsy > hh + h3 && this.tsy < hh + h3 * 2 && this.tsx > qw / 2) {
        this.keys[this.RIGHT] = 2;
        return this.pressTimes[this.RIGHT] = 0;
      }
    };


    /**
    * Initializes the input-system. Must be called before any input data can be received.
    *
    * @method initialize
    * @static
     */

    Input.prototype.initialize = function() {
      if (jQuery.browser.mobile) {
        this.C = this.A;
        this.touchButtonsEnabled = false;
        this.isTouchDevice = true;
      }
      this.leftFrameCount = 0;
      this.rightFrameCount = 0;
      this.upFrameCount = 0;
      this.downFrameCount = 0;
      jQuery(document).keydown((function(_this) {
        return function(e) {
          if (Graphics.frozen) {
            return;
          }
          _this.keyDown = true;
          _this.keys[e.keyCode] = 1;
          if (e.keyCode === _this.LEFT) {
            _this.leftFrameCount = Graphics.frameCount;
          }
          if (e.keyCode === _this.RIGHT) {
            _this.rightFrameCount = Graphics.frameCount;
          }
          if (e.keyCode === _this.UP) {
            _this.upFrameCount = Graphics.frameCount;
          }
          if (e.keyCode === _this.DOWN) {
            return _this.downFrameCount = Graphics.frameCount;
          }
        };
      })(this));
      return jQuery(document).keyup((function(_this) {
        return function(e) {
          _this.keyUp = true;
          _this.keys[e.keyCode] = 2;
          return _this.pressTimes[e.keyCode] = 0;
        };
      })(this));

      /*
      if navigator.getGamepads?
        $(window).on("gamepadconnected",  () =>
          @Gamepad = navigator.getGamepads()[0]
          console.log("CONNECTED: " + @Gamepad.id)
        )
      
        $(window).on("gamepaddisconnected", () =>
          @Gamepad = null
          console.log("DISCONNECTED")
        )
      
        checkGP = window.setInterval(() =>
          if navigator.getGamepads()[0]?
            if not @Gamepad? then $(window).trigger("gamepadconnected")
            window.clearInterval(checkGP)
        , 500)
       */
    };


    /**
    * Checked if the specified key is pressed.
    *
    * @method press
    * @param {number} key - The key to check.
    * @return {boolean} If <b>true</b> the key is pressed. Otherwise <b>false</b>.
    * @static
     */

    Input.prototype.press = function(key) {
      return this.keys[key] === 1;
    };


    /**
    * Checked if the specified key is released.
    *
    * @method release
    * @param {number} key - The key to check.
    * @return {boolean} If <b>true</b> the key is released. Otherwise <b>false</b>.
    * @static
     */

    Input.prototype.release = function(key) {
      return this.keys[key] === 2;
    };


    /**
    * Checked if the specified key is triggered.
    *
    * @method trigger
    * @param {number} key - The key to check.
    * @return {boolean} If <b>true</b> the key is triggered. Otherwise <b>false</b>.
    * @static
     */

    Input.prototype.trigger = function(key) {
      return this.triggeredKeys[key] === 1;
    };


    /**
    * Checkes if the specified key is pressed. It checks if the user held down a key for certain
    * time and then returns <b>true</b> in fixed time intervals.
    *
    * @method repeat
    * @param {number} key - The key to check.
    * @return {boolean} Return <b>true</b> if the key is pressed. It checks then if the user held down a key for certain
    * time and then returns <b>true</b> in fixed time intervals.
    * @static
     */

    Input.prototype.repeat = function(key) {
      return this.repeatedKeys[key] === 1;
    };


    /**
    * Checkes if one of the 4 direction keys/buttons is pressed.
    *
    * @method dir4
    * @return {number} 0 = NONE, 2 = DOWN, 4 = LEFT, 6 = UP, 8 = RIGHT
    * @static
     */

    Input.prototype.dir4 = function() {
      if (this.keys[this.LEFT] === 1 && (jQuery.browser.mobile || (this.leftFrameCount > this.rightFrameCount && this.leftFrameCount > this.upFrameCount && this.leftFrameCount > this.downFrameCount))) {
        return 4;
      }
      if (this.keys[this.RIGHT] === 1 && (jQuery.browser.mobile || (this.rightFrameCount > this.leftFrameCount && this.rightFrameCount > this.upFrameCount && this.rightFrameCount > this.downFrameCount))) {
        return 8;
      }
      if (this.keys[this.UP] === 1 && (jQuery.browser.mobile || (this.upFrameCount > this.downFrameCount && this.upFrameCount > this.leftFrameCount && this.upFrameCount > this.rightFrameCount))) {
        return 6;
      }
      if (this.keys[this.DOWN] === 1 && (jQuery.browser.mobile || (this.downFrameCount > this.upFrameCount && this.downFrameCount > this.leftFrameCount && this.downFrameCount > this.rightFrameCount))) {
        return 2;
      }
      return 0;
    };


    /**
    * Checkes if one of the 8 direction keys/buttons is pressed.
    *
    * @method dir8
    * @return {number} 0 = NONE, 1 = DOWN-LEFT, 2 = DOWN, 3 = DOWN-RIGHT, 4 = LEFT, 5 = RIGHT-UP, 6 = UP, 7 = LEFT-UP, 8 = RIGHT
    * @static
     */

    Input.prototype.dir8 = function() {
      if (this.keys[this.LEFT] === 1 && this.keys[this.UP] === 1) {
        return 7;
      }
      if (this.keys[this.RIGHT] === 1 && this.keys[this.UP] === 1) {
        return 5;
      }
      if (this.keys[this.RIGHT] === 1 && this.keys[this.DOWN] === 1) {
        return 3;
      }
      if (this.keys[this.LEFT] === 1 && this.keys[this.DOWN] === 1) {
        return 1;
      }
      return this.dir4();
    };


    /**
    * Clears all input states.
    *
    * @method clear
    * @static
     */

    Input.prototype.clear = function() {
      var i, j, ref;
      for (i = j = 0, ref = this.keys.length; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
        this.keys[i] = 0;
        this.triggeredKeys[i] = 0;
        this.pressTimes[i] = 0;
      }
      if (this.Mouse.initialized) {
        return this.Mouse.clear();
      }
    };


    /**
    * Updates the input-system. Should be called once per frame.
    *
    * @method update
    * @static
     */

    Input.prototype.update = function() {
      var i, j, k, key, pressed, ref, ref1;
      this.keyDown = false;
      this.keyUp = false;
      if (this.Gamepad != null) {
        this.Gamepad = navigator.getGamepads()[0];
        for (i = j = 0, ref = this.gamepadKeys.length; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
          key = this.gamepadKeys[i];
          pressed = this.Gamepad.buttons[this.gamepadButtons[i]].pressed;
          if (!pressed) {
            switch (i) {
              case 0:
                pressed = Math.round(this.Gamepad.axes[0]) < 0;
                break;
              case 1:
                pressed = Math.round(this.Gamepad.axes[0]) > 0;
                break;
              case 2:
                pressed = Math.round(this.Gamepad.axes[1]) < 0;
                break;
              case 3:
                pressed = Math.round(this.Gamepad.axes[1]) > 0;
            }
          }
          if (this.gamepadKeyStates[i] !== 1 && pressed) {
            this.keys[key] = 1;
            this.gamepadKeyStates[i] = 1;
            if (key === this.LEFT) {
              this.leftFrameCount = Graphics.frameCount;
            }
            if (key === this.RIGHT) {
              this.rightFrameCount = Graphics.frameCount;
            }
            if (key === this.UP) {
              this.upFrameCount = Graphics.frameCount;
            }
            if (key === this.DOWN) {
              this.downFrameCount = Graphics.frameCount;
            }
          } else if (this.gamepadKeyStates[i] === 1 && !pressed) {
            this.gamepadKeyStates[i] = 0;
            this.keys[key] = 2;
            this.pressTimes[key] = 0;
          }
        }
      }
      if (this.Mouse.initialized) {
        this.Mouse.update();
      }
      for (i = k = 0, ref1 = this.keys.length; 0 <= ref1 ? k < ref1 : k > ref1; i = 0 <= ref1 ? ++k : --k) {
        this.repeatedKeys[i] = 0;
        if (this.keys[i] === 2) {
          this.keys[i] = 0;
        }
        if (this.keys[i] === 1) {
          this.pressTimes[i]++;
        }
        if (this.pressTimes[i] === 1) {
          this.triggeredKeys[i] = 1;
        } else {
          this.triggeredKeys[i] = 0;
        }
        if (this.pressTimes[i] === 1 || (this.pressTimes[i] > 20 && (this.pressTimes[i] & 3) === 0)) {
          this.repeatedKeys[i] = 1;
        } else {
          this.repeatedKeys[i] = 0;
        }
      }
      if (this.keys[this.LEFT] === 0) {
        this.leftFrameCount = 0;
      }
      if (this.keys[this.RIGHT] === 0) {
        this.rightFrameCount = 0;
      }
      if (this.keys[this.UP] === 0) {
        this.upFrameCount = 0;
      }
      if (this.keys[this.DOWN] === 0) {
        return this.downFrameCount = 0;
      }
    };

    return Input;

  })();

  window.Input = new Input();

  gs.Input = Input;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var AudioBuffer,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  AudioBuffer = (function(superClass) {
    extend(AudioBuffer, superClass);

    AudioBuffer.fromDataBundle = function(data, context) {
      var result;
      result = null;
      if (data.filePath) {
        result = ResourceManager.getAudioBuffer(data.filePath);
        result.volume = data.volume;
        result.playbackRate = data.playbackRate;
        result.currentTime = data.currentTime;
        if (data.isPlaying) {
          result.playIfReady = true;
        }
      }
      return result;
    };

    AudioBuffer.toDataBundle = function(object, context) {
      return {
        filePath: object.filePath,
        volume: object.volume,
        playbackRate: object.playbackRate,
        currentTime: object.currentTime,
        isPlaying: object.isPlaying
      };
    };


    /**
    * An audio buffer is used to load an audio track from file. The supported
    * audio formats are MP3, M4A, AAC, OGG and WAV but it depends on the platforms. It is
    * recommended to have at least two versions for each audio file like one file in M4A and the
    * other file in OGG format because some platforms can only play OGG files by default while others
    * can only play MP3/M4A.
    *
    *
    * @namespace gs
    * @class AudioBuffer
    * @memberof gs
    * @constructor
    * @param {string} filePath - The path to the audio resource.
    * @param {Object} [reserved=null] - Should be <b>null</b>
    * @param {string} [languageCode=null] - The language code to load the audio-file depending on language.
     */

    function AudioBuffer(filePath, storeWavFileInMemory, languageCode) {
      AudioBuffer.__super__.constructor.apply(this, arguments);
      this.filePath = filePath;
      this.languageCode = languageCode;

      /**
      * Reference to the audio-context.
      * @property context
      * @type Object
      * @readOnly
       */
      this.context = GS.Audio.context;

      /**
      * Indicates if the audio-buffer is loaded.
      * @property loaded
      * @type boolean
      * @readOnly
       */
      this.loaded = false;
      this.playing = false;
      this.source = null;
      this.encodedBuffer = null;
      this.decodedBuffer = null;
      this.playIfReady = false;

      /**
      * Indicates if the audio-buffer is decoded.
      * @property decoded
      * @type boolean
      * @see gs.AudioBuffer.decode
      * @readOnly
       */
      this.decoded = false;

      /**
      * Current fade-in time.
      * @property fadeInTime
      * @type number
      * @default 0
      * @readOnly
       */
      this.fadeInTime = 0;

      /**
      * Current fade-out time.
      * @property fadeOutTime
      * @type number
      * @default 0
      * @readOnly
       */
      this.fadeOutTime = 0;
      this.fadeInVolume = 0;

      /**
      * Current play-time limit in frames.
      * @property playTime
      * @type number
      * @default 0
      * @readOnly
       */
      this.playTime = 0;

      /**
      * An object which allows to store custom data.
      * @property customData
      * @type Object
      * @default {}
       */
      this.customData = {};

      /**
      * Indicates if the audio-buffer repeats after finished.
      * @property loop
      * @type boolean
      * @readOnly
       */
      this.loop = false;
      gs.Audio.addAudioBuffer(this);
      this.gainNode = this.context.createGain != null ? this.context.createGain() : this.context.createGainNode();
      filePath = gs.Application.getPlatformSpecificAudioFilePath(filePath, languageCode);
      gs.Application.loadData(filePath, (function(_this) {
        return function(data) {
          var mediaData;
          _this.encodedBuffer = GS.DataPreparer.prepare(data);
          mediaData = null;
          if (filePath.toLowerCase().endsWith(".ogg")) {
            mediaData = new gs.OggData(new Uint8Array(data));
          } else if (filePath.toLowerCase().endsWith(".m4a") || filePath.toLowerCase().endsWith(".mp4")) {
            mediaData = new gs.Mp4Data(new Uint8Array(data));
          }
          if ((mediaData != null ? mediaData.loopLength : void 0) > 0 && (mediaData != null ? mediaData.sampleRate : void 0) > 0) {
            _this.loopLength = mediaData.loopLength / mediaData.sampleRate;
            _this.loopStart = mediaData.loopStart / mediaData.sampleRate;
          }
          return _this.decode();
        };
      })(this), (function(_this) {
        return function(e) {
          _this.loaded = true;
          _this.error = true;
          _this.errorObject = e;
          console.log("AUDIO ERROR");
          return console.log(e);
        };
      })(this));
    }


    /**
    * The volume. Goes from 0 to 100.
    * @property volume
    * @type number
     */

    Object.defineProperty(AudioBuffer.prototype, "volume", {
      set: function(value) {
        if (this.error) {
          return;
        }
        if (this.context != null) {
          return this.gainNode.gain.value = value;
        } else {
          return this.source.volume = value;
        }
      },
      get: function() {
        var ref, ref1;
        if (this.error) {
          return 0;
        }
        if (this.context != null) {
          return (ref = (ref1 = this.gainNode) != null ? ref1.gain.value : void 0) != null ? ref : 100;
        } else {
          return this.source.volume;
        }
      }
    });


    /**
    * Indicates if the audio buffer is currently playing.
    * @property isPlaying
    * @type boolean
    * @readOnly
     */

    Object.defineProperty(AudioBuffer.prototype, "isPlaying", {
      get: function() {
        if (this.context != null) {
          return this.playing;
        } else {
          return !this.source.paused;
        }
      }
    });

    Object.defineProperty(AudioBuffer.prototype, "loop", {
      get: function() {
        return this.loop_;
      },
      set: function(v) {
        var ref, ref1, ref2;
        if (this.loop_ !== v) {
          this.loop_ = v;
          if ((ref = this.source) != null) {
            ref.loop = this.loop;
          }
          if (this.loopLength > 0 && this.loop) {
            if ((ref1 = this.source) != null) {
              ref1.loopStart = this.loopStart;
            }
            return (ref2 = this.source) != null ? ref2.loopEnd = this.loopStart + this.loopLength : void 0;
          }
        }
      }
    });


    /**
    * The current play time.
    * @property currentTime
    * @type number
     */

    Object.defineProperty(AudioBuffer.prototype, "currentTime", {
      get: function() {
        var ref;
        return ((ref = this.source) != null ? ref.currentTime : void 0) || 0;
      },
      set: function(v) {
        var ref;
        return (ref = this.source) != null ? ref.currentTime = v : void 0;
      }
    });


    /**
    * The play-back rate in percent.
    * @property playbackRate
    * @type number
    * @readOnly
     */

    Object.defineProperty(AudioBuffer.prototype, "playbackRate", {
      set: function(value) {
        if (this.source) {
          if (this.context) {
            return this.source.playbackRate.value = value;
          } else {
            return this.source.playbackRate = value;
          }
        }
      },
      get: function() {
        if (this.source) {
          if (this.context) {
            return this.source.playbackRate.value;
          } else {
            return this.source.playbackRate;
          }
        } else {
          return 100;
        }
      }
    });

    AudioBuffer.prototype.bufferToBase64 = function(buffer) {
      var binary, bytes, i, j, len, ref;
      binary = '';
      bytes = new Uint8Array(buffer);
      len = bytes.byteLength;
      for (i = j = 0, ref = len; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
        binary += String.fromCharCode(bytes[i]);
      }
      return window.btoa(binary);
    };


    /**
    * Disposes the audio buffer and frees the resources/memory. Should be called
    * if the audio buffer is no longer needed.
    *
    * @method dispose
     */

    AudioBuffer.prototype.dispose = function() {
      var ref, ref1;
      if (!this.disposed) {
        this.stop();
        this.disposed = true;
        gs.Audio.removeAudioBuffer(this);
        if (((ref = this.source) != null ? ref.numberOfOutputs : void 0) > 0) {
          this.source.disconnect();
        }
        if (((ref1 = this.gainNode) != null ? ref1.numberOfOutputs : void 0) > 0) {
          this.gainNode.disconnect();
        }
        this.gainNode = null;
        this.encodedBuffer = null;
        this.decodedBuffer = null;
        this.source = null;
        return this.decoded = false;
      }
    };


    /**
    * Decodes the audio data if necessary. Needs to be called once before
    * the audio buffer can be played.
    *
    * @method decode
    * @see gs.AudioBuffer.onFinishDecode
     */

    AudioBuffer.prototype.decode = function() {
      if (this.context != null) {
        if (this.decoded) {
          if (this.onFinishDecode != null) {
            this.onFinishDecode(this);
          }
          this.emit("finishDecode");
          return;
        }
        try {
          return this.context.decodeAudioData(this.encodedBuffer, (function(_this) {
            return function(buffer) {
              _this.decodedBuffer = buffer;
              _this.loaded = true;
              _this.source = _this.context.createBufferSource();
              _this.source.buffer = _this.decodedBuffer;
              _this.source.onended = function() {
                _this.playing = false;
                if (_this.onEnd != null) {
                  return _this.onEnd(_this);
                }
              };
              _this.decoded = true;
              if (_this.onFinishDecode != null) {
                _this.onFinishDecode(_this);
              }
              return _this.emit("finishDecode");
            };
          })(this), (function(_this) {
            return function(e) {
              _this.loaded = true;
              _this.decoded = true;
              _this.error = true;
              console.log("DECODE ERROR");
              return console.log(e);
            };
          })(this));
        } catch (error) {
          this.loaded = true;
          return this.decoded = true;
        }
      } else {
        this.decoded = true;
        if (this.onFinishDecode != null) {
          this.onFinishDecode(this);
        }
        return this.emit("finishDecode");
      }
    };


    /**
    * Updates the audio-buffer. Needs to be called once per frame.
    *
    * @method update
     */

    AudioBuffer.prototype.update = function() {
      if (this.playIfReady && this.loaded) {
        this.playIfReady = false;
        this.play();
      }
      if (this.playTime > 0 && this.playing) {
        this.playTime--;
        if (this.playTime <= this.fadeOutTime) {
          this.fadeInTime = this.playTime;
          this.playTime = 0;
          this.fadeInVolume = -(this.volume / this.fadeInTime);
        }
      }
      if (this.fadeInTime > 0 && (!this.error)) {
        this.fadeInTime--;
        this.volume = Math.max(Math.min(this.volume + this.fadeInVolume, 1.0), 0);
        if (this.fadeInTime <= 0) {
          if (this.fadeInVolume < 0 && (this.onFadeOutEnd != null)) {
            this.onFadeOutEnd(this);
          }
          if (this.fadeInVolume > 0 && (this.onFadeInEnd != null)) {
            return this.onFadeInEnd(this);
          }
        }
      }
    };


    /**
    * Plays the audio buffer.
    *
    * @method play
    * @param {number} [fadeInTime=0] - The fade-in time in frames.
    * @param {number} [playTime=null] - The play-time in frames.
    * @param {number} [fadeOutTime=0] - The fade-out time in frames if play-time is over.
     */

    AudioBuffer.prototype.play = function(fadeInTime, playTime, fadeOutTime) {
      var rate, ref, ref1;
      if (this.error) {
        return;
      }
      if (this.context) {
        this.playTime = playTime;
        this.fadeInTime = fadeInTime || 0;
        this.fadeOutTime = fadeOutTime || 0;
        if (this.fadeInTime > 0) {
          this.fadeInVolume = this.volume / this.fadeInTime;
          this.volume = 0;
        }
        this.playing = true;
        rate = (ref = (ref1 = this.source) != null ? ref1.playbackRate.value : void 0) != null ? ref : 1.0;
        this.source = this.context.createBufferSource();
        this.source.buffer = this.decodedBuffer;
        this.source.loop = this.loop;
        if (this.loopLength > 0 && this.loop) {
          this.source.loopStart = this.loopStart;
          this.source.loopEnd = this.loopStart + this.loopLength;
        }
        this.source.onended = (function(_this) {
          return function() {
            _this.playing = false;
            if (_this.onEnd != null) {
              _this.onEnd(_this);
            }
            if (_this.loop) {
              return _this.play();
            }
          };
        })(this);
        this.source.connect(this.gainNode);
        this.gainNode.connect(this.context.destination);
        this.source.playbackRate.value = rate;
        this.startTime = this.context.currentTime;
        if (this.source.noteOn != null) {
          return this.source.noteOn(0);
        } else {
          return this.source.start(0);
        }
      } else {
        return this.source.play();
      }
    };


    /**
    * Resumes the audio buffer if paused.
    *
    * @method resume
    * @param {number} [fadeInTime=0] - The fade-in time in frames.
     */

    AudioBuffer.prototype.resume = function(fadeInTime) {
      if (this.error) {
        return;
      }
      if (this.context) {
        this.fadeInTime = fadeInTime || 0;
        if (this.fadeInTime > 0) {
          this.fadeInVolume = this.volume / this.fadeInTime;
          this.volume = 0;
        }
        this.playing = true;
        this.source = this.context.createBufferSource();
        this.source.loop = this.loop;
        if (this.loopLength > 0 && this.loop) {
          this.source.loopStart = this.loopStart;
          this.source.loopEnd = this.loopStart + this.loopLength;
        }
        this.source.onended = (function(_this) {
          return function() {
            _this.playing = false;
            if (_this.onEnd != null) {
              return _this.onEnd(_this);
            }
          };
        })(this);
        this.source.buffer = this.decodedBuffer;
        this.source.connect(this.gainNode);
        if ((this.stopTime != null) && (this.startTime != null)) {
          return this.source.start(0, this.stopTime - this.startTime);
        } else {
          return this.source.start(0);
        }
      } else {
        return this.source.paused = false;
      }
    };


    /**
    * Stops the audio buffer.
    *
    * @method resume
    * @param {number} [fadeOutTime=0] - The fade-out time in frames.
     */

    AudioBuffer.prototype.stop = function(fadeOutTime) {
      if (this.error) {
        return;
      }
      this.fadeInTime = fadeOutTime || 0;
      if (this.fadeInTime > 0) {
        this.fadeInVolume = -(this.volume / this.fadeInTime);
        return this.onFadeOutEnd = (function(_this) {
          return function() {
            if (_this.source != null) {
              if (_this.context != null) {
                _this.stopTime = _this.context.currentTime;
                if (_this.playing) {
                  _this.source.onended = null;
                  if (_this.source.noteOff != null) {
                    _this.source.noteOff(0);
                  } else {
                    _this.source.stop(0);
                  }
                  if (_this.source.numberOfOutputs > 0) {
                    _this.source.disconnect();
                  }
                  return _this.playing = false;
                }
              } else {
                return _this.source.paused = true;
              }
            }
          };
        })(this);
      } else {
        if (this.source != null) {
          if (this.context != null) {
            this.stopTime = this.context.currentTime;
            if (this.playing) {
              if (this.source.noteOff != null) {
                this.source.noteOff(0);
              } else {
                this.source.stop(0);
              }
              if (this.source.numberOfOutputs > 0) {
                this.source.disconnect();
              }
              return this.playing = false;
            }
          } else {
            return this.source.paused = true;
          }
        }
      }
    };

    AudioBuffer.prototype.toDataURL = function() {
      return this.source.src || "";
    };

    return AudioBuffer;

  })(gs.EventEmitter);

  window.GS.AudioBuffer = AudioBuffer;

  gs.AudioBuffer = AudioBuffer;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var Audio;

  Audio = (function() {

    /**
    * This class manages the basic audio system.
    *
    *
    * @namespace gs
    * @class Audio
    * @static
    * @final
    * @memberof gs
    * @constructor
     */
    function Audio() {

      /**
      * The audio context. It depends on the platform but in most cases it is an HTML5 Web-Audio context.
      * Accessing the context directly is not recommended. It should only be done if really necessary.
      * @property context
      * @type Object
       */
      var ex;
      this.context = null;
      this.gainNode = null;
      this.audioBuffers = [];
      try {
        if (window.webkitAudioContext != null) {
          this.context = new webkitAudioContext();
        } else if (window.mozAudioContext != null) {
          this.context = new mozAudioContext();
        } else if (window.oAudioContext != null) {
          this.context = new oAudioContext();
        } else {
          this.context = new AudioContext();
        }
        this.gainNode = this.context.createGain != null ? this.context.createGain() : this.context.createGainNode();
      } catch (error) {
        ex = error;
        this.context = null;
      }
    }

    Audio.prototype.addAudioBuffer = function(buffer) {
      return this.audioBuffers.push(buffer);
    };

    Audio.prototype.removeAudioBuffer = function(buffer) {
      return this.audioBuffers.remove(buffer);
    };

    Audio.prototype.pause = function() {
      var buffer, ex, i, len, ref, results;
      ref = this.audioBuffers;
      results = [];
      for (i = 0, len = ref.length; i < len; i++) {
        buffer = ref[i];
        if (buffer != null) {
          buffer.customData.wasPlaying = buffer.isPlaying;
          try {
            if (buffer.isPlaying) {
              results.push(buffer.stop(0));
            } else {
              results.push(void 0);
            }
          } catch (error) {
            ex = error;
            results.push(console.warn(ex));
          }
        } else {
          results.push(void 0);
        }
      }
      return results;
    };

    Audio.prototype.resume = function() {
      var buffer, i, len, ref, results;
      ref = this.audioBuffers;
      results = [];
      for (i = 0, len = ref.length; i < len; i++) {
        buffer = ref[i];
        if ((buffer != null) && buffer.customData.wasPlaying) {
          results.push(buffer != null ? buffer.resume(0) : void 0);
        } else {
          results.push(void 0);
        }
      }
      return results;
    };

    return Audio;

  })();

  window.GS.Audio = new Audio();

  gs.Audio = GS.Audio;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var AudioBufferReference;

  AudioBufferReference = (function() {
    AudioBufferReference.fromDataBundle = function(data, context) {
      var result;
      result = null;
      if (data.audioBuffer) {
        gs.ObjectCodec.decodeProperties(data, context);
        result = new gs.AudioBufferReference(null, data.useAnalyser);
        if (!data.audioBuffer.decoded) {
          data.audioBuffer.once("finishDecode", {
            audioBuffer: data.audioBuffer,
            reference: result
          }, function(e) {
            var ref;
            if ((ref = e.data.audioBuffer) != null ? ref.decoded : void 0) {
              e.data.reference.audioBuffer = e.data.audioBuffer;
              e.data.reference.initialize();
              if (e.data.reference.isPlaying) {
                return e.data.reference.play();
              }
            }
          });
        } else {
          result.audioBuffer = data.audioBuffer;
          result.initialize();
          result.volume = data.volume;
          result.playbackRate = data.playbackRate;
          if (data.isPlaying) {
            result.play();
          }
        }
        result.customData = data.customData;
      }
      return result;
    };

    AudioBufferReference.toDataBundle = function(object, context) {
      debugger;
      var r;
      r = {
        audioBuffer: object.audioBuffer,
        customData: object.customData,
        useAnalyzer: object.useAnalyzer,
        volume: object.volume,
        playbackRate: object.playbackRate,
        isPlaying: object.isPlaying
      };
      return gs.ObjectCodec.encodeProperties(r, context, null);
    };


    /**
    * An audio buffer reference is a reference to an already existing audio buffer. It
    * allows to play the audio buffer independent from other references. This is useful
    * for sound-effects which are played very often and quickly at the same time.
    *
    *
    * @namespace gs
    * @class AudioBufferReference
    * @memberof gs
    * @constructor
    * @param {gs.AudioBuffer} audioBuffer - The audio buffer to create a reference to.
    * @param {boolean} [useAnalyser=false] - If <b>true</b> the sound-data is analyzed in real-time to
    * get data like the average-volume.
     */

    function AudioBufferReference(audioBuffer, useAnalyser) {

      /**
      * Reference to the audio-context.
      * @property context
      * @type Object
      * @readOnly
       */
      this.context = GS.Audio.context;

      /**
      * Reference to the audio-buffer.
      * @property audioBuffer
      * @type gs.AudioBuffer
      * @readOnly
       */
      this.audioBuffer = audioBuffer;
      this.playing = false;
      this.source = null;

      /**
      * An object which allows to store custom data.
      * @property customData
      * @type Object
      * @default {}
       */
      this.customData = {};

      /**
      * Indicates if the sound is analyzed in real-time to get data like the average-volume.
      * @property useAnalyser
      * @default false
      * @type boolean
      * @readOnly
       */
      this.useAnalyser = useAnalyser != null ? useAnalyser : false;
      this.startTime = 0;
      this.stopTime = 0;
      gs.Audio.addAudioBuffer(this);
      if (this.audioBuffer != null) {
        this.initialize();
      }
    }


    /**
    * The volume. Goes from 0 to 100.
    * @property volume
    * @type number
     */

    Object.defineProperty(AudioBufferReference.prototype, "volume", {
      set: function(value) {
        if (this.audioBuffer.error) {
          return;
        }
        if (this.context != null) {
          return this.gainNode.gain.value = value;
        } else {
          return this.source.volume = value;
        }
      },
      get: function() {
        if (this.audioBuffer.error) {
          return 100;
        }
        if (this.context != null) {
          return this.gainNode.gain.value;
        } else {
          return this.source.volume;
        }
      }
    });


    /**
    * Indicates if the audio buffer is currently playing.
    * @property isPlaying
    * @type boolean
    * @readOnly
     */

    Object.defineProperty(AudioBufferReference.prototype, "isPlaying", {
      get: function() {
        if (this.audioBuffer.error) {
          return false;
        }
        if (this.context != null) {
          return this.playing;
        } else {
          return !this.source.paused;
        }
      }
    });


    /**
    * The play-back rate in percent.
    * @property playbackRate
    * @type number
    * @readOnly
     */

    Object.defineProperty(AudioBufferReference.prototype, "playbackRate", {
      set: function(value) {
        var ref, ref1;
        if (this.audioBuffer.error) {
          return;
        }
        if (this.context) {
          return (ref = this.source) != null ? ref.playbackRate.value = value : void 0;
        } else {
          return (ref1 = this.source) != null ? ref1.playbackRate = value : void 0;
        }
      },
      get: function() {
        var ref;
        if (this.audioBuffer.error) {
          return;
        }
        if (this.context) {
          return ((ref = this.source) != null ? ref.playbackRate.value : void 0) || 100;
        } else {
          return this.source.playbackRate;
        }
      }
    });

    AudioBufferReference.prototype._getAverageVolume = function(array) {
      var average, i, j, length, ref, values;
      values = 0;
      average = 0;
      length = array.length;
      for (i = j = 0, ref = length; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
        values += array[i];
      }
      average = values / length;
      return average;
    };

    AudioBufferReference.prototype.initialize = function() {
      if (!this.audioBuffer.error) {
        if (this.useAnalyser) {
          this.analyser = this.context.createAnalyser();
          this.analyser.smoothingTimeConstant = 0.3;
          this.analyser.fftSize = 1024;
          this.averageVolume = 0;
          this.scriptProcessor = this.context.createScriptProcessor(2048, 1, 1);
          this.scriptProcessor.onaudioprocess = (function(_this) {
            return function() {
              var array;
              array = new Uint8Array(_this.analyser.frequencyBinCount);
              _this.analyser.getByteFrequencyData(array);
              return _this.averageVolume = _this._getAverageVolume(array);
            };
          })(this);
          this.scriptProcessor.connect(this.context.destination);
          this.analyser.connect(this.scriptProcessor);
        }
        if (this.context) {
          this.gainNode = this.context.createGain != null ? this.context.createGain() : this.context.createGainNode();
          this.source = this.context.createBufferSource();
          if (this.analyser != null) {
            this.source.connect(this.analyser);
          }
          this.source.buffer = this.audioBuffer.decodedBuffer;
          return this.source.onended = (function(_this) {
            return function() {
              _this.playing = false;
              if (_this.onEnd != null) {
                return _this.onEnd(_this);
              }
            };
          })(this);
        } else {
          this.source = new Audio();
          return this.source.src = this.audioBuffer.toDataURL();
        }
      }
    };


    /**
    * Disposes the audio buffer reference and frees the resources/memory. Should be called
    * if the reference is no longer needed. The associated audio buffer is <b>not</b> disposed.
    *
    * @method dispose
     */

    AudioBufferReference.prototype.dispose = function() {
      gs.Audio.removeAudioBuffer(this);
      return this.source = null;
    };


    /**
    * Plays the audio buffer reference.
    *
    * @method play
     */

    AudioBufferReference.prototype.play = function() {
      var buffer, rate;
      if (this.audioBuffer.error) {
        return;
      }
      if (this.context) {
        this.startTime = this.context.currentTime;
        this.playing = true;
        rate = this.source.playbackRate.value;
        buffer = this.source.buffer;
        this.source = this.context.createBufferSource();
        if (this.analyser != null) {
          this.source.connect(this.analyser);
        }
        this.source.buffer = buffer;
        this.source.onended = (function(_this) {
          return function() {
            _this.playing = false;
            if (_this.onEnd != null) {
              return _this.onEnd(_this);
            }
          };
        })(this);
        this.source.connect(this.gainNode);
        this.gainNode.connect(this.context.destination);
        this.source.playbackRate.value = rate;
        if (this.source.noteOn != null) {
          return this.source.noteOn(0);
        } else {
          return this.source.start(0);
        }
      } else {
        return this.source.play();
      }
    };

    AudioBufferReference.prototype.resume = function() {
      var buffer, rate;
      if (this.audioBuffer.error) {
        return;
      }
      if (this.context) {
        this.playing = true;
        rate = this.source.playbackRate.value;
        buffer = this.source.buffer;
        this.source = this.context.createBufferSource();
        if (this.analyser != null) {
          this.source.connect(this.analyser);
        }
        this.source.buffer = buffer;
        this.source.onended = (function(_this) {
          return function() {
            _this.playing = false;
            if (_this.onEnd != null) {
              return _this.onEnd(_this);
            }
          };
        })(this);
        this.source.connect(this.gainNode);
        this.gainNode.connect(this.context.destination);
        this.source.playbackRate.value = rate;
        if (this.source.noteOn != null) {
          return this.source.noteOn(0, this.stopTime - this.startTime);
        } else {
          return this.source.start(0, this.stopTime - this.startTime);
        }
      } else {
        return this.source.paused = false;
      }
    };


    /**
    * Stops the audio buffer reference.
    *
    * @method stop
     */

    AudioBufferReference.prototype.stop = function() {
      if (this.audioBuffer.error) {
        return;
      }
      if (this.source) {
        if (this.context) {
          if (this.playing) {
            this.stopTime = this.context.currentTime;
            if (this.source.noteOff != null) {
              this.source.noteOff(0);
            } else {
              this.source.stop(0);
            }
            if (this.source.numberOfOutputs > 0) {
              this.source.disconnect();
            }
            return this.playing = false;
          }
        } else {
          this.source.pause();
          return this.source.currentTime = 0;
        }
      }
    };

    return AudioBufferReference;

  })();

  window.GS.AudioBufferReference = AudioBufferReference;

  gs.AudioBufferReference = AudioBufferReference;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var AudioBufferStream;

  AudioBufferStream = (function() {
    AudioBufferStream.fromDataBundle = function(data, context) {
      var result;
      result = null;
      if (data.filePath) {
        result = new gs.AudioBufferStream(data.filePath, false, data.languageCode);
        result.volume = data.volume;
        result.playbackRate = data.playbackRate;
        result.currentTime = data.currentTime;
        if (data.isPlaying) {
          result.play();
          result.currentTime = data.currentTime;
        }
      }
      return result;
    };

    AudioBufferStream.toDataBundle = function(object, context) {
      return {
        filePath: object.filePath,
        volume: object.volume,
        playbackRate: object.playbackRate,
        currentTime: object.currentTime,
        isPlaying: object.isPlaying
      };
    };


    /**
    * An audio buffer stream is used to stream an audio track from file. The supported
    * audio formats are MP3, M4A, AAC, OGG and WAV but it depends on the platforms. It is
    * recommended to have at least two versions for each audio file like one file in M4A and the
    * other file in OGG format because some platforms can only play OGG files by default while others
    * can only play MP3/M4A.
    *
    *
    * @namespace gs
    * @class AudioBuffer
    * @memberof gs
    * @constructor
    * @param {string} filePath - The path to the audio resource.
    * @param {Object} [reserved=null] - Should be <b>null</b>
    * @param {string} [languageCode=null] - The language code to load the audio-file depending on language.
     */

    function AudioBufferStream(filePath, languageCode) {
      var oFilePath;
      this.filePath = filePath;
      this.languageCode = languageCode;

      /**
      * Reference to the audio-context.
      * @property context
      * @type Object
      * @readOnly
       */
      this.context = GS.Audio.context;

      /**
      * Indicates if the audio-buffer is loaded.
      * @property loaded
      * @type boolean
      * @readOnly
       */
      this.loaded = false;
      this.playing = false;
      this.source = null;
      this.encodedBuffer = null;
      this.decodedBuffer = null;

      /**
      * Indicates if the audio-buffer is decoded.
      * @property decoded
      * @type boolean
      * @see gs.AudioBuffer.decode
      * @readOnly
       */
      this.decoded = false;

      /**
      * Current fade-in time.
      * @property fadeInTime
      * @type number
      * @default 0
      * @readOnly
       */
      this.fadeInTime = 0;

      /**
      * Current fade-out time.
      * @property fadeOutTime
      * @type number
      * @default 0
      * @readOnly
       */
      this.fadeOutTime = 0;
      this.fadeInVolume = 0;

      /**
      * Current play-time limit in frames.
      * @property playTime
      * @type number
      * @default 0
      * @readOnly
       */
      this.playTime = 0;

      /**
      * An object which allows to store custom data.
      * @property customData
      * @type Object
      * @default {}
       */
      this.customData = {};

      /**
      * Indicates if the audio-buffer repeats after finished.
      * @property loop
      * @type boolean
      * @readOnly
       */
      this.loop = false;
      this.audioStream = null;
      gs.Audio.addAudioBuffer(this);
      oFilePath = gs.Application.getPlatformSpecificAudioFilePath(filePath, languageCode, true);
      filePath = gs.Application.getPlatformSpecificAudioFilePath(filePath, languageCode);
      this.audioStream = new Audio();
      this.audioStream.addEventListener("loadeddata", (function(_this) {
        return function() {
          return _this.loaded = true;
        };
      })(this));
      this.audioStream.canplaythrough = (function(_this) {
        return function() {
          return _this.loaded = true;
        };
      })(this);
      this.audioStream.addEventListener("error", (function(_this) {
        return function() {
          _this.loaded = true;
          return _this.error = true;
        };
      })(this));
      this.audioStream.addEventListener("ended", (function(_this) {
        return function() {
          var ref;
          if (_this.onEnd != null) {
            _this.onEnd(_this);
          }
          if (_this.loop) {
            _this.play();
            return _this.audioStream.currentTime = (ref = _this.loopStart) != null ? ref : 0;
          }
        };
      })(this));
      if (gs.Platform.isSafariMobile && gs.Platform.isCordova) {
        if ($PARAMS.needsPreparation) {
          gs.Application.loadData(filePath, (function(_this) {
            return function(data, type) {
              var reader;
              data = GS.DataPreparer.prepare(data);
              reader = new FileReader();
              reader.onload = function(e) {
                _this.audioStream.src = e.target.result;
                return _this.audioStream.load();
              };
              reader.onerror = onerror;
              return reader.readAsDataURL(new Blob([data], {
                type: type
              }));
            };
          })(this), (function(_this) {
            return function(e) {
              return onerror();
            };
          })(this), 0, "audio/" + oFilePath.substring(oFilePath.lastIndexOf(".") + 1));
        } else {
          gs.Application.loadData(filePath, (function(_this) {
            return function(data) {
              _this.audioStream.src = data;
              return _this.audioStream.load();
            };
          })(this), (function(_this) {
            return function(e) {
              _this.loaded = true;
              return _this.error = true;
            };
          })(this), 1);
        }
      } else {
        if ($PARAMS.needsPreparation || true) {
          gs.Application.loadData(filePath, (function(_this) {
            return function(data, type) {
              var reader;
              data = GS.DataPreparer.prepare(data);
              reader = new FileReader();
              reader.onload = function(e) {
                _this.audioStream.src = e.target.result;
                return _this.audioStream.load();
              };
              reader.onerror = onerror;
              return reader.readAsDataURL(new Blob([data], {
                type: type
              }));
            };
          })(this), (function(_this) {
            return function(e) {
              return onerror();
            };
          })(this), 0, "audio/" + oFilePath.substring(oFilePath.lastIndexOf(".") + 1));
        } else {
          this.audioStream.src = filePath;
          this.audioStream.load();
        }
      }
    }


    /**
    * The volume. Goes from 0 to 100.
    * @property volume
    * @type number
     */

    Object.defineProperty(AudioBufferStream.prototype, "volume", {
      set: function(value) {
        if (this.error) {
          return 0;
        }
        return this.audioStream.volume = value;
      },
      get: function() {
        if (this.error) {
          return 0;
        }
        return this.audioStream.volume;
      }
    });


    /**
    * The duration as double in seconds.
    * @property duration
    * @type number
     */

    Object.defineProperty(AudioBufferStream.prototype, "duration", {
      get: function() {
        if (this.error) {
          return 0;
        }
        return this.audioStream.duration;
      }
    });


    /**
    * The current time as double in seconds.
    * @property currentTime
    * @type number
     */

    Object.defineProperty(AudioBufferStream.prototype, "currentTime", {
      get: function() {
        if (this.error) {
          return 0;
        }
        return this.audioStream.currentTime;
      },
      set: function(v) {
        return this.audioStream.currentTime = v;
      }
    });


    /**
    * Indicates if the audio buffer is currently playing.
    * @property isPlaying
    * @type boolean
    * @readOnly
     */

    Object.defineProperty(AudioBufferStream.prototype, "isPlaying", {
      get: function() {
        return !this.audioStream.paused;
      }
    });


    /**
    * The play-back rate in percent.
    * @property playbackRate
    * @type number
    * @readOnly
     */

    Object.defineProperty(AudioBufferStream.prototype, "playbackRate", {
      set: function(value) {
        if (this.audioStream) {
          return this.audioStream.playbackRate = value;
        }
      },
      get: function() {
        if (this.audioStream) {
          return this.audioStream.playbackRate;
        } else {
          return 100;
        }
      }
    });


    /**
    * Disposes the audio buffer and frees the resources/memory. Should be called
    * if the audio buffer is no longer needed.
    *
    * @method dispose
     */

    AudioBufferStream.prototype.dispose = function() {
      var ref, ref1;
      if (!this.disposed) {
        this.disposed = true;
        gs.Audio.removeAudioBuffer(this);
        this.decodedBuffer = null;
        this.source = null;
        this.decoded = false;
        if ((ref = this.audioStream) != null) {
          ref.pause();
        }
        if ((ref1 = this.audioStream) != null) {
          ref1.src = "";
        }
        return delete this.audioStream;
      }
    };


    /**
    * Decodes the audio data if necessary. Needs to be called once before
    * the audio buffer can be played.
    *
    * @method decode
    * @see gs.AudioBuffer.onFinishDecode
     */

    AudioBufferStream.prototype.decode = function() {
      this.decoded = true;
      if (this.onFinishDecode != null) {
        return this.onFinishDecode(this);
      }
    };


    /**
    * Updates the audio-buffer. Needs to be called once per frame.
    *
    * @method update
     */

    AudioBufferStream.prototype.update = function() {
      if (this.playTime > 0 && this.playing) {
        this.playTime--;
        if (this.playTime <= this.fadeOutTime) {
          this.fadeInTime = this.playTime;
          this.playTime = 0;
          this.fadeInVolume = -(this.volume / this.fadeInTime);
        }
      }
      if (this.fadeInTime > 0 && !this.error) {
        this.fadeInTime--;
        this.volume = Math.max(Math.min(this.volume + this.fadeInVolume, 1.0), 0);
        if (this.fadeInTime === 0) {
          if (this.fadeInVolume < 0 && (this.onFadeOutEnd != null)) {
            this.onFadeOutEnd(this);
          }
          if (this.fadeInVolume > 0 && (this.onFadeInEnd != null)) {
            this.onFadeInEnd(this);
          }
        }
      }
      if (this.playing && this.audioStream.currentTime >= (this.loopStart + this.loopLength)) {
        return this.audioStream.currentTime = this.loopStart;
      }
    };


    /**
    * Plays the audio buffer.
    *
    * @method play
    * @param {number} [fadeInTime=0] - The fade-in time in frames.
    * @param {number} [playTime=null] - The play-time in frames.
    * @param {number} [fadeOutTime=0] - The fade-out time in frames if play-time is over.
     */

    AudioBufferStream.prototype.play = function(fadeInTime, playTime, fadeOutTime) {
      if (this.error) {
        return;
      }
      if (this.fadeInTime > 0) {
        this.volume = Math.max(Math.min(this.volume + this.fadeInTime * this.fadeInVolume, 1.0), 0);
      }
      this.playTime = playTime;
      this.fadeInTime = fadeInTime || 0;
      this.fadeOutTime = fadeOutTime || 0;
      this.audioStream.currentTime = 0;
      if (this.fadeInTime > 0) {
        this.fadeInVolume = this.volume / this.fadeInTime;
        this.volume = 0;
      }
      this.playing = true;
      return this.audioStream.play();
    };


    /**
    * Resumes the audio buffer if paused.
    *
    * @method resume
    * @param {number} [fadeInTime=0] - The fade-in time in frames.
     */

    AudioBufferStream.prototype.resume = function(fadeInTime) {
      if (this.error) {
        return;
      }
      if (this.fadeInTime > 0) {
        this.volume = Math.max(Math.min(this.volume + this.fadeInTime * this.fadeInVolume, 1.0), 0);
      }
      this.fadeInTime = fadeInTime || 0;
      if (this.fadeInTime > 0) {
        this.fadeInVolume = this.volume / this.fadeInTime;
        this.volume = 0;
      }
      this.playing = true;
      return this.audioStream.play();
    };


    /**
    * Stops the audio buffer.
    *
    * @method resume
    * @param {number} [fadeOutTime=0] - The fade-out time in frames.
     */

    AudioBufferStream.prototype.stop = function(fadeOutTime) {
      var originalVolume;
      if (this.error || !this.isPlaying) {
        return;
      }
      if (this.fadeInTime > 0) {
        this.volume = Math.max(Math.min(this.volume + this.fadeInTime * this.fadeInVolume, 1.0), 0);
      }
      this.fadeInTime = fadeOutTime || 0;
      if (this.fadeInTime > 0) {
        originalVolume = this.volume;
        this.fadeInVolume = -(this.volume / this.fadeInTime);
        return this.onFadeOutEnd = (function(_this) {
          return function() {
            if (_this.audioStream != null) {
              _this.audioStream.pause();
            }
            return _this.volume = originalVolume;
          };
        })(this);
      } else {
        if (this.audioStream != null) {
          return this.audioStream.pause();
        }
      }
    };

    AudioBufferStream.prototype.toDataURL = function() {
      return this.source.src || "";
    };

    return AudioBufferStream;

  })();

  window.GS.AudioBufferStream = AudioBufferStream;

  window.gs.AudioBufferStream = AudioBufferStream;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var DataPreparer;

  DataPreparer = (function() {
    function DataPreparer() {}

    DataPreparer.prepare = function(data) {
      var buffer, dKey, i, j, ref;
      if (!$PARAMS.needsPreparation) {
        return data;
      }
      dKey = GS.DataPreparer.generateKey();
      buffer = new Uint8Array(data);
      for (i = j = 0, ref = buffer.length; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
        buffer[i] = buffer[i] ^ dKey[i % dKey.length];
      }
      return data;
    };

    DataPreparer.prepareFilePath = function(filePath) {
      return filePath;
      if ($PARAMS.needsPreparation) {
        return btoa(unescape(encodeURIComponent(filePath))).replace(/\//g, "-").replace(/[=]/g, "_");
      } else {
        return filePath;
      }
    };

    DataPreparer.generateKey = function() {
      var dKey, dataHeap, dataHeap2, keyArray, keyPtr, sigArray, sigPtr;
      if ((window.$PARAMS != null) && !$PARAMS.needsPreparation) {
        return [0];
      }
      keyArray = new Int32Array([0, 0, 0, 0, 0]);
      sigArray = new Int32Array([42, 11, 23, 88, 133]);
      keyPtr = Module._malloc(keyArray.length * keyArray.BYTES_PER_ELEMENT);
      sigPtr = Module._malloc(sigArray.length * sigArray.BYTES_PER_ELEMENT);
      dataHeap = new Uint8Array(Module.HEAPU8.buffer, keyPtr, keyArray.length * keyArray.BYTES_PER_ELEMENT);
      dataHeap.set(new Uint8Array(keyArray.buffer));
      dataHeap2 = new Uint8Array(Module.HEAPU8.buffer, sigPtr, sigArray.length * sigArray.BYTES_PER_ELEMENT);
      dataHeap2.set(new Uint8Array(sigArray.buffer));
      Module.ccall("ac6d78ea1cf7fe1", "void", ["number", "number", "number", "number"], [dataHeap.byteOffset, keyArray.length, dataHeap2.byteOffset, sigArray.length]);
      dKey = new Int32Array(dataHeap.buffer, dataHeap.byteOffset, keyArray.length);
      Module._free(dataHeap.byteOffset);
      Module._free(dataHeap2.byteOffset);
      return dKey;
    };

    return DataPreparer;

  })();

  GS.DataPreparer = DataPreparer;

  if (!window.gs) {
    window.gs = {};
  }

  window.gs.DataPreparer = DataPreparer;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var Color;

  Color = (function() {

    /**
    * A class to describe RGBA colors. It also offers some helper methods.
    *
    * @namespace gs
    * @class Color
    * @memberof gs
    * @constructor
    * @param {number} red/color - The red value of the color <b>OR</b> another color to create a copy from.
    * @param {number} [green=null] - The green value of the color.
    * @param {number} [blue=null] - The blue value of the color.
    * @param {number} [alpha=null] - The alpha value of the color.
     */
    function Color(red, green, blue, alpha) {
      if (red == null) {

        /**
        * The red value of the color.
        * @property red
        * @type number
         */
        this.red = 255;

        /**
        * The green value of the color.
        * @property green
        * @type number
         */
        this.green = 255;

        /**
        * The blue value of the color.
        * @property blue
        * @type number
         */
        this.blue = 255;

        /**
        * The alpha value of the color.
        * @property alpha
        * @type number
         */
        this.alpha = 255;
      } else if (green == null) {
        this.red = red.red;
        this.green = red.green;
        this.blue = red.blue;
        this.alpha = red.alpha;
      } else {
        this.red = red;
        this.green = green;
        this.blue = blue;
        if (alpha != null) {
          this.alpha = alpha;
        } else {
          this.alpha = 255;
        }
      }
    }

    Color.prototype.setFromArray = function(array) {
      this.red = array[0];
      this.green = array[1];
      this.blue = array[2];
      return this.alpha = array[3];
    };

    Color.prototype.setFromObject = function(object) {
      this.red = object.red;
      this.green = object.green;
      this.alpha = object.blue;
      return this.blue = object.alpha;
    };


    /**
    * Sets the color values.
    *
    * @method set
    * @param {number} [red/color=null] - The red value of the color <b>OR</b> another color to copy the color values from.
    * @param {number} [green=null] - The green value of the color.
    * @param {number} [blue=null] - The blue value of the color.
    * @param {number} [alpha=null] - The alpha value of the color.
     */

    Color.prototype.set = function(red, green, blue, alpha) {
      if (red == null) {
        this.red = 255;
        this.green = 255;
        this.blue = 255;
        return this.alpha = 255;
      } else if (green == null) {
        this.red = red.red;
        this.green = red.green;
        this.blue = red.blue;
        return this.alpha = red.alpha;
      } else {
        this.red = red;
        this.green = green;
        this.blue = blue;
        if (alpha != null) {
          return this.alpha = alpha;
        }
      }
    };

    Color.prototype.setFromObject = function(color) {
      var ref, ref1, ref2, ref3;
      this.red = (ref = color.red) != null ? ref : this.red;
      this.green = (ref1 = color.green) != null ? ref1 : this.green;
      this.blue = (ref2 = color.blue) != null ? ref2 : this.blue;
      return this.alpha = (ref3 = color.alpha) != null ? ref3 : this.alpha;
    };


    /**
    * Returns the color as CSS string like rgba(r, g, b, a).
    *
    * @method toString
    * @return {string} The CSS string.
     */

    Color.prototype.toString = function() {
      return "rgba({0}, {1}, {2}, {3})".format(this.red, this.green, this.blue, this.alpha);
    };

    Color.prototype.toFillStyle = function() {
      return "rgba({0}, {1}, {2}, {3})".format(this.red, this.green, this.blue, this.alpha * 0.00392156862745);
    };


    /**
    * Checks if the color is equal to the specified color.
    *
    * @method equals
    * @param {gs.Color} color - The color to compare to.
    * @return {boolean} If <b>true</b> both colors are equal. Otherwise <b>false</b>
     */

    Color.prototype.equals = function(color) {
      return this.red === color.red && this.green === color.green && this.blue === color.blue && this.alpha === color.alpha;
    };

    Color.compare = function(a, b) {
      return a && b && a.red === b.red && a.green === b.green && a.blue === b.blue && a.alpha === b.alpha;
    };


    /**
    * Creates a new color from the specified color-value array. The should have 4 elements
    * with the red, green, blue and alpha color-values in that order.
    *
    * @method fromArray
    * @static
    * @param {number[]} a - An array of color-values.
    * @return {gs.Color} A new created color with the specified color-values.
     */

    Color.fromArray = function(a) {
      return new Color(a[0], a[1], a[2], a[3]);
    };


    /**
    * Creates a color from an raw object. The object should contain 4 properties
    * named red, green, blue and alpha.
    *
    * @method fromObject
    * @static
    * @param {Object} o - The object with the red, green, blue and alpha properties of the color.
    * @return {gs.Color} The new created color.
     */

    Color.fromObject = function(o) {
      return new Color(o.red, o.green, o.blue, o.alpha);
    };


    /**
    * Converts the specified RGB value to HSL space.
    *
    * @method rgbToHsl
    * @static
    * @param {number} r - The red value of the color to convert.
    * @param {number} g - The green value of the color to convert.
    * @param {number} b - The blue value of the color to convert.
    * @return {Object} The HSL values as object -> { h, s, l }.
     */

    Color.rgbToHsl = function(r, g, b) {
      var cMax, cMin, delta, h, l, s;
      r = r / 255;
      g = g / 255;
      b = b / 255;
      cMax = Math.max(r, g, b);
      cMin = Math.min(r, g, b);
      delta = cMax - cMin;
      l = (cMax + cMin) / 2;
      h = 0;
      s = 0;
      if (delta === 0) {
        h = 0;
      } else if (cMax === r) {
        h = 60 * (((g - b) / delta) % 6);
      } else if (cMax === g) {
        h = 60 * (((b - r) / delta) + 2);
      } else {
        h = 60 * (((r - g) / delta) + 4);
      }
      if (delta === 0) {
        s = 0;
      } else {
        s = delta / (1 - Math.abs(2 * l - 1));
      }
      return {
        h: h,
        s: s,
        l: l
      };
    };


    /**
    * Converts the specified HSL value to RGB space.
    *
    * @method rgbToHsl
    * @static
    * @param {number} h - The HUE value of the color to convert.
    * @param {number} s - The saturation value of the color to convert.
    * @param {number} l - The lightness value of the color to convert.
    * @return {Object} The RGB values as object -> { r, g, b }.
     */

    Color.hslToRgb = function(h, s, l) {
      var b, c, g, m, r, x;
      c = (1 - Math.abs(2 * l - 1)) * s;
      x = c * (1 - Math.abs((h / 60) % 2 - 1));
      m = l - c / 2;
      r = 0;
      g = 0;
      b = 0;
      if (h < 60) {
        r = c;
        g = x;
        b = 0;
      } else if (h < 120) {
        r = x;
        g = c;
        b = 0;
      } else if (h < 180) {
        r = 0;
        g = c;
        b = x;
      } else if (h < 240) {
        r = 0;
        g = x;
        b = c;
      } else if (h < 300) {
        r = x;
        g = 0;
        b = c;
      } else {
        r = c;
        g = 0;
        b = x;
      }
      r = Math.max(Math.floor((r + m) * 255), 0);
      g = Math.max(Math.floor((g + m) * 255), 0);
      b = Math.max(Math.floor((b + m) * 255), 0);
      return {
        r: r,
        g: g,
        b: b
      };
    };

    return Color;

  })();


  /**
  * Transparent color
  * @property TRANSPARENT
  * @type gs.Color
  * @static
  * @final
   */

  Color.TRANSPARENT = new Color(0, 0, 0, 0);


  /**
  * Black color
  * @property BLACK
  * @type gs.Color
  * @static
  * @final
   */

  Color.BLACK = new Color(0, 0, 0, 255);


  /**
  * White color
  * @property WHITE
  * @type gs.Color
  * @static
  * @final
   */

  Color.WHITE = new Color(255, 255, 255, 255);

  window.Color = Color;

  gs.Color = Color;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var Point;

  Point = (function() {

    /**
    * A class to describe points. It also offers a lot of helper methods.
    *
    * @namespace gs
    * @class Point
    * @memberof gs
    * @constructor
    * @param {number} x - The x-coordinate of the point.
    * @param {number} y - The x-coordinate of the point.
     */
    function Point(x, y) {

      /**
      * The x-coordinate of the point.
      * @property x
      * @type number
       */
      this.x = x || 0;

      /**
      * The y-coordinate of the point.
      * @property y
      * @type number
       */
      this.y = y || 0;
    }


    /**
    * Sets the coordinates of the point.
    *
    * @method set
    * @param {number} x - The x-coordinate of the point.
    * @param {number} y - The x-coordinate of the point.
     */

    Point.prototype.set = function(x, y) {
      this.x = x;
      return this.y = y;
    };

    Point.prototype.setFromObject = function(p) {
      this.x = p.x;
      return this.y = p.y;
    };

    return Point;

  })();

  gs.Point = Point;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var Rect;

  Rect = (function() {

    /**
    * A class to describe rectangles. It also offers a lot of helper methods.
    *
    * @namespace gs
    * @class Rect
    * @memberof gs
    * @constructor
    * @param {number} x - The x-coordinate of the rectangle.
    * @param {number} y - The x-coordinate of the rectangle.
    * @param {number} width - The width of the rectangle.
    * @param {number} height - The height of the rectangle.
     */
    function Rect(x, y, width, height) {

      /**
      * The x-coordinate of the rectangle.
      * @property x
      * @type number
       */
      this.x = x || 0;

      /**
      * The y-coordinate of the rectangle.
      * @property y
      * @type number
       */
      this.y = y || 0;

      /**
      * The width of the rectangle.
      * @property width
      * @type number
       */
      this.width = width || 1;

      /**
      * The height of the rectangle.
      * @property height
      * @type number
       */
      this.height = height || 1;
    }


    /**
    * Sets the coordinates of the rectangle.
    *
    * @method set
    * @param {number} x - The x-coordinate of the rectangle.
    * @param {number} y - The x-coordinate of the rectangle.
    * @param {number} width - The width of the rectangle.
    * @param {number} height - The height of the rectangle.
     */

    Rect.prototype.set = function(x, y, width, height) {
      if (x.x != null) {
        this.x = x.x;
        this.y = x.y;
        this.width = x.width;
        return this.height = x.height;
      } else {
        this.x = x;
        this.y = y;
        this.width = width;
        return this.height = height;
      }
    };


    /**
    * Sets the coordinates of the rectangle from raw object.
    *
    * @method setFromObject
    * @param {Object} rect - A raw rectangle object.
     */

    Rect.prototype.setFromObject = function(rect) {
      this.x = rect.x;
      this.y = rect.y;
      this.width = rect.width;
      return this.height = rect.height;
    };


    /**
    * Expands the rectangle so that the specified rectangle-area is covered.
    *
    * @method expand
    * @param {number} x - The x-coordinate of the rectangle.
    * @param {number} y - The x-coordinate of the rectangle.
    * @param {number} width - The width of the rectangle.
    * @param {number} height - The height of the rectangle.
     */

    Rect.prototype.expand = function(x, y, width, height) {
      if (this.width === -1) {
        this.x = x;
        this.y = y;
        this.width = width;
        return this.height = height;
      } else {
        this.x = Math.min(this.x, x);
        this.y = Math.min(this.y, y);
        this.width = Math.max(this.x + this.width, x + width) - this.x;
        return this.height = Math.max(this.y + this.height, y + height) - this.y;
      }
    };


    /**
    * Rounds the rectangle coordinates using floor-method.
    *
    * @method floor
     */

    Rect.prototype.floor = function() {
      this.x = Math.round(this.x);
      this.y = Math.round(this.y);
      this.width = Math.round(this.width);
      return this.height = Math.round(this.height);
    };


    /**
    * Rounds the rectangle coordinates using round-method.
    *
    * @method round
     */

    Rect.prototype.round = function() {
      this.x = Math.round(this.x);
      this.y = Math.round(this.y);
      this.width = Math.round(this.width);
      return this.height = Math.round(this.height);
    };


    /**
    * Rounds the rectangle coordinates using ceil-method.
    *
    * @method ceil
     */

    Rect.prototype.ceil = function() {
      this.x = Math.ceil(this.x);
      this.y = Math.ceil(this.y);
      this.width = Math.ceil(this.width);
      return this.height = Math.ceil(this.height);
    };

    Rect.prototype.shrink = function(x, y, width, height) {
      var h, w;
      w = this.x + this.width;
      h = this.y + this.height;
      this.x = Math.max(this.x, x);
      this.y = Math.max(this.y, y);
      this.width = Math.max(Math.min(width + x, w) - this.x, 0);
      return this.height = Math.max(Math.min(height + y, h) - this.y, 0);
    };


    /**
    * Checks if the specified point is inside the rectangle.
    *
    * @method contains
    * @param {number} x - The x-coordinate of the point.
    * @param {number} y - The y-coordinate of the point.
    * @return {boolean} If <b>true</b> the point is inside the rectangle. Otherwise <b>false</b>.
     */

    Rect.prototype.contains = function(x, y) {
      return Rect.contains(this.x, this.y, this.width, this.height, x, y);
    };


    /**
    * Checks if the rectangle intersects with the specified rectangle.
    *
    * @method intersect
    * @param {number} x - The x-coordinate of the rectangle.
    * @param {number} y - The y-coordinate of the rectangle.
    * @param {number} width - The width of the rectangle.
    * @param {number} height - The height of the rectangle.
    * @return {boolean} If <b>true</b> the two rectangle intersect. Otherwise <b>false</b>.
     */

    Rect.prototype.intersect = function(x, y, width, height) {
      return !(x > this.x + this.width || x + width < this.x || y > this.y + this.height || y + height < this.y);
    };


    /**
    * Rotates the rectangle by the specified angle.
    *
    * @method intersect
    * @param {number} angle
    * @return {gs.Rect} The rotated rectangle.
     */

    Rect.prototype.rotate = function(angle) {
      var r;
      angle = Math.degreesToRadians(angle);
      r = new gs.Rect(0, 0, 1, 1);
      r.x = this.x;
      r.y = this.y;
      r.width = this.width * Math.abs(Math.cos(angle)) + this.height * Math.abs(Math.sin(angle));
      r.height = this.width * Math.abs(Math.sin(angle)) + this.height * Math.abs(Math.cos(angle));
      return r;
    };


    /**
    * Checks if the specified point is inside the specified rectangle.
    *
    * @method contains
    * @static
    * @param {number} x - The x-coordinate of the rectangle.
    * @param {number} y - The y-coordinate of the rectangle.
    * @params {number} width - The width of the rectangle.
    * @params {number} height - The height of the rectangle.
    * @param {number} px - The x-coordinate of the point.
    * @param {number} py - The y-coordinate of the point.
    * @return {boolean} If <b>true</b> the point is inside the rectangle. Otherwise <b>false</b>.
     */

    Rect.contains = function(x, y, width, height, px, py) {
      if (py == null) {
        px = y;
        py = width;
        y = x.y;
        width = x.width;
        height = x.height;
        x = x.x;
      }
      return px >= x && px < x + width && py >= y && py < y + height;
    };


    /**
    * Checks if the specified rectangle 1 intersects with the specified rectangle 2.
    *
    * @method intersect
    * @static
    * @param {number} x1 - The x-coordinate of the first rectangle.
    * @param {number} y1 - The y-coordinate of the first rectangle.
    * @param {number} width1 - The width of the first rectangle.
    * @param {number} height1 - The height of the first rectangle.
    * @param {number} x2 - The x-coordinate of the second rectangle.
    * @param {number} y2 - The y-coordinate of the second rectangle.
    * @param {number} width2 - The width of the second rectangle.
    * @param {number} height2 - The height of the second rectangle.
    * @return {boolean} If <b>true</b> the two rectangle intersect. Otherwise <b>false</b>.
     */

    Rect.intersect = function(x1, y1, width1, height1, x2, y2, width2, height2) {
      return !(x1 > x2 + width2 || x1 + width1 < x2 || y1 > y2 + height2 || y1 + height1 < y2);
    };


    /**
    * Creates a rectangle from an array. The array should contain 4 elements
    * for x, y, width and height in that order.
    *
    * @method fromArray
    * @static
    * @param {number[]} a - The array with the x, y, width and height of the rectangle.
    * @return {gs.Rect} The new created rectangle.
     */

    Rect.fromArray = function(a) {
      return new Rect(a[0], a[1], a[2], a[3]);
    };


    /**
    * Creates a rectangle from an raw object. The object should contain 4 properties
    * named x, y, width and height.
    *
    * @method fromObject
    * @static
    * @param {Object} o - The object with the x, y, width and height properties of the rectangle.
    * @return {gs.Rect} The new created rectangle.
     */

    Rect.fromObject = function(o) {
      return new Rect(o.x, o.y, o.width, o.height);
    };

    return Rect;

  })();

  window.Rect = Rect;

  gs.Rect = Rect;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var Tone;

  Tone = (function() {

    /**
    * Describes a color tone used to tint a graphic object or a screen.
    *
    * @namespace gs
    * @class Tone
    * @memberof gs
    * @constructor
    * @param {number|gs.Tone} red|tone - The tone's red-value <b>OR</b> a tone-object to copy values from.
    * @param {number} [green=null] - The tone's green-value.
    * @param {number} [blue=null] - The tone's blue-value.
    * @param {number} [grey=null] - The tone's grey-value.
     */
    function Tone(red, green, blue, grey) {
      if (red != null) {
        this.set(red, green, blue, grey);
      } else {
        this.set(0, 0, 0, 0);
      }
    }

    Tone.prototype.setFromObject = function(object) {
      this.red = object.red;
      this.green = object.green;
      this.alpha = object.blue;
      return this.grey = object.grey;
    };


    /**
    * Sets the color-values of the tone.
    *
    * @method set
    * @param {number|gs.Tone} red|tone - The tone's red-value <b>OR</b> a tone-object to copy values from.
    * @param {number} [green=null] - The tone's green-value.
    * @param {number} [blue=null] - The tone's blue-value.
    * @param {number} [grey=null] - The tone's grey-value.
     */

    Tone.prototype.set = function(red, green, blue, grey) {
      if (blue != null) {

        /**
        * The tone's red-value.
        * @property red
        * @type number
         */
        this.red = red;

        /**
        * The tone's green-value.
        * @property green
        * @type number
         */
        this.green = green;

        /**
        * The tone's blue-value.
        * @property blue
        * @type number
         */
        this.blue = blue;

        /**
        * The tone's grey-value.
        * @property grey
        * @type number
         */
        return this.grey = grey;
      } else if (green != null) {
        this.red = red.red;
        this.green = red.green;
        this.blue = red.blue;
        return this.grey = green;
      } else {
        this.red = red.red;
        this.green = red.green;
        this.blue = red.blue;
        return this.grey = red.grey;
      }
    };


    /**
    * Creates a new tone from the specified tone-value array. The should have 4 elements
    * with the red, green, blue and grey tone-values in that order.
    *
    * @method fromArray
    * @static
    * @param {number[]} a - An array of tone-values.
    * @return {gs.Tone} A new created tone with the specified tone-values.
     */

    Tone.fromArray = function(a) {
      return new gs.Tone(a[0], a[1], a[2], a[3]);
    };


    /**
    * Creates a tone from an raw object. The object should contain 4 properties
    * named red, green, blue and grey.
    *
    * @method fromObject
    * @static
    * @param {Object} o - The object with the red, green, blue and grey properties of the tone.
    * @return {gs.Tone} The new created tone.
     */

    Tone.fromObject = function(o) {
      return new Tone(o.red, o.green, o.blue, o.grey);
    };

    return Tone;

  })();


  /**
  * Normal tone. No tinting is executed using this tone.
  * @property NORMAL
  * @type gs.Tone
  * @static
  * @final
   */

  Tone.NORMAL = new Tone(0, 0, 0, 0);

  window.Tone = Tone;

  gs.Tone = Tone;

}).call(this);

// Generated by CoffeeScript 1.10.0
(function() {
  var Graphics, InputButton;

  InputButton = (function() {
    function InputButton(filePath, width, height, code) {
      this.keys = Input.keys;
      this.element = $(document.createElement("div"));
      this.element.css("position", "absolute");
      this.element.css("background-color", "rgba(255, 0, 0, 0.2)");
      this.element.width(32);
      this.element.height(32);
      this.code = code;
      this.element.bind("touchstart", (function(_this) {
        return function(e) {
          _this.keys[code] = 1;
          if (code === Input.LEFT) {
            Input.leftFrameCount = Graphics.frameCount;
          }
          if (code === Input.RIGHT) {
            Input.rightFrameCount = Graphics.frameCount;
          }
          if (code === Input.UP) {
            Input.upFrameCount = Graphics.frameCount;
          }
          if (code === Input.DOWN) {
            return Input.downFrameCount = Graphics.frameCount;
          }
        };
      })(this));
      this.element.bind("touchend", (function(_this) {
        return function(e) {
          _this.keys[code] = 2;
          return Input.pressTimes[_this.code] = 0;
        };
      })(this));
      this.element.bind("mouseout", (function(_this) {
        return function(e) {
          _this.keys[code] = 2;
          return Input.pressTimes[_this.code] = 0;
        };
      })(this));
      this.element.bind("touchmove", (function(_this) {
        return function(e) {
          _this.keys[code] = 1;
          if (code === Input.LEFT) {
            Input.leftFrameCount = Graphics.frameCount;
          }
          if (code === Input.RIGHT) {
            Input.rightFrameCount = Graphics.frameCount;
          }
          if (code === Input.UP) {
            Input.upFrameCount = Graphics.frameCount;
          }
          if (code === Input.DOWN) {
            return Input.downFrameCount = Graphics.frameCount;
          }
        };
      })(this));
    }

    InputButton.prototype.setPosition = function(x, y) {
      this.element.css("top", y);
      return this.element.css("left", x);
    };

    InputButton.prototype.width = function() {
      return this.element.width();
    };

    InputButton.prototype.height = function() {
      return this.element.height();
    };

    return InputButton;

  })();

  Graphics = (function() {
    function Graphics() {}

    Graphics.prototype.initialize = function(width, height, parent, keepRatio) {
      var canvas;
      this.width = width;
      this.height = height;
      this.keepRatio = keepRatio != null ? keepRatio : true;
      this.transitionData = null;
      this.canvas = document.createElement("canvas");
      this.canvas.width = this.width;
      this.canvas.height = this.height;
      this.canvas.style.position = "absolute";
      $(this.canvas).css("-webkit-font-smoothing", "none");
      this.screenDiff = $(window).height() - this.height;
      this.screenDiffW = $(window).width() - this.width;
      this.parent = parent;
      this.frameCount = 0;
      $(window).resize((function(_this) {
        return function() {
          return _this.onResize();
        };
      })(this));
      if (window.requestAnimationFrame != null) {
        this.onEachFrame = (function(_this) {
          return function(cb) {
            var _cb;
            _cb = function() {
              cb();
              return requestAnimationFrame(_cb);
            };
            return _cb();
          };
        })(this);
      } else if (window.webkitRequestAnimationFrame != null) {
        this.onEachFrame = (function(_this) {
          return function(cb) {
            var _cb;
            _cb = function() {
              cb();
              return webkitRequestAnimationFrame(_cb);
            };
            return _cb();
          };
        })(this);
      } else if (window.mozRequestAnimationFrame != null) {
        this.onEachFrame = (function(_this) {
          return function(cb) {
            var _cb;
            _cb = function() {
              cb();
              return mozRequestAnimationFrame(_cb);
            };
            return _cb();
          };
        })(this);
      } else {
        this.onEachFrame = (function(_this) {
          return function(cb) {
            return setInterval(cb, 16.666667);
          };
        })(this);
      }
      parent.appendChild(this.canvas);
      canvas = this.canvas;
      if ($MOBILE) {
        $(canvas).css("left", ($(window).width() - canvas.width) / 2);
        $(canvas).css("top", this.screenDiff / 2);
      } else {
        $(this.canvas).css("left", 0);
        $(this.canvas).css("top", 0);
      }
      this.context = this.canvas.getContext("2d");
      this.context.imageSmoothingEnabled = false;
      this.context.webkitImageSmoothingEnabled = false;
      this.context.mozImageSmoothingEnabled = false;
      this.renderer = new Renderer(this.context);
      this.viewport = new Viewport(0, 0, width, height);
      $("body").css("background-color", "black");
      return this.onResize();
    };

    Graphics.prototype.createButtons = function() {
      this.buttonA = new InputButton("ButtonA.png", null, null, Input.A);
      this.buttonA.setPosition($(window).width() - this.buttonA.width() - 10, $(window).height() - this.buttonA.height() - 100);
      this.buttonB = new InputButton("ButtonB.png", null, null, Input.B);
      this.buttonB.setPosition($(window).width() - this.buttonB.width() - 47, $(window).height() - this.buttonB.height() - 100);
      this.buttonX = new InputButton("ButtonX.png", null, null, Input.X);
      this.buttonX.setPosition($(window).width() - this.buttonA.width() - 10, $(window).height() - this.buttonA.height() - 105 - 32);
      this.buttonY = new InputButton("ButtonY.png", null, null, Input.Y);
      this.buttonY.setPosition($(window).width() - this.buttonB.width() - 47, $(window).height() - this.buttonB.height() - 105 - 32);
      this.buttonLeft = new InputButton("ButtonLeft.png", 94, 94, Input.LEFT);
      this.buttonLeft.setPosition(5, $(window).height() - this.buttonLeft.height() - 32 - 70);
      this.buttonUp = new InputButton("ButtonUp.png", 94, 94, Input.UP);
      this.buttonUp.setPosition(7 + this.buttonUp.width() / 2, $(window).height() - this.buttonUp.height() - 32 - 32 - 70 - 5);
      this.buttonRight = new InputButton("ButtonRight.png", 94, 94, Input.RIGHT);
      this.buttonRight.setPosition(10 + 32, $(window).height() - this.buttonLeft.height() - 32 - 70);
      this.buttonDown = new InputButton("ButtonDown.png", 94, 94, Input.DOWN);
      this.buttonDown.setPosition(7 + this.buttonUp.width() / 2, $(window).height() - this.buttonUp.height() - 70 + 5);
      this.buttonStart = new InputButton("ButtonStart.png", 183, 70, Input.START);
      this.buttonStart.setPosition(($(window).width() - this.buttonStart.width() * 2) / 2, $(window).height() - 64);
      this.buttonSelect = new InputButton("ButtonSelect.png", 183, 70, Input.SELECT);
      this.buttonSelect.setPosition(64 + ($(window).width() - this.buttonStart.width() * 2) / 2, $(window).height() - 64);
      $(this.parent).append(this.buttonA.element);
      $(this.parent).append(this.buttonB.element);
      $(this.parent).append(this.buttonX.element);
      $(this.parent).append(this.buttonY.element);
      $(this.parent).append(this.buttonLeft.element);
      $(this.parent).append(this.buttonUp.element);
      $(this.parent).append(this.buttonRight.element);
      $(this.parent).append(this.buttonDown.element);
      $(this.parent).append(this.buttonStart.element);
      return $(this.parent).append(this.buttonSelect.element);
    };

    Graphics.prototype.dispose = function() {};

    Graphics.prototype.onResize = function(width, height) {
      var scaleH;
      height = height || $(window).height();
      width = width || $(window).width();
      this.screenDiff = height - this.height;
      this.screenDiffW = width - this.width;
      scaleH = 1.0 + this.screenDiff / this.height;
      if (this.keepRatio) {
        scaleH = Math.min(scaleH, width / this.width);
      }
      $(this.canvas).css("-webkit-transform", "none");
      $(this.canvas).css("-ms-transform", "none");
      $(this.canvas).css("-moz-transform", "none");
      $(this.canvas).css("transform", "none");

      /*
      $(@canvas).css("image-rendering", "optimizeSpeed")
      $(@canvas).css("image-rendering", "crisp-edges")
      $(@canvas).css("image-rendering", "-moz-crisp-edges")
      $(@canvas).css("image-rendering", "-o-crisp-edges")
      $(@canvas).css("image-rendering", "optimize-contrast")
      $(@canvas).css("image-rendering", "-webkit-optimize-contrast")
      $(@canvas).css("image-rendering", "-o-optimize-contrast")
      $(@canvas).css("image-rendering", "-webkit-pixelated")
      $(@canvas).css("image-rendering", "pixelated")
      $(@canvas).css("image-rendering", "-webkit-crisp-edges")
      $(@canvas).css("image-rendering", "crisp-edges")
       */
      $(this.canvas).css("-webkit-transform", "scale(" + (this.keepRatio ? "" : (1.0 + (this.screenDiffW / this.width)) + ", ") + scaleH + ")");
      $(this.canvas).css("-ms-transform", "scale(" + (this.keepRatio ? "" : (1.0 + (this.screenDiffW / this.width)) + ", ") + scaleH + ")");
      $(this.canvas).css("-moz-transform", "scale(" + (this.keepRatio ? "" : (1.0 + (this.screenDiffW / this.width)) + ", ") + scaleH + ")");
      $(this.canvas).css("transform", "scale(" + (this.keepRatio ? "" : (1.0 + (this.screenDiffW / this.width)) + ", ") + scaleH + ")");
      $(this.canvas).css("left", (width - this.width) / 2);
      $(this.canvas).css("top", (height - this.height) / 2);
      this.scaleFactorX = this.width / this.canvas.width;
      return this.scaleFactorY = this.height / this.canvas.height;
    };

    Graphics.prototype.update = function() {
      this.context.fillRect(0, 0, this.width, this.height);
      this.viewport.update();
      if (this.transitionData != null) {
        return this.updateTransition();
      } else {
        return this.viewport.draw();
      }
    };

    Graphics.prototype.addGraphicObject = function(object) {
      if (this.viewport != null) {
        return this.viewport.addGraphicObject(object);
      }
    };

    Graphics.prototype.removeGraphicObject = function(object) {
      if (this.viewport != null) {
        return this.viewport.removeGraphicObject(object);
      }
    };

    Graphics.prototype.snapshot = function() {
      return new Bitmap(this.canvas);
    };

    Graphics.prototype.updateTransition = function() {
      if (this.transitionData.transitionData != null) {
        return this.updateBitmapTransition();
      } else {
        return this.updateDefaultTransition();
      }
    };

    Graphics.prototype.updateDefaultTransition = function() {
      this.context.drawImage(this.transitionData.sourceBitmap.image, 0, 0);
      this.context.globalAlpha = this.transitionData.currentAlpha / 255;
      this.context.drawImage(this.transitionData.targetBitmap.image, 0, 0);
      this.context.globalAlpha = 1.0;
      this.transitionData.currentAlpha += 255 / this.transitionData.duration;
      if (this.transitionData.currentAlpha >= 255) {
        this.transitionData = null;
        this.frozenScreenBitmap = null;
        return this.frozen = false;
      }
    };

    Graphics.prototype.updateBitmapTransition = function() {
      var alpha, diff, i, j, ref, screenData, sourceData, targetData, transitionData;
      screenData = this.transitionData.screenData;
      transitionData = this.transitionData.transitionData;
      targetData = this.transitionData.targetData;
      sourceData = this.transitionData.sourceData;
      for (i = j = 0, ref = screenData.length; j < ref; i = j += 4) {
        if (transitionData[i] >= this.transitionData.currentAlpha) {
          diff = transitionData[i] - this.transitionData.currentAlpha;
          if (this.transitionData.vague > diff) {
            alpha = diff / this.transitionData.vague * 255;
            screenData[i] = ((sourceData[i] * alpha) + targetData[i] * (255 - alpha)) / 255;
            screenData[i + 1] = ((sourceData[i + 1] * alpha) + targetData[i + 1] * (255 - alpha)) / 255;
            screenData[i + 2] = ((sourceData[i + 2] * alpha) + targetData[i + 2] * (255 - alpha)) / 255;
            screenData[i + 3] = sourceData[i + 3];
          } else {
            screenData[i] = sourceData[i];
            screenData[i + 1] = sourceData[i + 1];
            screenData[i + 2] = sourceData[i + 2];
            screenData[i + 3] = sourceData[i + 3];
          }
        } else {
          screenData[i] = targetData[i];
          screenData[i + 1] = targetData[i + 1];
          screenData[i + 2] = targetData[i + 2];
          screenData[i + 3] = targetData[i + 3];
        }
      }
      this.transitionData.currentAlpha += 255 / this.transitionData.duration;
      this.context.putImageData(this.transitionData.screenImageData, 0, 0);
      if (this.transitionData.currentAlpha >= 255) {
        this.transitionData = null;
        this.frozenScreenBitmap = null;
        return this.frozen = false;
      }
    };

    Graphics.prototype.freeze = function() {
      this.frozen = true;
      return this.frozenScreenBitmap = this.snapshot();
    };

    Graphics.prototype.transition = function(duration, transitionBitmap, vague) {
      if (this.frozenScreenBitmap != null) {
        this.context.fillStyle = "#000000";
        this.context.fillRect(0, 0, this.width, this.height);
        this.viewport.update();
        this.viewport.draw();
        return this._transition(this.frozenScreenBitmap, this.snapshot(), duration, transitionBitmap, vague);
      }
    };

    Graphics.prototype._transition = function(sourceBitmap, targetBitmap, duration, transitionBitmap, vague) {
      this.transitionData = {
        sourceBitmap: sourceBitmap,
        targetBitmap: targetBitmap,
        duration: duration,
        transitionBitmap: transitionBitmap,
        vague: vague
      };
      this.transitionData.currentAlpha = 0;
      if (this.transitionData.transitionBitmap != null) {
        this.transitionData.screenImageData = this.context.getImageData(0, 0, this.width, this.height);
        this.transitionData.screenData = this.transitionData.screenImageData.data;
        this.transitionData.transitionData = transitionBitmap.getImageData(0, 0, this.width, this.height).data;
        this.transitionData.targetData = targetBitmap.getImageData(0, 0, this.width, this.height).data;
        return this.transitionData.sourceData = sourceBitmap.getImageData(0, 0, this.width, this.height).data;
      }
    };

    Graphics.prototype._transitionWithBitmap = function(sourceBitmap, targetBitmap, duration, transitionBitmap, vague) {};

    return Graphics;

  })();

  window.Graphics_Canvas = Graphics;

  window.Graphics = new Graphics();

}).call(this);

// Generated by CoffeeScript 1.6.3
(function() {
  var CanvasContextInfo, Renderer_Canvas;

  CanvasContextInfo = (function() {
    function CanvasContextInfo() {
      this.globalAlpha = 1.0;
      this.globalCompositeOperation = 0;
    }

    return CanvasContextInfo;

  })();

  Renderer_Canvas = (function() {
    function Renderer_Canvas(context) {
      this.context = context;
      this.canvas = context.canvas;
      this.contextInfo = new CanvasContextInfo();
      this.ox = 0;
      this.oy = 0;
      this.zoom = {
        x: 1.0,
        y: 1.0
      };
      this.size = {
        width: 0,
        height: 0
      };
      this.angle = 0;
      this.mirror = false;
      this.color = new Color(255, 255, 255, 0);
      this.opacity = 255;
      this.tone = new Tone(0, 0, 0, 0);
      this.blendType = 0;
      this.preparationBitmap = new Bitmap(1024, 1024);
      this.preparationContext = this.preparationBitmap.context;
      this.clip = new Rect(0, 0, this.canvas.width, this.canvas.height);
    }

    Renderer_Canvas.prototype.reset = function() {
      this.blendType = 0;
      this.opacity = 255;
      if (this.contextInfo.globalCompositeOperation !== 0) {
        this.context.globalCompositeOperation = "source-over";
        this.contextInfo.globalCompositeOperation = 0;
      }
      if (this.contextInfo.globalAlpha !== 1.0) {
        this.context.globalAlpha = 1.0;
        return this.contextInfo.globalAlpha = 1.0;
      }
    };

    Renderer_Canvas.prototype.drawRect = function(rect) {
      this.context.fillStyle = this.color.toFillStyle();
      return this.context.fillRect(rect.x, rect.y, rect.width, rect.height);
    };

    Renderer_Canvas.prototype.clipRect = function(rect) {
      if (rect.x !== this.clip.x || rect.y !== this.clip.y || rect.width !== this.clip.width || rect.height !== this.clip.height) {
        this.context.restore();
        this.context.save();
        this.context.beginPath();
        this.context.rect(rect.x, rect.y, rect.width, rect.height);
        this.context.clip();
        return this.clip = rect;
      }
    };

    Renderer_Canvas.prototype._changeTone = function(imageData) {
      var a1, a2, avg, blue, data, green, grey, i, red, _i, _j, _k, _ref, _ref1, _ref2;
      data = imageData.data;
      red = this.tone.red;
      green = this.tone.green;
      blue = this.tone.blue;
      if (this.tone.grey !== 0) {
        grey = this.tone.grey * 0.00392156862745;
        if (this.tone.grey === 255) {
          for (i = _i = 0, _ref = data.length - 1; _i < _ref; i = _i += 4) {
            avg = (data[i] + data[i + 1] + data[i + 2]) * 0.3;
            data[i] = avg + red;
            data[i + 1] = avg + green;
            data[i + 2] = avg + blue;
          }
        } else {
          for (i = _j = 0, _ref1 = data.length - 1; _j < _ref1; i = _j += 4) {
            avg = (data[i] + data[i + 1] + data[i + 2]) * 0.3;
            a1 = avg * grey;
            a2 = 1.0 - grey;
            data[i] = (a1 + data[i] * a2) + red;
            data[i + 1] = (a1 + data[i + 1] * a2) + green;
            data[i + 2] = (a1 + data[i + 2] * a2) + blue;
          }
        }
      } else {
        for (i = _k = 0, _ref2 = data.length - 1; _k < _ref2; i = _k += 4) {
          data[i] += red;
          data[i + 1] += green;
          data[i + 2] += blue;
        }
      }
      return null;
    };

    Renderer_Canvas.prototype.changeTone = function(rect) {
      var imageData;
      this.preparationContext.globalCompositeOperation = "source-over";
      imageData = this.context.getImageData(rect.x, rect.y, rect.width, rect.height);
      this._changeTone(imageData);
      return this.context.putImageData(imageData, rect.x, rect.y);
    };

    Renderer_Canvas.prototype.drawTiledBitmap = function(bitmap, x, y, w, h, sx, sy, sh, sw) {
      var tilesX, tilesY, tx, ty, _i, _j;
      tilesX = Math.ceil(w / sw);
      tilesY = Math.ceil(h / sh);
      for (tx = _i = 0; 0 <= tilesX ? _i < tilesX : _i > tilesX; tx = 0 <= tilesX ? ++_i : --_i) {
        for (ty = _j = 0; 0 <= tilesY ? _j < tilesY : _j > tilesY; ty = 0 <= tilesY ? ++_j : --_j) {
          this.context.drawImage(bitmap.image, sx, sy, sw, sh, x + tx * sw, y + ty * sh, sw, sh);
        }
      }
      return null;
    };

    Renderer_Canvas.prototype.drawBitmapSimple = function(bitmap, x, y, w, h, sx, sy, sw, sh) {
      if (this.context.bitmap != null) {
        this.context.bitmap.sequenceNumber++;
      }
      return this.context.drawImage(bitmap.image, sx, sy, sw, sh, x - this.ox, y - this.oy, w, h);
    };

    Renderer_Canvas.prototype.drawBitmap = function(bitmap, x, y, srcRect, ox, oy) {
      var a, height, imageData, width;
      if (!bitmap.loaded) {
        return;
      }
      if (this.context.bitmap != null) {
        this.context.bitmap.sequenceNumber++;
      }
      ox = ox || 0;
      oy = oy || 0;
      x = x - this.ox - (ox * this.zoom.x);
      y = y - this.oy - (oy * this.zoom.y);
      srcRect.width = Math.min(Math.floor(srcRect.width), bitmap.width);
      srcRect.height = Math.min(Math.floor(srcRect.height), bitmap.height);
      width = this.size.width > 0 ? this.size.width : srcRect.width * this.zoom.x;
      height = this.size.height > 0 ? this.size.height : srcRect.height * this.zoom.y;
      imageData = null;
      if (this.tone.red !== 0 || this.tone.green !== 0 || this.tone.blue !== 0 || this.tone.grey !== 0) {
        imageData = bitmap.context.getImageData(srcRect.x, srcRect.y, srcRect.width, srcRect.height);
        this._changeTone(imageData);
        this.preparationContext.globalCompositeOperation = "source-over";
        this.preparationContext.putImageData(imageData, 0, 0);
      }
      if (this.color.alpha !== 0) {
        this.preparationContext.globalCompositeOperation = "source-over";
        if (imageData == null) {
          this.preparationContext.clearRect(0, 0, srcRect.width, srcRect.height);
          this.preparationContext.drawImage(bitmap.image, srcRect.x, srcRect.y, srcRect.width, srcRect.height, 0, 0, srcRect.width, srcRect.height);
        }
        this.preparationContext.globalCompositeOperation = "source-atop";
        this.preparationContext.fillStyle = this.color.toFillStyle();
        this.preparationContext.fillRect(0, 0, srcRect.width, srcRect.height);
      }
      if (this.opacity !== 255) {
        a = this.opacity * 0.00392156862745;
        if (this.contextInfo.globalAlpha !== a) {
          this.context.globalAlpha = a;
          this.contextInfo.globalAlpha = a;
        }
      } else {
        if (this.contextInfo.globalAlpha !== 1.0) {
          this.context.globalAlpha = 1.0;
          this.contextInfo.globalAlpha = 1.0;
        }
      }
      switch (this.blendType) {
        case 0:
          if (this.contextInfo.globalCompositeOperation !== 0) {
            this.context.globalCompositeOperation = "source-over";
            this.contextInfo.globalCompositeOperation = 0;
          }
          break;
        case 1:
          if (this.contextInfo.globalCompositeOperation !== 1) {
            this.context.globalCompositeOperation = "lighter";
            this.contextInfo.globalCompositeOperation = 1;
          }
          break;
        case 2:
          if (this.contextInfo.globalCompositeOperation !== 2) {
            this.context.globalCompositeOperation = "darker";
            this.contextInfo.globalCompositeOperation = 2;
          }
      }
      if (this.angle !== 0) {
        if (this.mirror) {
          this.context.scale(-1, 1);
          x = (x + width) * -1;
        }
        this.context.translate(x, y);
        this.context.translate(width * 0.5, height * 0.5);
        this.context.rotate(this.angle);
        this.context.translate(-(width * 0.5), -(height * 0.5));
        if (this.color.alpha === 0 && (imageData == null)) {
          this.context.drawImage(bitmap.image, srcRect.x, srcRect.y, srcRect.width, srcRect.height, 0, 0, width, height);
        } else {
          this.context.drawImage(this.preparationBitmap.image, 0, 0, srcRect.width, srcRect.height, 0, 0, width, height);
        }
        return this.context.setTransform(1, 0, 0, 1, 0, 0);
      } else {
        if (this.mirror) {
          this.context.scale(-1, 1);
          x = (x + width) * -1;
        }
        if (this.color.alpha === 0 && (imageData == null)) {
          this.context.drawImage(bitmap.image, srcRect.x, srcRect.y, srcRect.width, srcRect.height, x, y, width, height);
        } else {
          this.context.drawImage(this.preparationBitmap.image, 0, 0, srcRect.width, srcRect.height, x, y, width, height);
        }
        if (this.mirror) {
          return this.context.setTransform(1, 0, 0, 1, 0, 0);
        }
      }
    };

    return Renderer_Canvas;

  })();

  window.Renderer_Canvas = Renderer_Canvas;

  window.Renderer = Renderer_Canvas;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var Graphics_OpenGL;

  window.$gl = null;

  Graphics_OpenGL = (function() {

    /**
    * This class manages the graphics system. It needs to be initializes before any
    * graphical operations can be executed. It initializes the necessary graphic contexts
    * allocates necessary resources.
    *
    * It manages all graphic objects and renders them automatically on screen ordered
    * by the z-index of each object.
    *
    * @namespace gs
    * @class Graphics
    * @see gs.GraphicObject
    * @static
    * @final
    * @memberof gs
    * @constructor
     */
    function Graphics_OpenGL() {}


    /**
    * Initializes the graphics system. Must be called before any graphic object are created
    * or any other graphic operations are executed.
    *
    * @method initialize
    * @static
    * @param {number} width - The width of the game's resolution.
    * @param {number} height - The height of the game's resolution.
    * @param {HTMLElement} parent - The HTML element to which the canvas should be added.
    * @param {boolean} keepRatio - Indicates if the canvas should keep the ratio of the resolution on scaling.
     */

    Graphics_OpenGL.prototype.initialize = function(width, height, parent, keepRatio) {
      var canvas, delta, fps, interval, now, t;
      parent = parent || document.body;

      /**
      * The canvas/screen object. On most platforms that is an HTMLCanvasElement but there
      * is no guarantee. Accessing the canvas directly is not recommended, it should only be done
      * if really necessary.
      *
      * @property canvas
      * @type Object
      * @static
      * @readOnly
       */
      this.canvas = document.createElement("canvas");
      jQuery(this.canvas).css("background-color", "#000");
      this.canvas.width = width;
      this.canvas.height = height;

      /**
      * Indicates if the canvas/screen should keep the ratio of the resolution on scaling.
      * @property keepRatio
      * @static
      * @type boolean
       */
      this.keepRatio = keepRatio != null ? keepRatio : true;

      /**
      * The width of the game's resolution.
      * @property width
      * @type number
      * @static
      * @readOnly
       */
      this.width = this.canvas.width;

      /**
      * The width of the game's resolution.
      * @property height
      * @type number
      * @static
      * @readOnly
       */
      this.height = this.canvas.height;

      /**
      * The graphic context. This in regular is an OpenGL context but it depends on the platform.
      * The graphic context should only be accessed if really necessary.
      *
      * @property context
      * @type Object
      * @static
      * @readOnly
       */
      this.context = this.canvas.getContext("webgl", {
        alpha: true
      }) || this.canvas.getContext("experimental-webgl", {
        alpha: false,
        antialias: false
      });

      /**
      * The renderer to render graphical objects on screen.
      *
      * @property renderer
      * @type gs.Renderer
      * @static
      * @readOnly
       */
      this.renderer = null;
      this.frameStartTime = 0;
      this.frameTime = 0;

      /**
      * A callback function called whenever the game screen receives input focus.
      *
      * @property @onFocusReceive
      * @type Function
      * @static
       */
      this.onFocusReceive = null;

      /**
      * A callback function called whenever the game screen loses input focus.
      *
      * @property @onFocusLost
      * @type Function
      * @static
       */
      this.onFocusLost = null;

      /**
      * The target frame-rate. This is property doesn't reflect the actual current frame-rate
      * of the running game.
      *
      * @property frameRate
      * @default 60
      * @type number
      * @static
       */
      this.frameRate = 60;
      window.$gl = this.context;
      if (typeof $gl === "undefined" || $gl === null) {
        throw "No WebGL Support or WebGL is disabled. Check your browser configuration.";
      }
      $gl.clearColor(0.0, 0.0, 0.0, 1.0);
      $gl.viewport(0, 0, this.width, this.height);
      $gl.disable($gl.DEPTH_TEST);
      $gl.enable($gl.BLEND);
      $gl.blendFuncSeparate($gl.SRC_ALPHA, $gl.ONE_MINUS_SRC_ALPHA, $gl.ONE, $gl.ONE_MINUS_SRC_ALPHA);
      $gl.enable($gl.SCISSOR_TEST);
      $gl.scissor(0, 0, this.width, this.height);
      this.graphics = [];

      /**
      * The top-level viewport. Adding/Removing objects from that viewport is equivalent to
      * gs.Graphics.addGraphicObject/gs.Graphics.removeGraphicObject methods.
      *
      * @property viewport
      * @type gs.Viewport
      * @static
      * @readOnly
       */
      this.viewport = new Viewport(0, 0, width, height);
      this.renderer = new Renderer(this.context);
      this.screenTexture = new Texture2D(width, height, null, true, false);
      this.screenTexture.bind();
      this.screenTexture.setFilter(0, false);
      this.screenTexture.unbind();
      this.offscreenTexture = new Texture2D(Math.nextPow2(width), Math.nextPow2(height), null, false, true);
      this.frameBuffer = new gs.FrameBuffer();
      this.screenProjectionMatrix = mat4.create();
      mat4.identity(this.screenProjectionMatrix);
      mat4.ortho(this.screenProjectionMatrix, 0, this.offscreenTexture.realWidth, this.offscreenTexture.realHeight, 0, -1, 1);
      this.canvas.style.position = "absolute";
      jQuery(this.canvas).css("-webkit-font-smoothing", "none");
      this.screenDiff = jQuery(window).height() - this.height;
      this.screenDiffW = jQuery(window).width() - this.width;
      this.parent = parent;
      this.frameCount = 0;
      this.paused = false;
      this.stopped = false;
      this.screenRenderInfo = {
        position: vec3.create([0, 0, 0]),
        size: vec3.create([this.canvas.width, this.canvas.height, 1])
      };
      jQuery(window).resize((function(_this) {
        return function() {
          return _this.onResize();
        };
      })(this));
      if (window.nw) {
        nw.Window.get().on("resize", (function(_this) {
          return function(w, h) {
            return _this.onResize();
          };
        })(this));
      }
      fps = 60;
      now = 0;
      t = Date.now();
      interval = 1000 / fps;
      delta = 0;
      if (!$PARAMS.preview && !gs.Platform.isAtsumaru) {
        window.addEventListener("focus", (function(_this) {
          return function(e) {
            _this.paused = false;
            gs.Audio.resume();
            Input.clear();
            return typeof _this.onFocusReceive === "function" ? _this.onFocusReceive() : void 0;
          };
        })(this));
        window.addEventListener("blur", (function(_this) {
          return function(e) {
            _this.paused = true;
            gs.Audio.pause();
            Input.clear();
            return typeof _this.onFocusLost === "function" ? _this.onFocusLost() : void 0;
          };
        })(this));
        window.addEventListener("keydown", (function(_this) {
          return function(e) {
            if (e.keyCode === 9) {
              return e.preventDefault();
            }
          };
        })(this));
      }
      if (window.requestAnimationFrame != null) {
        this.onEachFrame = (function(_this) {
          return function(cb) {
            var _cb;
            _cb = function(time) {
              if (!_this.stopped) {
                if (!_this.paused || $PARAMS.preview) {
                  cb(time);
                }
                return requestAnimationFrame(_cb);
              }
            };
            return _cb();
          };
        })(this);
      } else if (window.webkitRequestAnimationFrame != null) {
        this.onEachFrame = (function(_this) {
          return function(cb) {
            var _cb;
            _cb = function() {
              cb();
              return webkitRequestAnimationFrame(_cb);
            };
            return _cb();
          };
        })(this);
      } else if (window.mozRequestAnimationFrame != null) {
        this.onEachFrame = (function(_this) {
          return function(cb) {
            var _cb;
            _cb = function() {
              cb();
              return mozRequestAnimationFrame(_cb);
            };
            return _cb();
          };
        })(this);
      } else {
        this.onEachFrame = (function(_this) {
          return function(cb) {
            return setInterval(cb, 16.666667);
          };
        })(this);
      }
      parent.appendChild(this.canvas);
      jQuery(document).mousedown((function(e) {
        e.preventDefault();
        e.stopPropagation();
        return e.stopImmediatePropagation();
      }));
      jQuery(document).mouseup((function(e) {
        e.preventDefault();
        e.stopPropagation();
        return e.stopImmediatePropagation();
      }));
      canvas = this.canvas;
      if ($MOBILE) {
        jQuery(canvas).css("left", (jQuery(window).width() - canvas.width) / 2);
        jQuery(canvas).css("top", this.screenDiff / 2);
      } else {
        jQuery(this.canvas).css("left", 0);
        jQuery(this.canvas).css("top", 0);
      }
      if ($MOBILE) {
        jQuery("canvas").css("-webkit-transform", "scale(" + (1 + (this.screenDiff / this.height)) + ")");
      }
      return this.onResize();
    };


    /**
    * Disposes the graphic system and frees resources/memory. Should be called before
    * the game quits. After that, no graphic objects should be created and no other
    * graphical operations should be executed.
    *
    * @method dispose
    * @static
     */

    Graphics_OpenGL.prototype.dispose = function() {
      if (this.onDispose != null) {
        this.onDispose();
      }
      this.renderer.dispose();
      this.screenTexture.dispose();
      this.offscreenTexture.dispose();
      return this.frameBuffer.dispose();
    };


    /*
    onResize: (width, height) ->
      jQuery(@canvas).css("-webkit-transform", "none")
      jQuery(@canvas).css("-ms-transform", "none")
      jQuery(@canvas).css("-moz-transform", "none")
      jQuery(@canvas).css("transform", "none")
    
      height = height || jQuery(window).height()
      width = width || jQuery(window).width()
    
      @screenDiff = height - (@height)
      @screenDiffW = width - (@width)
      scaleH = 1.0 + @screenDiff / @height
    
      if @keepRatio
        scaleH = Math.min(scaleH, width / @width)
    
      if @keepRatio
        @canvas.width = Math.floor(@width * (scaleH))
        @canvas.height = Math.floor(@height * (scaleH))
        @x = Math.floor((width - @canvas.width) / 2)
        @y = Math.floor((height - @canvas.height) / 2)
        jQuery(@canvas).css("left", @x)
        jQuery(@canvas).css("top", @y)
      else
        @x = 0
        @y = 0
        jQuery(@canvas).css("left", 0)
        jQuery(@canvas).css("top", 0)
        @canvas.width = width
        @canvas.height = height
    
      @scaleFactorX = @width / @canvas.width
      @scaleFactorY = @height / @canvas.height
    
    
      @renderer.projectionMatrix = mat4.create()
      mat4.identity(@projectionMatrix)
      mat4.ortho(0, @canvas.width, @canvas.height, 0, -1, 1, @renderer.projectionMatrix)
     */

    Graphics_OpenGL.prototype.onResize = function(width, height) {
      var scaleH, scaleW;
      height = height || jQuery(window).height();
      width = width || jQuery(window).width();
      if (!this.screenRenderInfo) {
        this.screenRenderInfo = {
          position: vec3.create([0, 0, 0]),
          size: vec3.create([this.canvas.width, this.canvas.height, 1])
        };
      }
      this.screenRenderInfo.size[0] = this.canvas.width;
      this.screenRenderInfo.size[1] = this.canvas.height;
      this.screenDiff = height - this.height;
      this.screenDiffW = width - this.width;
      scaleH = 1.0 + this.screenDiff / this.height;
      scaleW = 1.0 + (this.screenDiffW / this.width);
      if (this.keepRatio) {
        scaleH = Math.min(scaleH, width / this.width);
        scaleW = scaleH;
      }
      jQuery(this.canvas).css("-webkit-transform", "none");
      jQuery(this.canvas).css("-ms-transform", "none");
      jQuery(this.canvas).css("-moz-transform", "none");
      jQuery(this.canvas).css("transform", "none");
      jQuery(this.canvas).css("-webkit-transform", "scale(" + (this.keepRatio ? "" : (1.0 + (this.screenDiffW / this.width)) + ", ") + scaleH + ")");
      jQuery(this.canvas).css("-ms-transform", "scale(" + (this.keepRatio ? "" : (1.0 + (this.screenDiffW / this.width)) + ", ") + scaleH + ")");
      jQuery(this.canvas).css("-moz-transform", "scale(" + (this.keepRatio ? "" : (1.0 + (this.screenDiffW / this.width)) + ", ") + scaleH + ")");
      jQuery(this.canvas).css("transform", "scale(" + (this.keepRatio ? "" : (1.0 + (this.screenDiffW / this.width)) + ", ") + scaleH + ")");
      jQuery(this.canvas).css("left", (width - this.width) / 2);
      jQuery(this.canvas).css("top", (height - this.height) / 2);
      this.x = 0;
      this.y = 0;
      if (this.keepRatio) {
        this.x = Math.floor((width - this.canvas.width * scaleH) / 2);
        this.y = Math.floor((height - this.canvas.height * scaleH) / 2);
      }
      this.scaleFactorX = this.width / (this.width * scaleW);
      return this.scaleFactorY = this.height / (this.height * scaleH);
    };


    /**
    * Sets the current pointer/cursor image. The cursor's hotspot/click-point
    * starts at the top-left corner of the image like 0, 0 means the top-left corner
    * as hotspot/click-point.
    *
    * @method setCursorBitmap
    * @param {gs.Bitmap} bitmap - The bitmap used to display the current pointer/cursor.
    * @param {number} hotspotX - The x-coordinate of the cursor's hotspot/click-point.
    * @param {number} hotspotY - The y-coordinate of the cursor's hotspot/click-point.
    * @static
     */

    Graphics_OpenGL.prototype.setCursorBitmap = function(bitmap, hotspotX, hotspotY) {
      var url;
      if (bitmap && !bitmap.image) {
        bitmap.makeMutable();
      }
      if (bitmap != null ? bitmap.image : void 0) {
        url = bitmap.image.toDataURL("image/png");
        return jQuery(document.body).css("cursor", "url('" + url + "') " + (hotspotX || 0) + " " + (hotspotY || 0) + ", default");
      } else {
        return jQuery(document.body).css("cursor", "default");
      }
    };


    /**
    * Enters fullscreen mode.
    *
    * @method enterFullscreen
    * @static
     */

    Graphics_OpenGL.prototype.enterFullscreen = function() {
      var ex, fs, pack, win;
      if (window.nw && !$PARAMS.platform.isSteamOS) {
        win = nw.Window.get();
        win.setResizable(true);
        win.enterFullscreen();
        try {
          fs = require("fs");
          if (fs.existsSync("./package.json")) {
            pack = JSON.parse(fs.readFileSync("./package.json", "utf8"));
            pack.fullscreen = true;
            return fs.writeFileSync("./package.json", JSON.stringify(pack), "utf8");
          }
        } catch (error) {
          ex = error;
          console.log("ERROR: Cannot store display-settings in package.json. Do have write-access to the game's directory/folder?");
          return console.log(ex);
        }
      }
    };


    /**
    * Leaves fullscreen mode.
    *
    * @method leaveFullscreen
    * @static
     */

    Graphics_OpenGL.prototype.leaveFullscreen = function() {
      var ex, fs, pack, win;
      if (window.nw && !$PARAMS.platform.isSteamOS) {
        win = nw.Window.get();
        win.leaveFullscreen();
        try {
          fs = require("fs");
          if (fs.existsSync("./package.json")) {
            pack = JSON.parse(fs.readFileSync("./package.json", "utf8"));
            pack.fullscreen = false;
            fs.writeFileSync("./package.json", JSON.stringify(pack), "utf8");
          }
        } catch (error) {
          ex = error;
          console.log("ERROR: Cannot store display-settings in package.json. Do have write-access to the game's directory/folder?");
          console.log(ex);
        }
        win.setResizable(true);
        win.width = this.width;
        win.height = this.height;
        this.onResize();
        return setTimeout(((function(_this) {
          return function() {
            return _this.onResize();
          };
        })(this)), 1);
      }
    };


    /**
    * Indiciates if the game runs currently in fullscreen mode.
    *
    * @method isFullscreen
    * @return {boolean} Returns <b>true</b> if the game runs in fullscreen mode. Otherwise <b>false</b>.
    * @static
     */

    Graphics_OpenGL.prototype.isFullscreen = function() {
      if (window.nw != null) {
        return nw.Window.get().isFullscreen;
      } else {
        return true;
      }
    };


    /**
    * Clears the screen.
    *
    * @method clear
    * @static
     */

    Graphics_OpenGL.prototype.clear = function() {
      return $gl.clear($gl.COLOR_BUFFER_BIT | $gl.DEPTH_BUFFER_BIT);
    };


    /**
    * Updates the screen. Renders all graphic objects. Should be called once per frame.
    *
    * @method update
    * @static
     */

    Graphics_OpenGL.prototype.update = function() {
      this.frameBuffer.bind();
      this.frameBuffer.setTexture(this.screenTexture);
      $gl.viewport(0, 0, this.screenTexture.width, this.screenTexture.height);
      this.viewport.update();
      if (this.transitionData != null) {
        this.updateTransition();
      } else if (!this.frozen) {
        this.clear();
        this.viewport.draw();
        this.renderer.render();
      }
      this.renderer.clip.set(0, 0, this.width, this.height);
      this.frameBuffer.unbind();
      $gl.scissor(0, 0, this.canvas.width, this.canvas.height);
      this.renderer.setTexture(this.screenTexture);
      mat2d.identity(this.renderer.transform_);
      this.renderer.transform = this.renderer.transform_;
      this.renderer.setSourceRectangle(0, 0, Graphics.width, Graphics.height);
      mat2d.translate(this.renderer.transform, this.renderer.transform, [0, Graphics.height]);
      mat2d.scale(this.renderer.transform, this.renderer.transform, [Graphics.width, -Graphics.height]);
      this.renderer.drawTransform();
      return this.renderer.render();
    };


    /**
    * Adds a graphic object to the graphic system. The graphic object will be automatically
    * rendered and sorted by z-index.
    *
    * @method addGraphicObject
    * @static
    * @param {gs.GraphicObject} object - The graphic object to add.
     */

    Graphics_OpenGL.prototype.addGraphicObject = function(object) {
      if (this.viewport != null) {
        return this.viewport.addGraphicObject(object);
      }
    };


    /**
    * Removes a graphic object from the graphic system. The graphic object is no longer rendered
    * then.
    *
    * @method removeGraphicObject
    * @static
    * @param {gs.GraphicObject} object - The graphic object to remove.
     */

    Graphics_OpenGL.prototype.removeGraphicObject = function(object) {
      if (this.viewport != null) {
        return this.viewport.removeGraphicObject(object);
      }
    };

    Graphics_OpenGL.prototype.snapshotToTexture = function(texture) {
      this.frameBuffer.bind();
      this.frameBuffer.setTexture(this.screenTexture);
      this.clear();
      this.viewport.update();
      this.viewport.draw();
      this.renderer.render();
      texture.bind();
      $gl.copyTexSubImage2D($gl.TEXTURE_2D, 0, 0, 0, 0, 0, Graphics.width, Graphics.height);
      texture.unbind();
      return this.frameBuffer.unbind();
    };

    Graphics_OpenGL.prototype.snapshotAsTexture = function(texture) {
      var data;
      this.frameBuffer.bind();
      this.frameBuffer.setTexture(this.screenTexture);
      this.clear();
      this.viewport.update();
      this.viewport.draw();
      this.renderer.render();
      data = new Uint8Array(this.width * this.height * 4);
      $gl.pixelStorei($gl.UNPACK_FLIP_Y_WEBGL, true);
      $gl.readPixels(0, 0, this.width, this.height, $gl.RGBA, $gl.UNSIGNED_BYTE, data);
      this.frameBuffer.unbind();
      texture = new Texture2D(data, this.width, this.height, false, true);
      $gl.pixelStorei($gl.UNPACK_FLIP_Y_WEBGL, false);
      return texture;
    };


    /**
    * Creates a snapshot from the current screen.
    *
    * @method snapshot
    * @static
    * @return {gs.Bitmap} A snapshot from the current screen.
     */

    Graphics_OpenGL.prototype.snapshot = function() {
      var canvas, context, data, imageData;
      this.frameBuffer.bind();
      data = new Uint8Array(this.width * this.height * 4);
      $gl.readPixels(0, 0, this.width, this.height, $gl.RGBA, $gl.UNSIGNED_BYTE, data);
      this.frameBuffer.unbind();
      canvas = document.createElement('canvas');
      canvas.width = this.width;
      canvas.height = this.height;
      context = canvas.getContext('2d');
      context.clearRect(0, 0, this.width, this.height);
      imageData = context.createImageData(this.width, this.height);
      imageData.data.set(data);
      context.putImageData(imageData, 0, 0);
      return new Bitmap(canvas, true);
    };


    /**
    * Creates a snapshot from the current screen.
    *
    * @method snapshotAsData
    * @static
    * @return {Uint8Array} A snapshot from the current screen a uint8 pixel data array.
     */

    Graphics_OpenGL.prototype.snapshotAsData = function() {
      var data;
      this.viewport.update();
      this.viewport.draw();
      this.renderer.render();
      data = new Uint8Array(this.width * this.height * 4);
      $gl.readPixels(0, 0, this.width, this.height, $gl.RGBA, $gl.UNSIGNED_BYTE, data);
      return data;
    };


    /**
    * Freezes the current screen. Prepares for transition.
    *
    * @method freeze
    * @static
    * @see gs.Graphics.transition
     */

    Graphics_OpenGL.prototype.freeze = function() {
      if (!this.frozen) {
        $gl.scissor(0, 0, this.width, this.height);
        this.frozen = true;
        this.frozenScreenTexture = new Texture2D(Graphics.width, Graphics.height, null, true, false);
        return this.snapshotToTexture(this.frozenScreenTexture);
      }
    };


    /**
    * Executes a transition from the frozen state to the current state. If no
    * frozen state exists, no transition is executed. To freeze the current state
    * use gs.Graphics.freeze method.
    *
    * @method transition
    * @static
    * @see gs.Graphics.freeze
    * @param {number} duration - The transition duration in frames.
    * @param {gs.Bitmap} [transitionBitmap=null] - The mask-bitmap used for the transition. If <b>null</b> a simple
    * transition using blending is executed.
    * @param {number} [vague=null] - The vague of the masking if a mask-bitmap is specified.
     */

    Graphics_OpenGL.prototype.transition = function(duration, transitionBitmap, vague) {
      if (this.frozenScreenTexture != null) {
        $gl.scissor(0, 0, this.width, this.height);
        this.transitionSnapshot = new Texture2D(Graphics.width, Graphics.height, null, true, false);
        this.snapshotToTexture(this.transitionSnapshot);
        this._transition(this.frozenScreenTexture, this.transitionSnapshot, duration, transitionBitmap, vague);
        return this.update();
      }
    };

    Graphics_OpenGL.prototype._transition = function(sourceTexture, targetTexture, duration, transitionBitmap, vague) {
      this.transitionData = {
        sourceTexture: sourceTexture,
        targetTexture: targetTexture,
        duration: duration,
        transitionBitmap: transitionBitmap,
        vague: vague,
        mask: {
          ox: 0,
          oy: 0,
          source: transitionBitmap,
          type: 1,
          value: 0,
          vague: vague
        }
      };
      return this.transitionData.currentAlpha = 0;
    };

    Graphics_OpenGL.prototype.updateTransition = function() {
      var ref;
      $gl.scissor(0, 0, this.width, this.height);
      if ((ref = this.transitionData.transitionBitmap) != null ? ref.loaded : void 0) {
        return this.updateBitmapTransition();
      } else {
        return this.updateDefaultTransition();
      }
    };

    Graphics_OpenGL.prototype.updateDefaultTransition = function() {
      this.renderer.setTexture(this.transitionData.sourceTexture);
      this.renderer.setSourceRectangle(0, Graphics.height, Graphics.width, -Graphics.height);
      this.renderer.position[0] = 0;
      this.renderer.position[1] = 0;
      this.renderer.size[0] = this.width;
      this.renderer.size[1] = this.height;
      mat2d.identity(this.renderer.transform_);
      mat2d.scale(this.renderer.transform_, this.renderer.transform_, vec2.fromValues(this.width, this.height));
      this.renderer.transform = this.renderer.transform_;
      this.renderer.drawQuad(this.renderer.position, this.renderer.size);
      this.renderer.render();
      this.renderer.opacity = this.transitionData.currentAlpha;
      this.renderer.setTexture(this.transitionData.targetTexture);
      this.renderer.position[0] = 0;
      this.renderer.position[1] = 0;
      this.renderer.size[0] = this.width;
      this.renderer.size[1] = this.height;
      this.renderer.drawQuad(this.renderer.position, this.renderer.size);
      this.renderer.render();
      this.renderer.opacity = 255;
      if (this.transitionData.currentAlpha >= 255) {
        this.transitionData.sourceTexture.dispose();
        this.transitionData.targetTexture.dispose();
        this.transitionData = null;
        this.frozenScreenTexture = null;
        return this.frozen = false;
      } else {
        return this.transitionData.currentAlpha += 255 / this.transitionData.duration;
      }
    };

    Graphics_OpenGL.prototype.updateBitmapTransition = function() {
      this.transitionData.currentAlpha = Math.min(this.transitionData.currentAlpha + (255 / this.transitionData.duration), 255);
      $gl.disable($gl.BLEND);
      mat2d.identity(this.renderer.transform_);
      mat2d.scale(this.renderer.transform_, this.renderer.transform_, vec2.fromValues(this.width, this.height));
      this.renderer.transform = this.renderer.transform_;
      this.renderer.setSourceRectangle(0, Graphics.height, Graphics.width, -Graphics.height);
      this.renderer.setTexture(this.transitionData.targetTexture);
      this.renderer.drawQuad(this.renderer.position, this.renderer.size);
      this.renderer.render();
      $gl.enable($gl.BLEND);
      this.renderer.mask = this.transitionData.mask;
      this.renderer.mask.value = this.transitionData.currentAlpha;
      this.renderer.setTexture(this.transitionData.sourceTexture);
      this.renderer.drawQuad(this.renderer.position, this.renderer.size);
      this.renderer.render();
      this.renderer.mask = null;
      if (this.transitionData.currentAlpha >= 255) {
        this.transitionData.sourceTexture.dispose();
        this.transitionData.targetTexture.dispose();
        this.transitionData = null;
        this.frozenScreenTexture = null;
        return this.frozen = false;
      }
    };

    return Graphics_OpenGL;

  })();

  window.Graphics_OpenGL = Graphics_OpenGL;

  window.Graphics = window.$PARAMS != null ? new Graphics_Canvas() : new Graphics_Canvas();

  gs.Graphics = Graphics;

}).call(this);

// Generated by CoffeeScript 1.12.2

/**
* <p>A tilemap-render info holds more information about the tilemap being rendered on screen. This class
* is actually for internal usage only. The only exception
* is if you work with custom GLSL shaders.</p>
* <p>If you want to learn more about why certain classes of the Basic Engine are hidden and not editable, read the Basic Engine chapter
* in Help File.</p>
*
* @namespace gs
* @class TilemapRenderInfo
* @memberof gs
* @constructor
 */

(function() {
  var RenderTask, Renderer_OpenGL, TilemapRenderInfo;

  TilemapRenderInfo = (function() {
    function TilemapRenderInfo() {

      /**
      * The tileset for the tilemap.
      * @property tilesetBitmap
      * @default null
      * @type gs.Bitmap
       */
      this.tilesetBitmap = null;

      /**
      * A bitmap object which has the same size as the map. Each pixel describes a single tile. The red-component
      * of the pixel stores the column and the green-component stores the row of the tile on the tileset.
      * @property dataBitmap
      * @default null
      * @type gs.Bitmap
       */
      this.dataBitmap = null;

      /**
      * The offset-value on x-axis. Change this value to scroll the tilemap.
      * @property ox
      * @default 0
      * @type number
       */
      this.ox = 0;

      /**
      * The offset-value on y-axis. Change this value to scroll the tilemap.
      * @property oy
      * @default 0
      * @type number
       */
      this.oy = 0;

      /**
      * <p>The auto/animated/special animation frame-number to display. This is multiplied with the width of a special-tile to
      * determine which animation-frame/tile to display. Let's say an animated tile is 32x32 with 4 animation-frames,
      * then the calculation is:</p>
      * <code>source-rectangle.x = regular-tiles-area.x + autotileOffset * 32</code>
      *
      * @property autotileOffset
      * @default 0
      * @type number
       */
      this.autotileOffset = 0;

      /**
      * The x-position of the "red-line" separating the regular-tiles and special-tiles area. Check Scripter's Guide in
      * Help-File for more info.
      *
      * @property autotileAreaStart
      * @default 0
      * @type number
       */
      this.autotileAreaStart = 0;

      /**
      * The tile-size in pixels.
      *
      * @property tileSize
      * @default 32
      * @type number
       */
      this.tileSize = 32;
    }

    return TilemapRenderInfo;

  })();

  gs.TilemapRenderInfo = TilemapRenderInfo;


  /**
  * <p>A render-task holds information about a graphic object which should be rendered on screen. This class
  * is actually for internal usage only. The only exception
  * is if you work with custom GLSL shaders.</p>
  * <p>If you want to learn more about why certain classes of the Basic Engine are hidden and not editable, read the Basic Engine chapter
  * in Help File.</p>
  *
  * @namespace gs
  * @class RenderTask
  * @memberof gs
  * @constructor
   */

  RenderTask = (function() {
    function RenderTask() {
      this.clear = false;

      /**
      * The default texture to render.
      * @property texture
      * @default null
      * @type gs.Texture2D
       */
      this.texture = null;

      /**
      * The opacity to use.
      * @property opacity
      * @default 0
      * @type number
       */
      this.opacity = 0;

      /**
      * The blend-mode to use.
      * @property blend-mode
      * @default 0
      * @type number
       */
      this.blendMode = 0;

      /**
      * The clip-rect to use.
      * @property clip
      * @default new Rect(0, 0, Graphics.width, Graphics.height)
      * @type gs.Rect
       */
      this.clip = new Rect(0, 0, Graphics.width, Graphics.height);
      this.l2dObject = null;

      /**
      * If the render-task renders a tilemap, this property contains more info about it.
      * @property tilemap
      * @default null
      * @type gs.TilemapRenderInfo
       */
      this.tilemap = null;

      /**
      * Indicates if the texture should be tiled.
      * @property tiling
      * @default false
      * @type boolean
       */
      this.tiling = false;

      /**
      * The graphic effects to use.
      * @property effects
      * @default new gs.EffectCollection()
      * @type gs.EffectCollection
       */
      this.effects = new gs.EffectCollection();
    }

    return RenderTask;

  })();

  gs.RenderTask = RenderTask;

  Renderer_OpenGL = (function() {

    /**
    * The renderer is used to render graphical object on screen like images, colored rectangles, etc. It behaves
    * like a state-machine. So first step is to set origin, rotation-angle, color, etc. and then render
    * graphical objects with that settings.
    *
    * @namespace gs
    * @class Renderer
    * @memberof gs
    * @constructor
    * @param {Object} context - The graphics context.
     */
    function Renderer_OpenGL(context) {
      Effect.initialize();

      /**
      * The graphics context.
      * @property context
      * @type Object
      * @readOnly
       */
      this.context = context;
      this.canvas = context.canvas;
      this.clear = false;

      /**
      * The x-coordinate of the origin.
      * @property ox
      * @type number
       */
      this.ox = 0;

      /**
      * The y-coordinate of the origin.
      * @property oy
      * @type number
       */
      this.oy = 0;

      /**
      * The anchor-point.
      * @property anchor
      * @type gs.Vector2
       */
      this.anchor = {
        x: 0,
        y: 0

        /**
        * The zoom.
        * @property zoom
        * @type gs.Vector2
         */
      };
      this.zoom = {
        x: 1.0,
        y: 1.0
      };
      this.size = {
        width: 0,
        height: 0
      };
      this.textureOffset = {
        x: 0.0,
        y: 0.0
      };
      this.transform_ = mat2d.create();
      this.transformVector_ = vec2.create();
      this.effects = null;

      /**
      * The rotation angle.
      * @property angle
      * @type number
       */
      this.angle = 0;

      /**
      * Indicates if rendered images are mirrored/flipped horizontally.
      * @property mirror
      * @type boolean
       */
      this.mirror = false;
      this.mirrorY = false;

      /**
      * The color.
      * @property color
      * @type gs.Color
       */
      this.color = new Color(255, 255, 255, 0);

      /**
      * The opacity. Goes from 0 to 255.
      * @property opacity
      * @type number
       */
      this.opacity = 255;

      /**
      * The tone.
      * @property tone
      * @type gs.Tone
       */
      this.tone = new Tone(0, 0, 0, 0);

      /**
      * The blend mode.
      * @property blendType
      * @type gs.BlendMode
       */
      this.blendType = 0;

      /**
      * The current clip-rect
      * @property clip
      * @type gs.Rect
      * @readOnly
       */
      this.clip = new Rect(0, 0, this.canvas.width, this.canvas.height);
      this.transform = mat2d.create();
      this.texBuffers = [];
      this.defaultTone = this.tone;
      this.vertexBuffer = null;
      this.indexBuffer = null;

      /**
      * The effect settings for different kind of visual effect like wobble, etc.
      * @property effects
      * @type Object
       */
      this.maxRenderTasks = 1000;
      this.renderTasks = new Array(this.maxRenderTasks);
      this.renderTaskIndex = 0;
      this.vertexSize = 5;
      this.vertexColorData = null;
      this.vertexDataBuffer = null;
      this.renderTaskSize = 4 * (this.vertexSize * 4);
      this.createBufferObjects();
      this.rotationCenter = vec3.create([0, 0, 0]);
      this.rotationCenterInverse = vec3.create([0, 0, 0]);
      this.identity = mat4.create();
      mat4.identity(this.identity);
      this.width = this.canvas.width;
      this.height = this.canvas.height;
      this.textures = [null, null];
      this.sourceRectangle = [0.0, 0.0, 1.0, 1.0];
      this.position = vec3.create([0, 0, 0]);
      this.position1 = vec2.fromValues(0, 0);
      this.position2 = vec2.fromValues(1, 0);
      this.position3 = vec2.fromValues(0, 1);
      this.position4 = vec2.fromValues(1, 1);
      this.size = vec3.create([1, 1, 1]);
      this.color = [1.0, 1.0, 1.0, 1.0];
      this.colors = [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0];
      this.clipRectangle = [0, 0, this.width, this.height];
      this.transform = mat4.create();
      this.translation = mat4.create();
      this.scale = mat4.create();
      mat4.identity(this.translation);
      mat4.identity(this.scale);
      mat4.identity(this.transform);

      /*
      vertexShader = new VertexShader('shader-vs');
      vertexRectShader = new VertexShader('shader-rect-vs');
      vertexTilemapShader = new VertexShader(if Graphics.height == 240 then 'tilemap-vs' else 'tilemap-32-vs')
      fragmentShader = new FragmentShader('shader-fs');
      toneShader = new FragmentShader('tinting-fs');
      colorShader = new FragmentShader('color-fs');
      colorToneShader = new FragmentShader('color-tinting-fs');
      colorRectShader = new FragmentShader('color-rect-fs');
      transitionShader = new FragmentShader('transition-fs');
      fragmentTilemapShader = new FragmentShader(if Graphics.height == 240 then 'tilemap-fs' else 'tilemap-32-fs')
      maskingShader = new FragmentShader('masking-fs');
      dynamicMaskingShader = new FragmentShader('dynamic-masking-fs');
      tilingShader = new FragmentShader('tiling-fs');
       */

      /*
      @vertexShader = vertexShader
       *  @vertexRectShader = vertexRectShader
      @fragmentShader = fragmentShader
      @toneShader = toneShader
      @colorShader = colorShader
      @colorToneShader = colorToneShader
      @colorRectShader = colorRectShader
      @transitionShader = transitionShader
      @vertexTilemapShader = vertexTilemapShader
      @fragmentTilemapShader = fragmentTilemapShader
      @maskingShader = maskingShader
      @dynamicMaskingShader = dynamicMaskingShader
      @tilingShader = tilingShader
       */
      this.effect = Effect.get(1);
      this.projectionMatrix = mat4.create();
      mat4.identity(this.projectionMatrix);
      mat4.ortho(this.projectionMatrix, 0, this.width, this.height, 0, -1, 1);

      /*
      effect = new Effect()
      effect.vertexBuffer = @vertexBuffer
      effect.setVertexShader(vertexShader)
      effect.setFragmentShader(fragmentShader)
      effect.build()
      effect.prepare()
      effect.positionBuffer = @positionBuffer
      effect.textureCoordBuffer = @textureCoordBuffer
      @texBuffer = @textureCoordBuffer
      @basicEffect = effect
      @effect = effect
      
      
      @tilingEffect = new Effect()
      @tilingEffect.setVertexShader(vertexShader)
      @tilingEffect.setFragmentShader(@tilingShader)
      @tilingEffect.vertexBuffer = @vertexBuffer
      @tilingEffect.build()
      @tilingEffect.prepare()
      
      @tilemapEffect = new Effect()
      @tilemapEffect.setVertexShader(@vertexTilemapShader)
      @tilemapEffect.setFragmentShader(@fragmentTilemapShader)
      @tilemapEffect.vertexBuffer = @vertexBuffer
      @tilemapEffect.build()
      @tilemapEffect.prepare()
      
      @tintEffect = new Effect()
      @tintEffect.vertexBuffer = @vertexBuffer
      @tintEffect.setVertexShader(vertexShader)
      @tintEffect.setFragmentShader(toneShader)
      @tintEffect.build()
      @tintEffect.prepare()
      
      @colorEffect = new Effect()
      @colorEffect.vertexBuffer = @vertexBuffer
      @colorEffect.setVertexShader(vertexShader)
      @colorEffect.setFragmentShader(colorShader)
      @colorEffect.build()
      @colorEffect.prepare()
      
      
      @colorRectEffect = new Effect(true) # No Texture
      @colorRectEffect.vertexBuffer = @vertexBuffer
      @colorRectEffect.setVertexShader(vertexShader)
      @colorRectEffect.setFragmentShader(colorRectShader)
      @colorRectEffect.build()
      @colorRectEffect.prepare()
      
      
      @colorTintEffect = new Effect()
      @colorTintEffect.vertexBuffer = @vertexBuffer
      @colorTintEffect.setVertexShader(vertexShader)
      @colorTintEffect.setFragmentShader(colorToneShader)
      @colorTintEffect.build()
      @colorTintEffect.prepare()
      
      
      @transitionEffect = new Effect()
      @transitionEffect.vertexBuffer = @vertexBuffer
      @transitionEffect.setVertexShader(vertexShader)
      @transitionEffect.setFragmentShader(transitionShader)
      @transitionEffect.build()
      @transitionEffect.prepare()
      
      @maskEffect = new Effect()
      @maskEffect.vertexBuffer = @vertexBuffer
      @maskEffect.setVertexShader(vertexShader)
      @maskEffect.setFragmentShader(maskingShader)
      @maskEffect.build()
      @maskEffect.prepare()
      
      @dynamicMaskEffect = new Effect()
      @dynamicMaskEffect.vertexBuffer = @vertexBuffer
      @dynamicMaskEffect.setVertexShader(vertexShader)
      @dynamicMaskEffect.setFragmentShader(dynamicMaskingShader)
      @dynamicMaskEffect.build()
      @dynamicMaskEffect.prepare()
      
      
      @vBlurEffect = new Effect()
      @vBlurEffect.vertexBuffer = @vertexBuffer
      @vBlurEffect.setVertexShader(vertexShader)
      @vBlurEffect.setFragmentShader(new FragmentShader('v-blur-fs'))
      @vBlurEffect.build()
      @vBlurEffect.prepare()
      
      @hBlurEffect = new Effect()
      @hBlurEffect.vertexBuffer = @vertexBuffer
      @hBlurEffect.setVertexShader(vertexShader)
      @hBlurEffect.setFragmentShader(new FragmentShader('h-blur-fs'))
      @hBlurEffect.build()
      @hBlurEffect.prepare()
       */
    }


    /**
    * Disposes the renderer and frees all allocated resources/memory.
    *
    * @method dispose
     */

    Renderer_OpenGL.prototype.dispose = function() {
      var buffer, k, len, ref;
      this.effect.dispose();
      this.tintEffect.dispose();
      this.colorEffect.dispose();
      this.colorTintEffect.dispose();
      this.colorRectEffect.dispose();
      this.transitionEffect.dispose();
      ref = this.texBuffers;
      for (k = 0, len = ref.length; k < len; k++) {
        buffer = ref[k];
        buffer.dispose();
      }
      this.vertexShader.dispose();
      this.fragmentShader.dispose();
      this.toneShader.dispose();
      this.colorShader.dispose();
      this.colorToneShader.dispose();
      this.colorRectShader.dispose();
      return this.transitionShader.dispose();
    };

    Renderer_OpenGL.prototype.createBufferObjects = function() {
      var i, k, ref, v;
      for (i = k = 0, ref = this.renderTasks.length; 0 <= ref ? k < ref : k > ref; i = 0 <= ref ? ++k : --k) {
        this.renderTasks[i] = new gs.RenderTask();
      }
      this.vertexDataBuffer = new ArrayBuffer(this.maxRenderTasks * 4 * (this.vertexSize * 4));
      this.vertexData = new Float32Array(this.vertexDataBuffer);
      this.vertexColorData = new Uint32Array(this.vertexDataBuffer);
      this.indexData = new Uint16Array(this.maxRenderTasks * 6);
      i = 0;
      v = 0;
      while (i < (this.maxRenderTasks * 6)) {
        this.indexData[i + 0] = v + 0;
        this.indexData[i + 1] = v + 1;
        this.indexData[i + 2] = v + 3;
        this.indexData[i + 3] = v + 2;
        this.indexData[i + 4] = v + 0;
        this.indexData[i + 5] = v + 3;
        i += 6;
        v += 4;
      }
      this.vertexBuffer = new gs.Buffer($gl.ARRAY_BUFFER, this.vertexData, 5, this.maxRenderTasks * 4, $gl.DYNAMIC_DRAW);
      this.indexBuffer = new gs.Buffer($gl.ELEMENT_ARRAY_BUFFER, this.indexData, 3, this.maxRenderTasks * 6);
      this.vertexBuffer.bind();
      return this.indexBuffer.bind();
    };

    Renderer_OpenGL.prototype.resize = function() {
      this.width = this.canvas.width;
      this.height = this.canvas.height;
      this.projectionMatrix = mat4.create();
      mat4.identity(this.projectionMatrix);
      return mat4.ortho(this.projectionMatrix, 0, this.width, this.height, 0, -1, 1);
    };

    Renderer_OpenGL.prototype.reset = function() {
      this.blendType = 0;
      this.opacity = 255;
      return $gl.blendFuncSeparate($gl.SRC_ALPHA, $gl.ONE_MINUS_SRC_ALPHA, $gl.ONE, $gl.ONE_MINUS_SRC_ALPHA);
    };

    Renderer_OpenGL.prototype.drawRotatedQuad = function(position, size, angle, rotationCenter, rotationCenterInverse) {
      return this.drawQuad(position, size);
    };

    Renderer_OpenGL.prototype.drawBuffer = function(indexBuffer) {
      var color, effect, tone;
      effect = null;
      tone = (this.tone != null) && (this.tone.red !== 0 || this.tone.green !== 0 || this.tone.blue !== 0 || this.tone.grey !== 0);
      color = (this.color != null) && this.color.alpha > 0;
      if (color && tone) {
        effect = this.colorTintEffect;
        this.colorTintEffect.use();
        this.colorTintEffect.setVector4Value("Tone", this.tone.red, this.tone.green, this.tone.blue, this.tone.grey);
        this.colorTintEffect.setVector4Value("Color", this.color.red, this.color.green, this.color.blue, this.color.alpha);
      } else if (tone) {
        effect = this.tintEffect;
        this.tintEffect.use();
        this.tintEffect.setVector4Value("Tone", this.tone.red, this.tone.green, this.tone.blue, this.tone.grey);
      } else if (color) {
        effect = this.colorEffect;
        this.colorEffect.use();
        this.colorEffect.setFloatValue("Opacity", this.opacity);
        this.colorEffect.setVector4Value("Color", this.color.red, this.color.green, this.color.blue, this.color.alpha);
      } else {
        effect = this.effect;
      }
      if (this.currentEffect !== effect || true) {
        this.currentEffect = effect;
        effect.use();
        effect.setMatrixValue("pMatrix", this.projectionMatrix);
        effect.setFloatValue("Opacity", this.opacity);
        this.effectUsed = true;
        this.projectionMatrixOld = this.projectionMatrix;
        this.opacityOld = this.opacity;
      } else if (this.opacityOld !== this.opacity) {
        effect.setFloatValue("Opacity", this.opacity);
        this.opacityOld = this.opacity;
      } else if (this.projectionMatrixOld !== this.projectionMatrix) {
        effect.setMatrixValue("pMatrix", this.projectionMatrix);
        this.projectionMatrixOld = this.projectionMatrix;
      }
      return $gl.drawElements($gl.TRIANGLES, indexBuffer.itemCount, $gl.UNSIGNED_SHORT, 0);
    };

    Renderer_OpenGL.prototype.drawTextureTransform = function(rect) {
      var color, frameCount, index, task;
      if (this.effects && this.effects.wobble.enabled) {
        index = this.renderTaskIndex * (this.vertexSize * 4);
        frameCount = Graphics.frameCount * this.effects.wobble.speed;
        if (this.effects.wobble.horizontal) {
          this.vertexData[index + 2] += Math.sin(frameCount) * this.effects.wobble.power;
          this.vertexData[index + 7] += Math.sin(frameCount + 1) * this.effects.wobble.power;
          this.vertexData[index + 12] += Math.sin(frameCount + 2) * this.effects.wobble.power;
          this.vertexData[index + 17] += Math.sin(frameCount + 3) * this.effects.wobble.power;
        }
        if (this.effects.wobble.vertical) {
          this.vertexData[index + 3] += Math.cos(frameCount) * this.effects.wobble.power;
          this.vertexData[index + 8] += Math.cos(frameCount + 1) * this.effects.wobble.power;
          this.vertexData[index + 13] += Math.cos(frameCount + 2) * this.effects.wobble.power;
          this.vertexData[index + 18] += Math.cos(frameCount + 3) * this.effects.wobble.power;
        }
      }
      index = this.renderTaskIndex * (this.vertexSize * 4);
      color = (this.opacity << 24) | (0 << 16) | (0 << 8) | 0;
      this.position1[0] = 0;
      this.position1[1] = 0;
      this.position2[0] = 1;
      this.position2[1] = 0;
      this.position3[0] = 0;
      this.position3[1] = 1;
      this.position4[0] = 1;
      this.position4[1] = 1;
      vec2.transformMat2d(this.position1, this.position1, this.transform);
      vec2.transformMat2d(this.position2, this.position2, this.transform);
      vec2.transformMat2d(this.position3, this.position3, this.transform);
      vec2.transformMat2d(this.position4, this.position4, this.transform);
      this.vertexData[index + 2] = this.position1[0] / this.texture.realWidth;
      this.vertexData[index + 3] = this.position1[1] / this.texture.realHeight;
      this.vertexData[index + 7] = this.position2[0] / this.texture.realWidth;
      this.vertexData[index + 8] = this.position2[1] / this.texture.realHeight;
      this.vertexData[index + 12] = this.position3[0] / this.texture.realWidth;
      this.vertexData[index + 13] = this.position3[1] / this.texture.realHeight;
      this.vertexData[index + 17] = this.position4[0] / this.texture.realWidth;
      this.vertexData[index + 18] = this.position4[1] / this.texture.realHeight;
      this.vertexData[index] = rect.x;
      this.vertexData[index + 1] = rect.y;
      this.vertexColorData[index + 4] = color;
      this.vertexData[index + 5] = rect.x + rect.width;
      this.vertexData[index + 6] = rect.y;
      this.vertexColorData[index + 9] = color;
      this.vertexData[index + 10] = rect.x;
      this.vertexData[index + 11] = rect.y + rect.height;
      this.vertexColorData[index + 14] = color;
      this.vertexData[index + 15] = rect.x + rect.width;
      this.vertexData[index + 16] = rect.y + rect.height;
      this.vertexColorData[index + 19] = color;
      task = this.renderTasks[this.renderTaskIndex];
      task.l2dObject = null;
      task.tilemap = null;
      task.texture = this.texture;
      task.clip.set(this.clip.x, this.clip.y, this.clip.width, this.clip.height);
      task.tone = this.tone;
      task.color = this.color;
      task.mask = this.mask;
      task.opacity = this.opacity;
      task.blendMode = this.blendType;
      task.effect = this.visualEffect;
      task.effects = this.effects;
      task.clear = this.clear;
      task.tiling = true;
      this.renderTaskIndex++;
      if (this.renderTaskIndex >= this.maxRenderTasks) {
        return this.render();
      }
    };


    /**
    * Draws a quad. The quad can be textured, rotated, tinted, etc. depending on the
    * renderer settings.
    *
    * @method drawQuad
    * @param {number[]} position - The position as number-array.
    * @param {number[]} size - The size as number-array.
     */

    Renderer_OpenGL.prototype.drawQuad = function(position, size) {
      var color, effect, frameCount, index, task;
      effect = null;
      if (this.effects && this.effects.wobble.enabled) {
        index = this.renderTaskIndex * (this.vertexSize * 4);
        frameCount = Graphics.frameCount * this.effects.wobble.speed;
        if (this.effects.wobble.horizontal) {
          this.vertexData[index + 2] += Math.sin(frameCount) * this.effects.wobble.power;
          this.vertexData[index + 7] += Math.sin(frameCount + 1) * this.effects.wobble.power;
          this.vertexData[index + 12] += Math.sin(frameCount + 2) * this.effects.wobble.power;
          this.vertexData[index + 17] += Math.sin(frameCount + 3) * this.effects.wobble.power;
        }
        if (this.effects.wobble.vertical) {
          this.vertexData[index + 3] += Math.cos(frameCount) * this.effects.wobble.power;
          this.vertexData[index + 8] += Math.cos(frameCount + 1) * this.effects.wobble.power;
          this.vertexData[index + 13] += Math.cos(frameCount + 2) * this.effects.wobble.power;
          this.vertexData[index + 18] += Math.cos(frameCount + 3) * this.effects.wobble.power;
        }
      }
      index = this.renderTaskIndex * (this.vertexSize * 4);
      color = (this.opacity << 24) | (0 << 16) | (0 << 8) | 0;
      this.position1[0] = 0;
      this.position1[1] = 0;
      this.position2[0] = 1;
      this.position2[1] = 0;
      this.position3[0] = 0;
      this.position3[1] = 1;
      this.position4[0] = 1;
      this.position4[1] = 1;
      vec2.transformMat2d(this.position1, this.position1, this.transform);
      vec2.transformMat2d(this.position2, this.position2, this.transform);
      vec2.transformMat2d(this.position3, this.position3, this.transform);
      vec2.transformMat2d(this.position4, this.position4, this.transform);
      this.vertexData[index] = this.position1[0];
      this.vertexData[index + 1] = this.position1[1];
      this.vertexColorData[index + 4] = color;
      this.vertexData[index + 5] = this.position2[0];
      this.vertexData[index + 6] = this.position2[1];
      this.vertexColorData[index + 9] = color;
      this.vertexData[index + 10] = this.position3[0];
      this.vertexData[index + 11] = this.position3[1];
      this.vertexColorData[index + 14] = color;
      this.vertexData[index + 15] = this.position4[0];
      this.vertexData[index + 16] = this.position4[1];
      this.vertexColorData[index + 19] = color;
      task = this.renderTasks[this.renderTaskIndex];
      task.l2dObject = null;
      task.texture = this.texture;
      task.tiling = false;
      task.tilemap = null;
      task.clip.set(this.clip.x, this.clip.y, this.clip.width, this.clip.height);
      task.tone = this.tone;
      task.color = this.color;
      task.mask = this.mask;
      task.opacity = this.opacity;
      task.blendMode = this.blendType;
      task.effect = this.visualEffect;
      task.effects = this.effects;
      task.clear = this.clear;
      this.renderTaskIndex++;
      if (this.renderTaskIndex >= this.maxRenderTasks) {
        return this.render();
      }
    };

    Renderer_OpenGL.prototype.render = function() {
      var blendMode, clip, color, currentTexture, data, e, effect, effectId, filter, hasColor, hasCustomBaseEffect, hasEffects, hasMask, hasTone, i, isClipped, j, k, l, len, len1, mask, maskChange, offset, ref, ref1, task, tone, toneChange;
      currentTexture = null;
      i = 0;
      j = 0;
      clip = null;
      tone = null;
      color = null;
      mask = null;
      effect = this.effect;
      isClipped = 0;
      hasMask = false;
      maskChange = false;
      offset = 0;
      blendMode = -1;
      effect.use();
      data = this.vertexData.subarray(0, this.renderTaskIndex * (this.vertexSize * 4));
      this.vertexBuffer.setSubData(data, 0);
      while (i < this.renderTaskIndex) {
        task = this.renderTasks[i];
        clip = task.clip;
        toneChange = task.tone !== tone || task.color !== color;
        hasMask = task.mask && task.mask.source;
        hasEffects = task.effects !== null;
        isClipped = clip.x !== this.clip.x || clip.y !== this.clip.y || clip.width !== this.clip.width || clip.height !== this.clip.height;
        if ((task.effect || hasEffects || task.tilemap || task.tiling || isClipped) || task.blendMode !== blendMode || task.texture !== currentTexture || toneChange || maskChange !== hasMask) {
          this.drawTasks(currentTexture, effect, offset, j - offset, task.clear);
          offset = j;
          currentTexture = task.texture;
          tone = task.tone;
          color = task.color;
          maskChange = hasMask;
          mask = task.mask;
          effectId = 0;
          if (blendMode !== task.blendMode) {
            blendMode = task.blendMode;
            switch (blendMode) {
              case 1:
                $gl.blendFunc($gl.SRC_ALPHA, $gl.ONE);
                break;
              case 2:
                $gl.blendFunc($gl.SRC_ALPHA_SATURATE, $gl.ONE_MINUS_SRC_COLOR);
                break;
              default:
                $gl.blendFuncSeparate($gl.SRC_ALPHA, $gl.ONE_MINUS_SRC_ALPHA, $gl.ONE, $gl.ONE_MINUS_SRC_ALPHA);
            }
          }
          if (isClipped) {
            this.clip.set(clip.x, clip.y, clip.width, clip.height);
            $gl.scissor(clip.x, Graphics.height - clip.y - clip.height, clip.width, clip.height);
          }
          hasCustomBaseEffect = false;
          if (hasEffects) {
            ref = task.effects.list;
            for (k = 0, len = ref.length; k < len; k++) {
              e = ref[k];
              if (e.enabled) {
                effectId |= e.id;
                hasCustomBaseEffect = hasCustomBaseEffect || e.type === 0;
              }
            }
          }
          if (task.tilemap) {
            effectId |= 4;
          } else if (!currentTexture) {
            effectId |= 8;
          } else if (!hasCustomBaseEffect) {
            effectId = 1;
          }
          if (task.tiling) {
            effectId |= !currentTexture.isPowerOf2 ? 2 : 1;
          }
          if (toneChange) {
            hasTone = tone && (tone.red !== 0 || tone.green !== 0 || tone.blue !== 0 || tone.grey !== 0);
            hasColor = color && color.alpha > 0;
            if (hasTone) {
              effectId |= 16;
            }
            if (hasColor) {
              effectId |= 32;
            }
          }
          if (hasMask) {
            if (mask.type === 0) {
              effectId |= 64;
            } else if (mask.type === 1) {
              effectId |= 128;
            }
          }
          effect = gs.Effect.get(effectId);
          effect.use();
          effect.setMatrixValue("pMatrix", this.projectionMatrix);
          if (task.tilemap) {
            filter = Texture2D.filter;
            Texture2D.filter = 0;
            this.setBitmap(task.tilemap.dataBitmap, 2);
            this.setBitmap(task.tilemap.tilesetBitmap, 3);
            Texture2D.filter = filter;
            effect.setVector2Value("viewportSize", Graphics.width, Graphics.height);
            effect.setVector2Value("viewportOrigin", task.tilemap.ox, task.tilemap.oy);
            effect.setVector2Value("autotileSettings", task.tilemap.autotileAreaStart, task.tilemap.tileSize);
            effect.setFloatValue("autotileOffset", task.tilemap.autotileOffset);
            effect.setVector2Value("invTilesetTextureSize", 1.0 / task.tilemap.tilesetBitmap.texture.realWidth, 1.0 / task.tilemap.tilesetBitmap.texture.realHeight);
            effect.setVector2Value("invDataTextureSize", 1.0 / task.tilemap.dataBitmap.texture.realWidth, 1.0 / task.tilemap.dataBitmap.texture.realHeight);
            effect.setIntegerValue("DataTexture", 2);
            effect.setIntegerValue("TilesetTexture", 3);
          } else if (!currentTexture) {
            effect.setVector4Value("Color", color.red, color.green, color.blue, color.alpha);
          }
          if (task.tiling && !currentTexture.isPowerOf2) {
            effect.setVector2Value("TextureMax", currentTexture.width / currentTexture.realWidth, currentTexture.height / currentTexture.realHeight);
          }
          if (hasTone) {
            effect.setVector4Value("Tone", tone.red, tone.green, tone.blue, tone.grey);
          }
          if (hasColor) {
            effect.setVector4Value("Color", color.red, color.green, color.blue, color.alpha);
          }
          if (hasMask) {
            this.setBitmap(mask.source, 1);
            if (mask.type === 0) {
              effect.setVector2Value("MaskOffset", mask.ox / mask.source.width, mask.oy / mask.source.height);
              effect.setIntegerValue("Texture1", 1);
            } else if (mask.type === 1) {
              effect.setFloatValue("CurrentValue", mask.value / 255);
              effect.setFloatValue("Vague", mask.vague / 255);
              effect.setIntegerValue("Texture1", 1);
            }
          }
          if (hasEffects) {
            ref1 = task.effects.list;
            for (l = 0, len1 = ref1.length; l < len1; l++) {
              e = ref1[l];
              if (e.enabled) {
                e.setup(effect, task, currentTexture);
              }
            }
          }
        }
        i++;
        j++;
      }
      this.drawTasks(currentTexture, effect, offset, j - offset, false);
      return this.renderTaskIndex = 0;
    };

    Renderer_OpenGL.prototype.drawTasks = function(texture, effect, offset, count, clear) {
      if (!(offset >= 0)) {
        return;
      }
      this.setTexture(texture);
      this.effectUsed = true;
      this.projectionMatrixOld = this.projectionMatrix;
      this.opacityOld = this.opacity;
      if (clear) {
        $gl.clear($gl.COLOR_BUFFER_BIT | $gl.DEPTH_BUFFER_BIT);
      }
      return $gl.drawElements($gl.TRIANGLES, count * 6, $gl.UNSIGNED_SHORT, offset * 6 * 2);
    };

    Renderer_OpenGL.prototype.drawRect = function(rect, ox, oy) {
      var effect;
      this.setTexture(null);
      this.texture = null;
      this.drawQuad(this.position, this.size);
      return;
      this.currentEffect = null;
      effect = this.colorRectEffect;
      effect.use();
      effect.setFloatValue("Opacity", this.opacity);
      effect.setVector4Value("Color", this.color.red, this.color.green, this.color.blue, this.color.alpha);
      effect.setMatrixValue("pMatrix", this.projectionMatrix);
      switch (this.blendType) {
        case 1:
          $gl.blendFunc($gl.SRC_ALPHA, $gl.ONE);
          break;
        case 2:
          $gl.blendFunc($gl.SRC_ALPHA_SATURATE, $gl.ONE_MINUS_SRC_COLOR);
          break;
        default:
          $gl.blendFuncSeparate($gl.SRC_ALPHA, $gl.ONE_MINUS_SRC_ALPHA, $gl.ONE, $gl.ONE_MINUS_SRC_ALPHA);
      }
      return $gl.drawElements($gl.TRIANGLES, 6, $gl.UNSIGNED_SHORT, 0);
    };


    /**
    * Changes the color tone of the specified area. Needs to be called once per
    * frame to keep it up.
    *
    * @method changeTone
    * @param {gs.Rect} rect - The area to tint.
     */

    Renderer_OpenGL.prototype.changeTone = function(rect) {
      return this.drawScreenRect(rect);
    };

    Renderer_OpenGL.prototype.prepareScreenRect = function() {
      this.render();
      Graphics.frameBuffer.bind();
      Graphics.frameBuffer.setTexture(Graphics.offscreenTexture);
      this.setTexture(Graphics.screenTexture);
      this.transform = mat2d.create();
      mat2d.scale(this.transform, this.transform, [Graphics.screenTexture.width, Graphics.screenTexture.height]);
      this.setSourceRectangle(0, 0, Graphics.screenTexture.width, Graphics.screenTexture.height);
      this.drawTransform();
      this.render();
      return Graphics.frameBuffer.setTexture(Graphics.screenTexture);
    };

    Renderer_OpenGL.prototype.drawScreenRect = function(rect, mipmap, textureFilter) {
      Graphics.offscreenTexture.bind();
      if (mipmap) {
        $gl.generateMipmap($gl.TEXTURE_2D);
      }
      if (textureFilter !== null) {
        Graphics.offscreenTexture.setFilter(textureFilter, mipmap);
      }
      Graphics.offscreenTexture.unbind();
      mat2d.identity(this.transform_);
      this.transformVector_[0] = rect.x;
      this.transformVector_[1] = rect.y;
      mat2d.translate(this.transform_, this.transform_, this.transformVector_);
      this.transformVector_[0] = rect.width;
      this.transformVector_[1] = rect.height;
      mat2d.scale(this.transform_, this.transform_, this.transformVector_);
      this.transform = this.transform_;
      this.setTexture(Graphics.offscreenTexture);
      this.setSourceRectangle(rect.x, rect.y, rect.width, rect.height);
      $gl.disable($gl.BLEND);
      this.drawTransform();
      this.render();
      return $gl.enable($gl.BLEND);
    };

    Renderer_OpenGL.prototype.blur = function(rect, passes) {
      this.render();
      mat2d.identity(this.transform_);
      this.transformVector_[0] = rect.x;
      this.transformVector_[1] = rect.y;
      mat2d.translate(this.transform_, this.transform_, this.transformVector_);
      this.transformVector_[0] = rect.width;
      this.transformVector_[1] = rect.height;
      mat2d.scale(this.transform_, this.transform_, this.transformVector_);
      this.transform = this.transform_;
      this.blurV(rect, passes);
      return this.blurH(rect, passes);
    };

    Renderer_OpenGL.prototype.blurV = function(rect, passes) {
      var i, k, ref;
      this.setTexture(Graphics.offscreenTexture);
      this.setSourceRectangle(rect.x, rect.y, rect.width, rect.height);
      $gl.copyTexSubImage2D($gl.TEXTURE_2D, 0, 0, 0, 0, 0, Graphics.width, Graphics.height);
      this.visualEffect = this.vBlurEffect;
      for (i = k = 0, ref = passes; 0 <= ref ? k <= ref : k >= ref; i = 0 <= ref ? ++k : --k) {
        this.drawQuad(this.position, this.size);
        this.render();
        $gl.copyTexSubImage2D($gl.TEXTURE_2D, 0, 0, 0, 0, 0, Graphics.width, Graphics.height);
      }
      return this.visualEffect = null;
    };

    Renderer_OpenGL.prototype.blurH = function(rect, passes) {
      var i, k, ref;
      this.setTexture(Graphics.offscreenTexture);
      this.setSourceRectangle(rect.x, rect.y, rect.width, rect.height);
      $gl.copyTexSubImage2D($gl.TEXTURE_2D, 0, 0, 0, 0, 0, Graphics.width, Graphics.height);
      this.visualEffect = this.hBlurEffect;
      for (i = k = 0, ref = passes; 0 <= ref ? k <= ref : k >= ref; i = 0 <= ref ? ++k : --k) {
        this.drawQuad(this.position, this.size);
        this.render();
        $gl.copyTexSubImage2D($gl.TEXTURE_2D, 0, 0, 0, 0, 0, Graphics.width, Graphics.height);
      }
      return this.visualEffect = null;
    };

    Renderer_OpenGL.prototype.drawTransition = function(transitionData) {
      var effect, ref;
      if ((ref = this.texture) != null) {
        ref.unbind();
      }
      this.texture = null;
      this.setBitmap(transitionData.sourceBitmap, 1);
      this.setBitmap(transitionData.targetBitmap, 2);
      this.setBitmap(transitionData.transitionBitmap, 3);
      this.setSourceRectangleForTexture(transitionData.transitionBitmap.texture, 0, 0, transitionData.transitionBitmap.width, transitionData.transitionBitmap.height);
      this.vertexData[0] = 0;
      this.vertexData[1] = 0;
      this.vertexData[5] = 0 + Graphics.width;
      this.vertexData[6] = 0;
      this.vertexData[10] = 0;
      this.vertexData[11] = 0 + Graphics.height;
      this.vertexData[15] = 0 + Graphics.width;
      this.vertexData[16] = 0 + Graphics.height;
      this.vertexBuffer.setSubData(this.vertexData);
      this.currentEffect = null;
      effect = this.transitionEffect;
      effect.use();
      effect.setFloatValue("CurrentAlpha", transitionData.currentAlpha / 255);
      effect.setFloatValue("Vague", transitionData.vague / 255);
      effect.setIntegerValue("Texture0", 1);
      effect.setIntegerValue("Texture1", 2);
      effect.setIntegerValue("Texture2", 3);
      effect.setMatrixValue("pMatrix", this.projectionMatrix);
      return $gl.drawElements($gl.TRIANGLES, 6, $gl.UNSIGNED_SHORT, 0);
    };

    Renderer_OpenGL.prototype.drawTiles = function(ox, oy, dataBitmap, tilesetBitmap, autotileOffset, tileSize, tilesetConfig) {
      var color, effect, filter, index, ref, task;
      index = this.renderTaskIndex * (this.vertexSize * 4);
      color = (this.opacity << 24) | (0 << 16) | (0 << 8) | 0;
      this.position1[0] = 0;
      this.position1[1] = 0;
      this.position2[0] = 1;
      this.position2[1] = 0;
      this.position3[0] = 0;
      this.position3[1] = 1;
      this.position4[0] = 1;
      this.position4[1] = 1;
      vec2.transformMat2d(this.position1, this.position1, this.transform);
      vec2.transformMat2d(this.position2, this.position2, this.transform);
      vec2.transformMat2d(this.position3, this.position3, this.transform);
      vec2.transformMat2d(this.position4, this.position4, this.transform);
      this.vertexData[index] = this.position1[0];
      this.vertexData[index + 1] = this.position1[1];
      this.vertexColorData[index + 4] = color;
      this.vertexData[index + 5] = this.position2[0];
      this.vertexData[index + 6] = this.position2[1];
      this.vertexColorData[index + 9] = color;
      this.vertexData[index + 10] = this.position3[0];
      this.vertexData[index + 11] = this.position3[1];
      this.vertexColorData[index + 14] = color;
      this.vertexData[index + 15] = this.position4[0];
      this.vertexData[index + 16] = this.position4[1];
      this.vertexColorData[index + 19] = color;
      filter = Texture2D.filter;
      Texture2D.filter = 0;
      this.updateBitmap(dataBitmap);
      Texture2D.filter = filter;
      this.setSourceRectangleForTexture(dataBitmap.texture, 0, 0, dataBitmap.texture.realWidth, dataBitmap.texture.realHeight);
      task = this.renderTasks[this.renderTaskIndex];
      task.l2dObject = null;
      task.texture = dataBitmap.texture;
      task.tiling = false;
      task.tilemap = {
        tilesetBitmap: tilesetBitmap,
        dataBitmap: dataBitmap,
        ox: ox,
        oy: oy,
        autotileOffset: autotileOffset,
        autotileAreaStart: tilesetConfig.specialTiles.x,
        tileSize: tileSize
      };
      task.clip.set(this.clip.x, this.clip.y, this.clip.width, this.clip.height);
      task.tone = this.tone;
      task.color = this.color;
      task.mask = this.mask;
      task.opacity = this.opacity;
      task.blendMode = this.blendType;
      task.effect = this.visualEffect;
      task.clear = this.clear;
      this.renderTaskIndex++;
      this.render();
      return;
      if ((ref = this.texture) != null) {
        ref.unbind();
      }
      this.texture = null;
      filter = Texture2D.filter;
      Texture2D.filter = 0;
      this.setBitmap(dataBitmap, 1);
      this.setBitmap(tilesetBitmap, 2);
      Texture2D.filter = filter;
      this.vertexData[0] = x;
      this.vertexData[1] = y;
      this.vertexData[2] = 1.0;
      this.vertexData[5] = x + width;
      this.vertexData[6] = y;
      this.vertexData[7] = 1.0;
      this.vertexData[10] = x;
      this.vertexData[11] = y + height;
      this.vertexData[12] = 1.0;
      this.vertexData[15] = x + width;
      this.vertexData[16] = y + height;
      this.vertexData[17] = 1.0;
      this.vertexBuffer.setSubData(this.vertexData);
      this.effectUsed = false;
      this.currentEffect = null;
      tileSize = Graphics.height === 240 ? 16 : 32;
      effect = this.tilemapEffect;
      effect.use();
      effect.setVector2Value("viewportSize", width, height);
      effect.setVector2Value("viewportOrigin", ox, oy);
      effect.setFloatValue("autotileOffset", 8 * tileSize * autotileOffset);
      effect.setVector2Value("invTilesetTextureSize", 1.0 / tilesetBitmap.texture.realWidth, 1.0 / tilesetBitmap.texture.realHeight);
      effect.setVector2Value("invDataTextureSize", 1.0 / dataBitmap.texture.realWidth, 1.0 / dataBitmap.texture.realHeight);
      effect.setIntegerValue("DataTexture", 1);
      effect.setIntegerValue("TilesetTexture", 2);
      effect.setMatrixValue("pMatrix", this.projectionMatrix);
      return $gl.drawElements($gl.TRIANGLES, 6, $gl.UNSIGNED_SHORT, 0);
    };


    /**
    * Draws a Live2D object.
    *
    * @method drawLive2D
    * @param {gs.Live2DObject} object - The Live2D object to draw.
     */

    Renderer_OpenGL.prototype.drawLive2D = function(object) {
      return object.draw_();
    };

    Renderer_OpenGL.prototype.drawTiledBitmap = function(bitmap, x, y, w, h, sx, sy, sw, sh) {
      var d, k, l, ref, ref1, tilesX, tilesY, tx, ty;
      this.setBitmap(bitmap);
      x = Math.floor(x);
      y = Math.floor(y);
      w = Math.floor(w);
      h = Math.floor(h);
      tilesX = Math.ceil(w / sw);
      tilesY = Math.ceil(h / sh);
      d = 0;
      for (tx = k = 0, ref = tilesX; 0 <= ref ? k < ref : k > ref; tx = 0 <= ref ? ++k : --k) {
        for (ty = l = 0, ref1 = tilesY; 0 <= ref1 ? l < ref1 : l > ref1; ty = 0 <= ref1 ? ++l : --l) {
          this.position[0] = x + tx * sw;
          this.position[1] = y + ty * sh;
          this.size[0] = (d = (this.position[0] + sw) - (x + w)) > 0 ? sw - d : sw;
          this.size[1] = (d = (this.position[1] + sh) - (y + h)) > 0 ? sh - d : sh;
          this.setSourceRectangle(sx, sy, this.size[0], this.size[1]);
          this.drawQuad(this.position, this.size);
        }
      }
      return null;
    };


    /**
    * Draws a bitmap "simple" without any visual effects like rotation, tinting, alpha-blending, etc. Its faster
    * than regular gs.Renderer.drawBitmap.
    *
    * @method drawBitmapSimple
    * @param {gs.Bitmap} bitmap - The bitmap to draw.
    * @param {number} x - The x-coordinate of the bitmap's position.
    * @param {number} y - The y-coordinate of the bitmap's position.
    * @param {number} w - The width of the bitmap.
    * @param {number} h - The height of the bitmap.
    * @param {number} sx - The x-coordinate of the bitmap's area to draw.
    * @param {number} sy - The y-coordinate of the bitmap's area to draw.
    * @param {number} sw - The width of the bitmap's area to draw.
    * @param {number} sh - The height of the bitmap's area to draw.
     */

    Renderer_OpenGL.prototype.drawBitmapSimple = function(bitmap, x, y, w, h, sx, sy, sw, sh) {
      if (!bitmap.loaded) {
        return;
      }
      this.setBitmap(bitmap);
      this.setSourceRectangle(sx, sy, sw, sh);
      this.position[0] = x - this.ox;
      this.position[1] = y - this.oy;
      this.size[0] = w;
      this.size[1] = h;
      return this.drawQuad(this.position, this.size);
    };

    Renderer_OpenGL.prototype.drawTransform = function() {
      return this.drawQuad(this.position, this.size);
    };


    /**
    * Draws a specified bitmap on screen.
    *
    * @method drawBitmap
    * @param {gs.Bitmap} bitmap - The bitmap to draw.
    * @param {number} x - The x-coordinate of the bitmap's position.
    * @param {number} y - The y-coordinate of the bitmap's position.
    * @param {gs.Rect} srcRect - The area of the bitmap to draw.
    * @param {number} [ox=0] - The x-coordinate of the origin.
    * @param {number} [oy=0] - The xycoordinate of the origin.
     */

    Renderer_OpenGL.prototype.drawBitmap = function(bitmap, x, y, srcRect, ox, oy) {
      if (!bitmap.loaded) {
        return;
      }
      ox = ox || 0;
      oy = oy || 0;
      x = x - this.ox - ox;
      y = y - this.oy - oy;
      switch (this.blendType) {
        case 1:
          $gl.blendFunc($gl.SRC_ALPHA, $gl.ONE);
          break;
        case 2:
          $gl.blendFunc($gl.SRC_ALPHA_SATURATE, $gl.ONE_MINUS_SRC_COLOR);
          break;
        default:
          $gl.blendFuncSeparate($gl.SRC_ALPHA, $gl.ONE_MINUS_SRC_ALPHA, $gl.ONE, $gl.ONE_MINUS_SRC_ALPHA);
      }
      this.setBitmap(bitmap);
      if (this.mirror) {
        this.setSourceRectangle(srcRect.x + srcRect.width, srcRect.y, -srcRect.width, srcRect.height);
      } else {
        this.setSourceRectangle(srcRect.x, srcRect.y, srcRect.width, srcRect.height);
      }
      this.position[0] = x;
      this.position[1] = y;
      this.size[0] = srcRect.width * this.zoom.x;
      this.size[1] = srcRect.height * this.zoom.y;
      if (this.angle > 0) {
        this.rotationCenter[0] = this.size[0] * 0.5;
        this.rotationCenter[1] = this.size[1] * 0.5;
        this.rotationCenterInverse[0] = -this.rotationCenter[0];
        this.rotationCenterInverse[1] = -this.rotationCenter[1];
        return this.drawRotatedQuad(this.position, this.size, Math.degreesToRadians(this.angle), this.rotationCenter, this.rotationCenterInverse);
      } else {
        return this.drawQuad(this.position, this.size);
      }
    };

    Renderer_OpenGL.prototype._clipRect = function(rect) {
      this.clip.set(rect.x, rect.y, rect.width, rect.height);
      return $gl.scissor(rect.x, Graphics.height - rect.height, rect.width - rect.x, Graphics.height - rect.y - (Graphics.height - rect.height));
    };


    /**
    * Sets the current clip-rect. All render-calls outside of the clip-area are clipped.
    *
    * @method clipRect
    * @param {gs.Rect} rect - The clipping area.
     */

    Renderer_OpenGL.prototype.clipRect = function(rect) {
      this.clip.set(rect.x, rect.y, rect.width, rect.height);
      return $gl.scissor(rect.x, Graphics.height - rect.y - rect.height, rect.width, rect.height);
    };


    /**
    * Sets the current clip-rect. All render-calls outside of the clip-area are clipped. It is
    * equivalent to clipRect but takes direct coordinates instead of a gs.Rect.
    *
    * @method clipTo
    * @see clipRect
    * @param {number} x - The x coordinate of the clipping area.
    * @param {number} y - The y coordinate of the clipping area.
    * @param {number} width - The width of the clipping area.
    * @param {number} height - The height of the clipping area.
     */

    Renderer_OpenGL.prototype.clipTo = function(x, y, width, height) {
      this.clip.set(x, y, width, height);
      return $gl.scissor(x, Graphics.height - y - height, width, height);
    };

    Renderer_OpenGL.prototype.updateBitmap = function(bitmap) {
      if (bitmap.texture == null) {
        return bitmap.texture = new Texture2D(bitmap.image, bitmap._flipTextureY);
      } else if (bitmap.sequenceNumber !== bitmap.texture.sequenceNumber) {
        bitmap.texture.setImageData(bitmap);
        return bitmap.dirtyRect.set(-1, -1, -1, -1);
      }
    };


    /**
    * Sets the current bitmap to render.
    *
    * @method setBitmap
    * @param {gs.Bitmap} bitmap - The bitmap to render.
     */

    Renderer_OpenGL.prototype.setBitmap = function(bitmap, level) {
      if (!bitmap || !bitmap.width) {
        return;
      }
      if (bitmap.texture == null) {
        bitmap.texture = new Texture2D(bitmap.image, bitmap._flipTextureY);
      } else if (bitmap.sequenceNumber !== bitmap.texture.sequenceNumber) {
        bitmap.texture.setImageData(bitmap);
        bitmap.dirtyRect.set(-1, -1, -1, -1);
      }
      bitmap.texture.sequenceNumber = bitmap.sequenceNumber;
      return this.setTexture(bitmap.texture, level);
    };

    Renderer_OpenGL.prototype.setTexture = function(texture, level) {
      var ref;
      if (level == null) {
        if (this.texture === texture) {
          return;
        }
        this.texture = texture;
        return (ref = this.texture) != null ? ref.bind() : void 0;
      } else {
        $gl.activeTexture($gl.TEXTURE0 + level);
        if (texture != null) {
          texture.bind();
        }
        return $gl.activeTexture($gl.TEXTURE0);
      }
    };

    Renderer_OpenGL.prototype.setSourceRectangleForTexture = function(texture, sx, sy, sw, sh) {
      var index, th, tw, tx, ty;
      if (texture != null) {
        tx = sx / texture.realWidth;
        ty = sy / texture.realHeight;
        tw = sw / texture.realWidth;
        th = sh / texture.realHeight;
        index = this.renderTaskIndex * (this.vertexSize * 4);
        this.vertexData[index + 2] = tx;
        this.vertexData[index + 3] = ty;
        this.vertexData[index + 7] = tx + tw;
        this.vertexData[index + 8] = ty;
        this.vertexData[index + 12] = tx;
        this.vertexData[index + 13] = ty + th;
        this.vertexData[index + 17] = tx + tw;
        return this.vertexData[index + 18] = ty + th;
      }
    };


    /**
    * Sets the current source-rectangle to render.
    *
    * @method setSourceRectangle
    * @param {number} sx - The x-coordinate of the area to draw.
    * @param {number} sy - The y-coordinate of the area to draw.
    * @param {number} sw - The width of the area to draw.
    * @param {number} sh - The height of the area to draw.
     */

    Renderer_OpenGL.prototype.setSourceRectangle = function(sx, sy, sw, sh) {
      var index, th, tw, tx, ty;
      if (this.texture != null) {
        if (this.mirrorY) {
          ty = (sy + sh) / this.texture.realHeight;
          th = (-sh) / this.texture.realHeight;
        } else {
          ty = sy / this.texture.realHeight;
          th = sh / this.texture.realHeight;
        }
        if (this.mirror) {
          tx = (sx + sw) / this.texture.realWidth;
          tw = (-sw) / this.texture.realWidth;
        } else {
          tx = sx / this.texture.realWidth;
          tw = sw / this.texture.realWidth;
        }
        index = this.renderTaskIndex * (this.vertexSize * 4);
        this.vertexData[index + 2] = tx;
        this.vertexData[index + 3] = ty;
        this.vertexData[index + 7] = tx + tw;
        this.vertexData[index + 8] = ty;
        this.vertexData[index + 12] = tx;
        this.vertexData[index + 13] = ty + th;
        this.vertexData[index + 17] = tx + tw;
        return this.vertexData[index + 18] = ty + th;
      }
    };

    return Renderer_OpenGL;

  })();

  window.Renderer_OpenGL = Renderer_OpenGL;

  window.Renderer = window.$PARAMS != null ? Renderer_Canvas : Renderer_Canvas;

  gs.Renderer = Renderer_OpenGL;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var Shader, fragmentShaderTemplate, vertexShaderTemplate;

  vertexShaderTemplate = "precision mediump float;\n\nattribute vec3 vertexPosition;\nattribute vec2 vertexTexture;\nattribute vec4 vertexColor;\n\n{DEFINES}\n\n{UNIFORMS}\n{VARYING}\n\n#ifdef USE_POSITION\nvoid default_position()\n{\n    vColor = vertexColor;\n    gl_Position = pMatrix * vec4(vertexPosition, 1.0);\n    textureCoord = vertexTexture.st;\n}\n#endif\n#ifdef USE_TILEMAP\nvoid tilemap()\n{\n    pixelCoord = (vertexTexture * viewportSize) + viewportOrigin;\n    dataTextureCoord = pixelCoord * invDataTextureSize / autotileSettings.y;\n    textureCoord = vertexTexture;\n    vColor[3] = 1.0;\n\n    gl_Position = pMatrix * vec4(vertexPosition, 1.0);\n}\n#endif\n\n{FUNCTIONS}\n\nvoid main(void)\n{\n    {FUNCTION_CALLS}\n}";

  fragmentShaderTemplate = "precision mediump float;\n\n{DEFINES}\n\n{UNIFORMS}\n{VARYING}\n\n#ifdef USE_TEXTURE_COLOR\nvoid texture_color()\n{\n#ifdef USE_TEXTURE_TILING\n    gl_FragColor = texture2D(Texture0, mod(textureCoord, TextureMax));\n#else\n    gl_FragColor = texture2D(Texture0, textureCoord);\n#endif\n\n    gl_FragColor[3] *= vColor[3];\n}\n#endif\n\n#ifdef USE_TEXTURE_COLOR_TILING\nvoid texture_color_tiling()\n{\n    gl_FragColor = texture2D(Texture0, mod(textureCoord, TextureMax));\n    gl_FragColor[3] *= vColor[3];\n}\n#endif\n\n#ifdef USE_TINTING\nvoid tinting()\n{\n    vec4 c = gl_FragColor;\n    float avg = (c[0] + c[1] + c[2]) / 3.0;\n    float grey = Tone[3] / 255.0;\n    c.xyz = (avg * grey + c.xyz * (1.0 - grey)) + Tone.xyz / 255.0;\n    c[3] *= vColor[3];\n    gl_FragColor = c;\n}\n#endif\n\n#ifdef USE_COLOR\nvoid color()\n{\n    vec4 c = gl_FragColor;\n    float a = Color[3] / 255.0;\n\n    c.xyz = Color.xyz / 255.0 * a + c.xyz * (1.0 - a);\n    c[3] *= vColor[3];\n\n    gl_FragColor = c;\n}\n#endif\n#ifdef USE_DYNAMIC_MASK\nvoid dynamic_mask()\n{\n    vec4 sc = gl_FragColor;\n    vec4 tsc = texture2D(Texture1, textureCoord);\n    vec4 c = vec4(0.0, 0.0, 0.0, 0.0);\n\n    if(tsc[0] >= CurrentValue)\n    {\n      if(Vague > tsc[0] - CurrentValue)\n      {\n        float diff = tsc[0] - CurrentValue;\n        float alpha = diff / Vague;\n\n        c[0] = sc[0];\n        c[1] = sc[1];\n        c[2] = sc[2];\n        c[3] = sc[3] * alpha * (vColor[3]);\n      }\n      else\n      {\n        c = sc;\n        c[3] *= (vColor[3]);\n      }\n    }\n\n    gl_FragColor = c;\n}\n#endif\n#ifdef USE_STATIC_MASK\nvoid static_mask()\n{\n    vec4 sc = gl_FragColor;\n    vec4 tsc = texture2D(Texture1, vec2(textureCoord.s + MaskOffset.x, textureCoord.t + MaskOffset.y));\n\n    sc[3] *= tsc[0] * (vColor[3]);\n\n    gl_FragColor = sc;\n}\n#endif\n#ifdef USE_COLOR_QUAD\nvoid color_quad()\n{\n    gl_FragColor = Color / 255.0;\n}\n#endif\n#ifdef USE_TILEMAP\nvoid tilemap()\n{\n    vec4 tile = texture2D(DataTexture, dataTextureCoord);\n    if(tile.x == 1.0 && tile.y == 1.0) { discard; }\n\n    vec2 tilesetOffset = floor(tile.xy * 256.0) * autotileSettings.y;\n    vec2 tilesetCoord = mod(pixelCoord, autotileSettings.y);\n\n    if(tilesetOffset.x >= autotileSettings.x) tilesetOffset.x = tilesetOffset.x + autotileOffset;\n\n    gl_FragColor = texture2D(TilesetTexture, (tilesetOffset + tilesetCoord) * invTilesetTextureSize);\n}\n#endif\n#ifdef USE_LOD_BLUR\nvoid lod_blur()\n{\n	  float bias = (5.0 + 5.0 * sin(LodBlurPower));\n#ifdef USE_TEXTURE_TILING\n    vec4 col = texture2D(Texture0, mod(textureCoord, TextureMax), bias );\n#else\n	  vec4 col = texture2D(Texture0, textureCoord , bias );\n#endif\n\n	  gl_FragColor = vec4( col.xyz, 1.0 );\n}\n#endif\n#ifdef USE_PIXELATE\nvoid pixelate()\n{\n    vec2 pixelatedCoord = vec2(0.0, 0.0);\n    pixelatedCoord.x = floor(textureCoord.x / PixelateSize.x) * PixelateSize.x;\n    pixelatedCoord.y = floor(textureCoord.y / PixelateSize.y) * PixelateSize.y;\n\n#ifdef USE_TEXTURE_TILING\n    gl_FragColor = texture2D(Texture0, mod(pixelatedCoord, TextureMax));\n#else\n    gl_FragColor = texture2D(Texture0, pixelatedCoord);\n#endif\n}\n#endif\n\n{FUNCTIONS}\n\nvoid main(void)\n{\n    {FUNCTION_CALLS}\n}";

  Shader = (function() {
    Shader.defaultShaders = {
      "fragment-shader-template": fragmentShaderTemplate,
      "vertex-shader-template": vertexShaderTemplate
    };

    function Shader(source, type) {
      this.id = $gl.createShader(type);
      $gl.shaderSource(this.id, source);
      $gl.compileShader(this.id);
      if (!$gl.getShaderParameter(this.id, $gl.COMPILE_STATUS)) {
        throw 'Error while compliling shader "' + name + '". ' + $gl.getShaderInfoLog(this.id);
      }
    }

    Shader.prototype.dispose = function() {
      $gl.deleteShader(this.id);
      return this.id = null;
    };

    return Shader;

  })();

  window.Shader = Shader;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var VertexShader,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  VertexShader = (function(superClass) {
    extend(VertexShader, superClass);

    function VertexShader(info) {
      var source;
      source = Shader.defaultShaders["vertex-shader-template"];
      source = source.replace(/\{DEFINES\}/gi, info.defines.join("\n"));
      source = source.replace(/\{UNIFORMS\}/gi, info.uniforms.join("\n"));
      source = source.replace(/\{VARYING\}/gi, info.varying.join("\n"));
      source = source.replace(/\{FUNCTIONS\}/gi, info.functions.join("\n"));
      source = source.replace(/\{FUNCTION_CALLS\}/gi, info.functionCalls.join("\n"));
      VertexShader.__super__.constructor.call(this, source, $gl.VERTEX_SHADER);
    }

    return VertexShader;

  })(Shader);

  window.VertexShader = VertexShader;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var FragmentShader,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  FragmentShader = (function(superClass) {
    extend(FragmentShader, superClass);

    function FragmentShader(info) {
      var source;
      source = Shader.defaultShaders["fragment-shader-template"];
      source = source.replace(/\{DEFINES\}/gi, info.defines.join("\n"));
      source = source.replace(/\{UNIFORMS\}/gi, info.uniforms.join("\n"));
      source = source.replace(/\{VARYING\}/gi, info.varying.join("\n"));
      source = source.replace(/\{FUNCTIONS\}/gi, info.functions.join("\n"));
      source = source.replace(/\{FUNCTION_CALLS\}/gi, info.functionCalls.join("\n"));
      FragmentShader.__super__.constructor.call(this, source, $gl.FRAGMENT_SHADER);
    }

    return FragmentShader;

  })(Shader);

  window.FragmentShader = FragmentShader;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var Effect, EffectCollection, EffectInfo;

  EffectInfo = (function() {

    /**
    * An effect-info encapsulates all necessary information to register and compile
    * a custom GLSL shader effect. To actually use a custom effect, it is necessary create a custom
    * effect class as well and derive from gs.GraphicEffect class.
    *
    * @namespace gs
    * @class EffectInfo
    * @memberof gs
    * @constructor
    * @param {Object} [data=null] - An optional data-bundle to create the effect-info from. Can be <b>null</b>.
     */
    function EffectInfo(data) {

      /**
      * The internal ID number.
      * @property id
      * @type number
       */
      var ref, ref1, ref2, ref3, ref4, ref5, ref6, ref7;
      this.id = (ref = data != null ? data.id : void 0) != null ? ref : 0;

      /**
      * The effect type.
      * @property type
      * @type gs.GraphicEffectType
       */
      this.type = (ref1 = data != null ? data.type : void 0) != null ? ref1 : gs.GraphicEffectType.ADDON;

      /**
      * The effect name. That name must match the name defined in the effect class.
      * @property name
      * @type string
       */
      this.name = (ref2 = data != null ? data.name : void 0) != null ? ref2 : "unknown";

      /**
      * <p>An array of GLSL uniform declarations. Each entry represents a single uniform declaration. The
      * uniform declaration must be in correct syntax including the trailing semicolon. For Example:</p>
      *
      * <code>
      * uniform float MyValue;
      * </code>
      * @property uniforms
      * @type string[]
       */
      this.uniforms = (ref3 = data != null ? data.uniforms : void 0) != null ? ref3 : [];

      /**
      * <p>An array of GLSL varying declarations. Each entry represents a single varying declaration. The
      * varying declaration must be in correct syntax including the trailing semicolon. For Example:</p>
      *
      * <code>
      * varying vec2 textureCoord;
      * </code>
      * @property varying
      * @type string[]
       */
      this.varying = (ref4 = data != null ? data.varying : void 0) != null ? ref4 : [];

      /**
      * <p>An array of GLSL preprocessor definitions. Each entry represents a single preprocessor definition. The
      * preprocessor definition must be in correct syntax. For Example:</p>
      *
      * <code>
      * #define PI 3.14159
      * </code>
      * @property defines
      * @type string[]
       */
      this.defines = (ref5 = data != null ? data.defines : void 0) != null ? ref5 : [];

      /**
      * <p>An array of GLSL function calls. Each entry represents a single function call. The
      * function call must be in correct syntax including the trailing semicolon. For Example:</p>
      *
      * <code>
      * my_powerful_effect();
      * </code>
      * <p>The function calls are paste into the main-function of the fragment-shader.
      * @property functionCalls
      * @type string[]
       */
      this.functionCalls = (ref6 = data != null ? data.functionCalls : void 0) != null ? ref6 : [];

      /**
      * <p>An array of GLSL function definitions. Each entry represents a single function definition. The
      * function definition must be in correct syntax. For Example:</p>
      *
      * <code>
      * void texture_color()
      * {
      *     gl_FragColor = texture2D(Texture0, textureCoord);
      * }
      * </code>
      * <p>The function definitions are pasted after all define, uniform and varying declarations.</p>
      * @property functions
      * @type string[]
       */
      this.functions = (ref7 = data != null ? data.functions : void 0) != null ? ref7 : [];
    }

    return EffectInfo;

  })();

  gs.EffectInfo = EffectInfo;

  EffectCollection = (function() {

    /**
    * An effect collection stores all effects by name. So if you add custom effects, you
    * have to create an own class and inherit from this class and add your
    * own effects in the constructor. It is important to put the super-call at the end and not
    * at the beginning.
    *
    * @namespace gs
    * @class EffectCollection
    * @memberof gs
    * @constructor
     */
    function EffectCollection() {
      this.wobble = new gs.WobbleEffect();
      this.blur = new gs.BlurEffect();
      this.pixelate = new gs.PixelateEffect();
      this.initialize();
    }


    /**
    * Initializes the collection. This method is automatically called in the constructor and doesn't
    * need to be called manually. It only needs to be called once. It is important that all custom effects
    * are already registered using gs.Effect.register before this method is called.
    *
    * @method initialize
     */

    EffectCollection.prototype.initialize = function() {
      var e, i, j, len, ref;
      this.list = Object.values(this).where(function(e) {
        return e instanceof gs.GraphicEffect && e.type !== gs.GraphicEffectType.INTERNAL;
      });
      ref = this.list;
      for (i = j = 0, len = ref.length; j < len; i = ++j) {
        e = ref[i];
        e.id = Effect.fragmentShaderInfos[e.name].id;
      }
      return null;
    };

    return EffectCollection;

  })();

  gs.EffectCollection = EffectCollection;

  Effect = (function() {
    Effect.nextEffectId = 8;

    Effect.registerEffect = function(info) {
      Effect.fragmentShaderInfos[info.name] = info;
      Effect.fragmentShaderInfosByID[1 << Effect.nextEffectId] = info;
      Effect.vertexShaderInfosByID[1 << Effect.nextEffectId] = Effect.vertexShaderInfos.position;
      info.id = 1 << Effect.nextEffectId;
      return Effect.nextEffectId++;
    };

    Effect.effects = new Array(513);

    Effect.vertexShaderInfos = {
      tilemap: {
        defines: ["#define USE_TILEMAP"],
        uniforms: ["uniform mat4 pMatrix;", "uniform vec2 viewportSize;", "uniform vec2 viewportOrigin;", "uniform vec2 invDataTextureSize;", "uniform vec2 autotileSettings;"],
        varying: ["varying vec2 dataTextureCoord;", "varying vec2 pixelCoord;", "varying vec2 textureCoord;", "varying vec4 vColor;"],
        functionCalls: ["tilemap();"],
        functions: []
      },
      position: {
        defines: ["#define USE_POSITION"],
        uniforms: ["uniform mat4 pMatrix;"],
        varying: ["varying vec2 textureCoord;", "varying vec4 vColor;"],
        functionCalls: ["default_position();"],
        functions: []
      }
    };

    Effect.fragmentShaderInfos = {
      texture_color: {
        type: 0,
        defines: ["#define USE_TEXTURE_COLOR"],
        uniforms: ["uniform sampler2D Texture0;"],
        varying: ["varying vec2 textureCoord;", "varying vec4 vColor;"],
        functionCalls: ["texture_color();"],
        functions: []
      },
      texture_color_tiling: {
        type: 1,
        defines: ["#define USE_TEXTURE_TILING"],
        uniforms: ["uniform vec2 TextureMax;"],
        varying: [],
        functionCalls: [],
        functions: []
      },
      tinting: {
        type: 1,
        defines: ["#define USE_TINTING"],
        uniforms: ["uniform sampler2D Texture0;", "uniform vec4 Tone;"],
        varying: ["varying vec2 textureCoord;", "varying vec4 vColor;"],
        functionCalls: ["tinting();"],
        functions: []
      },
      color: {
        type: 1,
        defines: ["#define USE_COLOR"],
        uniforms: ["uniform sampler2D Texture0;", "uniform vec4 Color;"],
        varying: ["varying vec2 textureCoord;", "varying vec4 vColor;"],
        functionCalls: ["color();"],
        functions: []
      },
      dynamic_mask: {
        type: 1,
        defines: ["#define USE_DYNAMIC_MASK"],
        uniforms: ["uniform sampler2D Texture0;", "uniform sampler2D Texture1;", "uniform float Vague;", "uniform float CurrentValue;"],
        varying: ["varying vec2 textureCoord;", "varying vec4 vColor;"],
        functionCalls: ["dynamic_mask();"],
        functions: []
      },
      static_mask: {
        type: 1,
        defines: ["#define USE_STATIC_MASK"],
        uniforms: ["uniform sampler2D Texture0;", "uniform sampler2D Texture1;", "uniform vec2 MaskOffset;"],
        varying: ["varying vec2 textureCoord;", "varying vec4 vColor;"],
        functionCalls: ["static_mask();"],
        functions: []
      },
      tilemap: {
        type: 0,
        defines: ["#define USE_TILEMAP"],
        uniforms: ["uniform sampler2D DataTexture;", "uniform sampler2D TilesetTexture;", "uniform vec2 invTilesetTextureSize;", "uniform float autotileOffset;", "uniform vec2 autotileSettings;"],
        varying: ["varying vec2 dataTextureCoord;", "varying vec2 pixelCoord;"],
        functionCalls: ["tilemap();"],
        functions: []
      },
      color_quad: {
        type: 0,
        defines: ["#define USE_COLOR_QUAD"],
        uniforms: ["uniform vec4 Color;"],
        varying: [],
        functionCalls: ["color_quad();"],
        functions: []
      },
      lod_blur: {
        type: 0,
        name: "lod_blur",
        defines: ["#define USE_LOD_BLUR"],
        uniforms: ["uniform sampler2D Texture0;", "uniform float LodBlurPower;"],
        varying: ["varying vec2 textureCoord;", "varying vec4 vColor;"],
        functionCalls: ["lod_blur();"],
        functions: []
      },
      pixelate: {
        type: 0,
        name: "pixelate",
        defines: ["#define USE_PIXELATE"],
        uniforms: ["uniform sampler2D Texture0;", "uniform vec2 PixelateSize;"],
        varying: ["varying vec2 textureCoord;", "varying vec4 vColor;"],
        functionCalls: ["pixelate();"],
        functions: []
      }
    };

    Effect.fragmentShaderInfosByID = new Array(513);

    Effect.vertexShaderInfosByID = new Array(513);

    Effect.initialize = function() {
      Effect.fragmentShaderInfosByID[1] = Effect.fragmentShaderInfos.texture_color;
      Effect.fragmentShaderInfosByID[2] = Effect.fragmentShaderInfos.texture_color_tiling;
      Effect.fragmentShaderInfosByID[4] = Effect.fragmentShaderInfos.tilemap;
      Effect.fragmentShaderInfosByID[8] = Effect.fragmentShaderInfos.color_quad;
      Effect.fragmentShaderInfosByID[16] = Effect.fragmentShaderInfos.tinting;
      Effect.fragmentShaderInfosByID[32] = Effect.fragmentShaderInfos.color;
      Effect.fragmentShaderInfosByID[64] = Effect.fragmentShaderInfos.static_mask;
      Effect.fragmentShaderInfosByID[128] = Effect.fragmentShaderInfos.dynamic_mask;
      Effect.vertexShaderInfosByID[1] = Effect.vertexShaderInfos.position;
      Effect.vertexShaderInfosByID[2] = Effect.vertexShaderInfos.position;
      Effect.vertexShaderInfosByID[4] = Effect.vertexShaderInfos.tilemap;
      Effect.vertexShaderInfosByID[8] = Effect.vertexShaderInfos.position;
      Effect.vertexShaderInfosByID[16] = Effect.vertexShaderInfos.position;
      Effect.vertexShaderInfosByID[32] = Effect.vertexShaderInfos.position;
      Effect.vertexShaderInfosByID[64] = Effect.vertexShaderInfos.position;
      return Effect.vertexShaderInfosByID[128] = Effect.vertexShaderInfos.position;
    };

    Effect.createFragmentInfo = function(id) {
      var i, info, infos, j, k, len, result;
      infos = [];
      result = new gs.EffectInfo();
      for (i = j = 0; j <= 10; i = ++j) {
        if (id & (1 << i)) {
          info = Effect.fragmentShaderInfosByID[1 << i];
          if (!info) {
            break;
          }
          infos.push(info);
        }
      }
      infos.sort(function(a, b) {
        return a.type > b.type;
      });
      for (k = 0, len = infos.length; k < len; k++) {
        info = infos[k];
        result.defines.pushArray(info.defines);
        result.uniforms.pushArray(info.uniforms);
        result.varying.pushArray(info.varying);
        result.functions.pushArray(info.functions);
        result.functionCalls.pushArray(info.functionCalls);
      }
      result.defines = result.defines.filter(function(v, pos, self) {
        return self.indexOf(v) === pos;
      });
      result.uniforms = result.uniforms.filter(function(v, pos, self) {
        return self.indexOf(v) === pos;
      });
      result.varying = result.varying.filter(function(v, pos, self) {
        return self.indexOf(v) === pos;
      });
      result.functions = result.functions.filter(function(v, pos, self) {
        return self.indexOf(v) === pos;
      });
      result.functionCalls = result.functionCalls.filter(function(v, pos, self) {
        return self.indexOf(v) === pos;
      });
      return result;
    };

    Effect.createVertexInfo = function(id) {
      var info, result;
      result = new gs.EffectInfo();
      if (id & 4) {
        info = Effect.vertexShaderInfosByID[4];
        result.defines.pushArray(info.defines);
        result.uniforms.pushArray(info.uniforms);
        result.varying.pushArray(info.varying);
        result.functions.pushArray(info.functions);
        result.functionCalls.pushArray(info.functionCalls);
      } else {
        info = Effect.vertexShaderInfosByID[1];
        result.defines.pushArray(info.defines);
        result.uniforms.pushArray(info.uniforms);
        result.varying.pushArray(info.varying);
        result.functions.pushArray(info.functions);
        result.functionCalls.pushArray(info.functionCalls);
      }
      return result;
    };

    Effect.get = function(id) {
      var effect, fragmentInfo, fragmentShader, vertexInfo, vertexShader;
      effect = Effect.effects[id];
      if (!effect) {
        fragmentInfo = Effect.createFragmentInfo(id);
        vertexInfo = Effect.createVertexInfo(id);
        effect = new Effect();
        vertexShader = new VertexShader(vertexInfo);
        fragmentShader = new FragmentShader(fragmentInfo);
        effect.setVertexShader(vertexShader);
        effect.setFragmentShader(fragmentShader);
        effect.build();
        effect.prepare();
        Effect.effects[id] = effect;
      }
      return effect;
    };


    /**
    * A class to handle Open/GLSL shader-based effect. It is important to not confuse this class
    * with gs.GraphicEffect, which is for own custom shader-based effects. This class is for internal usage
    * only and encapsulates the OpenGL shader program. You should not create instance of this class manually.
    *
    * @namespace gs
    * @class Effect
    * @memberof gs
    * @constructor
     */

    function Effect(noTexture) {
      this.textured = noTexture != null ? false : true;
      this.programId = $gl.createProgram();
      this.locations = {};
      this.pointerSet = false;
    }

    Effect.prototype.dispose = function() {
      return $gl.deleteProgram(this.programId);
    };

    Effect.prototype.prepare = function() {
      this.vertexPositionAttribute = $gl.getAttribLocation(this.programId, "vertexPosition");
      this.vertexTextureAttribute = $gl.getAttribLocation(this.programId, "vertexTexture");
      this.vertexColorAttribute = $gl.getAttribLocation(this.programId, "vertexColor");
      $gl.enableVertexAttribArray(this.vertexPositionAttribute);
      $gl.enableVertexAttribArray(this.vertexTextureAttribute);
      if (this.vertexColorAttribute !== -1) {
        return $gl.enableVertexAttribArray(this.vertexColorAttribute);
      }
    };

    Effect.prototype.build = function() {
      $gl.linkProgram(this.programId);
      if (!$gl.getProgramParameter(this.programId, $gl.LINK_STATUS)) {
        throw 'Error while linking program: ' + $gl.getProgramInfoLog(this.programId);
      }
    };

    Effect.prototype.use = function() {
      $gl.useProgram(this.programId);
      $gl.vertexAttribPointer(this.vertexPositionAttribute, 2, $gl.FLOAT, false, 20, 0);
      $gl.vertexAttribPointer(this.vertexTextureAttribute, 2, $gl.FLOAT, false, 20, 8);
      if (this.vertexColorAttribute !== -1) {
        return $gl.vertexAttribPointer(this.vertexColorAttribute, 4, $gl.UNSIGNED_BYTE, true, 20, 16);
      }
    };


    /**
    * Sets an integer uniform value.
    *
    * @method setIntegerValue
    * @param {string} name - The name of the uniform to set.
    * @param {number} value - The integer value to set as the uniform's value.
     */

    Effect.prototype.setIntegerValue = function(name, value) {
      if (this.locations[name] == null) {
        this.locations[name] = $gl.getUniformLocation(this.programId, name);
      }
      return $gl.uniform1i(this.locations[name], value);
    };


    /**
    * Sets an array of integers.
    *
    * @method setIntegerValues
    * @param {string} name - The name of the uniform to set.
    * @param {number[]} values - The integer values to set as the uniform's value.
     */

    Effect.prototype.setIntegerValues = function(name, values) {
      if (this.locations[name] == null) {
        this.locations[name] = $gl.getUniformLocation(this.programId, name);
      }
      return $gl.uniform1iv(this.locations[name], values);
    };


    /**
    * Sets a float uniform value.
    *
    * @method setFloatValue
    * @param {string} name - The name of the uniform to set.
    * @param {number} value - The float value to set as the uniform's value.
     */

    Effect.prototype.setFloatValue = function(name, value) {
      if (this.locations[name] == null) {
        this.locations[name] = $gl.getUniformLocation(this.programId, name);
      }
      return $gl.uniform1f(this.locations[name], value);
    };


    /**
    * Sets an array of floats.
    *
    * @method setFloatValues
    * @param {string} name - The name of the uniform to set.
    * @param {number[]} value - The float values to set as the uniform's value.
     */

    Effect.prototype.setFloatValues = function(name, values) {
      if (this.locations[name] == null) {
        this.locations[name] = $gl.getUniformLocation(this.programId, name);
      }
      return $gl.uniform1fv(this.locations[name], values);
    };


    /**
    * Sets a vec2 uniform value.
    *
    * @method setVector2Value
    * @param {string} name - The name of the uniform to set.
    * @param {number} v1 - The x-coordinate of the vec2 value to set as the uniform's value.
    * @param {number} v2 - The y-coordinate of the vec2 value to set as the uniform's value.
     */

    Effect.prototype.setVector2Value = function(name, v1, v2) {
      if (this.locations[name] == null) {
        this.locations[name] = $gl.getUniformLocation(this.programId, name);
      }
      return $gl.uniform2f(this.locations[name], v1, v2);
    };


    /**
    * Sets an uniform array of vec2 values.
    *
    * @method setVector2Value
    * @param {string} name - The name of the uniform to set.
    * @param {number[]} values - An array of coordinate-pairs to set as the uniforms value.
     */

    Effect.prototype.setVector2Values = function(name, values) {
      if (this.locations[name] == null) {
        this.locations[name] = $gl.getUniformLocation(this.programId, name);
      }
      return $gl.uniform2fv(this.locations[name], values);
    };


    /**
    * Sets a vec3 uniform value.
    *
    * @method setVector3Value
    * @param {string} name - The name of the uniform to set.
    * @param {number} v1 - The x-coordinate of the vec3 value to set as the uniform's value.
    * @param {number} v2 - The y-coordinate of the vec3 value to set as the uniform's value.
    * @param {number} v3 - The z-coordinate of the vec3 value to set as the uniform's value.
     */

    Effect.prototype.setVector3Value = function(name, v1, v2, v3) {
      if (this.locations[name] == null) {
        this.locations[name] = $gl.getUniformLocation(this.programId, name);
      }
      return $gl.uniform3f(this.locations[name], v1, v2, v3);
    };


    /**
    * Sets an uniform array of vec3 values.
    *
    * @method setVector3Value
    * @param {string} name - The name of the uniform to set.
    * @param {number[]} values - An array of coordinate-pairs to set as the uniforms value.
     */

    Effect.prototype.setVector3Values = function(name, values) {
      if (this.locations[name] == null) {
        this.locations[name] = $gl.getUniformLocation(this.programId, name);
      }
      return $gl.uniform3fv(this.locations[name], values);
    };


    /**
    * Sets a vec4 uniform value.
    *
    * @method setVector4Value
    * @param {string} name - The name of the uniform to set.
    * @param {number} v1 - The x-coordinate of the vec4 value to set as the uniform's value.
    * @param {number} v2 - The y-coordinate of the vec4 value to set as the uniform's value.
    * @param {number} v3 - The z-coordinate of the vec4 value to set as the uniform's value.
    * @param {number} v4 - The w-coordinate of the vec4 value to set as the uniform's value.
     */

    Effect.prototype.setVector4Value = function(name, v1, v2, v3, v4) {
      if (this.locations[name] == null) {
        this.locations[name] = $gl.getUniformLocation(this.programId, name);
      }
      return $gl.uniform4f(this.locations[name], v1, v2, v3, v4);
    };


    /**
    * Sets an uniform array of vec4 values.
    *
    * @method setVector4Value
    * @param {string} name - The name of the uniform to set.
    * @param {number[]} values - An array of coordinate-pairs to set as the uniforms value.
     */

    Effect.prototype.setVector4Values = function(name, values) {
      if (this.locations[name] == null) {
        this.locations[name] = $gl.getUniformLocation(this.programId, name);
      }
      return $gl.uniform2fv(this.locations[name], values);
    };


    /**
    * Sets a mat4 uniform value.
    *
    * @method setMatrixValue
    * @param {string} name - The name of the uniform to set.
    * @param {number[]} value - The matrix to set as the uniform's value.
     */

    Effect.prototype.setMatrixValue = function(name, value) {
      if (this.locations[name] == null) {
        this.locations[name] = $gl.getUniformLocation(this.programId, name);
      }
      return $gl.uniformMatrix4fv(this.locations[name], false, value);
    };

    Effect.prototype.setVertexShader = function(shader) {
      if ((shader == null) && (this.vertexShader != null)) {
        $gl.detachShader(this.programId, this.vertexShader.id);
      }
      this.vertexShader = shader;
      if (this.vertexShader != null) {
        return $gl.attachShader(this.programId, this.vertexShader.id);
      }
    };

    Effect.prototype.setFragmentShader = function(shader) {
      if ((shader == null) && (this.fragmentShader != null)) {
        $gl.detachShader(this.programId, this.fragmentShader.id);
      }
      this.fragmentShader = shader;
      if (this.fragmentShader != null) {
        return $gl.attachShader(this.programId, this.fragmentShader.id);
      }
    };

    return Effect;

  })();

  window.Effect = Effect;

  gs.Effect = Effect;

}).call(this);

// Generated by CoffeeScript 1.12.2

/**
* Describes different kind of texture filters.
*
* @namespace gs
* @class TextureFilter
* @memberof gs
* @constructor
 */

(function() {
  var Texture2D, TextureFilter;

  TextureFilter = (function() {

    /**
    * Nearest-Neighbor
    * @property NEAREST
    * @type number
    * @static
     */
    function TextureFilter() {}

    TextureFilter.NEAREST = 0;


    /**
    * Linear Interpolation
    * @property LINEAR
    * @type number
    * @static
     */

    TextureFilter.LINEAR = 1;

    return TextureFilter;

  })();

  gs.TextureFilter = TextureFilter;


  /**
  * <p>A class to deal with textures. This class has a lot of more functions and properties than
  * actually documented. That is because this class is actually for internal usage only. The only exception
  * is if you work with custom GLSL shaders.</p>
  * <p>If you want to learn more about why certain classes of the Basic Engine are hidden and not editable, read the Basic Engine chapter
  * in Help File.</p>
  *
  * @namespace gs
  * @class Texture2D
  * @memberof gs
  * @constructor
   */

  Texture2D = (function() {

    /**
    * The default texture filter used for newly created textures. If you want to create a game
    * with pixel-look, it is recommended to set gs.TextureFilter.NEAREST.
    * @property filter
    * @type gs.TextureFilter
    * @readOnly
    * @static
     */
    Texture2D.filter = 0;


    /**
    * Sets the filter for the texture.
    *
    * @method setFilter
    * @param {gs.TextureFilter} filter - The filter to set
    * @param {boolean} mipmap - Indicates if mip-mapping should be enabled.
     */

    Texture2D.prototype.setFilter = function(filter, mipmap) {
      if (filter !== this.filter) {
        this.filter = filter;
        switch (this.filter) {
          case 0:
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MAG_FILTER, $gl.NEAREST);
            if (mipmap) {
              return $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.NEAREST_MIPMAP_NEAREST);
            } else {
              return $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.NEAREST);
            }
            break;
          case 1:
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MAG_FILTER, $gl.LINEAR);
            if (mipmap) {
              return $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.LINEAR_MIPMAP_LINEAR);
            } else {
              return $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.LINEAR);
            }
        }
      }
    };

    function Texture2D(width, height, format, noPower2, mipmap) {

      /**
      * Indicates if the dimensions of the texture are power-of-2.
      * @property isPowerOf2
      * @type boolean
       */
      this.isPowerOf2 = true;

      /**
      * The current texture filter. If you want to set the filter, use setFilter method.
      * @property filter
      * @type gs.TextureFilter
      * @readOnly
       */
      this.filter = Texture2D.filter;

      /**
      * Indicates if mip-mapping is enabled for this texture. If you want to change mip-mapping, use setFilter method.
      * @property mipmap
      * @readOnly
      * @type boolean
       */
      this.mipmap = mipmap != null ? mipmap : false;
      if (typeof width === "string") {
        this.createFromFile(width, height);
      } else if (typeof width === "number") {
        this.createTexture(width, height, format, noPower2, mipmap);
      } else if (width instanceof Uint8Array) {
        this.createFromPixels(width, height, format, noPower2, mipmap);
      } else if (typeof width === "object") {
        this.createFromImage(width, false, height);
      }
    }

    Texture2D.prototype.bind = function() {
      return $gl.bindTexture($gl.TEXTURE_2D, this.id);
    };

    Texture2D.prototype.unbind = function() {
      return $gl.bindTexture($gl.TEXTURE_2D, null);
    };

    Texture2D.prototype.dispose = function() {
      return $gl.deleteTexture(this.id);
    };

    Texture2D.prototype.createTexture = function(width, height, format, noPower2, mipmap) {

      /**
      * The OpenGL ID of the texture.
      * @property id
      * @type number
       */
      this.id = $gl.createTexture();

      /**
      * The original width of the texture.
      * @property width
      * @type number
       */
      this.width = width;

      /**
      * The original height of the texture.
      * @property height
      * @type number
       */
      this.height = height;

      /**
      * The real width of the texture.
      * @property realWidth
      * @type number
       */
      this.realWidth = width;

      /**
      * The real width of the texture.
      * @property realHeight
      * @type number
       */
      this.realHeight = height;
      this.isLoaded = true;
      this.isPowerOf2 = Math.nextPow2(this.width) === this.width && Math.nextPow2(this.height) === this.height;
      $gl.bindTexture($gl.TEXTURE_2D, this.id);
      if (!noPower2) {
        $gl.texImage2D($gl.TEXTURE_2D, 0, format != null ? format : $gl.RGBA, width, height, 0, format != null ? format : $gl.RGBA, $gl.UNSIGNED_BYTE, null);
        if (mipmap) {
          $gl.generateMipmap($gl.TEXTURE_2D);
        }
        switch (this.filter) {
          case 0:
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MAG_FILTER, $gl.NEAREST);
            if (mipmap) {
              $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.NEAREST_MIPMAP_NEAREST);
            } else {
              $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.NEAREST);
            }
            break;
          case 1:
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MAG_FILTER, $gl.LINEAR);
            if (mipmap) {
              $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.LINEAR_MIPMAP_LINEAR);
            } else {
              $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.LINEAR);
            }
        }
      } else {
        switch (this.filter) {
          case 0:
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MAG_FILTER, $gl.NEAREST);
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.NEAREST);
            break;
          case 1:
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MAG_FILTER, $gl.LINEAR);
            if (mipmap) {
              $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.LINEAR_MIPMAP_LINEAR);
            } else {
              $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.LINEAR);
            }
        }
        $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_WRAP_T, $gl.CLAMP_TO_EDGE);
        $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_WRAP_S, $gl.CLAMP_TO_EDGE);
        $gl.texImage2D($gl.TEXTURE_2D, 0, $gl.RGBA, this.realWidth, this.realHeight, 0, $gl.RGBA, $gl.UNSIGNED_BYTE, null);
        if (mipmap) {
          $gl.generateMipmap($gl.TEXTURE_2D);
        }
      }
      return $gl.bindTexture($gl.TEXTURE_2D, null);
    };

    Texture2D.prototype.setImageData = function(image) {
      if (image == null) {
        return;
      }
      $gl.bindTexture($gl.TEXTURE_2D, this.id);
      $gl.texSubImage2D($gl.TEXTURE_2D, 0, 0, 0, $gl.RGBA, $gl.UNSIGNED_BYTE, image.image);
      return $gl.bindTexture($gl.TEXTURE_2D, null);
    };

    Texture2D.prototype.createFromPixels = function(pixels, width, height, mipmap, flip) {
      var isPow2;
      this.id = $gl.createTexture();
      this.width = width || 1;
      this.height = height || 1;
      if (mipmap) {
        this.realHeight = Math.nextPow2(this.height);
        this.realWidth = Math.nextPow2(this.width);
      } else {
        this.realWidth = this.width;
        this.realHeight = this.height;
      }
      this.isLoaded = true;
      isPow2 = Math.nextPow2(this.width) === this.width && Math.nextPow2(this.height) === this.height;
      this.isPowerOf2 = isPow2;
      $gl.bindTexture($gl.TEXTURE_2D, this.id);
      $gl.pixelStorei($gl.UNPACK_FLIP_Y_WEBGL, flip);
      $gl.texImage2D($gl.TEXTURE_2D, 0, $gl.RGBA, width, height, 0, $gl.RGBA, $gl.UNSIGNED_BYTE, pixels);
      if (mipmap) {
        $gl.generateMipmap($gl.TEXTURE_2D);
      }
      $gl.pixelStorei($gl.UNPACK_FLIP_Y_WEBGL, false);
      switch (this.filter) {
        case 0:
          $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MAG_FILTER, $gl.NEAREST);
          if (mipmap) {
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.NEAREST_MIPMAP_NEAREST);
          } else {
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.NEAREST);
          }
          $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_WRAP_T, $gl.CLAMP_TO_EDGE);
          $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_WRAP_S, $gl.CLAMP_TO_EDGE);
          break;
        case 1:
          $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MAG_FILTER, $gl.LINEAR);
          if (mipmap) {
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.LINEAR_MIPMAP_LINEAR);
          } else {
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.LINEAR);
          }
          if (isPow2) {
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_WRAP_T, $gl.REPEAT);
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_WRAP_S, $gl.REPEAT);
          } else {
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_WRAP_T, $gl.CLAMP_TO_EDGE);
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_WRAP_S, $gl.CLAMP_TO_EDGE);
          }
      }
      return $gl.bindTexture($gl.TEXTURE_2D, null);
    };

    Texture2D.prototype.createFromImage = function(image, mipmap, flipY) {
      var canvas, context, imageData, isPow2;
      this.id = $gl.createTexture();
      this.width = image.width || 1;
      this.height = image.height || 1;
      if (mipmap) {
        this.realHeight = Math.nextPow2(this.height);
        this.realWidth = Math.nextPow2(this.width);
      } else {
        this.realWidth = this.width;
        this.realHeight = this.height;
      }
      this.isLoaded = true;
      isPow2 = Math.nextPow2(this.width) === this.width && Math.nextPow2(this.height) === this.height;
      this.isPowerOf2 = isPow2;
      canvas = image;
      if (mipmap && this.width !== this.realWidth || this.height !== this.realHeight) {
        canvas = document.createElement('canvas');
        canvas.width = Math.max(1, this.realWidth);
        canvas.height = Math.max(1, this.realHeight);
        context = canvas.getContext('2d');
        context.clearRect(0, 0, this.realWidth, this.realHeight);
        if (image.width > 0 && image.height > 0) {
          context.drawImage(image, 0, 0);
        }
        image = canvas;
      }
      if (image instanceof Image) {
        imageData = image;
      } else {
        imageData = canvas.getContext("2d").getImageData(0, 0, this.realWidth, this.realHeight);
      }
      $gl.bindTexture($gl.TEXTURE_2D, this.id);
      $gl.texImage2D($gl.TEXTURE_2D, 0, $gl.RGBA, $gl.RGBA, $gl.UNSIGNED_BYTE, imageData);
      if (mipmap) {
        $gl.generateMipmap($gl.TEXTURE_2D);
      }
      switch (this.filter) {
        case 0:
          $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MAG_FILTER, $gl.NEAREST);
          if (mipmap) {
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.NEAREST_MIPMAP_NEAREST);
          } else {
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.NEAREST);
          }
          $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_WRAP_T, $gl.CLAMP_TO_EDGE);
          $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_WRAP_S, $gl.CLAMP_TO_EDGE);
          break;
        case 1:
          $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MAG_FILTER, $gl.LINEAR);
          if (mipmap) {
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.LINEAR_MIPMAP_LINEAR);
          } else {
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.LINEAR);
          }
          if (isPow2) {
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_WRAP_T, $gl.REPEAT);
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_WRAP_S, $gl.REPEAT);
          } else {
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_WRAP_T, $gl.CLAMP_TO_EDGE);
            $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_WRAP_S, $gl.CLAMP_TO_EDGE);
          }
      }
      return $gl.bindTexture($gl.TEXTURE_2D, null);
    };

    Texture2D.prototype.createFromFile = function(filePath) {
      this.filePath = filePath;
      this.isLoaded = false;
      this.id = $gl.createTexture();
      this.image = new Image();
      this.image.onload = (function(_this) {
        return function() {
          var canvas, context;
          _this.width = _this.image.width;
          _this.height = _this.image.height;
          _this.realWidth = Math.nextPow2(_this.width);
          _this.realHeight = Math.nextPow2(_this.height);
          _this.isPowerOf2 = Math.nextPow2(_this.width) === _this.width && Math.nextPow2(_this.height) === _this.height;
          canvas = document.createElement('canvas');
          canvas.width = _this.realWidth;
          canvas.height = _this.realHeight;
          context = canvas.getContext('2d');
          context.drawImage(_this.image, 0, 0);
          $gl.bindTexture($gl.TEXTURE_2D, _this.id);
          $gl.texImage2D($gl.TEXTURE_2D, 0, $gl.RGBA, $gl.RGBA, $gl.UNSIGNED_BYTE, canvas);
          switch (_this.filter) {
            case 0:
              $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MAG_FILTER, $gl.NEAREST);
              $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.NEAREST);
              break;
            case 1:
              $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MAG_FILTER, $gl.LINEAR);
              $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.LINEAR);
          }
          $gl.bindTexture($gl.TEXTURE_2D, null);
          return _this.isLoaded = true;
        };
      })(this);
      return this.image.src = filePath;
    };

    return Texture2D;

  })();

  window.Texture2D = Texture2D;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var Buffer;

  Buffer = (function() {
    function Buffer(type, data, itemSize, itemCount, usage) {
      this.id = $gl.createBuffer();
      this.type = type;
      this.itemSize = itemSize;
      this.itemCount = itemCount;
      this.usage = usage != null ? usage : $gl.STATIC_DRAW;
      $gl.bindBuffer(type, this.id);
      $gl.bufferData(type, data, this.usage);
      this.id.itemSize = itemSize;
      this.id.numItems = itemCount;
      $gl.bindBuffer(type, null);
    }

    Buffer.prototype.dispose = function() {
      if (this.id != null) {
        $gl.deleteBuffer(this.id);
        return this.id = null;
      }
    };

    Buffer.prototype.setData = function(data) {
      return $gl.bufferData(this.type, new Float32Array(data), this.usage);
    };

    Buffer.prototype.setSubData = function(data, offset) {
      return $gl.bufferSubData(this.type, offset, data);
    };

    Buffer.prototype.bind = function() {
      return $gl.bindBuffer(this.type, this.id);
    };

    Buffer.prototype.unbind = function() {
      return $gl.bindBuffer(this.type, null);
    };

    return Buffer;

  })();

  if (window.gs == null) {
    window.gs = {};
  }

  window.gs.Buffer = Buffer;

}).call(this);

// Generated by CoffeeScript 1.6.3
(function() {
  var FrameBuffer;

  FrameBuffer = (function() {
    function FrameBuffer() {
      this.id = $gl.createFramebuffer();
      this.texture = null;
    }

    FrameBuffer.prototype.dispose = function() {
      if (this.id != null) {
        $gl.deleteFramebuffer(this.id);
        return this.id = null;
      }
    };

    FrameBuffer.prototype.setTexture = function(texture) {
      if (this.texture != null) {
        $gl.framebufferTexture2D($gl.FRAMEBUFFER, $gl.COLOR_ATTACHMENT0, $gl.TEXTURE_2D, null, 0);
      }
      this.texture = texture;
      if (this.texture != null) {
        return $gl.framebufferTexture2D($gl.FRAMEBUFFER, $gl.COLOR_ATTACHMENT0, $gl.TEXTURE_2D, this.texture.id, 0);
      }
    };

    FrameBuffer.prototype.bind = function() {
      return $gl.bindFramebuffer($gl.FRAMEBUFFER, this.id);
    };

    FrameBuffer.prototype.unbind = function() {
      return $gl.bindFramebuffer($gl.FRAMEBUFFER, null);
    };

    return FrameBuffer;

  })();

  if (window.gs == null) {
    window.gs = {};
  }

  window.gs.FrameBuffer = FrameBuffer;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var Font;

  Font = (function() {
    Font.CONTROL_SEQUENCE_COLOR = 1;

    Font.CONTROL_SEQUENCE_STYLE = 2;

    Font.sequenceColors = [Color.BLACK, Color.WHITE];


    /**
    * The default font name.
    * @property defaultName
    * @type string
    * @static
     */

    Font.defaultName = "Courier";


    /**
    * The default font size.
    * @property defaultSize
    * @type number
    * @static
     */

    Font.defaultSize = 18;


    /**
    * Indicates if the the default style is bold.
    * @property defaultBold
    * @type boolean
    * @static
     */

    Font.defaultBold = false;


    /**
    * Indicates if the the default style is italic.
    * @property defaultItalic
    * @type boolean
    * @static
     */

    Font.defaultItalic = false;


    /**
    * Indicates if the the default style is underline.
    * @property @defaultUnderline
    * @type boolean
    * @static
     */

    Font.defaultUnderline = false;


    /**
    * Indicates if the the default style is strike-through.
    * @property @defaultStrikeThrough
    * @type boolean
    * @static
     */

    Font.defaultStrikeThrough = false;


    /**
    * Indicates if the default font has shadow.
    * @property defaultShadow
    * @type boolean
    * @static
     */

    Font.defaultShadow = false;


    /**
    * Indicates if the default font has an outline.
    * @property defaultBorder
    * @type boolean
    * @static
     */

    Font.defaultBorder = false;


    /**
    * The default font color.
    * @property defaultColor
    * @type gs.Color
    * @static
     */

    Font.defaultColor = new Color(255, 255, 255, 255);


    /**
    * The default font outline color.
    * @property defaultBorderColor
    * @type gs.Color
    * @static
     */

    Font.defaultBorderColor = new Color(0, 0, 0, 255);


    /**
    * The default font shadow color.
    * @property defaultShadowColor
    * @type gs.Color
    * @static
     */

    Font.defaultShadowColor = new Color(0, 0, 0, 128);

    Font.cache = {};

    Font.bitmapCache = {};

    Object.defineProperty(Font.prototype, "borderColor", {
      set: function(value) {
        var ref;
        if (!value.equals(this.borderColor_)) {
          if ((ref = this.outlineBitmap) != null) {
            ref.changeColor(this.borderColor_);
          }
          return this.borderColor_ = value;
        }
      },
      get: function() {
        return this.borderColor_;
      }
    });

    Object.defineProperty(Font.prototype, "shadowColor", {
      set: function(value) {
        var ref;
        if (!value.equals(this.shadowColor_)) {
          if ((ref = this.shadowBitmap) != null) {
            ref.changeColor(this.shadowColor_);
          }
          return this.shadowColor_ = value;
        }
      },
      get: function() {
        return this.shadowColor_;
      }
    });

    Font.prototype.height = function() {};


    /**
    * The height of the font.
    * @property height
    * @type number
    * @readOnly
     */

    Object.defineProperty(Font.prototype, "height", {
      get: function() {
        if (this.metrics == null) {
          this.metrics = this.fontMetrics_();
        }
        return this.metrics.height;
      }
    });


    /**
    * The ascent-value of the font.
    * @property ascent
    * @type number
    * @readOnly
     */

    Object.defineProperty(Font.prototype, "ascent", {
      get: function() {
        if (this.metrics == null) {
          this.metrics = this.fontMetrics_();
        }
        return this.metrics.ascent;
      }
    });


    /**
    * The descent-value of the font.
    * @property descent
    * @type number
    * @readOnly
     */

    Object.defineProperty(Font.prototype, "descent", {
      get: function() {
        if (this.metrics == null) {
          this.metrics = this.fontMetrics_();
        }
        return this.metrics.descent;
      }
    });


    /**
    * The line-height of the font.
    * @property lineHeight
    * @type number
    * @readOnly
     */

    Object.defineProperty(Font.prototype, "lineHeight", {
      get: function() {
        if (this.bitmaps.length > 0) {
          return this.bitmapFontData.common.lineHeight;
        }
        if (this.metrics == null) {
          this.metrics = this.fontMetrics_();
        }
        if (this.border) {
          return this.metrics.height + this.borderSize;
        } else {
          return this.metrics.height;
        }
      }
    });

    Object.defineProperty(Font.prototype, "size", {
      set: function(v) {
        if (this.size_ !== v) {
          this.metrics = null;
          return this.size_ = v;
        }
      },
      get: function() {
        if (this.bitmaps.length > 0) {
          return this.bitmapFontData.info.size;
        } else {
          return this.size_;
        }
      }
    });

    Object.defineProperty(Font.prototype, "name", {
      set: function(v) {
        if (this.name_ !== v) {
          this.metrics = null;
          this.name_ = v;
          return this.update();
        }
      },
      get: function() {
        return this.name_;
      }
    });

    Object.defineProperty(Font.prototype, "bold", {
      set: function(v) {
        if (this.bold_ !== v) {
          this.metrics = null;
          return this.bold_ = v;
        }
      },
      get: function() {
        return this.bold_;
      }
    });

    Object.defineProperty(Font.prototype, "italic", {
      set: function(v) {
        if (this.italic_ !== v) {
          this.metrics = null;
          return this.italic_ = v;
        }
      },
      get: function() {
        return this.italic_;
      }
    });

    Object.defineProperty(Font.prototype, "smallCaps", {
      set: function(v) {
        if (this.smallCaps_ !== v) {
          this.metrics = null;
          return this.smallCaps_ = v;
        }
      },
      get: function() {
        return this.smallCaps_;
      }
    });


    /**
    * A class to describe fonts. Fonts can be loaded from installed
    * fonts or from imported fonts. (Graphics/Fonts)
    *
    * @namespace gs
    * @class Font
    * @memberof gs
    * @constructor
    * @param {string} name - The font name.
    * @param {string} size - The font size in pixels.
     */

    function Font(name, size) {
      var ext, filePath, languageCode, resources;
      this.outlineBitmap_ = null;
      this.shadowBitmap_ = null;

      /**
      * Font metrics contains additional info about the font.
      * @property metrics
      * @type gs.FontMetrics
       */
      this.metrics = null;
      if (Font.compositeBuffer == null) {
        Font.compositeBuffer = document.createElement("canvas");
        Font.compositeBuffer.width = 128;
        Font.compositeBuffer.height = 128;
        Font.compositeContext = Font.compositeBuffer.getContext("2d");
        Font.compositeContext.imageSmoothingEnabled = false;
        Font.compositeContext.webkitImageSmoothingEnabled = false;
        Font.compositeContext.mozImageSmoothingEnabled = false;
      }
      if ((name != null) && (size == null)) {

        /**
        * The font name.
        * @property name
        * @type string
         */
        this.name_ = name.name;

        /**
        * The font size in pixels.
        * @property size
        * @type number
         */
        this.size = name.size;

        /**
        * Indicates if the font-style is bold.
        * @property bold
        * @type boolean
         */
        this.bold = name.bold;

        /**
        * Indicates if the font-style is italic.
        * @property italic
        * @type boolean
         */
        this.italic = name.italic;

        /**
        * Indicates if the font-style is underline.
        * @property underline
        * @type boolean
         */
        this.underline = name.underline;

        /**
        * Indicates if the font-style is strike-through.
        * @property strikeThrough
        * @type boolean
         */
        this.strikeThrough = name.strikeThrough;

        /**
        * Indicates if the font-style is small-caps.
        * @property smallCaps
        * @type boolean
         */
        this.smallCaps = name.smallCaps;

        /**
        * The font's color.
        * @property color
        * @type gs.Color
         */
        this.color = Color.fromObject(name.color);

        /**
        * Indicates if the font has an outline/border.
        * @property border
        * @type boolean
         */
        this.border = name.border;

        /**
        * The outline color.
        * @property borderColor
        * @type gs.Color
         */
        this.borderColor_ = name.borderColor;

        /**
        * The outline size.
        * @property borderSize
        * @type number
         */
        this.borderSize = name.borderSize;

        /**
        * Indicates if the font has a shadow.
        * @property shadow
        * @type boolean
         */
        this.shadow = name.shadow;

        /**
        * The shadow color.
        * @property shadowColor
        * @type gs.Color
         */
        this.shadowColor_ = name.shadowColor;

        /**
        * The horizontal shadow offset.
        * @property @shadowOffsetX
        * @type number
         */
        this.shadowOffsetX = 1;

        /**
        * The vertical shadow offset.
        * @property @shadowOffsetX
        * @type number
         */
        this.shadowOffsetY = 1;
        this.measureBitmap = null;
      } else {
        this.name_ = name;
        this.size = size;
        this.bold = false;
        this.italic = false;
        this.underline = false;
        this.strikethrough = false;
        this.smallCaps = false;
        this.color = new Color(255, 255, 255, 255);
        this.border = false;
        this.borderColor_ = new Color(0, 0, 0, 255);
        this.borderSize = 4;
        this.shadow = false;
        this.shadowColor_ = new Color(0, 0, 0, 128);
        this.shadowOffsetX = 1;
        this.shadowOffsetY = 1;
        this.measureBitmap = null;
      }
      this.bitmap = null;
      this.bitmaps = [];

      /**
      * Indicates if the font is loaded.
      * @property loaded
      * @type boolean
       */
      this.loaded = true;
      if (GS.dataCache && GS.dataCache["RESOURCES"]) {
        languageCode = null;
        resources = GS.dataCache["RESOURCES"].items.resources;
        filePath = "Graphics/Fonts/" + this.name;
        if (resources[filePath + "_" + languageCode] != null) {
          ext = GS.dataCache["RESOURCES"].items.resources[filePath + "_" + languageCode] || ".woff";
          filePath = "resources/" + GS.DataPreparer.prepareFilePath(filePath + "_" + languageCode + ext);
        } else {
          ext = GS.dataCache["RESOURCES"].items.resources[filePath] || ".woff";
          filePath = "resources/" + GS.DataPreparer.prepareFilePath(filePath + ext);
        }
        if (ext === ".fnt") {
          if (Font.bitmapCache[this.name + this.size] != null) {
            this.bitmap = Font.bitmapCache[this.name + this.size].bitmap;
            this.bitmapFontData = Font.bitmapCache[this.name + this.size].bitmapFontData;
          } else {
            this.loaded = false;
            this.loadBitmapFontData(filePath, languageCode);
          }
        }
      }
    }

    Font.prototype.update = function() {
      var bitmap, j, len, page, ref, results;
      this.bitmapFontData = Font.cache[this.name];
      if (this.bitmapFontData) {
        ref = this.bitmapFontData.pages;
        results = [];
        for (j = 0, len = ref.length; j < len; j++) {
          page = ref[j];
          bitmap = Font.bitmapCache[page.file];
          results.push(this.bitmaps[page.id] = bitmap);
        }
        return results;
      } else {
        return this.bitmaps = [];
      }
    };

    Font.prototype.parseBitmapFontData = function(text) {
      var chunks, data;
      data = {
        pages: [],
        charData: [],
        kerningData: []
      };
      chunks = text.split("\n");
      chunks = chunks.select(function(c) {
        var obj;
        obj = {};
        if (c.startsWith("info")) {
          c = c.replace(/["][^"]*["]/gm, "0");
        }
        c.split(" ").select(function(i) {
          var pair;
          if (i.length === 0) {
            return;
          }
          pair = i.split("=");
          if (!pair[1]) {
            obj.cid = pair[0];
            if (obj.cid !== "char" && obj.cid !== "page") {
              return data[obj.cid] = obj;
            }
          } else {
            if (obj.cid === "char") {
              if (pair[0] === "id") {
                return obj.id = parseInt(pair[1]);
              } else {
                if (!data.charData[obj.id]) {
                  data.charData[obj.id] = obj;
                }
                return data.charData[obj.id][pair[0]] = isNaN(pair[1]) ? pair[1] : parseInt(pair[1]);
              }
            } else if (obj.cid === "kerning") {
              if (pair[0] === "first") {
                obj.first = parseInt(pair[1]);
                if (!data.kerningData[obj.first]) {
                  data.kerningData[obj.first] = [];
                }
              }
              if (pair[0] === "second") {
                obj.second = parseInt(pair[1]);
              }
              if (pair[0] === "amount") {
                obj.amount = parseInt(pair[1]);
                return data.kerningData[obj.first][obj.second] = obj.amount;
              }
            } else if (obj.cid === "page") {
              if (pair[0] === "id") {
                return obj.id = parseInt(pair[1]);
              } else {
                if (!data.pages[obj.id]) {
                  data.pages[obj.id] = obj;
                }
                return data.pages[obj.id][pair[0]] = isNaN(pair[1]) ? pair[1].replace(/"/gm, "") : parseInt(pair[1]);
              }
            } else {
              return obj[pair[0]] = isNaN(pair[1]) ? pair[1] : parseInt(pair[1]);
            }
          }
        });
        return obj;
      });
      return data;
    };

    Font.prototype.loadBitmapFontPages = function(data) {
      var b, bitmap, j, k, len, len1, page, ref, ref1, resourcePath, results;
      ref = data.pages;
      results = [];
      for (j = 0, len = ref.length; j < len; j++) {
        page = ref[j];
        bitmap = Font.bitmapCache[page.file];
        if (!bitmap) {
          resourcePath = "Graphics/Fonts/" + (page.file.substring(0, page.file.lastIndexOf(".")));
          if (!GS.dataCache.RESOURCES.items.resources[resourcePath]) {
            GS.dataCache.RESOURCES.items.resources[resourcePath] = ".png";
          }
          bitmap = new Bitmap("Graphics/Fonts/" + (page.file.substring(0, page.file.lastIndexOf("."))));
          bitmap.immutable = false;
          bitmap.onload = (function(_this) {
            return function() {
              var b, k, len1, ref1, results1;
              _this.loaded = true;
              ref1 = _this.bitmaps;
              results1 = [];
              for (k = 0, len1 = ref1.length; k < len1; k++) {
                b = ref1[k];
                if (!b.loaded) {
                  _this.loaded = false;
                  break;
                } else {
                  results1.push(void 0);
                }
              }
              return results1;
            };
          })(this);
        } else {
          this.loaded = true;
          ref1 = this.bitmaps;
          for (k = 0, len1 = ref1.length; k < len1; k++) {
            b = ref1[k];
            if (!b.loaded) {
              this.loaded = false;
              break;
            }
          }
        }
        Font.bitmapCache[page.file] = bitmap;
        results.push(this.bitmaps[page.id] = bitmap);
      }
      return results;
    };

    Font.prototype.loadBitmapFontData = function(filePath, languageCode) {
      var request;
      this.bitmapFontData = Font.cache[this.name];
      if (!this.bitmapFontData) {
        request = new XMLHttpRequest();
        request.open('GET', filePath, true);
        request.responseType = 'arraybuffer';
        request.onload = (function(_this) {
          return function() {
            var buffer, reader;
            buffer = GS.DataPreparer.prepare(request.response);
            reader = new FileReader();
            reader.onload = function(e) {
              _this.bitmapFontData = _this.parseBitmapFontData(e.target.result);
              Font.cache[_this.name] = _this.bitmapFontData;
              return _this.loadBitmapFontPages(_this.bitmapFontData);
            };
            reader.onerror = onerror;
            return reader.readAsText(new Blob([buffer]));
          };
        })(this);
        request.onerror = onerror;
        return request.send();
      } else {
        return this.update();
      }
    };

    Font.prototype._fontHeight = function() {
      return this.fontMetrics_();
    };

    Font.prototype.fontMetrics_ = function() {
      var block, body, common, div, result, span;
      if (this.bitmaps.length > 0) {
        common = this.bitmapFontData.common;
        return {
          descent: common.lineHeight - common.base,
          ascent: common.base,
          height: common.lineHeight
        };
      }
      span = jQuery('<span>' + 'hg' + '</span>').css({
        fontFamily: this.name,
        fontSize: this.size.toString() + "px",
        fontWeight: this.bold ? "bold" : "normal",
        fontStyle: this.italic ? "italic" : "normal",
        fontVariant: this.smallCaps ? "small-caps" : "normal"
      });
      block = jQuery('<div style="display: inline-block; width: 1px; height: 0px;"></div>');
      div = jQuery('<div></div>');
      div.append(span, block);
      body = jQuery('body');
      body.append(div);
      try {
        result = {};
        block.css({
          verticalAlign: 'baseline'
        });
        result.ascent = block.offset().top - span.offset().top;
        block.css({
          verticalAlign: 'bottom'
        });
        result.height = block.offset().top - span.offset().top;
        result.descent = result.height - result.ascent;
      } finally {
        div.remove();
      }
      return result;
    };

    Font.exists = function() {
      return true;
    };


    /**
    * Creates a new default font.
    *
    * @method createDefault
    * @static
    * @return {gs.Font} A new created default font.
     */

    Font.createDefault = function() {
      var font;
      font = new Font(this.defaultName, this.defaultSize);
      font.bold = this.defaultBold;
      font.italic = this.defaultItalic;
      font.underline = this.defaultUnderline;
      font.strikeThrough = this.defaultStrikeThrough;
      font.shadow = this.defaultShadow;
      font.border = this.defaultBorder;
      font.color = this.defaultColor;
      font.borderColor = this.defaultBorderColor;
      font.shadowColor = this.defaultShadowColor;
      return font;
    };


    /**
    * Loads custom fonts and makes them ready to use.
    *
    * @method loadCustomFonts
    * @param {Function} A callback called if the loading-process has been done. It has the following
    * signature:
    * <pre><code>
    * // error - Indicates if an error occurred during the loading-process
    * function(error) {}
    * </code></pre>
     */

    Font.loadCustomFonts = function(callback) {
      var families, parts, path, resources, url;
      url = "";
      families = [];
      url = 'resources/fonts.css';
      resources = GS.dataCache["RESOURCES"].items.resources;
      for (path in resources) {
        if (path.startsWith("Graphics/Fonts/")) {
          parts = path.split("/");
          if (resources[path] === ".woff") {
            families.push(parts[parts.length - 1]);
          } else if (resources[path] === ".fnt") {
            new Font(path.split("/").last(), 1);
          }
        }
      }
      if (families.length > 0) {
        return WebFont.load({
          custom: {
            families: families,
            urls: [url]
          },
          active: (function(_this) {
            return function() {
              return typeof callback === "function" ? callback(false) : void 0;
            };
          })(this),
          inactive: (function(_this) {
            return function() {
              return typeof callback === "function" ? callback(true) : void 0;
            };
          })(this)
        });
      } else {
        return typeof callback === "function" ? callback(false) : void 0;
      }
    };


    /**
    * Returns a CSS string representation of the font like "italic bold 12px Arial".
    *
    * @method toString
    * @return {string} A CSS representation of the font.
     */

    Font.prototype.toString = function() {
      return (this.italic ? "italic" : "") + " " + (this.bold ? "bold" : "") + " " + (this.smallCaps ? "small-caps" : "") + " " + this.size + "px " + this.name;
    };


    /**
    * Assigns all properties from the specified font.
    *
    * @method set
    * @param {gs.Font} font - The font to get all properties from.
     */

    Font.prototype.set = function(font) {
      this.name = font.name;
      this.size = font.size;
      this.bold = font.bold;
      this.italic = font.italic;
      this.underline = font.underline;
      this.strikeThrough = font.strikeThrough;
      this.smallCaps = font.smallCaps;
      this.color.setFromObject(font.color);
      this.border = font.border;
      this.borderColor_ = font.borderColor;
      this.borderSize = font.borderSize;
      this.shadow = font.shadow;
      this.shadowColor_ = font.shadowColor;
      return this.measureBitmap = null;
    };


    /**
    * Converts the font into a data bundle.
    *
    * @method toDataBundle
    * @return {Object} The data bundle.
     */

    Font.prototype.toDataBundle = function() {
      return {
        name: this.name,
        size: this.size,
        bold: this.bold,
        italic: this.italic,
        underline: this.underline,
        strikeThrough: this.strikeThrough,
        smallCaps: this.smallCaps,
        color: this.color,
        border: this.border,
        borderColor: this.borderColor,
        borderSize: this.borderSize,
        shadow: this.shadow,
        shadowColor: this.shadowColor
      };
    };


    /**
    * Restores the font from a data bundle.
    *
    * @method toDataBundle
    * @param {Object} data - The data bundle.
     */

    Font.prototype.restore = function(data) {
      this.name = data.name;
      this.size = data.size;
      this.bold = data.bold;
      this.italic = data.italic;
      this.underline = data.underline;
      this.strikeThrough = data.strikeThrough;
      this.smallCaps = data.smallCaps;
      this.color.setFromObject(data.color);
      this.border = data.border;
      this.borderColor_.setFromObject(data.borderColor);
      this.borderSize = data.borderSize;
      this.shadow = data.shadow;
      this.shadowColor_.setFromObject(data.shadowColor);
      return this.measureBitmap = null;
    };

    Font.prototype.measureBitmapChar = function(c) {
      var code, rect;
      code = c.charCodeAt(0);
      rect = this.bitmapFontData.charData[code];
      if (rect != null) {
        return {
          width: rect.xadvance,
          height: this.bitmapFontData.common.lineHeight + 1
        };
      } else {
        return {
          width: 0,
          height: 0
        };
      }
    };

    Font.prototype.measureBitmapText = function(text) {
      var code, h, i, j, rect, ref, size;
      size = {
        width: 0,
        height: 0
      };
      h = 0;
      for (i = j = 0, ref = text.length; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
        code = text.charCodeAt(i);
        rect = this.bitmapFontData.charData[code];
        if (rect != null) {
          size.width = size.width + rect.xadvance - rect.xoffset;
          size.height = this.bitmapFontData.common.lineHeight + 1;
        }
      }
      return size;
    };


    /**
    * Measures the size of a single character. The result isn't pixel-perfect.
    *
    * @method measureChar
    * @param {string} c - A single character.
    * @return {gs.Size} The size of the character.
     */

    Font.prototype.measureChar = function(c) {
      var code, s, size;
      if (this.bitmaps.length > 0) {
        return this.measureBitmapChar(c);
      }
      if (this.measureBitmap == null) {
        this.measureBitmap = new Bitmap(1, 1);
        this.measureBitmap.font = this;
        this.measureBitmap.context.font = this.toString();
      }
      size = {
        width: 0,
        height: 0
      };
      code = c.charCodeAt(0);
      this.measureBitmap.context.font = this.toString();
      s = this.measureBitmap.context.measureText(c.charAt(0));
      s.height = this.lineHeight;
      size.width = s.width;
      size.height = s.height;
      if (this.border) {
        size.width += this.borderSize - 1 + this.borderSize - 1;
      }
      return size;
    };


    /**
    * Compares the font to the specified font. Only name and size are compared.
    *
    * @method compare
    * @param {gs.Font} font - The font to compare to.
    * @return {boolean} If <b>true</b> both fonts are equal. Otherwise <b>false</b>
     */

    Font.prototype.compare = function(font) {
      return font && font.name === this.name && font.size === this.size && font.bold === this.bold && font.italic === this.italic && font.underline === this.underline && font.strikeThrough === this.strikeThrough && font.smallCaps === this.smallCaps && font.border === this.border && font.shadow === this.shadow && font.borderSize === this.borderSize && font.shadowOffsetX === this.shadowOffsetX && font.shadowOffsetY === this.shadowOffsetY && gs.Color.compare(this.color, font.color) && gs.Color.compare(this.shadowColor, font.shadowColor) && gs.Color.compare(this.borderColor, font.borderColor);
    };


    /**
    * Measures a specified text as plain text. Formatting like outlines, style, etc.
    * is ignored. The result isn't pixel-perfect.
    *
    * @method measureTextPlain
    * @param {string} text - The text to measure.
    * @return {gs.Size} The size of the text.
     */

    Font.prototype.measureTextPlain = function(text) {
      var s;
      if (this.bitmaps.length > 0) {
        return this.measureBitmapText(text);
      }
      if (this.measureBitmap == null) {
        this.measureBitmap = new Bitmap(1, 1);
        this.measureBitmap.font = new Font(this);
        this.measureBitmap.context.font = this.toString();
      }
      this.measureBitmap.context.font = this.toString();
      s = this.measureBitmap.context.measureText(text);
      return {
        width: s.width,
        height: this.lineHeight
      };
    };


    /**
    * Measures a specified text. The result isn't pixel-perfect.
    *
    * @method measureText
    * @param {string} text - The text to measure.
    * @return {gs.Size} The size of the text.
     */

    Font.prototype.measureText = function(text) {
      var s, size;
      if (this.bitmaps.length > 0) {
        return this.measureBitmapText(text);
      }
      size = {
        width: 0,
        height: 0
      };
      if (this.measureBitmap == null) {
        this.measureBitmap = new Bitmap(1, 1);
        this.measureBitmap.font = new Font(this);
        this.measureBitmap.context.font = this.toString();
      }
      this.measureBitmap.context.font = this.toString();
      if (this.shadow) {
        this.measureBitmap.context.shadowOffsetX = 1;
        this.measureBitmap.context.shadowOffsetY = 1;
        this.measureBitmap.context.shadowBlur = 2;
      } else {
        this.measureBitmap.context.shadowOffsetX = 0;
        this.measureBitmap.context.shadowOffsetY = 0;
        this.measureBitmap.context.shadowBlur = 0;
      }
      s = this.measureBitmap.context.measureText(text);
      s.height = this.lineHeight;
      size.width += s.width;
      size.height = s.height;
      size.width = Math.round(size.width);
      size.height = Math.round(size.height);
      return size;
    };

    return Font;

  })();

  window.Font = Font;

  gs.Font = Font;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var Alignment, Bitmap, TextAlignment;

  TextAlignment = (function() {
    function TextAlignment() {}

    TextAlignment.LEFT = 0;

    TextAlignment.TOP = 0;

    TextAlignment.CENTER = 1;

    TextAlignment.RIGHT = 2;

    TextAlignment.BOTTOM = 2;

    return TextAlignment;

  })();

  gs.TextAlignment = TextAlignment;

  Alignment = (function() {
    function Alignment() {}

    Alignment.LEFT = 0;

    Alignment.TOP = 1;

    Alignment.CENTER = 2;

    Alignment.RIGHT = 3;

    Alignment.BOTTOM = 4;

    return Alignment;

  })();

  gs.Alignment = TextAlignment;

  Bitmap = (function() {
    Bitmap.fromDataBundle = function(data, context) {
      var result;
      result = null;
      if (data.filePath) {
        result = ResourceManager.getBitmap(data.filePath);
        result.font = gs.ObjectCodec.decode(data.font, context);
      }
      return result;
    };

    Bitmap.toDataBundle = function(object, context) {
      var r;
      r = {
        filePath: object.filePath,
        languageCode: object.languageCode,
        font: gs.ObjectCodec.encode(object.font, context)
      };
      return r;
    };


    /**
    * The bitmap class allows to load images from files or creating
    * in-memory bitmap. It offers different methods to directly draw
    * to the bitmap or recolor it. You can attach a bitmap to a gs.Sprite
    * object to display it on screen.
    *
    * <p>All bitmaps which are loaded from image files are immutable by default to keep memory usage
    * as low as possible. The pixel-data of an immutable bitmap cannot be changed and the bitmap cannot be
    * drawn onto another bitmap. However, an immutable bitmap will be automatically turned into a
    * mutable bitmap whenever you try to draw onto that bitmap or use it as a source-bitmap. A mutable bitmap may take the double/triple
    * amount of memory compared to an immutable one.</p>
    *
    * <p>You can make a bitmap immutable/mutable manually anytime using the makeMutable/makeImmutable methods. Notice that
    * a conversion from immutable to mutable or vice-versa may be time-consuming.</p>
    *
    * @namespace gs
    * @class Bitmap
    * @memberof gs
    * @constructor
    * @param {Object} width/filePath/image - The width of the bitmap for in-memory creation <b>OR</b> the filePath to an
    * image resource to load it from file <b>OR</b> an HTMLImageElement to create the bitmap from.
    * @param {number} [height/languageCode=null] - The height of the bitmap <b>OR</b> the language code if the bitmap
    * is created from filePath.
     */

    function Bitmap(width, height, imageWrap) {

      /**
      * Indicates if the bitmap is loaded.
      * @property loaded
      * @type boolean
      * @readOnly
       */
      this.loaded = false;

      /**
      * The font used to draw text on the bitmap.
      * @property font
      * @type gs.Font
       */
      this.font = null;

      /**
      * Indicates if the bitmap is immutable. An immutable bitmap's pixel-data cannot be
      * anymore after creation which is the default for bitmaps created from image files. For in-memory
      * bitmaps, it defaults to <b>false</b>. You can change this property before the bitmap has been loaded to create an mutable bitmap. You
      * can also check makeImmutable/makeMutable method to make a bitmap immutable/mutable anytime after creation.
      * @property immutable
      * @type boolean
       */
      this.immutable = false;
      this.disposed = false;
      this.imageData = null;
      this.bounds = new gs.Rect(0, 0, 1, 1);
      this.sequenceNumber = 1;
      this.dirtyRect = new Rect(-1, -1, -1, -1);
      if ((height != null) && !(typeof height === 'string') && !(typeof height === 'boolean')) {
        this.createWithSize(width, height);
        this.loaded = true;
      } else if (typeof width === 'string') {
        this.loaded = false;
        this.createFromFile(width, height, imageWrap);
      } else if (typeof width === 'object') {
        this.loaded = true;
        if (imageWrap) {
          this.createWithImage(width);
        } else {
          this.createFromImage(width, height);
        }
      }
    }


    /**
    * Makes the bitmap immutable if it was mutable before. An immutable bitmap's pixel-data
    * cannot be changed anymore and it cannot be drawn on other bitmaps but the memory consumption is much less. This action cannot be undone. You don't
    * need to call this for bitmap's created from image files since those are immutable by default.
    * @method makeImmutable
     */

    Bitmap.prototype.makeImmutable = function() {
      if (this.immutable || this.error) {
        return;
      }
      this.immutable = true;
      if (this.texture == null) {
        this.texture = new Texture2D(this.image);
      } else {
        this.texture.setImageData(bitmap);
        this.dirtyRect.set(-1, -1, -1, -1);
      }
      this.image = null;
      this.context = null;
      return this.texture.sequenceNumber = this.sequenceNumber;
    };


    /**
    * Makes the bitmap mutable if it was immutable before. An mutable bitmap's pixel-data
    * can be changed anytime and it can be drawn on other mutable bitmaps but the memory consumption is much higher.
    * @method makeMutable
     */

    Bitmap.prototype.makeMutable = function() {
      var canvas, data, framebuffer, imageData;
      if (!this.immutable || this.error) {
        return;
      }
      this.immutable = false;
      framebuffer = $gl.createFramebuffer();
      $gl.bindFramebuffer($gl.FRAMEBUFFER, framebuffer);
      $gl.framebufferTexture2D($gl.FRAMEBUFFER, $gl.COLOR_ATTACHMENT0, $gl.TEXTURE_2D, this.texture.id, 0);
      data = new Uint8Array(this.texture.realWidth * this.texture.realHeight * 4);
      $gl.readPixels(0, 0, this.texture.realWidth, this.texture.realHeight, $gl.RGBA, $gl.UNSIGNED_BYTE, data);
      $gl.deleteFramebuffer(framebuffer);
      canvas = document.createElement('canvas');
      canvas.width = this.texture.realWidth;
      canvas.height = this.texture.realHeight;
      this.context = canvas.getContext('2d');
      imageData = this.context.createImageData(this.texture.realWidth, this.texture.realHeight);
      imageData.data.set(data);
      this.context.putImageData(imageData, 0, 0);
      return this.image = canvas;
    };

    Bitmap.prototype.createWithImage = function(image) {
      this.width = image.width;
      this.height = image.height;
      this.bounds.set(0, 0, this.width, this.height);
      this.image = image;
      this.context = image.getContext("2d");
      this.context.imageSmoothingEnabled = false;
      this.context.webkitImageSmoothingEnabled = false;
      this.context.mozImageSmoothingEnabled = false;
      return this.context.bitmap = this;
    };

    Bitmap.prototype.createFromImage = function(image, flipV) {
      this.createWithSize(image.width, image.height);
      if (flipV) {
        this.context.translate(0, this.height);
        this.context.scale(1, -1);
      }
      this.context.drawImage(image, 0, 0);
      if (flipV) {
        return this.context.setTransform(1, 0, 0, 1, 0, 0);
      }
    };

    Bitmap.prototype.createFromFile = function(filePath, languageCode) {
      var onerror, request, src;
      this.immutable = true;
      this.filePath = filePath;
      this.languageCode = languageCode;
      this.font = Font.createDefault();
      this.image = new Image();
      this.image.onload = (function(_this) {
        return function() {
          var canvas;
          if (!_this.immutable) {
            canvas = document.createElement("canvas");
            canvas.width = _this.image.width;
            canvas.height = _this.image.height;
            _this.context = canvas.getContext("2d");
            _this.context.imageSmoothingEnabled = false;
            _this.context.webkitImageSmoothingEnabled = false;
            _this.context.mozImageSmoothingEnabled = false;
            _this.context.bitmap = _this;
            _this.width = _this.image.width;
            _this.height = _this.image.height;
            _this.bounds.set(0, 0, _this.width, _this.height);
            _this.context.clearRect(0, 0, _this.width, _this.height);
            _this.context.drawImage(_this.image, 0, 0);
            _this.image = canvas;
          } else {
            _this.width = _this.image.width;
            _this.height = _this.image.height;
            _this.bounds.set(0, 0, _this.width, _this.height);
          }
          _this.loaded = true;
          if (_this.immutable) {
            _this.texture = new Texture2D(_this.image);
            _this.texture.sequenceNumber = _this.sequenceNumber;
            _this.image = null;
            _this.context = null;
          }
          if (_this.onload != null) {
            return _this.onload();
          }
        };
      })(this);
      onerror = (function(_this) {
        return function() {
          var canvas;
          if (filePath && filePath.startsWith("data:")) {
            console.log("CANNOT LOAD IMAGE");
          } else {
            console.log("CANNOT LOAD IMAGE: " + filePath);
          }
          canvas = document.createElement("canvas");
          canvas.width = 32;
          canvas.height = 32;
          _this.context = canvas.getContext("2d");
          _this.context.imageSmoothingEnabled = false;
          _this.context.webkitImageSmoothingEnabled = false;
          _this.context.mozImageSmoothingEnabled = false;
          _this.context.bitmap = _this;
          _this.width = 32;
          _this.height = 32;
          _this.error = true;
          _this.context.clearRect(0, 0, _this.width, _this.height);
          _this.image = canvas;
          _this.loaded = true;
          if (_this.onerror != null) {
            return _this.onerror();
          }
        };
      })(this);
      this.image.onerror = onerror;
      src = filePath;
      if (filePath.startsWith("data:")) {
        src = filePath;
      } else {
        src = gs.Application.getPlatformSpecificGraphicsFilePath(filePath, languageCode);
      }
      if (gs.Platform.isSafariMobile && gs.Platform.isCordova) {
        if ($PARAMS.needsPreparation) {
          return gs.Application.loadData(src, (function(_this) {
            return function(data) {
              var reader;
              data = GS.DataPreparer.prepare(data);
              reader = new FileReader();
              reader.onload = function(e) {
                return _this.image.src = e.target.result;
              };
              reader.onerror = onerror;
              return reader.readAsDataURL(new Blob([data]));
            };
          })(this), (function(_this) {
            return function(e) {
              return onerror();
            };
          })(this));
        } else {
          return gs.Application.loadData(src, (function(_this) {
            return function(dataURL) {
              return _this.image.src = dataURL;
            };
          })(this), (function(_this) {
            return function(e) {
              return onerror();
            };
          })(this), 1);
        }
      } else if ($PARAMS.needsPreparation && !filePath.startsWith("data:")) {
        request = new XMLHttpRequest();
        request.open('GET', src, true);
        request.responseType = 'arraybuffer';
        request.onload = (function(_this) {
          return function() {
            var buffer, reader;
            buffer = GS.DataPreparer.prepare(request.response);
            reader = new FileReader();
            reader.onload = function(e) {
              return _this.image.src = e.target.result;
            };
            reader.onerror = onerror;
            return reader.readAsDataURL(new Blob([buffer]));
          };
        })(this);
        request.onerror = onerror;
        return request.send();
      } else {
        return this.image.src = src;
      }
    };

    Bitmap.prototype.createWithSize = function(width, height) {
      this.font = Font.createDefault();
      this.context = null;
      this.image = document.createElement("canvas");
      this.width = width;
      this.height = height;
      this.bounds.set(0, 0, this.width, this.height);
      this.image.width = this.width;
      this.image.height = this.height;
      this.context = this.image.getContext("2d");
      this.context.imageSmoothingEnabled = false;
      this.context.webkitImageSmoothingEnabled = false;
      this.context.mozImageSmoothingEnabled = false;
      this.context.bitmap = this;
      return this.clear();
    };


    /**
    * Disposes the bitmap and frees the memory. Should be called
    * if the bitmap is no longer needed to free resources which are
    * not handled by the garbage collector.
    *
    * @method dispose
     */

    Bitmap.prototype.dispose = function() {
      this.disposed = true;
      if (this.texture != null) {
        this.texture.dispose();
        this.texture = null;
      }
      this.image = null;
      return this.context = null;
    };


    /**
    * Checks if the pixel at the specified position has an alpha-value other than 0.
    *
    * @method isPixelSet
    * @param {number} x - The x-coordinate of the pixel to check
    * @param {number} y - The y-coordinate of the pixel to check
    * @return {boolean} If <b>true</b> the pixel is set(alpha > 0). Otherwise <b>false</b>
     */

    Bitmap.prototype.isPixelSet = function(x, y) {
      this.makeMutable();
      if (!this.imageData || this.imageData.sequenceNumber !== this.sequenceNumber) {
        this.imageData = this.context.getImageData(0, 0, this.width, this.height);
        this.imageData.sequenceNumber = this.sequenceNumber;
      }
      return this.imageData.data[(y * this.width + x) * 4 + 3] !== 0;
    };


    /**
    * Gets the color of the pixel at the specified position.
    *
    * @method getPixel
    * @param {number} x - The x-coordinate of the pixel to get
    * @param {number} y - The y-coordinate of the pixel to get
    * @return {gs.Color} The color of the pixel.
     */

    Bitmap.prototype.getPixel = function(x, y) {
      this.makeMutable();
      if (!this.imageData || this.imageData.sequenceNumber !== this.sequenceNumber) {
        this.imageData = this.context.getImageData(0, 0, this.width, this.height);
        this.imageData.sequenceNumber = this.sequenceNumber;
      }
      return new gs.Color(this.imageData.data[(y * this.width + x) * 4], this.imageData.data[(y * this.width + x) * 4 + 1], this.imageData.data[(y * this.width + x) * 4 + 2], this.imageData.data[(y * this.width + x) * 4 + 3]);
    };


    /**
    * Gets the image data of the specified rectangle-area.
    *
    * @method getImageData
    * @param {number} x - The x-coordinate of the rectangle.
    * @param {number} y - The x-coordinate of the rectangle.
    * @param {number} width - The width of the rectangle.
    * @param {number} height - The height of the rectangle.
    * @return {ImageData} The image data of the specified rectangle-area.
     */

    Bitmap.prototype.getImageData = function(x, y, width, height) {
      this.makeMutable();
      return this.context.getImageData(x, y, width, height);
    };


    /**
    * Puts the image data at a specified point.
    *
    * @method setImageData
    * @param {ImageData} imageData - The image data to put.
    * @param {number} x - The x-coordinate of the point.
    * @param {number} y - The y-coordinate of the point.
     */

    Bitmap.prototype.setImageData = function(imageData, x, y) {
      this.makeMutable();
      this.context.putImageData(imageData, x, y);
      return this.dirtyRect.expand(0, 0, this.width, this.height);
    };


    /**
    * Scales the bitmap by the specified factor.
    *
    * @method scale
    * @param {number} factor - The scale factor.
     */

    Bitmap.prototype.scale = function(factor) {
      var context, image;
      image = document.createElement("canvas");
      image.width = Math.floor(this.width * factor);
      image.height = Math.floor(this.height * factor);
      context = image.getContext("2d");
      context.imageSmoothingEnabled = false;
      context.webkitImageSmoothingEnabled = false;
      context.mozImageSmoothingEnabled = false;
      context.drawImage(this.image, 0, 0, this.width, this.height, 0, 0, image.width, image.height);
      this.image = image;
      this.width = image.width;
      this.height = image.height;
      this.context = context;
      return this.sequenceNumber++;
    };


    /**
    * Clears the bitmap.
    *
    * @method clear
     */

    Bitmap.prototype.clear = function() {
      this.clearRect(0, 0, this.width, this.height);
      return this.sequenceNumber++;
    };


    /**
    * Clears the specified rectangle-area.
    *
    * @method clearRect
    * @param {number} x - The x-coordinate of the rectangle.
    * @param {number} y - The x-coordinate of the rectangle.
    * @param {number} width - The width of the rectangle.
    * @param {number} height - The height of the rectangle.
     */

    Bitmap.prototype.clearRect = function(x, y, w, h) {
      this.makeMutable();
      if (y != null) {
        this.context.clearRect(x, y, w, h);
        this.dirtyRect.expand(x, y, w, h);
      } else {
        this.context.clearRect(x.x, x.y, x.width, x.height);
        this.dirtyRect.expand(x.x, x.y, x.width, x.height);
      }
      return this.sequenceNumber++;
    };


    /**
    * Fills a rectangle-area with the specified color.
    *
    * @method fillRect
    * @param {number} x - The x-coordinate of the rectangle.
    * @param {number} y - The x-coordinate of the rectangle.
    * @param {number} width - The width of the rectangle.
    * @param {number} height - The height of the rectangle.
    * @param {gs.Color} color - The fill color.
     */

    Bitmap.prototype.fillRect = function(x, y, w, h, color) {
      this.makeMutable();
      if (w != null) {
        this.context.fillStyle = "rgba({0}, {1}, {2}, {3})".format(color.red, color.green, color.blue, color.alpha);
        this.context.fillRect(x, y, w, h);
        this.dirtyRect.expand(x, y, w, h);
      } else {
        this.context.fillStyle = "rgba({0}, {1}, {2}, {3})".format(y.red, y.green, y.blue, y.alpha);
        this.context.fillRect(x.x, x.y, x.width, x.height);
        this.dirtyRect.expand(x.x, x.y, x.width, x.height);
      }
      return this.sequenceNumber++;
    };


    /**
    * Fills a rectangle-area with a linear-gradient.
    *
    * @method gradientFillRect
    * @param {number} x - The x-coordinate of the rectangle.
    * @param {number} y - The x-coordinate of the rectangle.
    * @param {number} width - The width of the rectangle.
    * @param {number} height - The height of the rectangle.
    * @param {gs.Color} color1 - The first gradient color.
    * @param {gs.Color} color2 - The second gradient color.
    * @param {boolean} vertical - If <b>true</b> the gradient is vertical. Otherwise horizontal.
     */

    Bitmap.prototype.gradientFillRect = function(x, y, w, h, color1, color2, vertical) {
      var gradient;
      this.makeMutable();
      if (color1 != null) {
        gradient = vertical != null ? this.context.createLinearGradient(x, y, x, y + h) : this.context.createLinearGradient(x, 0, x + w, h);
        gradient.addColorStop(0, color1.toString());
        gradient.addColorStop(1, color2.toString());
        this.context.fillStyle = gradient;
        this.context.fillRect(x, y, w, h);
        this.dirtyRect.expand(x, y, w, h);
      } else {
        gradient = h != null ? this.context.createLinearGradient(0, this.width, 0, this.height) : this.context.createLinearGradient(0, this.height, this.width, 0);
        gradient.addColorStop(0, y.toString());
        gradient.addColorStop(1, w.toString());
        this.context.fillStyle = gradient;
        this.context.fillRect(x.x, x.y, x.width, x.height);
        this.dirtyRect.expand(x.x, x.y, x.width, x.height);
      }
      return this.sequenceNumber++;
    };


    /**
    * Blits/Draws the specified bitmap on this bitmap.
    *
    * @method blt
    * @param {number} x - The x-coordinate of the position.
    * @param {number} y - The y-coordinate of the position.
    * @param {gs.Bitmap} srcBitmap - The bitmap to blit/draw.
    * @param {gs.Rect} srcRect- The source-rectangle. Only that area will be blitted/drawn.
    * @param {number} opacity - The opacity
     */

    Bitmap.prototype.blt = function(x, y, srcBitmap, srcRect, opacity) {
      this.makeMutable();
      srcBitmap.makeMutable();
      this.context.globalAlpha = opacity != null ? opacity / 255 : 1.0;
      this.context.drawImage(srcBitmap.image, srcRect.x, srcRect.y, Math.min(srcRect.width, srcBitmap.width), Math.min(srcRect.height, srcBitmap.height), x, y, srcRect.width, srcRect.height);
      this.sequenceNumber++;
      return this.dirtyRect.expand(x, y, srcRect.width, srcRect.height);
    };


    /**
    * Blits/Draws the specified bitmap on this bitmap and stretches it to fit into the specified dest-rect.
    *
    * @method stretchBlt
    * @param {gs.Rect} destRect - The destination-rectangle for the bitmap to draw. If smaller/larger than the actual bitmap-size then it is automatically scaled.
    * @param {gs.Bitmap} srcBitmap - The bitmap to blit/draw.
    * @param {gs.Rect} srcRect- The source-rectangle. Only that area will be blitted/drawn.
    * @param {number} opacity - The opacity
     */

    Bitmap.prototype.stretchBlt = function(destRect, srcBitmap, srcRect, opacity) {
      this.makeMutable();
      srcBitmap.makeMutable();
      this.context.globalAlpha = opacity != null ? opacity / 255 : 1.0;
      this.context.drawImage(srcBitmap.image, srcRect.x, srcRect.y, Math.min(srcRect.width, srcBitmap.width), Math.min(srcRect.height, srcBitmap.height), destRect.x, destRect.y, destRect.width, destRect.height);
      this.sequenceNumber++;
      return this.dirtyRect.expand(destRect.x, destRect.y, destRect.width, destRect.height);
    };

    Bitmap.prototype.changeColor = function(color) {
      this.makeMutable();
      this.context.globalCompositeOperation = "source-atop";
      this.context.fillStyle = color.toString();
      this.context.fillRect(0, 0, this.width, this.height);
      this.context.globalCompositeOperation = "source-over";
      this.dirtyRect.expand(0, 0, this.width, this.height);
      return this.sequenceNumber++;
    };


    /**
    * Changes the HUE of the bitmap. Once the HUE was changed it cannot changed back by
    * passing 0 for hue or something. The HUE change is always executed on the current bitmap-data.
    *
    * @method changeHue
    * @param {number} hue - The HUE in degrees.
     */

    Bitmap.prototype.changeHue = function(hue) {
      var data, hsl, i, imageData, j, ref, rgb;
      this.makeMutable();
      imageData = this.context.getImageData(0, 0, this.width, this.height);
      data = imageData.data;
      hsl = null;
      rgb = null;
      for (i = j = 0, ref = data.length; j < ref; i = j += 4) {
        hsl = Color.rgbToHsl(data[i], data[i + 1], data[i + 2]);
        hsl.h = hsl.h + hue;
        hsl.h = hsl.h > 359 ? hsl.h - 360 : hsl.h;
        rgb = Color.hslToRgb(hsl.h, hsl.s, hsl.l);
        data[i] = rgb.r;
        data[i + 1] = rgb.g;
        data[i + 2] = rgb.b;
      }
      this.sequenceNumber++;
      this.dirtyRect.expand(0, 0, this.width, this.height);
      return this.context.putImageData(imageData, 0, 0);
    };

    Bitmap.prototype.drawBitmapFontText = function(x, y, w, h, text, alignment) {
      var code, color, compositeContext, fontData, glyph, i, j, kerning, lastCode, lastGlyph, ref, size;
      if (h == null) {
        y = x.y;
        w = x.width;
        h = x.height;
        x = x.x;
      }
      this.dirtyRect.expand(x, y, w, h);
      if (this.currentFont !== this.font) {
        this.currentFont = this.font;
      }
      size = this.font.measureText(text);
      color = this.font.color.toString();
      lastGlyph = null;
      lastCode = 0;
      compositeContext = Font.compositeContext;
      switch (alignment) {
        case 0:
          x = x;
          break;
        case 1:
          x = Math.floor(x + (w - size.width) / 2);
          break;
        case 2:
          x = Math.floor(x + w - size.width);
      }
      fontData = this.font.bitmapFontData;
      y = y + Math.floor((h - size.height) / 2);
      this.context.globalAlpha = this.font.color.alpha * 0.00392156862745;
      for (i = j = 0, ref = text.length; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
        code = text.charCodeAt(i);
        glyph = this.font.bitmapFontData.charData[code];
        if (!glyph) {
          continue;
        }
        if (lastGlyph) {
          kerning = fontData.kerningData[lastCode];
          if (kerning && kerning[code]) {
            x += kerning[code];
          }
        }
        if (this.font.color.equals(Color.WHITE)) {
          if (i === 0) {
            this.context.drawImage(this.font.bitmaps[glyph.page].image, glyph.x, glyph.y, glyph.width, glyph.height, x, y + glyph.yoffset, glyph.width, glyph.height);
          } else {
            this.context.drawImage(this.font.bitmaps[glyph.page].image, glyph.x, glyph.y, glyph.width, glyph.height, x + glyph.xoffset, y + glyph.yoffset, glyph.width, glyph.height);
          }
        } else {
          compositeContext.drawImage(this.font.bitmaps[glyph.page].image, glyph.x, glyph.y, glyph.width, glyph.height, 0, 0, glyph.width, glyph.height);
          compositeContext.globalCompositeOperation = "source-atop";
          compositeContext.fillStyle = color;
          compositeContext.fillRect(0, 0, glyph.width, glyph.height);
          compositeContext.globalCompositeOperation = "source-over";
          this.context.drawImage(Font.compositeBuffer, 0, 0, glyph.width, glyph.height, x + glyph.xoffset, y + glyph.yoffset, glyph.width, glyph.height);
          compositeContext.clearRect(0, 0, glyph.width, glyph.height);
        }
        lastGlyph = glyph;
        lastCode = code;
        x = x + glyph.xadvance;
      }
      this.sequenceNumber++;
      return null;
    };

    Bitmap.prototype.drawTextF = function(x, y, w, h, text, hAlign, vAlign) {
      var borderColor, color, lineThickness, shadowColor, size;
      this.makeMutable();
      if (this.font.bitmaps.length > 0) {
        return this.drawBitmapFontText(x, y, w, h, text, hAlign, vAlign);
      }
      hAlign = hAlign != null ? hAlign : 0;
      vAlign = vAlign != null ? vAlign : 1;
      if (h == null) {
        y = x.y;
        w = x.width;
        h = x.height;
        x = x.x;
      }
      this.dirtyRect.expand(x, y, w, h);
      this.context.font = this.font.toString();
      this.context.textBaseline = "top";
      this.currentFont = this.font;
      size = this.font.measureText(text);
      borderColor = this.font.borderColor.toFillStyle();
      shadowColor = this.font.shadowColor.toString();
      color = this.font.color.toString();
      switch (hAlign) {
        case 0:
          x = x;
          break;
        case 1:
          x = Math.floor(x + (w - size.width) / 2);
          break;
        case 2:
          x = Math.floor(x + w - size.width);
      }
      switch (vAlign) {
        case 0:
          y = y;
          break;
        case 1:
          y = Math.floor(y + (h - size.height) / 2);
          break;
        case 2:
          y = Math.floor(y + h - size.height);
      }
      lineThickness = this.font.size / 10;
      if (this.font.shadow) {
        this.context.shadowColor = shadowColor;
        this.context.shadowOffsetX = this.font.shadowOffsetX;
        this.context.shadowOffsetY = this.font.shadowOffsetY;
        this.context.shadowBlur = 2;
      } else {
        this.context.shadowColor = "";
        this.context.shadowOffsetX = 0;
        this.context.shadowOffsetY = 0;
        this.context.shadowBlur = 0;
      }
      if (this.font.border) {
        this.context.strokeStyle = borderColor;
        this.context.fillStyle = color;
        this.context.lineWidth = this.font.borderSize;
        this.context.lineJoin = "round";
        this.context.lineCap = "round";
        this.context.strokeText(text, x, y, w);
        this.context.fillText(text, x, y, w);
        if (this.font.underline) {
          this.context.strokeRect(x + this.font.borderSize, y + size.height - this.font.descent, size.width - this.font.borderSize * 2, lineThickness);
          this.context.fillRect(x + this.font.borderSize, y + size.height - this.font.descent, size.width - this.font.borderSize * 2, lineThickness);
        }
        if (this.font.strikeThrough) {
          this.context.strokeRect(x + this.font.borderSize, y + size.height / 2, size.width - this.font.borderSize * 2, lineThickness);
          this.context.fillRect(x + this.font.borderSize, y + size.height / 2, size.width - this.font.borderSize * 2, lineThickness);
        }
        this.context.lineWidth = 1;
      } else {
        this.context.fillStyle = color;
        this.context.lineWidth = 1;
        this.context.fillText(text, x, y, w);
        if (this.font.underline) {
          this.context.fillRect(x, y + size.height - lineThickness - 1 - this.font.descent, size.width, 2);
        }
        if (this.font.strikeThrough) {
          this.context.fillRect(x, y + size.height / 2, size.width, 2);
        }
      }
      this.sequenceNumber++;
      return null;
    };


    /**
    * Draws the specified text on the bitmap.
    *
    * @method drawText
    * @param {number} x - The x-coordinate of the text's position.
    * @param {number} y - The y-coordinate of the text's position.
    * @param {number} w - The width of text-box used to align the text.
    * @param {number} h - The height of text-box used to align the text.
    * @param {string} text - The text to draw.
    * @param {gs.Alignment} [hAlign=gs.Alignment.LEFT] - The horizontal alignment of the text.
    * @param {gs.Alignment} [vAlign=gs.Alignment.TOP] - The vertical alignment of the text.
     */

    Bitmap.prototype.drawText = function(x, y, w, h, text, hAlign, vAlign) {
      return this.drawTextF(x, y, w, h, text, hAlign, vAlign);
    };


    /**
    * Measures the size of the specified text.
    *
    * @method textSize
    * @param {string} text - The text to measure
    * @return {gs.Size} size - The size of the specified text.
     */

    Bitmap.prototype.textSize = function(text) {
      var size;
      size = this.font.measureText(text);
      if (this.font.border) {
        size.width += 0;
        size.height -= 0;
      } else if (this.font.shadow) {
        size.width += 0;
        size.height += 0;
      }
      return size;
    };

    Bitmap.prototype._fontHeight = function(text) {
      var block, body, div, result, span;
      span = $('<span>' + text + '</span>').css({
        fontFamily: this.font.name,
        fontSize: this.font.size.toString() + "pt",
        fontWeight: this.font.bold ? "bold" : "normal"
      });
      block = $('<div style="display: inline-block; width: 1px; height: 0px;"></div>');
      div = $('<div></div>');
      div.append(span, block);
      body = $('body');
      body.append(div);
      try {
        result = {};
        block.css({
          verticalAlign: 'baseline'
        });
        result.ascent = block.offset().top - span.offset().top;
        block.css({
          verticalAlign: 'bottom'
        });
        result.height = block.offset().top - span.offset().top;
        result.descent = result.height - result.ascent;
      } finally {
        div.remove();
      }
      return result;
    };

    return Bitmap;

  })();

  window.Bitmap = Bitmap;

  gs.Bitmap = Bitmap;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var GraphicObject;

  GraphicObject = (function() {
    GraphicObject.instanceCount = 0;

    GraphicObject.prototype.onDataBundleRestore = function(data, context) {};

    Object.defineProperty(GraphicObject.prototype, "viewport", {
      set: function(viewport) {
        if (this.viewport_ === viewport) {
          return;
        }
        if (this.managed) {
          if (this.viewport_ != null) {
            this.viewport_.removeGraphicObject(this);
          } else {
            Graphics.removeGraphicObject(this);
          }
        }
        this.viewport_ = viewport;
        if (this.managed) {
          if (this.viewport_ == null) {
            return Graphics.addGraphicObject(this);
          } else {
            return this.viewport_.addGraphicObject(this);
          }
        }
      },
      get: function() {
        return this.viewport_;
      }
    });

    Object.defineProperty(GraphicObject.prototype, "z", {
      set: function(z) {
        var ref;
        if (z !== this.z_) {
          this.z_ = z;
          return (ref = this.viewport) != null ? ref.needsSort = true : void 0;
        }
      },
      get: function() {
        return this.z_;
      }
    });


    /**
    * The base class of all graphical object. Each graphic object is automatically
    * rendered and sorted by it z-value.
    *
    * @namespace gs
    * @class GraphicObject
    * @memberof gs
    * @constructor
     */

    function GraphicObject(viewport, managed) {
      this.viewport_ = viewport;
      this.managed = managed != null ? managed : true;

      /*
      * A factor controlling how much influence the viewport's transformation has on the graphic object's tranformation. The
      * default value is 1.0. A value of 0.0 means "No Influence". In that case, the viewport's transformation doesn't affect
      * graphic object at all.
      * @property viewportInfluence
      * @default 1.0
      * @type number
       */

      /**
      * The z-index controls rendering-order/image-overlapping. An object with a smaller z-index is rendered
      * before an object with a larger index. For example: To make sure a graphic object is always on top of the screen, it
      * should have the largest z-index of all graphic objects.
      * @property z
      * @type number
       */
      this.z = 0;

      /**
      * The current index in the global rendering list. This property is read-only and automatically
      * calculated at runtime depending on z-property.
      * @property index
      * @readOnly
      * @type number
       */
      this.index = 0;

      /**
      * Fired if the graphic object's current index in the global rendering list was changed.
      * @property onIndexChange
      * @default null
      * @type Function
       */
      this.onIndexChange = null;

      /**
      * The x-coordinate of the origin.
      * @property ox
      * @type number
       */
      this.ox = 0;

      /**
      * The y-coordinate of the origin.
      * @property oy
      * @type number
       */
      this.oy = 0;

      /**
      * The transformation matrix of the graphic object.
      * @property transform
      * @type mat2d
       */
      this.transform = mat2d.create();

      /**
      * The anchor point. For example: An anchor-point with 0,0 rotates the object around its top-left corner
      * but with an 0.5, 0.5 anchor-point the object rotates around its center. An anchor-point of 1,1
      * rotates the object around its lower-right corner. It has not effect on the object's positioning. For positioning, take
      * a look at <b>positionAnchor</b> property.
      *
      * @property anchor
      * @type gs.Point
       */
      this.anchor = new gs.Point(0, 0);

      /**
      * The position anchor point. For example: An anchor-point with 0,0 places the object with its top-left corner
      * at its position but with an 0.5, 0.5 anchor-point the object will be placed with its center. An anchor-point of 1,1
      * will place the object with its lower-right corner. It has not effect on the object's rotation/zoom anchor. For that, take
      * a look at <b>anchor</b> property.
      *
      * @property positionAnchor
      * @type gs.Point
       */
      this.positionAnchor = new gs.Point(0, 0);

      /**
      * Indicates if the object is rendered.
      * @property visible
      * @type boolean
       */
      this.visible = true;

      /**
      * Indicates if the object is disposed. A disposed object is not rendered.
      * @property disposed
      * @type boolean
       */
      this.disposed = false;
      this.position = vec2.create();

      /**
      * The effect settings for different kind of visual effect like wobble, etc.
      * @property effects
      * @type gs.EffectCollection
       */
      this.effects = new gs.EffectCollection();

      /**
      * The unique identifier of the graphic object.
      * @property id
      * @type number
       */
      this.id = GraphicObject.instanceCount;
      GraphicObject.instanceCount++;
      this.initialize();
    }


    /**
    * Initializes the graphic object.
    *
    * @method initialize
    * @protected
     */

    GraphicObject.prototype.initialize = function() {
      if (this.managed) {
        if (this.viewport == null) {
          Graphics.addGraphicObject(this);
          return this.viewport = Graphics.viewport;
        } else {
          return this.viewport.addGraphicObject(this);
        }
      }
    };


    /**
    * Disposes the graphic object.
    *
    * @method dispose
     */

    GraphicObject.prototype.dispose = function() {
      this.disposed = true;
      if (this.managed) {
        if (this.viewport != null) {
          return this.viewport.removeGraphicObject(this);
        } else {
          return Graphics.removeGraphicObject(this);
        }
      }
    };

    GraphicObject.prototype.updateTransform = function(x, y, width, height) {
      mat2d.identity(this.transform);
      mat2d.mul(this.transform, this.transform, this.viewport.transform);
      if (this.angle !== 0 || this.zoomX !== 1 || this.zoomY !== 1) {
        this.position[0] = x - (width * (this.zoomX - 1.0) * this.anchor.x);
        this.position[1] = y - (height * (this.zoomY - 1.0) * this.anchor.y);
        mat2d.translate(this.transform, this.transform, this.position);
        if (this.angle !== 0) {
          this.position[0] = width * this.zoomX * this.anchor.x;
          this.position[1] = height * this.zoomY * this.anchor.y;
          mat2d.translate(this.transform, this.transform, this.position);
          mat2d.rotate(this.transform, this.transform, Math.degreesToRadians(this.angle));
          this.position[0] = -this.position[0];
          this.position[1] = -this.position[1];
          mat2d.translate(this.transform, this.transform, this.position);
        }
        this.position[0] = width * this.zoomX;
        this.position[1] = height * this.zoomY;
        return mat2d.scale(this.transform, this.transform, this.position);
      } else {
        this.position[0] = x - width * this.positionAnchor.x;
        this.position[1] = y - height * this.positionAnchor.y;
        mat2d.translate(this.transform, this.transform, this.position);
        this.position[0] = width;
        this.position[1] = height;
        return mat2d.scale(this.transform, this.transform, this.position);
      }
    };


    /**
    * Updates the graphic object.
    *
    * @method update
     */

    GraphicObject.prototype.update = function() {};


    /**
    * Draws the graphic object.
    *
    * @method draw
    * @abstract
     */

    GraphicObject.prototype.draw = function() {};

    return GraphicObject;

  })();

  window.GraphicObject = GraphicObject;

  gs.GraphicObject = GraphicObject;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var Video;

  Video = (function() {
    Video.fromDataBundle = function(data, context) {
      var result;
      result = null;
      if (data.filePath) {
        result = new gs.Video(data.filePath, null);
        if (data.isPlaying) {
          result.videoElement.currentTime = data.currentTime;
          result.volume = data.volume;
          result.playbackRate = data.playbackRate;
          result.loop = data.loop;
          result.play();
        }
      }
      return result;
    };

    Video.toDataBundle = function(object, context) {
      var r, ref;
      r = {
        filePath: object.filePath,
        loop: object.loop,
        isPlaying: object.isPlaying,
        currentTime: (ref = object.videoElement) != null ? ref.currentTime : void 0,
        volume: object.volume,
        playbackRate: object.playbackRate
      };
      return r;
    };


    /**
    * Allows to display videos on screen. The supported video formats depends on the
    * platform. Only WEBM is supported right now.
    *
    *
    * @namespace gs
    * @class Video
    * @memberof gs
    * @constructor
    * @param {string} filePath - The path to the video resource.
    * @param {string} [languageCode=null] - The language code to load the video resource depending on language.
     */

    function Video(filePath, languageCode) {
      var request;
      this.languageCode = languageCode;
      this.lastCurrentTime = -1;

      /**
      * The path to the video resource.
      * @property filePath
      * @type string
      * @readOnly
       */
      this.filePath = filePath;

      /**
      * Indicates if the video is loaded.
      * @property loaded
      * @type boolean
      * @readOnly
       */
      this.loaded = false;

      /**
      * Indicates if the video is disposed.
      * @property disposed
      * @type boolean
      * @readOnly
       */
      this.disposed = false;
      this.texture = null;

      /**
      * The video's width.
      * @property width
      * @type number
      * @readOnly
       */
      this.width = 0;

      /**
      * The video's height.
      * @property height
      * @type number
      * @readOnly
       */
      this.height = 0;
      this.sequenceNumber = 0;

      /**
      * Callback function called if an error occurred.
      * @property onError
      * @type Function
       */
      this.onError = null;

      /**
      * Callback function called if an the video finished playing.
      * @property onEnded
      * @type Function
       */
      this.onEnded = null;

      /**
      * Callback function called if the video is ready to play.
      * @property onCanPlay
      * @type Function
       */
      this.onCanPlay = null;
      this.videoElement = document.createElement("video");
      this.videoElement.autoplay = false;
      this.videoElement.preload = "auto";
      jQuery(this.videoElement).attr("playsinline", true);
      if (gs.Platform.isSafariMobile && !gs.Platform.isCordova) {
        jQuery(this.videoElement).attr("muted", true);
      }
      jQuery(this.videoElement).attr("webkit-playsinline", true);
      this.videoElement.playsInline = true;
      this.videoElement.oncanplaythrough = (function(_this) {
        return function() {
          _this.loaded = true;
          _this.texture = new Texture2D(_this.videoElement.videoWidth, _this.videoElement.videoHeight, false, true);
          _this.texture.sequenceNumber = 0;
          _this.width = _this.videoElement.videoWidth;
          _this.height = _this.videoElement.videoHeight;
          _this.sequenceNumber = 0;
          return typeof _this.onCanPlay === "function" ? _this.onCanPlay() : void 0;
        };
      })(this);
      this.videoElement.onerror = (function(_this) {
        return function(e) {
          _this.loaded = true;
          _this.error = true;
          console.log("ERROR: ");
          console.log(e);
          return typeof _this.onError === "function" ? _this.onError() : void 0;
        };
      })(this);
      this.videoElement.onended = (function(_this) {
        return function(e) {
          return typeof _this.onEnded === "function" ? _this.onEnded() : void 0;
        };
      })(this);
      filePath = gs.Application.getPlatformSpecificVideoFilePath(filePath, languageCode);
      if (gs.Platform.isSafariMobile && gs.Platform.isCordova) {
        if ($PARAMS.needsPreparation) {
          gs.Application.loadData(filePath, (function(_this) {
            return function(data) {
              var reader;
              data = GS.DataPreparer.prepare(data);
              reader = new FileReader();
              reader.onload = function(e) {
                if (gs.Platform.isSafariMobile) {
                  _this.videoElement.src = e.target.result.replace("data:", "data:video/webm");
                } else {
                  _this.videoElement.src = e.target.result.replace("data:", "data:video/mp4");
                }
                return _this.videoElement.load();
              };
              reader.onerror = onerror;
              return reader.readAsDataURL(new Blob([data]));
            };
          })(this), (function(_this) {
            return function(e) {
              return _this.videoElement.onerror(e);
            };
          })(this));
        } else {
          gs.Application.loadData(filePath, (function(_this) {
            return function(data) {
              _this.videoElement.src = data;
              return _this.videoElement.load();
            };
          })(this), (function(_this) {
            return function(e) {
              return _this.videoElement.onerror(e);
            };
          })(this), 1);
        }
      } else if ($PARAMS.needsPreparation) {
        request = new XMLHttpRequest();
        request.open('GET', filePath, true);
        request.responseType = 'arraybuffer';
        request.onload = (function(_this) {
          return function() {
            var buffer, reader;
            buffer = GS.DataPreparer.prepare(request.response);
            reader = new FileReader();
            reader.onload = function(e) {
              if (gs.Platform.isSafari || gs.Platform.isEdge) {
                return _this.videoElement.src = e.target.result.replace("data:", "data:video/mp4");
              } else {
                return _this.videoElement.src = e.target.result.replace("data:", "data:video/webm");
              }
            };
            reader.onerror = onerror;
            return reader.readAsDataURL(new Blob([buffer]));
          };
        })(this);
        request.onerror = (function(_this) {
          return function(e) {
            console.log("ERROR: ");
            console.log(e);
            return typeof _this.onError === "function" ? _this.onError() : void 0;
          };
        })(this);
        request.send();
      } else {
        this.videoElement.src = filePath;
        this.videoElement.load();
      }
    }


    /**
    * Indicates if the video is currently playing.
    * @property isPlaying
    * @type boolean
    * @readOnly
     */

    Object.defineProperty(Video.prototype, "isPlaying", {
      get: function() {
        var ref;
        return !((ref = this.videoElement) != null ? ref.paused : void 0);
      }
    });


    /**
    * The volume of the video's audio. Goes from 0 to 100 percent.
    * @property volume
    * @type number
     */

    Object.defineProperty(Video.prototype, "volume", {
      get: function() {
        var ref;
        return (ref = this.videoElement) != null ? ref.volume : void 0;
      },
      set: function(volume) {
        var ref;
        return (ref = this.videoElement) != null ? ref.volume = volume : void 0;
      }
    });


    /**
    * The play-back rate in percent.
    * @property playbackRate
    * @type number
     */

    Object.defineProperty(Video.prototype, "playbackRate", {
      get: function() {
        var ref;
        return (ref = this.videoElement) != null ? ref.playbackRate : void 0;
      },
      set: function(playbackRate) {
        var ref;
        return (ref = this.videoElement) != null ? ref.playbackRate = playbackRate : void 0;
      }
    });


    /**
    * Indicates if the video repeats after finished playing.
    * @property loop
    * @type boolean
     */

    Object.defineProperty(Video.prototype, "loop", {
      get: function() {
        var ref;
        return (ref = this.videoElement) != null ? ref.loop : void 0;
      },
      set: function(v) {
        var ref;
        return (ref = this.videoElement) != null ? ref.loop = v : void 0;
      }
    });


    /**
    * Disposes the video and frees resources/memory. Should be called if the video is no
    * longer needed.
    *
    * @method dispose
     */

    Video.prototype.dispose = function() {
      var ref;
      if (!this.disposed) {
        if ((ref = this.texture) != null) {
          ref.dispose();
        }
        this.disposed = true;
        this.videoElement.oncanplaythrough = null;
        this.videoElement.onerror = null;
        this.videoElement.onend = null;
        this.videoElement.pause();
        this.videoElement.src = "";
        return delete this.videoElement;
      }
    };


    /**
    * Disposes the video and frees resources/memory. Should be called if the video is no
    * longer needed.
    *
    * @method play
     */

    Video.prototype.play = function() {
      var ref;
      if (!this.error) {
        return (ref = this.videoElement) != null ? ref.play() : void 0;
      }
    };


    /**
    * Pauses the video.
    *
    * @method pause
     */

    Video.prototype.pause = function() {
      var ref;
      if (!this.error) {
        return (ref = this.videoElement) != null ? ref.pause() : void 0;
      }
    };


    /**
    * Stops the video.
    *
    * @method stop
     */

    Video.prototype.stop = function() {
      var ref;
      if (!this.error) {
        if ((ref = this.videoElement) != null) {
          ref.pause();
        }
      }
      return typeof this.onEnded === "function" ? this.onEnded() : void 0;
    };


    /**
    * Updates the video. Needs to be called once per frame.
    *
    * @method update
     */

    Video.prototype.update = function() {
      var renderer;
      if (this.loaded && !this.disposed) {
        if (this.videoElement.currentTime !== this.lastCurrentTime) {
          renderer = Graphics.renderer;
          renderer.setTexture(this.texture);
          $gl.texImage2D($gl.TEXTURE_2D, 0, $gl.RGBA, $gl.RGBA, $gl.UNSIGNED_BYTE, this.videoElement);
          return this.lastCurrentTime = this.videoElement.currentTime;
        }
      }
    };

    return Video;

  })();

  gs.Video = Video;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var Viewport,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  Viewport = (function(superClass) {
    extend(Viewport, superClass);


    /**
    * A viewport manages a list of graphic objects and automatically renders
    * them ordered by z-index. All graphic object outside of the viewport's bounds
    * are clipped.
    *
    * @namespace gs
    * @class Viewport
    * @extends gs.GraphicObject
    * @memberof gs
    * @constructor
    * @param {number|gs.Rect} x|rect - The x-coordinate of the viewport's rect <b>OR</b> the whole viewport-rect.
    * @param {number} [y=null] - The y-coordinate of the viewport's rect.
    * @param {number} [width=null] - The width of the viewport's rect.
    * @param {number} [height=null] - The height of the viewport's rect.
     */

    function Viewport(x, y, width, height, viewport) {
      Viewport.__super__.constructor.call(this, viewport);

      /**
      * A rectangle defining the area of the viewport. All graphic objects outside of the rect are clipped.
      * @property rect
      * @type gs.Rect
       */
      if (y != null) {
        this.rect = new Rect(x, y, width, height);
      } else if (x != null) {
        this.rect = new Rect(x.x, x.y, x.width, x.height);
      } else {
        this.rect = new Rect(0, 0, Graphics.width, Graphics.height);
      }

      /**
      * The viewport's color useful for flashing-effects, etc.
      * @property color
      * @type gs.Color
       */
      this.color = new Color(255, 255, 255, 0);

      /**
      * The viewport's tone to tint the whole viewport-area.
      * @property tone
      * @type gs.Tone
       */
      this.tone = new Tone(0, 0, 0, 0);

      /**
      * The rotation-angle in degrees. Rotates all objects assigned to this viewport around the viewport's anchor-point.
      * @property angle
      * @type number
       */
      this.angle = 0;

      /**
      * The zoom on x-axis.
      * @property zoomX
      * @type number
       */
      this.zoomX = 1.0;

      /**
      * The zoom on y-axis.
      * @property zoomY
      * @type number
       */
      this.zoomY = 1.0;
      this.objects = [];
      this.needsSort = true;
      this.position = vec2.create();
    }


    /**
    * Disposes the viewport. All graphic objects which are part of the viewport are disposed too.
    *
    * @method dispose
     */

    Viewport.prototype.dispose = function() {
      var i, object;
      Viewport.__super__.dispose.call(this);
      this.disposed = true;
      i = 0;
      while (i < this.objects.length) {
        object = this.objects[i];
        if (!(object != null ? object.disposed : void 0)) {
          if (object != null) {
            object.dispose();
          }
        } else {
          i++;
        }
      }
      return null;
    };

    Viewport.prototype._sort = function(a, b) {
      if (a.z < b.z) {
        return -1;
      } else if (a.z > b.z) {
        return 1;
      } else if (a.id > b.id) {
        return 1;
      } else {
        return -1;
      }
    };


    /**
    * Adds a graphic object to the viewport.
    *
    * @method addGraphicObject
    * @param {gs.GraphicObject} object - The graphic object to add.
     */

    Viewport.prototype.addGraphicObject = function(object) {
      if (this !== Graphics.viewport) {
        Graphics.addGraphicObject(object);
      } else {
        this.needsSort = true;
      }
      if (this.objects.indexOf(object) === -1) {
        this.objects.push(object);
      }
      return object.viewport_ = this;
    };


    /**
    * Removes a graphic object from the viewport.
    *
    * @method removeGraphicObject
    * @param {gs.GraphicObject} object - The graphic object to remove.
     */

    Viewport.prototype.removeGraphicObject = function(object) {
      if (this !== Graphics.viewport) {
        Graphics.removeGraphicObject(object);
      }
      return Array.remove(this.objects, object);
    };

    Viewport.prototype.updateTransform = function() {
      mat2d.identity(this.transform);
      if (this.viewport) {
        mat2d.mul(this.transform, this.viewport.transform, this.transform);
      }
      if (this.angle !== 0 || this.zoomX !== 1 || this.zoomY !== 1) {
        this.position[0] = this.ox - (this.rect.width * (this.zoomX - 1.0) * (this.anchor.x - this.ox / this.rect.width));
        this.position[1] = this.oy - (this.rect.height * (this.zoomY - 1.0) * (this.anchor.y - this.oy / this.rect.height));
        mat2d.translate(this.transform, this.transform, this.position);
        if (this.angle !== 0) {
          this.position[0] = this.rect.width * this.zoomX * (this.anchor.x - this.ox / this.rect.width);
          this.position[1] = this.rect.height * this.zoomY * (this.anchor.y - this.oy / this.rect.height);
          mat2d.translate(this.transform, this.transform, this.position);
          mat2d.rotate(this.transform, this.transform, Math.degreesToRadians(this.angle));
          this.position[0] = -this.position[0];
          this.position[1] = -this.position[1];
          mat2d.translate(this.transform, this.transform, this.position);
        }
        this.position[0] = this.zoomX;
        this.position[1] = this.zoomY;
        return mat2d.scale(this.transform, this.transform, this.position);
      } else {
        this.position[0] = this.ox;
        this.position[1] = this.oy;
        return mat2d.translate(this.transform, this.transform, this.position);
      }
    };

    Viewport.prototype.updateGhostTransform = function(ghost) {
      var x, y;
      mat2d.identity(this.transform);
      x = ghost.x;
      y = ghost.y;
      if (this.viewport) {
        mat2d.mul(this.transform, this.viewport.transform, this.transform);
      }
      if (this.angle !== 0 || ghost.zoomX !== 1 || ghost.zoomY !== 1) {
        this.position[0] = x - (this.rect.width * (ghost.zoomX - 1.0) * (this.anchor.x - x / this.rect.width));
        this.position[1] = y - (this.rect.height * (ghost.zoomY - 1.0) * (this.anchor.y - y / this.rect.height));
        mat2d.translate(this.transform, this.transform, this.position);
        if (this.angle !== 0) {
          this.position[0] = this.rect.width * ghost.zoomX * (this.anchor.x - x / this.rect.width);
          this.position[1] = this.rect.height * ghost.zoomY * (this.anchor.y - y / this.rect.height);
          mat2d.translate(this.transform, this.transform, this.position);
          mat2d.rotate(this.transform, this.transform, Math.degreesToRadians(ghost.angle));
          this.position[0] = -this.position[0];
          this.position[1] = -this.position[1];
          mat2d.translate(this.transform, this.transform, this.position);
        }
        this.position[0] = ghost.zoomX;
        this.position[1] = ghost.zoomY;
        return mat2d.scale(this.transform, this.transform, this.position);
      } else {
        this.position[0] = x;
        this.position[1] = y;
        return mat2d.translate(this.transform, this.transform, this.position);
      }
    };

    Viewport.prototype.updateMotionBlur = function() {
      var ghost, i, results;
      if (this.motionBlur.enabled) {
        if ((Graphics.frameCount + 1) % this.motionBlur.delay === 0) {
          ghost = {
            srcRect: new Rect(this.srcRect.x, this.srcRect.y, this.srcRect.width, this.srcRect.height),
            x: this.x,
            y: this.y,
            zoomX: this.zoomX,
            zoomY: this.zoomY,
            opacity: this.motionBlur.opacity,
            angle: this.angle
          };
          this.motionBlurGhosts.push(ghost);
        }
        i = 0;
        results = [];
        while (i < this.motionBlurGhosts.length) {
          ghost = this.motionBlurGhosts[i];
          if (ghost.opacity === 0) {
            results.push(this.motionBlurGhosts.remove(ghost));
          } else {
            ghost.opacity = Math.max(ghost.opacity - this.motionBlur.dissolveSpeed, 0);
            results.push(i++);
          }
        }
        return results;
      }
    };


    /**
    * Updates the viewport.
    *
    * @method update
     */

    Viewport.prototype.update = function() {
      Viewport.__super__.update.call(this);
      return this.updateTransform();
    };


    /**
    * Draws the viewport with all its graphic objects.
    *
    * @method draw
     */

    Viewport.prototype.draw = function() {
      var color, effect, effectUsed, effects, filter, i, j, k, len, len1, mipmap, object, ref, ref1, renderer, tone;
      renderer = Graphics.renderer;
      tone = renderer.tone;
      effects = renderer.effects;
      renderer.reset();
      if (this.needsSort) {
        this.objects.sort(this._sort);
        this.needsSort = false;
      }
      if (this === Graphics.viewport) {
        ref = this.objects;
        for (i = j = 0, len = ref.length; j < len; i = ++j) {
          object = ref[i];
          if (object.index !== i) {
            object.index = i;
            if (typeof object.onIndexChange === "function") {
              object.onIndexChange();
            }
          }
          if (object.visible) {
            object.draw();
          }
        }
      }
      effectUsed = this.effects.wobble.enabled;
      mipmap = false;
      filter = null;
      ref1 = this.effects.list;
      for (k = 0, len1 = ref1.length; k < len1; k++) {
        effect = ref1[k];
        if (effect.enabled) {
          effectUsed = true;
          mipmap = effect.mipmap;
          filter = effect.textureFilter;
          break;
        }
      }
      if (effectUsed || this.tone.red !== 0 || this.tone.green !== 0 || this.tone.blue !== 0 || this.tone.grey !== 0) {
        renderer.prepareScreenRect();
        renderer.effects = this.effects;
        renderer.tone = this.tone;
        renderer.drawScreenRect(this.rect, mipmap, filter);
        renderer.clear = false;
        renderer.effects = effects;
      }
      if (this.color.alpha !== 0) {
        color = renderer.color;
        renderer.color = this.color;
        renderer.transform = renderer.transform_;
        mat2d.identity(renderer.transform_);
        this.position[0] = this.rect.x;
        this.position[1] = this.rect.y;
        mat2d.translate(renderer.transform_, renderer.transform_, this.position);
        this.position[0] = this.rect.width;
        this.position[1] = this.rect.height;
        mat2d.scale(renderer.transform_, renderer.transform_, this.position);
        renderer.drawRect(this.rect, 0, 0);
        renderer.color = color;
      }
      return renderer.tone = tone;
    };

    return Viewport;

  })(GraphicObject);

  window.Viewport = Viewport;

  gs.Viewport = Viewport;

}).call(this);

// Generated by CoffeeScript 1.12.2

/**
* Describes different kind of standard blend modes.
*
* @namespace gs
* @class BlendMode
* @memberof gs
* @constructor
 */

(function() {
  var BlendMode, BlurEffect, GraphicEffect, GraphicEffectType, Mask, MaskingType, MotionBlur, MotionBlurBehavior, PixelateEffect, Quad, Sprite, WobbleEffect,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  BlendMode = (function() {

    /**
    * Normal blending.
    * @property NORMAL
    * @type number
    * @static
     */
    function BlendMode() {}

    BlendMode.NORMAL = 0;


    /**
    * Additive blending.
    * @property NORMAL
    * @type number
    * @static
     */

    BlendMode.ADDITIVE = 1;


    /**
    * Subtractive blending.
    * @property NORMAL
    * @type number
    * @static
     */

    BlendMode.SUBTRACTIVE = 2;

    return BlendMode;

  })();

  gs.BlendMode = BlendMode;


  /**
  * Describes different kind of shader-based graphic-effect types.
  *
  * @namespace gs
  * @class GraphicEffectType
  * @memberof gs
  * @constructor
   */

  GraphicEffectType = (function() {

    /**
    * A base effect overrides/replaces the content of gl_FragColor and does not use gl_FragColor as input. This is the
    * case for effects such as pixelate or blur.
    * @property BASE
    * @type number
    * @static
     */
    function GraphicEffectType() {}

    GraphicEffectType.BASE = 0;


    /**
    * A add-on effect uses gl_FragColor as input. Means it just adds it's effect to the already existing
    * color-value in gl_FragColor. This is the case for effects such as tinting or masking.
    * @property ADDON
    * @type number
    * @static
     */

    GraphicEffectType.ADDON = 1;


    /**
    * An internal effect. This is only used for certain built-in effects of the engine.
    * @property INTERNAL
    * @type number
    * @static
     */

    GraphicEffectType.INTERNAL = 2;

    return GraphicEffectType;

  })();

  gs.GraphicEffectType = GraphicEffectType;


  /**
  * The base class of all shader-based graphic effect. If you want to create an own custom
  * graphic effect you have to inherit from this class. See Effects > CircularDistortionEffect in
  * Script Editor for an example.
  *
  * @namespace gs
  * @class GraphicEffect
  * @memberof gs
  * @constructor
   */

  GraphicEffect = (function() {
    function GraphicEffect(data) {

      /**
      * The type of the graphic effect.
      * @property type
      * @default gs.GraphicEffectType.ADDON
      * @type gs.GraphicEffectType
       */
      var ref, ref1, ref2;
      this.type = (ref = data != null ? data.type : void 0) != null ? ref : gs.GraphicEffectType.ADDON;

      /**
      * Indicates if the effect is enabled.
      * @property enabled
      * @default false
      * @type boolean
       */
      this.enabled = (ref1 = data != null ? data.enabled : void 0) != null ? ref1 : false;
      this.id = data != null ? data.id : void 0;

      /**
      * The name of the graphic effect. This name must match the name used
      * in the gs.EffectInfo structure when register the effect using gs.Effect.register.
      * @property name
      * @default "unknown"
      * @type string
       */
      this.name = (ref2 = data != null ? data.name : void 0) != null ? ref2 : "unknown";

      /**
      * Indicates if mip-mapping should be enabled for the effect. That is necessary for certain kind of effects
      * such as LOD blur.
      * @property mipmap
      * @default false
      * @type boolean
       */
      this.mipmap = false;

      /**
      * The texture-filter to use for this effect. If <b>null</b>, the current filter will not change.
      * @property textureFilter
      * @default null
      * @type gs.TextureFilter
       */
      this.textureFilter = null;
    }


    /**
    * Setup the effect for rendering. All uniforms/shader-inputs need to be set here.
    *
    * @method setup
    * @param {gs.Effect} effect - The shader-effect/program object which allows you to set uniforms.
    * @param {gs.RenderTask} task - The render-task object contains additional information about the object being rendered.
    * @param {gs.Texture2D} texture - The texture used for rendering.
     */

    GraphicEffect.prototype.setup = function(effect, task, texture) {};

    return GraphicEffect;

  })();

  gs.GraphicEffect = GraphicEffect;


  /**
  * Describes a wobble-effect. A wobble-effects animates the texture-coordinates
  * using sin/cos functions to generate a "wobble"-like effect.
  *
  * @namespace gs
  * @class WobbleEffect
  * @memberof gs
  * @constructor
   */

  WobbleEffect = (function(superClass) {
    extend(WobbleEffect, superClass);

    function WobbleEffect(data) {
      var ref, ref1, ref2, ref3;
      WobbleEffect.__super__.constructor.call(this, data);
      this.type = gs.GraphicEffectType.INTERNAL;
      this.name = "wobble";

      /**
      * The wobble-power.
      * @property power
      * @default 0.001
      * @type number
       */
      this.power = (ref = data != null ? data.power : void 0) != null ? ref : 0.001;

      /**
      * The wobble-speed.
      * @property speed
      * @default 0.1
      * @type number
       */
      this.speed = (ref1 = data != null ? data.speed : void 0) != null ? ref1 : 0.1;

      /**
      * Indicates if wobble-effect is vertically.
      * @property vertical
      * @default true
      * @type boolean
       */
      this.vertical = (ref2 = data != null ? data.vertcial : void 0) != null ? ref2 : true;

      /**
      * Indicates if wobble-effect is horizontally.
      * @property horizontal
      * @default true
      * @type boolean
       */
      this.horizontal = (ref3 = data != null ? data.horizontal : void 0) != null ? ref3 : true;
    }


    /**
    * Creates a wobble-effect from an raw object. The object should contain all the properties defined
    * by the WobbleEffect class.
    *
    * @method fromObject
    * @static
    * @param {Object} o - The object with all necessary properties of the wobble-effect.
    * @return {gs.WobbleEffect} The new created wobble-effect.
     */

    WobbleEffect.fromObject = function(o) {
      return new gs.WobbleEffect(o);
    };

    return WobbleEffect;

  })(gs.GraphicEffect);

  gs.WobbleEffect = WobbleEffect;


  /**
  * Describes a blur-effect which blurs the screen using LOD.
  *
  * @namespace gs
  * @class BlurEffect
  * @memberof gs
  * @constructor
   */

  BlurEffect = (function(superClass) {
    extend(BlurEffect, superClass);

    function BlurEffect(data) {
      var ref;
      BlurEffect.__super__.constructor.call(this, data);
      this.type = gs.GraphicEffectType.BASE;
      this.name = "lod_blur";
      this.mipmap = true;
      this.textureFilter = gs.TextureFilter.LINEAR;

      /**
      * The blur-power.
      * @property power
      * @default 0.001
      * @type number
       */
      this.power = (ref = data != null ? data.power : void 0) != null ? ref : 0;
    }

    BlurEffect.prototype.setup = function(effect, task) {
      return effect.setFloatValue("LodBlurPower", task.effects.blur.power - 1.0);
    };


    /**
    * Creates a blur-effect from an raw object. The object should contain all the properties defined
    * by the BlurEffect class.
    *
    * @method fromObject
    * @static
    * @param {Object} o - The object with all necessary properties of the blur-effect.
    * @return {gs.BlurEffect} The new created blur-effect.
     */

    BlurEffect.fromObject = function(o) {
      return new gs.BlurEffect(o);
    };

    return BlurEffect;

  })(gs.GraphicEffect);

  gs.BlurEffect = BlurEffect;


  /**
  * Describes a pixelate-effect which pixelates the game object.
  *
  * @namespace gs
  * @class PixelateEffect
  * @memberof gs
  * @constructor
   */

  PixelateEffect = (function(superClass) {
    extend(PixelateEffect, superClass);

    function PixelateEffect(data) {
      var ref, ref1;
      PixelateEffect.__super__.constructor.call(this, data);
      this.type = gs.GraphicEffectType.BASE;
      this.name = "pixelate";
      this.textureFilter = gs.TextureFilter.NEAREST;

      /**
      * The pixelate-power.
      * @property power
      * @default 0
      * @type number
       */
      this.width = (ref = data != null ? data.width : void 0) != null ? ref : 0;
      this.height = (ref1 = data != null ? data.height : void 0) != null ? ref1 : 0;
    }

    PixelateEffect.prototype.setup = function(effect, task, texture) {
      return effect.setVector2Value("PixelateSize", Math.max(task.effects.pixelate.width, 1) / texture.realWidth, Math.max(task.effects.pixelate.height, 1) / texture.realHeight);
    };


    /**
    * Creates a pixelate-effect from an raw object. The object should contain all the properties defined
    * by the PixelateEffect class.
    *
    * @method fromObject
    * @static
    * @param {Object} o - The object with all necessary properties of the pixelate-effect.
    * @return {gs.PixelateEffect} The new created pixelate-effect.
     */

    PixelateEffect.fromObject = function(o) {
      return new gs.PixelateEffect(o);
    };

    return PixelateEffect;

  })(gs.GraphicEffect);

  gs.PixelateEffect = PixelateEffect;


  /**
  * Enumeration of different masking types.
  *
  * @namespace gs
  * @class MaskingType
  * @memberof gs
  * @constructor
  * @static
  * @final
   */

  MaskingType = (function() {

    /**
    * Static masking.
    * @property STATIC
    * @type number
    * @static
    * @final
     */
    function MaskingType() {}

    MaskingType.STATIC = 0;


    /**
    * Dynamic masking.
    * @property DYNAMIC
    * @type number
    * @static
    * @final
     */

    MaskingType.DYNAMIC = 1;

    return MaskingType;

  })();

  gs.MaskingType = MaskingType;

  Mask = (function() {

    /**
    * Stores mask-settings for masking-effects. There are two types of masking,
    * static and dynamic. Static masking just applies the mask to the target
    * using the vague. Dynamic masking applies the mask depending on the <b>value</b>
    * property. For more info about masking, see help.file.
    *
    * @namespace gs
    * @class Mask
    * @memberof gs
    * @constructor
    * @param {Object} [data=null] - Optional data-bundle to create the motion-blur settings from.
     */
    function Mask(data) {

      /**
      * The x-value of the mask-bitmap offset.
      * @property ox
      * @default 0
      * @type number
       */
      var ref, ref1, ref2, ref3, ref4;
      this.ox = (ref = data != null ? data.ox : void 0) != null ? ref : 0;

      /**
      * The y-value of the mask-bitmap offset.
      * @property oy
      * @default 0
      * @type number
       */
      this.oy = (ref1 = data != null ? data.oy : void 0) != null ? ref1 : 0;

      /**
      * The source/mask-bitmap. This is a greyscaled image defining the mask.
      * @property source
      * @default null
      * @type gs.Bitmap
       */
      this.source = null;

      /**
      * The type of masking, can be static or dynamic.
      * @property type
      * @default gs.MaskingType.STATIC
      * @type gs.MaskingType
       */
      this.type = (ref2 = data != null ? data.type : void 0) != null ? ref2 : 0;

      /**
      * The current value. Goes from 0 to 255.
      * @property value
      * @default 0
      * @type number
       */
      this.value = (ref3 = data != null ? data.value : void 0) != null ? ref3 : 0;

      /**
      * The vague/smoothing.
      * @property vague
      * @default 0
      * @type number
       */
      this.vague = (ref4 = data != null ? data.vague : void 0) != null ? ref4 : 0;
    }

    Mask.prototype.set = function(mask) {
      if (mask) {
        this.ox = mask.ox;
        this.oy = mask.oy;
        this.source = mask.source;
        this.type = mask.type;
        this.value = mask.value;
        return this.vague = mask.vague;
      } else {
        this.ox = 0;
        this.oy = 0;
        this.source = null;
        this.type = 0;
        this.value = 0;
        return this.vague = 0;
      }
    };


    /**
    * Serializes the mask-settings into a data-bundle
    *
    * @method toDataBundle
    * @return {Object} The data-bundle
     */

    Mask.prototype.toDataBundle = function() {
      var ref;
      return {
        ox: this.ox,
        oy: this.oy,
        type: this.type,
        value: this.value,
        vague: this.vague,
        source: (ref = this.source) != null ? ref.filePath : void 0
      };
    };


    /**
    * Creates a mask from an raw object. The object should contain all the properties defined
    * by the Mask class.
    *
    * @method fromObject
    * @static
    * @param {Object} o - The object with all necessary properties of the mask.
    * @return {gs.Mask} The new created mask
     */

    Mask.fromObject = function(o) {
      return new gs.Mask(o);
    };

    return Mask;

  })();

  gs.Mask = Mask;

  MotionBlur = (function() {

    /**
    * Stores motion-blur settings.
    *
    * @namespace gs
    * @class MotionBlur
    * @memberof gs
    * @constructor
    * @param {Object} [data=null] - Optional data-bundle to create the motion-blur settings from.
     */
    function MotionBlur(data) {

      /**
      * Indicates if the motion-blurring is enabled.
      * @property enabled
      * @type boolean
       */
      var ref, ref1, ref2, ref3;
      this.enabled = (ref = data != null ? data.enabled : void 0) != null ? ref : false;

      /**
      * Indicates how fast the motion-blur shadow dissolves.
      * @property dissolveSpeed
      * @default 3
      * @type number
       */
      this.dissolveSpeed = (ref1 = data != null ? data.dissolveSpeed : void 0) != null ? ref1 : 3;

      /**
      * The time in frame before a new motion-blur shadow is created.
      * @property delay
      * @default 2
      * @type number
       */
      this.delay = (ref2 = data != null ? data.delay : void 0) != null ? ref2 : 2;

      /**
      * The shadow start-opacity.
      * @property opacity
      * @default 100
      * @type number
       */
      this.opacity = (ref3 = data != null ? data.opacity : void 0) != null ? ref3 : 100;

      /**
      * The shadow color.
      * @property color
      * @default Color.TRANSPARENT
      * @type gs.Color
       */
      this.color = Color.TRANSPARENT;
    }


    /**
    * Sets the motion-blur settings. If no data-bundle is specified
    *
    * @method set
    * @param {Object} [data=null] - Optional data-bundle to create the motion-blur settings from.
     */

    MotionBlur.prototype.set = function(data) {
      var ref, ref1, ref2, ref3;
      this.enabled = (ref = data != null ? data.enabled : void 0) != null ? ref : false;
      this.dissolveSpeed = (ref1 = data != null ? data.dissolveSpeed : void 0) != null ? ref1 : 3;
      this.delay = (ref2 = data != null ? data.delay : void 0) != null ? ref2 : 2;
      this.opacity = (ref3 = data != null ? data.opacity : void 0) != null ? ref3 : 100;
      return this.color = Color.TRANSPARENT;
    };


    /**
    * Serializes the motion-blur-settings into a data-bundle
    *
    * @method toDataBundle
    * @return {Object} The data-bundle
     */

    MotionBlur.prototype.toDataBundle = function() {
      var result;
      result = {};
      Object.mixin(result, this);
      return result;
    };


    /**
    * Creates a mask from an raw object. The object should contain all the properties defined
    * by the Mask class.
    *
    * @method fromObject
    * @static
    * @param {Object} o - The object with all necessary properties of the mask.
    * @return {gs.Mask} The new created mask
     */

    MotionBlur.fromObject = function(o) {
      return new gs.MotionBlur(o);
    };

    return MotionBlur;

  })();

  gs.MotionBlur = MotionBlur;

  MotionBlurBehavior = (function() {
    function MotionBlurBehavior() {}

    return MotionBlurBehavior;

  })();

  Sprite = (function(superClass) {
    extend(Sprite, superClass);

    Sprite.objectCodecBlackList = ["onIndexChange"];


    /**
    * A sprite displays a bitmap or video on screen. It also supports different kind of
    * visual effects like zooming, rotating, tinting, etc.
    *
    * @namespace gs
    * @class Sprite
    * @extends gs.GraphicObject
    * @memberof gs
    * @constructor
    * @param {gs.Viewport} [viewport=gs.Graphics.viewport] - The viewport.
     */

    function Sprite(viewport, managed) {
      Sprite.__super__.constructor.call(this, viewport, managed);

      /**
      * The bitmap to render.
      * @property bitmap
      * @type gs.Bitmap
       */
      this.bitmap = null;

      /**
      * The video to render.
      * @property video
      * @type gs.Video
       */
      this.video_ = null;

      /**
      * The source-rectangle. Only that area of the bitmap is rendered on screen.
      * @property srcRect
      * @type gs.Rect
       */
      this.srcRect = null;

      /**
      * The mask used for masking-effects.
      * @property mask
      * @type gs.Mask
       */
      this.mask = {
        ox: 0,
        oy: 0,
        source: null,
        type: 0,
        value: 0,
        vague: 0
      };

      /**
      * The x-coordinate of the sprite's position.
      * @property x
      * @type number
       */
      this.x = 0;

      /**
      * The y-coordinate of the sprite's position.
      * @property y
      * @type number
       */
      this.y = 0;
      this.width = 0;
      this.height = 0;

      /**
      * The zoom on x-axis.
      * @property zoomX
      * @type number
       */
      this.zoomX = 1.0;

      /**
      * The zoom on y-axis.
      * @property zoomY
      * @type number
       */
      this.zoomY = 1.0;

      /**
      * The rotation-angle in degrees.
      * @property angle
      * @type number
       */
      this.angle = 0;

      /**
      * Indicates if the sprite is mirrored/flipped horizontally.
      * @property mirror
      * @type boolean
       */
      this.mirror = false;

      /**
      * The color. Can be used for flashing-effects, etc.
      * @property color
      * @type boolean
       */
      this.color = new Color(255, 255, 255, 0);

      /**
      * The opacity goes from 0 to 255. For example: A value of 0 means fully transparent, 255 means fully opaque and 128 means semi-transparent.
      * @property opacity
      * @type number
       */
      this.opacity = 255;

      /**
      * The color tone.
      * @property tone
      * @type gs.Tone
       */
      this.tone = new Tone(0, 0, 0, 0);

      /**
      * The blend mode used for alpha-blending.
      * @property blendingMode
      * @type gs.BlendMode
       */
      this.blendingMode = 0;

      /**
      * The motion-blur settings.
      * @property motionBlur
      * @type gs.MotionBlur
       */
      this.motionBlur = new gs.MotionBlur();
      this.motionBlurGhosts = [];
      this.wobble = this.effects.wobble;

      /**
      * The clip-rect. Everything outside the clip-rect is clipped and not visible.
      * @property clipRect
      * @type gs.Rect
       */
      this.clipRect = new Rect(0, 0, Graphics.width, Graphics.height);
    }


    /**
    * The video to render.
    * @property video
    * @type gs.Video
     */

    Object.defineProperty(Sprite.prototype, "video", {
      set: function(value) {
        this.video_ = value;
        return this.bitmap = value;
      },
      get: function() {
        return this.video_;
      }
    });

    Sprite.prototype.updateGhostTransform = function(ghost) {
      var h, w, x, y;
      mat2d.identity(this.transform);
      mat2d.mul(this.transform, this.transform, this.viewport.transform);
      w = ghost.srcRect.width;
      h = ghost.srcRect.height;
      x = ghost.x - this.ox;
      y = ghost.y - this.oy;
      if (ghost.angle !== 0 || ghost.zoomX !== 1 || ghost.zoomY !== 1) {
        this.position[0] = x - (w * (ghost.zoomX - 1.0) * this.anchor.x);
        this.position[1] = y - (h * (ghost.zoomY - 1.0) * this.anchor.y);
        mat2d.translate(this.transform, this.transform, this.position);
        if (ghost.angle !== 0) {
          this.position[0] = w * this.anchor.x;
          this.position[1] = h * this.anchor.y;
          mat2d.translate(this.transform, this.transform, this.position);
          mat2d.rotate(this.transform, this.transform, Math.degreesToRadians(ghost.angle));
          this.position[0] = -(w * this.anchor.x);
          this.position[1] = -(h * this.anchor.y);
          mat2d.translate(this.transform, this.transform, this.position);
        }
        this.position[0] = w * ghost.zoomX;
        this.position[1] = h * ghost.zoomY;
        return mat2d.scale(this.transform, this.transform, this.position);
      } else {
        this.position[0] = x;
        this.position[1] = y;
        mat2d.translate(this.transform, this.transform, this.position);
        this.position[0] = w;
        this.position[1] = h;
        return mat2d.scale(this.transform, this.transform, this.position);
      }
    };


    /**
    * Updates the sprite's motion-blur and other effects. Needs to be called once per frame.
    *
    * @method update
     */

    Sprite.prototype.update = function() {
      var ghost, i, results;
      Sprite.__super__.update.call(this);
      if (this.motionBlur.enabled) {
        if ((Graphics.frameCount + 1) % this.motionBlur.delay === 0) {
          ghost = {
            srcRect: new Rect(this.srcRect.x, this.srcRect.y, this.srcRect.width, this.srcRect.height),
            transform: mat2d.clone(this.transform),
            opacity: this.motionBlur.opacity
          };
          this.motionBlurGhosts.push(ghost);
        }
        i = 0;
        results = [];
        while (i < this.motionBlurGhosts.length) {
          ghost = this.motionBlurGhosts[i];
          if (ghost.opacity === 0) {
            results.push(this.motionBlurGhosts.remove(ghost));
          } else {
            ghost.opacity = Math.max(ghost.opacity - this.motionBlur.dissolveSpeed, 0);
            results.push(i++);
          }
        }
        return results;
      }
    };


    /**
    * Draws the sprite. Shouldn't be called manually.
    *
    * @method draw
     */

    Sprite.prototype.draw = function() {
      var blendingMode, color, effects, ghost, i, j, ref, renderer, tone;
      if (this.srcRect == null) {
        return;
      }
      renderer = Graphics.renderer;
      if ((this.video != null) && this.video.loaded) {
        this.bitmap = this.video;
      }
      color = renderer.color;
      tone = renderer.tone;
      blendingMode = renderer.blendType;
      effects = renderer.effects;
      this.updateTransform(~~(this.x - this.ox), ~~(this.y - this.oy), this.width || this.srcRect.width, this.height || this.srcRect.height);
      renderer.transform = this.transform;
      renderer.mirror = this.mirror;
      renderer.color = this.color;
      renderer.opacity = this.opacity;
      renderer.tone = this.tone;
      renderer.blendType = this.blendingMode;
      renderer.mask = this.mask;
      renderer.wobble = this.wobble;
      renderer.effects = this.effects;
      if (this.clipRect != null) {
        renderer.clipRect(this.clipRect);
      } else {
        renderer.clipRect(Graphics.viewport.rect);
      }
      if ((this.bitmap != null) && !this.bitmap.disposed) {
        renderer.setBitmap(this.bitmap);
        renderer.setSourceRectangle(this.srcRect.x, this.srcRect.y, this.srcRect.width, this.srcRect.height);
        renderer.drawTransform();
      }
      if (this.motionBlur.enabled) {
        for (i = j = 0, ref = this.motionBlurGhosts.length; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
          ghost = this.motionBlurGhosts[i];
          renderer.mirror = this.mirror;
          renderer.color = this.motionBlur.color;
          renderer.opacity = ghost.opacity;
          renderer.tone = this.tone;
          renderer.blendType = this.blendingMode;
          if (this.bitmap != null) {
            renderer.transform = ghost.transform;
            renderer.setSourceRectangle(this.srcRect.x, this.srcRect.y, this.srcRect.width, this.srcRect.height);
            renderer.drawTransform();
          }
        }
        renderer.color = Color.TRANSPARENT;
        renderer.opacity = 255;
      }
      renderer.opacity = 255;
      renderer.tone = tone;
      renderer.color = color;
      renderer.mask = null;
      renderer.blendType = blendingMode;
      renderer.effects = effects;
      return renderer.mirror = false;
    };

    return Sprite;

  })(GraphicObject);

  window.Sprite = Sprite;

  gs.Sprite = Sprite;

  Quad = (function(superClass) {
    extend(Quad, superClass);

    Quad.objectCodecBlackList = ["onIndexChange"];


    /**
    * @class Quad
    * @namespace gs
    * @extends gs.GraphicObject
    * @constructor
    * @param {gs.Viewport} viewport - The viewport. Defaults to gs.Graphics.viewport.
     */

    function Quad(viewport) {
      Quad.__super__.constructor.call(this, viewport);

      /**
      * The the position and size of the rectangle.
      * @property color
      * @type gs.Color
       */
      this.rect = new Rect(0, 0, 100, 100);

      /**
      * The color of the rectangle.
      * @property color
      * @type gs.Color
       */
      this.color = new Color(255, 255, 255, 0);

      /**
      * The rectangle's opacity (0 - 255).
      * @property opacity
      * @type number
       */
      this.opacity = 255;

      /**
      * The rectangle's blend mode.
      * @property blendingMode
      * @type gs.BlendMode
       */
      this.blendingMode = 0;
      this.clipRect = null;
      this.zoomX = 1.0;
      this.zoomY = 1.0;
      this.angle = 0;
    }

    Quad.prototype.update = function() {
      Quad.__super__.update.call(this);
      if (this.flashCounter > 0) {
        this.flashCounter--;
        this.flashColor.alpha = (this.flashCounter / this.flashDuration) * 255;
        return this.color = this.flashColor;
      }
    };

    Quad.prototype.draw = function() {
      var blendingMode, color, effects, renderer;
      renderer = Graphics.renderer;
      if (this.clipRect) {
        renderer.clipRect(this.clipRect);
      } else {
        renderer.clipRect(Graphics.viewport.rect);
      }
      blendingMode = renderer.blendType;
      effects = renderer.effects;
      renderer.blendType = this.blendingMode;
      renderer.mask = this.mask;
      renderer.wobble = this.wobble;
      renderer.effects = this.effects;
      if (this.color.alpha !== 0) {
        color = renderer.color;
        renderer.color = this.color;
        renderer.transform = this.transform;
        renderer.opacity = this.opacity;
        this.updateTransform(this.rect.x - this.ox, this.rect.y - this.oy, this.rect.width, this.rect.height);
        renderer.drawRect(this.rect, this.ox, this.oy);
        renderer.color = color;
        renderer.opacity = 255;
      }
      renderer.color = color;
      renderer.blendType = blendingMode;
      renderer.mask = null;
      renderer.effects = effects;
      return renderer.opacity = 255;
    };

    return Quad;

  })(GraphicObject);

  window.gs.Quad = Quad;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var TilingObject,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  TilingObject = (function(superClass) {
    extend(TilingObject, superClass);

    TilingObject.objectCodecBlackList = ["onIndexChange"];


    /**
    * A tiling sprite tiles a bitmap or video over the screen. It also supports different kind of
    * visual effects like tinting, blending, etc.
    *
    * @namespace gs
    * @class TilingSprite
    * @extends gs.GraphicObject
    * @memberof gs
    * @constructor
    * @param {gs.Viewport} [viewport=gs.Graphics.viewport] - The viewport.
    * @param {boolean} [managed=yes] - Indicates if the sprite should be managed by the graphic-system.
     */

    function TilingObject(viewport, managed) {
      TilingObject.__super__.constructor.call(this, viewport, managed);

      /**
      * The bitmap to render.
      * @property bitmap
      * @type gs.Bitmap
       */
      this.bitmap = null;

      /**
      * The video to render.
      * @property video
      * @type gs.Video
       */
      this.video = null;

      /**
      * The mask used for masking-effects.
      * @property mask
      * @type gs.Mask
       */
      this.mask = {
        ox: 0,
        oy: 0,
        source: null,
        type: 0,
        value: 0,
        vague: 0
      };

      /**
      * The color. Can be used for flashing-effects, etc.
      * @property mirror
      * @type boolean
       */
      this.color = new Color(255, 255, 255, 0);

      /**
      * The opacity goes from 0 to 255. For example: A value of 0 means fully transparent, 255 means fully opaque and 128 means semi-transparent.
      * @property opacity
      * @type number
       */
      this.opacity = 255;

      /**
      * The color tone.
      * @property tone
      * @type gs.Tone
       */
      this.tone = new Tone(0, 0, 0, 0);

      /**
      * The blend mode used for alpha-blending.
      * @property blendingMode
      * @type gs.BlendMode
       */
      this.blendingMode = 0;

      /**
      * Indicates if the sprite tiles vertically.
      * @property vertical
      * @type boolean
       */
      this.vertical = true;

      /**
      * Indicates if the sprite tiles horizontally.
      * @property horizontal
      * @type boolean
       */
      this.horizontal = true;

      /**
      * The effect settings for different kind of visual effect like wobble, etc.
      * @property effects
      * @type Object
       */
      this.effects = new gs.EffectCollection();

      /**
      * The motion-blur settings.
      * @property motionBlur
      * @type gs.MotionBlur
       */
      this.motionBlur = new gs.MotionBlur();
      this.motionBlurGhosts = [];

      /**
      * The clip-rect. Everything outside the clip-rect is clipped and not visible.
      * @property clipRect
      * @type gs.Rect
       */
      this.clipRect = new Rect(0, 0, Graphics.width, Graphics.height);

      /**
      * The source-rectangle. Only that area of the bitmap/video is tiled over the screen.
      * @property srcRect
      * @type gs.Rect
       */
      this.srcRect = null;

      /**
      * The x-coordinate of the scrolling.
      * @property x
      * @type number
       */
      this.x = 0;

      /**
      * The y-coordinate of the scrolling.
      * @property y
      * @type number
       */
      this.y = 0;

      /**
      * The zoom on x-axis.
      * @property zoomX
      * @type number
       */
      this.zoomX = 1.0;

      /**
      * The zoom on y-axis.
      * @property zoomY
      * @type number
       */
      this.zoomY = 1.0;

      /**
      * The rotation-angle in degrees. Rotates all objects assigned to this viewport around the viewport's anchor-point.
      * @property angle
      * @type number
       */
      this.angle = 0;
    }


    /**
    * Updates the sprite's motion-blur and other effects. Needs to be called once per frame.
    *
    * @method update
     */

    TilingObject.prototype.update = function() {
      return TilingObject.__super__.update.call(this);
    };


    /*
    if @motionBlur.enabled
      if (Graphics.frameCount+1) % @motionBlur.delay == 0
        ghost = { srcRect: new Rect(@srcRect.x, @srcRect.y, @srcRect.width, @srcRect.height), x: @x, y: @y, zoomX: @zoomX, zoomY: @zoomY, opacity: @motionBlur.opacity }
        @motionBlurGhosts.push(ghost)
    
      i = 0
      while i < @motionBlurGhosts.length
        ghost = @motionBlurGhosts[i]
        if ghost.opacity == 0
          @motionBlurGhosts.remove(ghost)
        else
          ghost.opacity = Math.max(ghost.opacity - @motionBlur.dissolveSpeed, 0)
          i++
     */

    TilingObject.prototype.updateTransform = function(x, y, width, height) {
      mat2d.identity(this.transform);
      mat2d.mul(this.transform, this.transform, this.viewport.transform);
      if (this.angle !== 0) {
        this.position[0] = x + this.rect.width * this.anchor.x;
        this.position[1] = y + this.rect.height * this.anchor.y;
        mat2d.translate(this.transform, this.transform, this.position);
        mat2d.rotate(this.transform, this.transform, Math.degreesToRadians(this.angle));
        this.position[0] = -this.position[0];
        this.position[1] = -this.position[1];
        mat2d.translate(this.transform, this.transform, this.position);
      }
      if (this.angle !== 0 || this.zoomX !== 1 || this.zoomY !== 1) {
        this.position[0] = x - (this.rect.width * (this.zoomX - 1.0) * this.anchor.x);
        this.position[1] = y - (this.rect.height * (this.zoomY - 1.0) * this.anchor.y);
        mat2d.translate(this.transform, this.transform, this.position);
        this.position[0] = width * this.zoomX;
        this.position[1] = height * this.zoomY;
        return mat2d.scale(this.transform, this.transform, this.position);
      } else {
        this.position[0] = x;
        this.position[1] = y;
        mat2d.translate(this.transform, this.transform, this.position);
        this.position[0] = width;
        this.position[1] = height;
        return mat2d.scale(this.transform, this.transform, this.position);
      }
    };

    TilingObject.prototype._drawTiles = function(rect) {
      var blendType, renderer, rx, ry;
      renderer = Graphics.renderer;
      blendType = renderer.blendType;
      renderer.blendType = this.blendingMode;
      rx = rect.x - this.ox;
      ry = rect.y - this.oy;
      this.updateTransform(rx, ry, rect.width, rect.height);
      renderer.transform = this.transform;
      renderer.setBitmap(this.bitmap);
      renderer.setSourceRectangle(this.x, this.y, rect.width, rect.height);
      renderer.drawTransform();
      renderer.blendType = blendType;
      return null;
    };

    TilingObject.prototype._draw = function() {
      if (this.bitmap != null) {
        return this._drawTiles(Graphics.viewport.rect);
      }
    };


    /**
    * Draws the sprite. Shouldn't be called manually.
    *
    * @method draw
     */

    TilingObject.prototype.draw = function() {
      var blendType, color, effects, ref, renderer, tone;
      if (!((ref = this.bitmap) != null ? ref.loaded : void 0)) {
        return;
      }
      if (this.srcRect == null) {
        this.srcRect = new Rect(0, 0, this.bitmap.width, this.bitmap.height);
      }
      renderer = Graphics.renderer;
      if ((this.video != null) && this.video.loaded) {
        this.bitmap = this.video;
      }
      color = renderer.color;
      tone = renderer.tone;
      blendType = renderer.blendType;
      effects = renderer.effects;
      renderer.mirror = false;
      renderer.color = this.color;
      renderer.opacity = this.opacity;
      renderer.tone = this.tone;
      renderer.blendType = this.blendingMode;
      renderer.mask = this.mask;
      renderer.wobble = this.wobble;
      renderer.effects = this.effects;
      if (this.clipRect != null) {
        renderer.clipRect(this.clipRect);
      } else {
        renderer.clipRect(Graphics.viewport.rect);
      }
      this._draw();
      renderer.opacity = 255;
      renderer.tone = tone;
      renderer.color = color;
      renderer.mask = null;
      renderer.blendType = blendType;
      return renderer.effects = effects;
    };

    return TilingObject;

  })(GraphicObject);

  gs.TilingObject = TilingObject;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var TilingSprite,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  TilingSprite = (function(superClass) {
    extend(TilingSprite, superClass);

    TilingSprite.objectCodecBlackList = ["onIndexChange"];

    function TilingSprite() {
      TilingSprite.__super__.constructor.apply(this, arguments);

      /**
      * The tiling-area. The sprite will only be tiled over that area. To tile over the whole screen just the rectangle to the screen-size.
      * @property rect
      * @type gs.Rect
       */
      this.rect = new Rect(0, 0, Graphics.width, Graphics.height);
    }

    TilingSprite.prototype._draw = function() {
      if (this.bitmap != null) {
        return this._drawTiles(this.rect);
      }
    };

    return TilingSprite;

  })(gs.TilingObject);

  gs.TilingSprite = TilingSprite;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var TilingPlane,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  TilingPlane = (function(superClass) {
    extend(TilingPlane, superClass);

    TilingPlane.objectCodecBlackList = ["onIndexChange"];

    function TilingPlane() {
      TilingPlane.__super__.constructor.apply(this, arguments);

      /**
      * The tiling-area. The sprite will only be tiled over that area. To tile over the whole screen just the rectangle to the screen-size.
      * @property rect
      * @type gs.Rect
       */
      this.rect = new Rect(0, 0, Graphics.width, Graphics.height);
      this.x = 0;
      this.y = 0;
      this.viewportInverse_ = mat2d.create();
    }

    TilingPlane.prototype.updateTransform = function(x, y, width, height) {
      mat2d.identity(this.transform);
      mat2d.invert(this.viewportInverse_, this.viewport.transform);
      mat2d.mul(this.transform, this.transform, this.viewportInverse_);
      if (this.angle !== 0 || this.zoomX !== 1 || this.zoomY !== 1) {
        this.position[0] = x - (width * (this.zoomX - 1.0) * this.anchor.x);
        this.position[1] = y - (height * (this.zoomY - 1.0) * this.anchor.y);
        mat2d.translate(this.transform, this.transform, this.position);
        if (this.angle !== 0) {
          this.position[0] = width * this.anchor.x;
          this.position[1] = height * this.anchor.y;
          mat2d.translate(this.transform, this.transform, this.position);
          mat2d.rotate(this.transform, this.transform, Math.degreesToRadians(this.angle));
          this.position[0] = -(width * this.anchor.x);
          this.position[1] = -(height * this.anchor.y);
          mat2d.translate(this.transform, this.transform, this.position);
        }
        this.position[0] = width * this.zoomX;
        this.position[1] = height * this.zoomY;
        return mat2d.scale(this.transform, this.transform, this.position);
      } else {
        this.position[0] = x;
        this.position[1] = y;
        mat2d.translate(this.transform, this.transform, this.position);
        this.position[0] = width;
        this.position[1] = height;
        return mat2d.scale(this.transform, this.transform, this.position);
      }
    };

    TilingPlane.prototype._draw = function() {
      if (this.bitmap != null) {
        return this._drawTiles(this.rect);
      }
    };

    TilingPlane.prototype._drawTiles = function(rect) {
      var blendType, effects, renderer;
      renderer = Graphics.renderer;
      blendType = renderer.blendType;
      effects = renderer.effects;
      renderer.blendType = this.blendingMode;
      this.updateTransform(~~(-this.x), ~~this.y, this.rect.width, this.rect.height);
      renderer.transform = this.transform;
      renderer.effects = this.effects;
      renderer.setBitmap(this.bitmap);
      renderer.drawTextureTransform(this.rect);
      renderer.blendType = blendType;
      renderer.effects = effects;

      /*
      if @vertical and @horizonal
        for x in [stx..tilesX]
          for y in [sty..tilesY]
            px = 0 + (rv[0]%w) + x * w
            py = 0 + (rv[1]%h) + y * h
            #if px < 0+rw and py < 0+rh
            #if (x+1 == tilesX+10) or (y+1 == tilesY+10)
             *  sw = @srcRect.width -  Math.max((px + w - (0+rw) ), 0)
             *  sh = @srcRect.height - Math.max((py + h - (0+rh) ), 0)
            #else
            sw = @srcRect.width
            sh = @srcRect.height
      
            if px-rv[0]+rsw > -(rect.width*1.0) and px-rv[0] < rw+rect.width*1.0 and
               py-rv[1]+rsh > -(rect.height*1.0) and py-rv[1] < rh+rect.height*1.0
              @updateTransform(px - rv[0], py - rv[1], sw, sh)
              renderer.setBitmap(@bitmap)
              renderer.setSourceRectangle(@srcRect.x, @srcRect.y, sw, sh)
              renderer.drawTransform()
      else if @vertical
        px = rx
        for y in [0..tilesY]
          py = ry + y * h
          if px + w > rx and px < rx+rw and py + h > ry and py < ry+rh
            if (y+1 == tilesY)
              sw = @srcRect.width -  Math.max((px + @srcRect.width - (rx+rect.width) ), 0)
              sh = @srcRect.height - Math.max((py + @srcRect.height - (ry+rect.height) ), 0)
            else
              sw = @srcRect.width
              sh = @srcRect.height
      
            @updateTransform(px, py, sw, sh)
            renderer.setBitmap(@bitmap)
            renderer.setSourceRectangle(@srcRect.x, @srcRect.y, sw, sh)
            renderer.drawTransform()
      else if @horizontal
        py = ry
        for x in [0..tilesX]
          px = rx + (x * w)
          if px + w > rx and px < rx+rw and py + h > ry and py < ry+rh
            if (y+1 == tilesY)
              sw = @srcRect.width -  Math.max((px + @srcRect.width - (rx+rect.width) ), 0)
              sh = @srcRect.height - Math.max((py + @srcRect.height - (ry+rect.height) ), 0)
            else
              sw = @srcRect.width
              sh = @srcRect.height
      
            @updateTransform(px, py, sw, sh)
            renderer.setBitmap(@bitmap)
            renderer.setSourceRectangle(@srcRect.x, @srcRect.y, sw, sh)
            renderer.drawTransform()
      else
        @updateTransform(px, py, @srcRect.width, @srcRect.height)
        renderer.setBitmap(@bitmap)
        renderer.setSourceRectangle(@srcRect.x, @srcRect.y, sw, sh)
        renderer.drawTransform()
       */
      return null;
    };

    return TilingPlane;

  })(gs.TilingObject);

  gs.TilingPlane = TilingPlane;

}).call(this);

// Generated by CoffeeScript 1.6.3
(function() {
  var Plane,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Plane = (function(_super) {
    __extends(Plane, _super);

    function Plane() {
      Plane.__super__.constructor.apply(this, arguments);
      this.bitmap = null;
      this.zoomX = 1.0;
      this.zoomY = 1.0;
      this.color = new Color(255, 255, 255, 0);
      this.opacity = 255;
      this.tone = new Tone(0, 0, 0, 0);
      this.blendType = 0;
      this.srcRect = null;
      this.loopVertical = true;
      this.loopHorizontal = true;
    }

    Plane.prototype.draw = function() {
      var h, px, py, renderer, screenH, screenW, tilesX, tilesY, w, x, y, _i, _j, _k, _l, _ref, _ref1, _ref2, _ref3;
      if ((this.bitmap == null) || !this.bitmap.loaded) {
        return;
      }
      if (!this.srcRect) {
        this.srcRect = new Rect(0, 0, this.bitmap.width, this.bitmap.height);
      }
      renderer = Graphics.renderer;
      w = this.srcRect.width * this.zoomX;
      h = this.srcRect.height * this.zoomY;
      screenW = Graphics.width;
      screenH = Graphics.height;
      tilesX = this.loopHorizontal ? Math.ceil(screenW / w) : 1;
      tilesY = this.loopVertical ? Math.ceil(screenH / h) : 1;
      px = 0;
      py = 0;
      renderer.zoom.x = this.zoomX;
      renderer.zoom.y = this.zoomY;
      renderer.angle = 0;
      renderer.mirror = false;
      renderer.color = this.color;
      renderer.opacity = this.opacity;
      renderer.tone = this.tone;
      renderer.blendingMode = this.blendType;
      if (this.loopVertical && this.loopHorizontal) {
        for (x = _i = -1, _ref = tilesX + 2; -1 <= _ref ? _i <= _ref : _i >= _ref; x = -1 <= _ref ? ++_i : --_i) {
          for (y = _j = -1, _ref1 = tilesY + 2; -1 <= _ref1 ? _j <= _ref1 : _j >= _ref1; y = -1 <= _ref1 ? ++_j : --_j) {
            px = x * w + -this.ox % w;
            py = y * h + -this.oy % h;
            if (px + w > 0 && px < screenW && py + h > 0 && py < screenH) {
              renderer.drawBitmap(this.bitmap, px, py, this.srcRect);
            }
          }
        }
      } else if (this.loopVertical) {
        for (y = _k = -1, _ref2 = tilesY + 2; -1 <= _ref2 ? _k <= _ref2 : _k >= _ref2; y = -1 <= _ref2 ? ++_k : --_k) {
          px = x;
          py = y * h + -this.oy % h;
          if (px + w > 0 && px < screenW && py + h > 0 && py < screenH) {
            renderer.drawBitmap(this.bitmap, px, py, this.srcRect);
          }
        }
      } else if (this.loopHorizontal) {
        for (x = _l = -1, _ref3 = tilesX + 2; -1 <= _ref3 ? _l <= _ref3 : _l >= _ref3; x = -1 <= _ref3 ? ++_l : --_l) {
          px = x * w + -this.ox % w;
          py = y;
          if (px + w > 0 && px < screenW && py + h > 0 && py < screenH) {
            renderer.drawBitmap(this.bitmap, px, py, this.srcRect);
          }
        }
      }
      return null;
    };

    return Plane;

  })(GraphicObject);

  window.Plane = Plane;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var L2DPlatformManager, Live2DModel,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  L2DPlatformManager = (function() {
    function L2DPlatformManager() {}

    L2DPlatformManager.prototype.jsonParseFromBytes = function(buf) {
      var bomCode, jsonObj, jsonStr;
      jsonStr = null;
      bomCode = new Uint8Array(buf, 0, 3);
      if (bomCode[0] === 239 && bomCode[1] === 187 && bomCode[2] === 191) {
        jsonStr = String.fromCharCode.apply(null, new Uint8Array(buf, 3));
      } else {
        jsonStr = String.fromCharCode.apply(null, new Uint8Array(buf));
      }
      jsonObj = JSON.parse(jsonStr);
      return jsonObj;
    };

    L2DPlatformManager.prototype.log = function(txt) {
      return console.log(txt);
    };

    return L2DPlatformManager;

  })();

  window.L2DPlatformManager = L2DPlatformManager;

  Live2DModel = (function(superClass) {
    extend(Live2DModel, superClass);

    Live2DModel.fromDataBundle = function(data, context) {
      var i, j, len, p, ref, result;
      result = null;
      if (data.filePath) {
        result = ResourceManager.getLive2DModel(data.filePath);
        if (!result.loaded) {
          result.once("modelDataLoaded", {
            params: data.params,
            model: result
          }, function(e) {
            var i, j, len, p, ref, results;
            ref = e.data.params;
            results = [];
            for (i = j = 0, len = ref.length; j < len; i = ++j) {
              p = ref[i];
              results.push(e.data.model.live2DModel.setParamFloat(i, p));
            }
            return results;
          });
        } else {
          ref = data.params;
          for (i = j = 0, len = ref.length; j < len; i = ++j) {
            p = ref[i];
            result.live2DModel.setParamFloat(i, p);
          }
        }
      }
      return result;
    };

    Live2DModel.toDataBundle = function(object, context) {
      var i, j, modelParams, params, r, ref;
      params = [];
      modelParams = object.live2DModel._$5S._$_2;
      for (i = j = 0, ref = modelParams.length; 0 <= ref ? j <= ref : j >= ref; i = 0 <= ref ? ++j : --j) {
        params[i] = modelParams[i];
      }
      r = {
        filePath: object.filePath,
        params: params
      };
      return r;
    };


    /**
    * This class is used to load a Live2D model from .live2d archive. See Resource Standards in Help File for
    * more information.
    *
    * @namespace gs
    * @class Live2DModel
    * @memberof gs
    * @constructor
    * @param {string} filePath - The path to the Live2D resource.
    * @param {string} [languageCode=null] - The language code to load the Live2D resource depending on language.
     */

    function Live2DModel(filePath, languageCode, glId) {
      Live2DModel.__super__.constructor.apply(this, arguments);
      this.filePath = filePath;
      this.live2DModel = null;
      this.params = [];
      glId = glId != null ? glId : 0;

      /**
      * Indicates if the Live2D model is disposed.
      * @property disposed
      * @type boolean
      * @readOnly
       */
      this.disposed = false;

      /**
      * Indicates if the Live2D model is initialized.
      * @property initialized
      * @type boolean
      * @readOnly
       */
      this.initialized = false;
      this.textures = [];

      /**
      * Live2D motions by name/key.
      * @property motions
      * @type Object
       */
      this.motions = {};

      /**
      * Live2D motions by group.
      * @property motionsByGroup
      * @type Object
       */
      this.motionsByGroup = {};

      /**
      * Live2D expressions.
      * @property expressions
      * @type gs.Live2DExpression[]
       */
      this.expressions = [];

      /**
      * Live2D pose data.
      * @property pose
      * @type gs.Live2DPose
       */
      this.pose = null;
      if (typeof filePath === "string") {
        this.loadFromFile(filePath, languageCode, glId);
      } else {
        this.loadFromBuffer(filePath, glId);
      }
    }


    /**
    * Indicates if the Live2D model is loaded.
    * @property loaded
    * @type boolean
    * @readOnly
     */

    Object.defineProperty(Live2DModel.prototype, "loaded", {
      get: function() {
        var j, len, ref, result, texture;
        result = this.textures.length > 0;
        ref = this.textures;
        for (j = 0, len = ref.length; j < len; j++) {
          texture = ref[j];
          if (!texture || !texture.loaded) {
            return false;
          }
        }
        return result;
      }
    });


    /**
    * Initializes the Live2D model for use.
    *
    * @method initialize
     */

    Live2DModel.prototype.initialize = function() {
      var j, len, ref, t, texture;
      if (!window.Live2DModelWebGL) {
        return;
      }
      ref = this.textures;
      for (j = 0, len = ref.length; j < len; j++) {
        t = ref[j];
        texture = $gl.createTexture();
        $gl.pixelStorei($gl.UNPACK_PREMULTIPLY_ALPHA_WEBGL, 1);
        $gl.pixelStorei($gl.UNPACK_FLIP_Y_WEBGL, 1);
        $gl.activeTexture($gl.TEXTURE0);
        $gl.bindTexture($gl.TEXTURE_2D, texture);
        $gl.texImage2D($gl.TEXTURE_2D, 0, $gl.RGBA, $gl.RGBA, $gl.UNSIGNED_BYTE, t);
        $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MAG_FILTER, $gl.LINEAR);
        $gl.texParameteri($gl.TEXTURE_2D, $gl.TEXTURE_MIN_FILTER, $gl.LINEAR_MIPMAP_NEAREST);
        $gl.generateMipmap($gl.TEXTURE_2D);
        $gl.bindTexture($gl.TEXTURE_2D, null);
        $gl.pixelStorei($gl.UNPACK_PREMULTIPLY_ALPHA_WEBGL, 0);
        $gl.pixelStorei($gl.UNPACK_FLIP_Y_WEBGL, 0);
        t.textureId = texture;
      }
      return this.initialized = true;
    };


    /**
    * Disposes the Live2D model and frees allocated resources/memory.
    *
    * @method dispose
     */

    Live2DModel.prototype.dispose = function() {
      var i, j, len, ref, results, texture;
      if (!this.disposed) {
        this.disposed = true;
        ref = this.textures;
        results = [];
        for (i = j = 0, len = ref.length; j < len; i = ++j) {
          texture = ref[i];
          if ((texture != null) && (texture.textureId != null)) {
            $gl.deleteTexture(texture.textureId);
            results.push(delete texture.textureId);
          } else {
            results.push(void 0);
          }
        }
        return results;
      }
    };

    Live2DModel.prototype.reset = function() {
      var i, j, len, p, ref, results;
      ref = this.params;
      results = [];
      for (i = j = 0, len = ref.length; j < len; i = ++j) {
        p = ref[i];
        results.push(this.live2DModel.setParamFloat(i, p));
      }
      return results;
    };

    Live2DModel.prototype.loadFromBuffer = function(modelBuffer, glId) {
      var buffer, exp, f, i, j, k, l, len, len1, len2, len3, len4, m, model, motion, n, o, param, params, q, ref, ref1, ref2, ref3, ref4, ref5, s, settings, t, zip;
      if (!window.Live2DModelWebGL) {
        this.textures = [
          {
            loaded: true
          }
        ];
        return;
      }
      buffer = modelBuffer;
      zip = new JSZip(buffer);
      settings = JSON.parse(zip.file("settings.json").asText());
      model = zip.file("model.moc").asArrayBuffer();
      this.live2DModel = Live2DModelWebGL.loadModel(model, glId != null ? glId : 0);
      if (settings.motions != null) {
        for (k in settings.motions) {
          this.motionsByGroup[k] = [];
          ref = settings.motions[k];
          for (j = 0, len = ref.length; j < len; j++) {
            motion = ref[j];
            buffer = zip.file(motion.file).asArrayBuffer();
            m = Live2DMotion.loadMotion(buffer);
            this.motions[motion.file] = m;
            this.motionsByGroup[k].push(m);
          }
        }
      }
      if (settings.expressions != null) {
        ref1 = settings.expressions;
        for (l = 0, len1 = ref1.length; l < len1; l++) {
          exp = ref1[l];
          buffer = zip.file(exp.file).asArrayBuffer();
          this.expressions[exp.name] = L2DExpressionMotion.loadJson(buffer);
        }
      }
      if (settings.pose != null) {
        buffer = zip.file(settings.pose).asArrayBuffer();
        this.pose = L2DPose.load(buffer);
      }
      if (settings.physics != null) {
        buffer = zip.file(settings.physics).asArrayBuffer();
        this.physics = L2DPhysics.load(buffer);
      }
      if (settings.init_param != null) {
        ref2 = settings.init_param;
        for (n = 0, len2 = ref2.length; n < len2; n++) {
          param = ref2[n];
          this.live2DModel.setParamFloat(param.id, param.val);
        }
      }
      if (settings.init_parts_visible != null) {
        ref3 = settings.init_parts_visible;
        for (o = 0, len3 = ref3.length; o < len3; o++) {
          param = ref3[o];
          this.live2DModel.setPartsOpacity(param.id, param.val);
        }
      }
      params = this.live2DModel._$5S._$_2;
      for (i = q = 0, ref4 = params.length; 0 <= ref4 ? q <= ref4 : q >= ref4; i = 0 <= ref4 ? ++q : --q) {
        this.params[i] = params[i];
      }
      this.textures = new Array(settings.textures.length);
      ref5 = settings.textures;
      for (i = s = 0, len4 = ref5.length; s < len4; i = ++s) {
        t = ref5[i];
        buffer = zip.file(t).asArrayBuffer();
        f = (function(_this) {
          return function(tex, index) {
            var reader;
            reader = new FileReader();
            reader.onload = function(e) {
              var image;
              image = new Image();
              _this.textures[index] = image;
              image.onload = function() {
                return _this.textures[index].loaded = true;
              };
              return image.src = e.target.result.replace("data:", "data:image/png");
            };
            reader.onerror = function(e) {
              return console.log(e);
            };
            return reader.readAsDataURL(new Blob([buffer]));
          };
        })(this);
        f(t, i);
      }
      return this.emit("modelDataLoaded");
    };

    Live2DModel.prototype.loadFromFile = function(filePath, languageCode, glId) {
      var ext, resources, src;
      resources = GS.dataCache["RESOURCES"].items.resources;
      if (resources[filePath + "_" + languageCode] != null) {
        ext = resources[filePath + "_" + languageCode];
        src = "resources/" + GS.DataPreparer.prepareFilePath(filePath + "_" + languageCode + ext);
      } else {
        ext = resources[filePath] || ".live2d";
        src = "resources/" + GS.DataPreparer.prepareFilePath(filePath + ext);
      }
      return gs.Application.loadData(src, (function(_this) {
        return function(data) {
          return _this.loadFromBuffer(GS.DataPreparer.prepare(data), glId != null ? glId : 0);
        };
      })(this), (function(_this) {
        return function(e) {
          return console.log(e);
        };
      })(this));
    };


    /**
    * Updates the Live2D model. Should be called once per frame.
    *
    * @method update
     */

    Live2DModel.prototype.update = function() {
      return this.live2DModel.update();
    };

    return Live2DModel;

  })(gs.EventEmitter);

  window.gs.Live2DModel = Live2DModel;

}).call(this);

// Generated by CoffeeScript 1.12.2

/**
* Enumeration of different Live2D motion-group play types.
*
* @namespace gs
* @class Live2DMotionPlayTypes
* @memberof gs
* @constructor
* @static
* @final
 */

(function() {
  var Live2DMotionManager, Live2DMotionPlayTypes,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  Live2DMotionPlayTypes = (function() {

    /**
    * Plays all motions of a group in their order.
    * @property ORDERED
    * @type number
    * @static
    * @final
     */
    function Live2DMotionPlayTypes() {}

    Live2DMotionPlayTypes.ORDERED = 0;


    /**
    * Plays all motions of a group randomly.
    * @property RANDOM
    * @type number
    * @static
    * @final
     */

    Live2DMotionPlayTypes.RANDOM = 1;

    return Live2DMotionPlayTypes;

  })();

  window.gs.Live2DMotionPlayTypes = Live2DMotionPlayTypes;

  if (!window.MotionQueueManager) {
    window.MotionQueueManager = Object;
    window.Live2D = {
      isEnabled: false,
      init: function() {
        return null;
      },
      setGL: function(gl) {
        return null;
      },
      getGL: function() {
        return null;
      }
    };
    window.Live2DFramework = {
      setPlatformManager: function() {
        return null;
      }
    };
  }

  Live2DMotionManager = (function(superClass) {
    extend(Live2DMotionManager, superClass);


    /**
    * Manages Live2D motions and plays them on a model.
    *
    * @namespace gs
    * @class Live2DMotionManager
    * @memberof gs
    * @constructor
     */

    function Live2DMotionManager() {
      Live2DMotionManager.__super__.constructor.apply(this, arguments);
      this.motion = null;

      /**
      * The current motion-group.
      * @property motion
      * @type gs.Live2DMotionGroup
      * @readOnly
       */
      this.group = null;

      /**
      * The current motion play-type.
      * @property playType
      * @type gs.Live2DMotionPlayTypes
      * @readOnly
       */
      this.playType = 0;

      /**
      * The current motion.
      * @property currentMotion
      * @type gs.Live2DMotion
      * @readOnly
       */
      this.currentMotion = null;
      this.loop = true;
    }


    /**
    * Starts to play a specified motion-group.
    *
    * @method startMotionGroup
    * @param {gs.Live2DMotionGroup} group - The motion-group to play.
    * @param {gs.Live2DMotionPlayTypes} playType - The play type to use.
     */

    Live2DMotionManager.prototype.startMotionGroup = function(group, playType) {
      this.group = group;
      this.playType = playType;
      this.currentMotion = group[0];
      return this.startNextMotion();
    };


    /**
    * Starts the next motion of the motion-group depending on the play-type.
    *
    * @method startNextMotion
     */

    Live2DMotionManager.prototype.startNextMotion = function() {
      switch (this.playType) {
        case 0:
          this.currentMotion = this.group[this.group.indexOf(this.currentMotion) + 1 % this.group.length];
          break;
        case 1:
          this.currentMotion = this.group[Math.round(Math.random() * this.group.length)];
      }
      return this.startMotion(this.currentMotion, false);
    };


    /**
    * Updates the motion-manager and animation. Needs to be called once per frame.
    *
    * @method updateParam
    * @param {Object} model - Live2D model.
     */

    Live2DMotionManager.prototype.updateParam = function(model) {
      Live2DMotionManager.__super__.updateParam.call(this, model);
      if ((this.group != null) && this.isFinished() && this.loop) {
        return this.startNextMotion();
      }
    };

    return Live2DMotionManager;

  })(MotionQueueManager);

  window.gs.Live2DMotionManager = Live2DMotionManager;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var Live2DObject,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  if (window.L2DEyeBlink) {
    L2DEyeBlink.prototype.enabled = true;
  }

  Live2DObject = (function(superClass) {
    extend(Live2DObject, superClass);

    Object.defineProperty(Live2DObject.prototype, "width", {
      get: function() {
        var ref, result, s;
        result = 1;
        if ((ref = this.model) != null ? ref.live2DModel : void 0) {
          s = Graphics.height / this.model.live2DModel.getCanvasHeight();
          result = this.model.live2DModel.getCanvasWidth() * s * this.zoomX;
        }
        return result;
      }
    });

    Object.defineProperty(Live2DObject.prototype, "height", {
      get: function() {
        var ref, result, s;
        result = 1;
        if ((ref = this.model) != null ? ref.live2DModel : void 0) {
          s = Graphics.height / this.model.live2DModel.getCanvasHeight();
          result = this.model.live2DModel.getCanvasHeight() * s * this.zoomY;
        }
        return result;
      }
    });

    Object.defineProperty(Live2DObject.prototype, "talking", {
      get: function() {
        return this.talking_;
      },
      set: function(v) {
        var ref;
        if (v !== this.talking_) {
          this.talking_ = v;
          return (ref = this.model) != null ? ref.live2DModel.setParamFloat("PARAM_MOUTH_OPEN_Y", 0) : void 0;
        }
      }
    });

    Live2DObject.fromDataBundle = function(data, context) {
      var result;
      result = null;
      gs.ObjectCodec.decodeProperties(data, context);
      result = new gs.Live2DObject();
      Object.mixin(result, data);
      return result;
    };

    Live2DObject.toDataBundle = function(object, context) {
      var r;
      r = gs.ObjectCodec.encodeProperties(object, context, ["motionManager", "expressionManager"]);
      return r;
    };


    /**
    * Displays an associated Live2D model on screen.
    *
    *
    * @namespace gs
    * @class Live2DObject
    * @extends gs.GraphicObject
    * @memberof gs
    * @constructor
    * @param {gs.Viewport} [viewport=Graphics.viewport] - The viewport.
     */

    function Live2DObject(viewport) {
      Live2DObject.__super__.constructor.call(this, viewport);
      this.useRenderTexture = true;

      /**
      * The color tone.
      * @property tone
      * @type gs.Tone
       */
      this.tone = new Tone(0, 0, 0, 0);

      /**
      * The color. Can be used for flashing-effects, etc.
      * @property color
      * @type boolean
       */
      this.color = new Color(255, 255, 255, 0);

      /**
      * The mask used for masking-effects.
      * @property mask
      * @type gs.Mask
       */
      this.mask = {
        ox: 0,
        oy: 0,
        source: null,
        type: 0,
        value: 0,
        vague: 0
      };

      /**
      * The motion manager.
      * @property motionManager
      * @type gs.Live2DMotionManager
      * @readOnly
       */
      this.motionManager = new gs.Live2DMotionManager();

      /**
      * The expression manager.
      * @property expressionManager
      * @type MotionQueueManager
      * @readOnly
       */
      this.expressionManager = new MotionQueueManager();

      /**
      * Eye-Blink object to support eye blinking.
      * @property eyeBlink
      * @type L2DEyeBlink
       */
      this.eyeBlink = new L2DEyeBlink();

      /**
      * The Live2D model to render.
      * @property model
      * @type gs.Live2DModel
       */
      this.model = null;

      /**
      * The current motion.
      * @property currentMotion
      * @type gs.Live2DMotion
       */
      this.currentMotion = null;

      /**
      * Indicates if the motion should be looped.
      * @property loopMotion
      * @type boolean
       */
      this.loopMotion = true;

      /**
      * Indicates if the Live2D model is currently talking.
      * @property talking
      * @type boolean
       */
      this.talking = false;

      /**
      * The talking-volume if the Live2D model is talking. Goes from 0.0 to 1.0. The larger the value the more open is the mouth of the Live2D model.
      * @property talkingVolume
      * @type number
       */
      this.talkingVolume = 0.0;

      /**
      * The sensitivity of the lip-sync if the Live2D model is talking. The higher the value the more open the mouth
      * will become on high volume. A value of 0 disables lip-sync. The default is 1.0.
      * @property lipSyncSensitivity
      * @type number
      * @default 1.0
       */
      this.lipSyncSensitivity = 1.0;

      /**
      * The intensity of the breath-animation. The higher the value the more intensive the breath-animation
      * will become. A value of 0 disables the breath-animation. The default is 1.0.
      * @property breathIntensity
      * @type number
      * @default 1.0
       */
      this.breathIntensity = 1.0;

      /**
      * The intensity of the default idle-animation. The higher the value the more intensive the idle-animation
      * will become. A value of 0 disables the default idle-animation. The default is 1.0.
      * @property idleIntensity
      * @type number
      * @default 1.0
       */
      this.idleIntensity = 1.0;

      /**
      * The opacity. Goes from 0 to 255.
      * @property opacity
      * @type number
       */
      this.opacity = 255;

      /**
      * The x-coordinate of the object's position.
      * @property x
      * @type number
       */
      this.x = 0;

      /**
      * The y-coordinate of the object's position.
      * @property y
      * @type number
       */
      this.y = 0;

      /**
      * The zoom-factor on x-axis.
      * @property zoomX
      * @type number
       */
      this.zoomX = 1;

      /**
      * The zoom-factor on y-axis.
      * @property zoomY
      * @type number
       */
      this.zoomY = 1;

      /**
      * The rotation-angle in degrees.
      * @property angle
      * @type number
       */
      this.angle = 0;

      /**
      * The blend mode used for alpha-blending.
      * @property blendingMode
      * @type gs.BlendMode
       */
      this.blendingMode = 0;
      this.transform4 = mat4.create();
      this.viewportTransform4 = mat4.create();
      this.position3 = vec3.create();
      this.proj = mat4.create();
    }


    /**
    * Plays the specified motion-group.
    *
    * @method playMotionGroup
    * @param {string} name - The name of the motion.group to play.
    * @param {gs.Live2DMotionPlayTypes} playType - The play type to use.
     */

    Live2DObject.prototype.playMotionGroup = function(name, playType) {
      var group;
      group = this.model.motionsByGroup[name];
      if (group != null) {
        return this.motionManager.startMotionGroup(group, playType || 0);
      }
    };


    /**
    * Plays the specified motion.
    *
    * @method playMotionGroup
    * @param {string} name - The name of the motion to play
    * @param {number} fadeTime - The fade-in time of the motion
     */

    Live2DObject.prototype.playMotion = function(name, fadeTime) {
      var motion, ref;
      motion = (ref = this.model.motions[name]) != null ? ref : this.model.motions[Object.keys(this.model.motions).first()];
      if (motion != null) {
        motion.setFadeIn(fadeTime);
        return this.motionManager.startMotionGroup([motion], gs.Live2DMotionPlayTypes.ORDERED);
      }
    };


    /**
    * Sets the specified pose.
    *
    * @method setPose
    * @param {string} index - The index of the pose to set.
     */

    Live2DObject.prototype.setPose = function(index) {
      var i, j, len, part, partsGroup, ref, results;
      if (this.model.pose.lastModel !== this.model.live2DModel) {
        this.model.pose.initParam(this.model.live2DModel);
        this.model.pose.lastModel = this.model.live2DModel;
      }
      ref = this.model.pose.partsGroups;
      results = [];
      for (j = 0, len = ref.length; j < len; j++) {
        partsGroup = ref[j];
        results.push((function() {
          var k, len1, results1;
          results1 = [];
          for (i = k = 0, len1 = partsGroup.length; k < len1; i = ++k) {
            part = partsGroup[i];
            if (i === index) {
              this.model.live2DModel.setParamFloat(part.paramIndex, 1);
              results1.push(this.model.live2DModel.setPartsOpacity(part.partsIndex, 1));
            } else {
              this.model.live2DModel.setParamFloat(part.paramIndex, 0);
              results1.push(this.model.live2DModel.setPartsOpacity(part.partsIndex, 0));
            }
          }
          return results1;
        }).call(this));
      }
      return results;
    };


    /**
    * Sets the specified face expression.
    *
    * @method setExpression
    * @param {string} name - The name of the face expression to set.
    * @param {number} fadeTime - The fade-in time of the expression.
     */

    Live2DObject.prototype.setExpression = function(name, fadeTime) {
      var exp, ref;
      exp = (ref = this.model.expressions[name]) != null ? ref : this.model.expressions[Object.keys(this.model.expressions).first()];
      if (exp != null) {
        exp.setFadeIn(fadeTime != null ? fadeTime : 0);
        return this.expressionManager.startMotion(exp, false);
      }
    };


    /**
    * Passes a Live2D parameter to the Live2D model like "PARAM_ANGLE_X", etc.
    *
    * @method setParameter
    * @param {string} name - The name of the parameter to set.
    * @param {number} value - The value.
     */

    Live2DObject.prototype.setParameter = function(name, value) {
      return this.model.live2DModel.setParamFloat(name, value);
    };


    /**
    * Gets a Live2D parameter from the Live2D model like "PARAM_ANGLE_X", etc.
    *
    * @method getParameter
    * @param {string} name - The name of the parameter to get.
    * @return {number} The value of the parameter.
     */

    Live2DObject.prototype.getParameter = function(name) {
      return this.model.live2DModel.getParamFloat(name);
    };


    /**
    * Updates the Live2D object. Needs to be called once per frame.
    *
    * @method update
     */

    Live2DObject.prototype.update = function() {
      var ref, t;
      if (!((ref = this.model) != null ? ref.live2DModel : void 0)) {
        return;
      }
      this.motionManager.loop = this.loopMotion;
      this.motionManager.updateParam(this.model.live2DModel);
      if (!this.expressionManager.isFinished()) {
        this.expressionManager.updateParam(this.model.live2DModel);
      }
      this.model.live2DModel.saveParam();
      t = Date.now() / 1000 * 2 * Math.PI;
      this.model.live2DModel.addToParamFloat("PARAM_ANGLE_X", Number(15 * Math.sin(t / 6.5345)) * this.idleIntensity, 0.5);
      this.model.live2DModel.addToParamFloat("PARAM_ANGLE_Y", Number(8 * Math.sin(t / 3.5345)) * this.idleIntensity, 0.5);
      this.model.live2DModel.addToParamFloat("PARAM_ANGLE_Z", Number(10 * Math.sin(t / 5.5345)) * this.idleIntensity, 0.5);
      this.model.live2DModel.addToParamFloat("PARAM_BODY_ANGLE_X", Number(4 * Math.sin(t / 15.5345)) * this.idleIntensity, 0.5);
      if (this.breathIntensity > 0) {
        this.model.live2DModel.setParamFloat("PARAM_BREATH", Number(0.5 + 0.5 * Math.sin(t / 3.2345)) * this.breathIntensity, 1);
      }
      if (this.talking) {
        this.model.live2DModel.setParamFloat("PARAM_MOUTH_OPEN_Y", this.talkingVolume * this.lipSyncSensitivity);
      }
      if (this.model.pose != null) {
        this.model.pose.updateParam(this.model.live2DModel);
      }
      if (this.eyeBlink.enabled) {
        this.eyeBlink.updateParam(this.model.live2DModel);
      }
      if (this.model.physics) {
        return this.model.physics.updateParam(this.model.live2DModel);
      }
    };

    Live2DObject.prototype.draw_ = function() {
      var blendType, color, effects, h, height, i, j, len, logicalH, logicalW, model, oldMatrix, ref, ref1, ref2, s, texture, tone, w, width, x, y, zoomX, zoomY;
      if (!this.model || !this.model.live2DModel) {
        return;
      }
      this.update();
      model = this.model.live2DModel;
      ref = this.model.textures;
      for (i = j = 0, len = ref.length; j < len; i = ++j) {
        texture = ref[i];
        model.setTexture(i, texture.textureId);
      }
      mat4.identity(this.transform4);
      s = Graphics.height / model.getCanvasHeight();
      logicalW = Graphics.width;
      logicalH = Graphics.height;
      mat4.identity(this.proj);
      mat4.ortho(this.proj, 0, logicalW, logicalH, 0, -0.5, 0.5);
      y = this.y;
      width = model.getCanvasWidth() * s;
      height = model.getCanvasHeight() * s;
      x = this.x;
      zoomX = this.zoomX;
      zoomY = this.zoomY;
      this.position3[0] = x - (width * (zoomX - 1.0) * this.anchor.x);
      this.position3[1] = y - (height * (zoomY - 1.0) * this.anchor.y);
      mat4.translate(this.transform4, this.transform4, this.position3);
      if (this.angle !== 0) {
        this.position3[0] = width * zoomX * this.anchor.x;
        this.position3[1] = height * zoomY * this.anchor.y;
        mat4.translate(this.transform4, this.transform4, this.position3);
        mat4.rotateZ(this.transform4, this.transform4, Math.degreesToRadians(this.angle));
        this.position3[0] = -this.position3[0];
        this.position3[1] = -this.position3[1];
        mat4.translate(this.transform4, this.transform4, this.position3);
      }
      this.position3[0] = Graphics.height / model.getCanvasHeight() * zoomX;
      this.position3[1] = Graphics.height / model.getCanvasHeight() * zoomY;
      mat4.scale(this.transform4, this.transform4, this.position3);
      mat4.identity(this.viewportTransform4);
      if (this.useRenderTexture) {
        this.viewportTransform4[0] = this.viewport.transform[0];
        this.viewportTransform4[3] = this.viewport.transform[1];
        this.viewportTransform4[4] = this.viewport.transform[2];
        this.viewportTransform4[5] = -this.viewport.transform[3];
        this.viewportTransform4[12] = this.viewport.transform[4];
        this.viewportTransform4[13] = -this.viewport.transform[5] + height;
      } else {
        this.viewportTransform4[0] = this.viewport.transform[0];
        this.viewportTransform4[3] = this.viewport.transform[1];
        this.viewportTransform4[4] = this.viewport.transform[2];
        this.viewportTransform4[5] = this.viewport.transform[3];
        this.viewportTransform4[12] = this.viewport.transform[4];
        this.viewportTransform4[13] = this.viewport.transform[5];
      }
      mat4.multiply(this.transform4, this.viewportTransform4, this.transform4);
      mat4.multiply(this.transform4, this.proj, this.transform4);
      model.setMatrix(this.transform4);
      if (this.useRenderTexture) {
        $gl.cullFace($gl.FRONT);
        Graphics.frameBuffer.setTexture(Graphics.offscreenTexture);
        $gl.viewport(0, 0, Graphics.width, Graphics.height);
        $gl.scissor(0, 0, Graphics.width, Graphics.height);
        oldMatrix = Graphics.renderer.projectionMatrix;
        $gl.clearColor(0, 0, 0, 0);
        $gl.clear($gl.COLOR_BUFFER_BIT | $gl.DEPTH_BUFFER_BIT);
        $gl.clearColor(0, 0, 0, 1);
        model.update();
        model.draw();
        $gl.activeTexture($gl.TEXTURE0, 0);
        x = 0;
        y = 0;
        w = Graphics.offscreenTexture.width;
        h = Graphics.offscreenTexture.height;
        Graphics.renderer.vertexBuffer.bind();
        Graphics.renderer.indexBuffer.bind();
        Graphics.renderer.currentEffect = null;
        if ((ref1 = Graphics.renderer.texture) != null) {
          ref1.unbind();
        }
        Graphics.renderer.texture = null;
        Graphics.frameBuffer.bind();
        Graphics.frameBuffer.setTexture(Graphics.screenTexture);
        $gl.viewport(0, 0, Graphics.width, Graphics.height);
        $gl.scissor(0, 0, Graphics.width, Graphics.height);
        Graphics.renderer.projectionMatrix = oldMatrix;
        Graphics.renderer.setTexture(Graphics.offscreenTexture);
        Graphics.renderer.setSourceRectangle(x, y, w, h);
        Graphics.renderer.transform = this.transform;
        mat2d.identity(this.transform);
        this.position[0] = 0;
        this.position[1] = 0;
        mat2d.translate(this.transform, this.transform, this.position);
        this.position[0] = w;
        this.position[1] = h;
        mat2d.scale(this.transform, this.transform, this.position);
        Graphics.renderer.opacity = this.opacity;
        Graphics.renderer.blendType = 0;
        color = Graphics.renderer.color;
        tone = Graphics.renderer.tone;
        effects = Graphics.renderer.effects;
        Graphics.renderer.color = this.color;
        Graphics.renderer.tone = this.tone;
        Graphics.renderer.mask = this.mask;
        Graphics.renderer.effects = this.effects;
        Graphics.renderer.projectionMatrix = oldMatrix;
        $gl.blendFunc($gl.SRC_ALPHA, $gl.ONE_MINUS_SRC_ALPHA);
        if (this.effects.pixelate.enabled && this.effects.pixelate.textureFilter !== null) {
          Graphics.offscreenTexture.setFilter(this.effects.pixelate.textureFilter, this.effects.pixelate.mipmap);
        }
        $gl.cullFace($gl.BACK);
        blendType = Graphics.renderer.blendType;
        Graphics.renderer.blendType = this.blendingMode;
        Graphics.renderer.drawTransform();
        Graphics.renderer.render();
        Graphics.renderer.setTexture(null);
        Graphics.renderer.blendType = blendType;
        Graphics.renderer.tone = tone;
        Graphics.renderer.color = color;
        Graphics.renderer.mask = null;
        Graphics.renderer.effects = effects;
        $gl.enable($gl.SCISSOR_TEST);
      } else {
        model.update();
        model.draw();
        $gl.activeTexture($gl.TEXTURE0);
        $gl.bindTexture($gl.TEXTURE_2D, null);
        $gl.activeTexture($gl.TEXTURE1);
        $gl.bindTexture($gl.TEXTURE_2D, null);
        $gl.activeTexture($gl.TEXTURE2);
        $gl.bindTexture($gl.TEXTURE_2D, null);
        $gl.activeTexture($gl.TEXTURE0);
      }
      Graphics.renderer.setTexture(null);
      Graphics.renderer.vertexBuffer.bind();
      Graphics.renderer.indexBuffer.bind();
      Graphics.renderer.currentEffect = null;
      if ((ref2 = Graphics.renderer.texture) != null) {
        ref2.unbind();
      }
      Graphics.renderer.texture = null;
      this.model.live2DModel.loadParam();
      Graphics.frameBuffer.bind();
      Graphics.frameBuffer.setTexture(Graphics.screenTexture);
      $gl.disable($gl.CULL_FACE);
      $gl.enable($gl.SCISSOR_TEST);
      return $gl.scissor(0, 0, Graphics.width, Graphics.height);
    };


    /**
    * Draws the Live2D object. Needs to be called once per frame.
    *
    * @method draw
     */

    Live2DObject.prototype.draw = function() {
      if (this.model == null) {
        return;
      }
      return Graphics.renderer.drawLive2D(this);
    };

    return Live2DObject;

  })(GraphicObject);

  window.gs.Live2DObject = Live2DObject;

}).call(this);


function AutotileExpander()
{
}

AutotileExpander.substitutableFrames = [34, 36, 38, 40, 16, 20, 24, 28, 46]

/**
 * The size of a single tile.
 * @property tileSize
 * @type number
 * @static
 * @default 32
*/
AutotileExpander.tileSize = 32
/**
 * The current tileset configuration.
 * @property tilesetConfig
 * @type gs.TilesetConfiguration
 * @static
 */
AutotileExpander.tilesetConfig = null

/**
 * Expands an autotile.
 * @method expand
 * @param {gs.Bitmap} anAutotile - The autotile bitmap to expand
 * @return {gs.Bitmap} The expanded autotile bitmap.
 * @static
 */
AutotileExpander.expand = function(anAutotile)
{
    var th = AutotileExpander.tileSize;
    var tw = AutotileExpander.tileSize;

    if(anAutotile.height > AutotileExpander.tileSize*4)
    {
        if(anAutotile.width <= AutotileExpander.tileSize*8)
        {
            var bitmap = new Bitmap(tw*8*4, 8*th)
            for(var i = 0; i < 4; i++) { bitmap.blt(tw*8*i, 0, anAutotile, new Rect(0, 0, anAutotile.width, anAutotile.height)); }
            anAutotile = bitmap;
        }

        return anAutotile;
    }

    var autoTilesArea = AutotileExpander.tilesetConfig.autoTiles;
    var animationFrames = Math.floor(autoTilesArea.width / (tw*8))
    var width = tw * 3;
    var size = { width: anAutotile.width, height: anAutotile.height };
    var frames = size.width > width ? Math.floor(size.width / (tw*3)) : Math.floor(size.width / (tw*3));
    var autoTileset = new Bitmap(tw*8 * animationFrames, 8*th); //document.createElement("canvas"); //new Texture2D(256 * frames, 192);

    var gpx = autoTileset.context

    var autoTile = anAutotile;

    window.tw = tw;
    window.th = th;

    for(var i = 0; i < animationFrames; i++)
    {
    GSAutotileExpanderDrawImage.originX = i * (tw*8);
    GSAutotileExpanderDrawImage.sourceOriginX = Math.min(i, frames-1) * (3*tw);
    GSAutotileExpanderDrawImage.originY = 0;
    GSAutotileExpanderDrawImage.sourceOriginY = 0;

    // All
    GSAutotileExpanderDrawBackground(0, 0, gpx, autoTile);

    // CornerLeftTop
    GSAutotileExpanderDrawBackground(tw * 1, th * 0, gpx, autoTile);
    GSAutotileExpanderDrawLeftTopCorner(tw * 1, th * 0, gpx, autoTile);

    // CornerRightTop
    GSAutotileExpanderDrawBackground(tw * 2, th * 0, gpx, autoTile);
    GSAutotileExpanderDrawRightTopCorner(tw * 2 + Math.floor(tw / 2), th * 0, gpx, autoTile);

    // CornerLeftTop-CornerRightTop
    GSAutotileExpanderDrawBackground(tw * 3, th * 0, gpx, autoTile);
    GSAutotileExpanderDrawLeftTopCorner(tw * 3, th * 0, gpx, autoTile);
    GSAutotileExpanderDrawRightTopCorner(tw * 3 + Math.floor(tw / 2), th * 0, gpx, autoTile);

    // CornerRightBottom
    GSAutotileExpanderDrawBackground(tw * 4, th * 0, gpx, autoTile);
    GSAutotileExpanderDrawRightBottomCorner(tw * 4 + Math.floor(tw / 2), th * 0 + Math.floor(th / 2), gpx, autoTile);

    // CornerLeftTop-CornerRightBottom
    GSAutotileExpanderDrawBackground(tw * 5, th * 0, gpx, autoTile);
    GSAutotileExpanderDrawLeftTopCorner(tw * 5, th * 0, gpx, autoTile);
    GSAutotileExpanderDrawRightBottomCorner(tw * 5 + Math.floor(tw / 2), th * 0 + Math.floor(th / 2), gpx, autoTile);

    // CornerRightTop-CornerRightBottom
    GSAutotileExpanderDrawBackground(tw * 6, th * 0, gpx, autoTile);
    GSAutotileExpanderDrawRightTopCorner(tw * 6 + Math.floor(tw / 2), th * 0, gpx, autoTile);
    GSAutotileExpanderDrawRightBottomCorner(tw * 6 + Math.floor(tw / 2), th * 0 + Math.floor(th / 2), gpx, autoTile);

    // CornerLeftTop-CornerRightTop-CornerRightBottom
    GSAutotileExpanderDrawBackground(tw * 7, th * 0, gpx, autoTile);
    GSAutotileExpanderDrawLeftTopCorner(tw * 7, th * 0, gpx, autoTile);
    GSAutotileExpanderDrawRightTopCorner(tw * 7 + Math.floor(tw / 2), th * 0, gpx, autoTile);
    GSAutotileExpanderDrawRightBottomCorner(tw * 7 + Math.floor(tw / 2), th * 0 + Math.floor(th / 2), gpx, autoTile);

    // CornerLeftBottom
    GSAutotileExpanderDrawBackground(tw * 0, th * 1, gpx, autoTile);
    GSAutotileExpanderDrawLeftBottomCorner(tw * 0, th * 1 + Math.floor(th / 2), gpx, autoTile);

    // CornerLeftTop-CornerLeftBottom
    GSAutotileExpanderDrawBackground(tw * 1, th * 1, gpx, autoTile);
    GSAutotileExpanderDrawLeftTopCorner(tw * 1, th * 1, gpx, autoTile);
    GSAutotileExpanderDrawLeftBottomCorner(tw * 1, th * 1 + Math.floor(th / 2), gpx, autoTile);

    // CornerLeftBottom-CornerRightTop
    GSAutotileExpanderDrawBackground(tw * 2, th * 1, gpx, autoTile);
    GSAutotileExpanderDrawLeftBottomCorner(tw * 2, th * 1 + Math.floor(th / 2), gpx, autoTile);
    GSAutotileExpanderDrawRightTopCorner(tw * 2 + Math.floor(tw / 2), th * 1, gpx, autoTile);

    // CornerLeftTop-CornerRightTop-CornerLeftBottom
    GSAutotileExpanderDrawBackground(tw * 3, th * 1, gpx, autoTile);
    GSAutotileExpanderDrawLeftTopCorner(tw * 3, th * 1, gpx, autoTile);
    GSAutotileExpanderDrawRightTopCorner(tw * 3 + Math.floor(tw / 2), th * 1, gpx, autoTile);
    GSAutotileExpanderDrawLeftBottomCorner(tw * 3, th * 1 + Math.floor(th / 2), gpx, autoTile);

    // CornerRightBottom-CornerLeftBottom
    GSAutotileExpanderDrawBackground(tw * 4, th * 1, gpx, autoTile);
    GSAutotileExpanderDrawRightBottomCorner(tw * 4 + Math.floor(tw / 2), th * 1 + Math.floor(th / 2), gpx, autoTile);
    GSAutotileExpanderDrawLeftBottomCorner(tw * 4, th * 1 + Math.floor(th / 2), gpx, autoTile);

    // CornerLeftTop-CornerRightBottom-CornerLeftBottom
    GSAutotileExpanderDrawBackground(tw * 5, th * 1, gpx, autoTile);
    GSAutotileExpanderDrawLeftTopCorner(tw * 5, th * 1, gpx, autoTile);
    GSAutotileExpanderDrawRightBottomCorner(tw * 5 + Math.floor(tw / 2), th * 1 + Math.floor(th / 2), gpx, autoTile);
    GSAutotileExpanderDrawLeftBottomCorner(tw * 5, th * 1 + Math.floor(th / 2), gpx, autoTile);

    // CornerRightTop-CornerRightBottom-CornerLeftBottom
    GSAutotileExpanderDrawBackground(tw * 6, th * 1, gpx, autoTile);
    GSAutotileExpanderDrawRightTopCorner(tw * 6 + Math.floor(tw / 2), th * 1, gpx, autoTile);
    GSAutotileExpanderDrawRightBottomCorner(tw * 6 + Math.floor(tw / 2), th * 1 + Math.floor(th / 2), gpx, autoTile);
    GSAutotileExpanderDrawLeftBottomCorner(tw * 6, th * 1 + Math.floor(th / 2), gpx, autoTile);

    // Up-Down-Left-Right-CornerAll
    GSAutotileExpanderDrawLeftTopCorner(tw * 7, th * 1, gpx, autoTile);
    GSAutotileExpanderDrawRightTopCorner(tw * 7 + Math.floor(tw / 2), th * 1, gpx, autoTile);
    GSAutotileExpanderDrawLeftBottomCorner(tw * 7, th * 1 + Math.floor(th / 2), gpx, autoTile);
    GSAutotileExpanderDrawRightBottomCorner(tw * 7 + Math.floor(tw / 2), th * 1 + Math.floor(th / 2), gpx, autoTile);

    // Up-Down-Right
    GSAutotileExpanderDrawLeftSide(tw * 0, th * 2, gpx, autoTile);

    // Up-Down-Right-CornerUp
    GSAutotileExpanderDrawLeftSide(tw * 1, th * 2, gpx, autoTile);
    GSAutotileExpanderDrawRightTopCorner(tw * 1 + Math.floor(tw / 2), th * 2, gpx, autoTile);

    // Up-Down-Right-CornerDown
    GSAutotileExpanderDrawLeftSide(tw * 2, th * 2, gpx, autoTile);
    GSAutotileExpanderDrawRightBottomCorner(tw * 2 + Math.floor(tw / 2), th * 2 + Math.floor(th / 2), gpx, autoTile);

    // Up-Down-Right-CornerUpDown
    GSAutotileExpanderDrawLeftSide(tw * 3, th * 2, gpx, autoTile);
    GSAutotileExpanderDrawRightTopCorner(tw * 3 + Math.floor(tw / 2), th * 2, gpx, autoTile);
    GSAutotileExpanderDrawRightBottomCorner(tw * 3 + Math.floor(tw / 2), th * 2 + Math.floor(th / 2), gpx, autoTile);

    // Left-Right-Down
    GSAutotileExpanderDrawUpSide(tw * 4, th * 2, gpx, autoTile);

    // Left-Right-Down-CornerRight
    GSAutotileExpanderDrawUpSide(tw * 5, th * 2, gpx, autoTile);
    GSAutotileExpanderDrawRightBottomCorner(tw * 5 + Math.floor(tw / 2), th * 2 + Math.floor(th / 2), gpx, autoTile);

    // Left-Right-Down-CornerLeft
    GSAutotileExpanderDrawUpSide(tw * 6, th * 2, gpx, autoTile);
    GSAutotileExpanderDrawLeftBottomCorner(tw * 6, th * 2 + Math.floor(th / 2), gpx, autoTile);

    // Left-Right-Down-CornerLeftRight
    GSAutotileExpanderDrawUpSide(tw * 7, th * 2, gpx, autoTile);
    GSAutotileExpanderDrawLeftBottomCorner(tw * 7, th * 2 + Math.floor(th / 2), gpx, autoTile);
    GSAutotileExpanderDrawRightBottomCorner(tw * 7 + Math.floor(tw / 2), th * 2 + Math.floor(th / 2), gpx, autoTile);

    // Up-Down-Left
    GSAutotileExpanderDrawRightSide(tw * 0, th * 3, gpx, autoTile);

    // Up-Down-Left-CornerDown
    GSAutotileExpanderDrawRightSide(tw * 1, th * 3, gpx, autoTile);
    GSAutotileExpanderDrawLeftBottomCorner(tw * 1, th * 3 + Math.floor(th / 2), gpx, autoTile);

    // Up-Down-Left-CornerUp
    GSAutotileExpanderDrawRightSide(tw * 2, th * 3, gpx, autoTile);
    GSAutotileExpanderDrawLeftTopCorner(tw * 2, th * 3, gpx, autoTile);

    // Up-Down-Left-CornerUpDown
    GSAutotileExpanderDrawRightSide(tw * 3, th * 3, gpx, autoTile);
    GSAutotileExpanderDrawLeftTopCorner(tw * 3, th * 3, gpx, autoTile);
    GSAutotileExpanderDrawLeftBottomCorner(tw * 3, th * 3 + Math.floor(th / 2), gpx, autoTile);

    // Left-Right-Up
    GSAutotileExpanderDrawDownSide(tw * 4, th * 3, gpx, autoTile);

    // Left-Right-Up-CornerLeft
    GSAutotileExpanderDrawDownSide(tw * 5, th * 3, gpx, autoTile);
    GSAutotileExpanderDrawLeftTopCorner(tw * 5, th * 3, gpx, autoTile);

    // Left-Right-Up-CornerRight
    GSAutotileExpanderDrawDownSide(tw * 6, th * 3, gpx, autoTile);
    GSAutotileExpanderDrawRightTopCorner(tw * 6 + Math.floor(tw / 2), th * 3, gpx, autoTile);

    // Left-Right-Up-CornerLeftRight
    GSAutotileExpanderDrawDownSide(tw * 7, th * 3, gpx, autoTile);
    GSAutotileExpanderDrawLeftTopCorner(tw * 7, th * 3, gpx, autoTile);
    GSAutotileExpanderDrawRightTopCorner(tw * 7 + Math.floor(tw / 2), th * 3, gpx, autoTile);

    // Up-Down
    GSAutotileExpanderDrawLeftSideSmall(tw * 0, th * 4, gpx, autoTile);
    GSAutotileExpanderDrawRightSideSmall(tw * 0 + Math.floor(tw / 2), th * 4, gpx, autoTile);

    // Left-Right
    GSAutotileExpanderDrawUpSideSmall(tw * 1, th * 4, gpx, autoTile);
    GSAutotileExpanderDrawDownSideSmall(tw * 1, th * 4 + Math.floor(th / 2), gpx, autoTile);

    // Right-Down
    GSAutotileExpanderDrawLeftTop(tw * 2, th * 4, gpx, autoTile);

    // Right-Down-Corner
    GSAutotileExpanderDrawLeftTop(tw * 3, th * 4, gpx, autoTile);
    GSAutotileExpanderDrawRightBottomCorner(tw * 3 + Math.floor(tw / 2), th * 4 + Math.floor(th / 2), gpx, autoTile);

    // Left-Down
    GSAutotileExpanderDrawRightTop(tw * 4, th * 4, gpx, autoTile);

    // Left-Down-Corner
    GSAutotileExpanderDrawRightTop(tw * 5, th * 4, gpx, autoTile);
    GSAutotileExpanderDrawLeftBottomCorner(tw * 5, th * 4 + Math.floor(th / 2), gpx, autoTile);

    // Left-Up
    GSAutotileExpanderDrawRightBottom(tw * 6, th * 4, gpx, autoTile);

    // Left-Up-Corner
    GSAutotileExpanderDrawRightBottom(tw * 7, th * 4, gpx, autoTile);
    GSAutotileExpanderDrawLeftTopCorner(tw * 7, th * 4, gpx, autoTile);

    // Right-Up
    GSAutotileExpanderDrawLeftBottom(tw * 0, th * 5, gpx, autoTile);

    // Right-Up-Corner
    GSAutotileExpanderDrawLeftBottom(tw * 1, th * 5, gpx, autoTile);
    GSAutotileExpanderDrawRightTopCorner(tw * 1 + Math.floor(tw / 2), th * 5, gpx, autoTile);

    // Down
    GSAutotileExpanderDrawLeftSideTop(tw * 2, th * 5, gpx, autoTile);
    GSAutotileExpanderDrawRightSideTop(tw * 2 + Math.floor(tw / 2), th * 5, gpx, autoTile);

    // Right
    GSAutotileExpanderDrawUpSideLeft(tw * 3, th * 5, gpx, autoTile);
    GSAutotileExpanderDrawDownSideLeft(tw * 3, th * 5 + Math.floor(th / 2), gpx, autoTile);

    // Up
    GSAutotileExpanderDrawLeftSideBottom(tw * 4, th * 5, gpx, autoTile);
    GSAutotileExpanderDrawRightSideBottom(tw * 4 + Math.floor(tw / 2), th * 5, gpx, autoTile);

    // Left
    GSAutotileExpanderDrawUpSideRight(tw * 5, th * 5, gpx, autoTile);
    GSAutotileExpanderDrawDownSideRight(tw * 5, th * 5 + Math.floor(th / 2), gpx, autoTile);

    // None
    GSAutotileExpanderDrawLeftTopSmall(tw * 6, th * 5, gpx, autoTile);
    GSAutotileExpanderDrawLeftBottomSmall(tw * 6, th * 5 + Math.floor(th / 2), gpx, autoTile);
    GSAutotileExpanderDrawRightTopSmall(tw * 6 + Math.floor(tw / 2), th * 5, gpx, autoTile);
    GSAutotileExpanderDrawRightBottomSmall(tw * 6 + Math.floor(tw / 2), th * 5 + Math.floor(th / 2), gpx, autoTile);

    // None
    GSAutotileExpanderDrawLeftTopSmall(tw * 7, th * 5, gpx, autoTile);
    GSAutotileExpanderDrawLeftBottomSmall(tw * 7, th * 5 + Math.floor(th / 2), gpx, autoTile);
    GSAutotileExpanderDrawRightTopSmall(tw * 7 + Math.floor(tw / 2), th * 5, gpx, autoTile);
    GSAutotileExpanderDrawRightBottomSmall(tw * 7 + Math.floor(tw / 2), th * 5 + Math.floor(th / 2), gpx, autoTile);
    }

 /*   for(var k = 0; k < 4; k++) {
        for (var i = 0; i < 8; i++)
            gpx.drawImage(autoTileset.image, 8*tw*k, 0, tw, th, (k*8+i) * tw, 7 * th, tw, th)

        for (var j = 0; j < AutotileExpander.substitutableFrames.length - 1; j++) {
            var f = AutotileExpander.substitutableFrames[j];
            gpx.drawImage(autoTileset.image, 8*tw*k + (f % 8) * tw, Math.floor(AutotileExpander.substitutableFrames[j]/8) * th, tw, th, (k*8+j)*tw, 6 * th, tw, th)
        }
    }

    console.log(autoTileset.image.toDataURL("image/png"))*/

    return autoTileset;
}

function GSAutotileExpanderDrawImage(context, image, x, y, w, h, sx, sy, sw, sh)
{
    var sourceOriginX = GSAutotileExpanderDrawImage.sourceOriginX;
    var sourceOriginY = GSAutotileExpanderDrawImage.sourceOriginY;
    var originX = GSAutotileExpanderDrawImage.originX;
    var originY = GSAutotileExpanderDrawImage.originY;

    context.drawImage(image.image, sourceOriginX + sx, sourceOriginY + sy, sw - sx, sh - sy, originX + x, originY + y, sw - sx, sh - sy)
}

function GSAutotileExpanderDrawLeftTopSmall(x, y, g, autoTile)
{
    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + Math.floor(Math.floor(tw / 2)), y + Math.floor(th / 2), 0, th, Math.floor(tw / 2), th + Math.floor(th / 2));
    }

function GSAutotileExpanderDrawRightTopSmall(x, y, g, autoTile)
{
    var sx = tw * 2 + Math.floor(tw / 2);
    var sy = th;
    var sw = sx + Math.floor(tw / 2);
    var sh = sy + Math.floor(th / 2);

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + Math.floor(tw / 2), y + Math.floor(th / 2), sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawLeftBottomSmall(x, y, g, autoTile)
{
    var sx = 0;
    var sy = th * 3 + Math.floor(th / 2);
    var sw = sx + Math.floor(tw / 2);
    var sh = sy + Math.floor(th / 2);

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + Math.floor(tw / 2), y + Math.floor(th / 2), sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawRightBottomSmall(x, y, g, autoTile)
{
    var sx = tw * 2 + Math.floor(tw / 2);
    var sy = th * 3 + Math.floor(th / 2);
    var sw = sx + Math.floor(tw / 2);
    var sh = sy + Math.floor(th / 2);

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + Math.floor(tw / 2), y + Math.floor(th / 2), sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawLeftTopCorner(x, y, g, autoTile)
{
    var sx = tw * 2;
    var sy = 0;
    var sw = sx + Math.floor(tw / 2);
    var sh = sy + Math.floor(th / 2);

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + Math.floor(tw / 2), y + Math.floor(th / 2), sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawRightTopCorner(x, y, g, autoTile)
{
    var sx = tw * 2 + Math.floor(tw / 2);
    var sy = 0;
    var sw = sx + Math.floor(tw / 2);
    var sh = sy + Math.floor(th / 2);

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + Math.floor(tw / 2), y + Math.floor(th / 2), sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawLeftBottomCorner(x, y, g, autoTile)
{
    var sx = tw * 2;
    var sy = Math.floor(th / 2);
    var sw = sx + Math.floor(tw / 2);
    var sh = sy + Math.floor(th / 2);

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + Math.floor(tw / 2), y + Math.floor(th / 2), sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawRightBottomCorner(x, y, g, autoTile)
{
    var sx = tw * 2 + Math.floor(tw / 2);
    var sy = Math.floor(th / 2);
    var sw = sx + Math.floor(tw / 2);
    var sh = sy + Math.floor(th / 2);

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + Math.floor(tw / 2), y + Math.floor(th / 2), sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawLeftSideTop(x, y, g, autoTile)
{
    var sx = 0;
    var sy = th;
    var sw = sx + Math.floor(tw / 2);
    var sh = sy + th;

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + Math.floor(tw / 2), y + th, sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawRightSideTop(x, y, g, autoTile)
{
    var sx = tw * 2 + Math.floor(tw / 2);
    var sy = th;
    var sw = sx + Math.floor(tw / 2);
    var sh = sy + th;

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + Math.floor(tw / 2), y + th, sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawLeftSideBottom(x, y, g, autoTile)
{
    var sx = 0;
    var sy = th * 3;
    var sw = sx + Math.floor(tw / 2);
    var sh = sy + th;

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + Math.floor(tw / 2), y + th, sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawRightSideBottom(x, y, g, autoTile)
{
    var sx = tw * 2 + Math.floor(tw / 2);
    var sy = th * 3;
    var sw = sx + Math.floor(tw / 2);
    var sh = sy + th;

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + Math.floor(tw / 2), y + th, sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawUpSideLeft(x, y, g, autoTile)
{
    var sx = 0;
    var sy = th;
    var sw = sx + tw;
    var sh = sy + Math.floor(th / 2);

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + tw, y + Math.floor(th / 2), sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawUpSideRight(x, y, g, autoTile)
{
    var sx = tw * 2;
    var sy = th;
    var sw = sx + tw;
    var sh = sy + Math.floor(th / 2);

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + tw, y + Math.floor(th / 2), sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawDownSideLeft(x, y, g, autoTile)
{
    var sx = 0;
    var sy = th * 3 + Math.floor(th / 2);
    var sw = sx + tw;
    var sh = sy + Math.floor(th / 2);

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + tw, y + Math.floor(th / 2), sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawDownSideRight(x, y, g, autoTile)
{
    var sx = tw * 2;
    var sy = th * 3 + Math.floor(th / 2);
    var sw = sx + tw;
    var sh = sy + Math.floor(th / 2);

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + tw, y + Math.floor(th / 2), sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawLeftSideSmall(x, y, g, autoTile)
{
    var sx = 0;
    var sy = th * 2;
    var sw = sx + Math.floor(tw / 2);
    var sh = sy + th;

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + Math.floor(tw / 2), y + th, sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawRightSideSmall(x, y, g, autoTile)
{
    var sx = tw * 2 + Math.floor(tw / 2);
    var sy = th * 2;
    var sw = sx + Math.floor(tw / 2);
    var sh = sy + th;

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + Math.floor(tw / 2), y + th, sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawUpSideSmall(x, y, g, autoTile)
{
    var sx = tw;
    var sy = th;
    var sw = sx + tw;
    var sh = sy + Math.floor(th / 2);

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + tw, y + Math.floor(th / 2), sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawDownSideSmall(x, y, g, autoTile)
{
    var sx = tw;
    var sy = th * 3 + Math.floor(th / 2);
    var sw = sx + tw;
    var sh = sy + Math.floor(th / 2);

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + tw, y + Math.floor(th / 2), sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawLeftSide(x, y, g, autoTile)
{
    var sx = 0;
    var sy = th * 2;
    var sw = sx + tw;
    var sh = sy + th;

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + tw, y + th, sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawRightSide(x, y, g, autoTile)
{
    var sx = tw * 2;
    var sy = th * 2;
    var sw = sx + tw;
    var sh = sy + th;

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + tw, y + th, sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawUpSide(x, y, g, autoTile)
{
    var sx = tw;
    var sy = th;
    var sw = sx + tw;
    var sh = sy + th;

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + tw, y + th, sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawDownSide(x, y, g, autoTile)
{
    var sx = tw;
    var sy = th * 3;
    var sw = sx + tw;
    var sh = sy + th;

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + tw, y + th, sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawLeftTop(x, y, g, autoTile)
{
    var sx = 0;
    var sy = th;
    var sw = sx + tw;
    var sh = sy + th;

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + tw, y + th, sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawRightTop(x, y, g, autoTile)
{
    var sx = tw * 2;
    var sy = th;
    var sw = sx + tw;
    var sh = sy + th;

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + tw, y + th, sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawLeftBottom(x, y, g, autoTile)
{
    var sx = 0;
    var sy = th * 3;
    var sw = sx + tw;
    var sh = sy + th;

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + tw, y + th, sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawRightBottom(x, y, g, autoTile)
{
    var sx = tw * 2;
    var sy = th * 3;
    var sw = sx + tw;
    var sh = sy + th;

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + tw, y + th, sx, sy, sw, sh);
    }

function GSAutotileExpanderDrawBackground(x, y, g, autoTile)
{
    var sx = tw;
    var sy = th * 2;
    var sw = sx + tw;
    var sh = sy + th;

    GSAutotileExpanderDrawImage(g, autoTile, x, y, x + tw, y + th, sx, sy, sw, sh);
}

gs.AutotileExpander = AutotileExpander;
// Generated by CoffeeScript 1.12.2
(function() {
  var Tilemap, TilemapLayer_OpenGL, TilemapSprite, Tilemap_OpenGL, TilesetConfiguration,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  TilemapSprite = (function(superClass) {
    extend(TilemapSprite, superClass);

    function TilemapSprite(viewport, tileSize) {
      TilemapSprite.__super__.constructor.call(this, viewport);
      this.bitmap = null;
      this.x = 0;
      this.y = 0;
      this.z = -50;
      this.srcRect = new Rect(0, 0, 0, 0);
      this.visible = false;
      this.tileSize = tileSize;
      this.contextInfo = Graphics.renderer.contextInfo;
      this.context = Graphics.renderer.context;
    }

    TilemapSprite.prototype.draw = function() {
      if (this.bitmap == null) {
        return;
      }
      if ((this.contextInfo != null) && this.contextInfo.globalCompositeOperation !== 0) {
        this.context.globalCompositeOperation = "source-over";
        this.contextInfo.globalCompositeOperation = 0;
      }
      if ((this.contextInfo != null) && this.contextInfo.globalAlpha !== 1.0) {
        this.context.globalAlpha = 1.0;
        this.contextInfo.globalAlpha = 1.0;
      }
      return Graphics.renderer.drawBitmapSimple(this.bitmap, this.x - this.ox, this.y - this.oy, this.tileSize, this.tileSize, this.srcRect.x, this.srcRect.y, this.tileSize, this.tileSize);
    };

    return TilemapSprite;

  })(GraphicObject);

  Tilemap = (function(superClass) {
    extend(Tilemap, superClass);

    function Tilemap(viewport) {
      Tilemap.__super__.constructor.call(this, viewport);
      this.autotiles = new Array(7);
      this.expandedAutotiles = new Array(7);
      this.tileset = null;
      this.priorities = null;
      this.passages = null;
      this.mapData = null;
      this.tileSize = 32;
      this.sprites = [];
      this.autotileOffset = 0;
      this.autotileOffset2 = 0;
      this.depth = 3;
      this.cache = [];

      /*
      tilesX = Math.ceil(Graphics.width / @tileSize) + 2
      tilesY = Math.ceil(Graphics.height / @tileSize) + 2
      tilesZ = 3
      tileCount = tilesX * tilesY * 3
      
      for x in [0...tilesX]
        for y in [0...tilesY]
          for z in [0...tilesZ]
            @sprites[y * tilesX + x + (tilesX * tilesY * z)] = new TilemapSprite(viewport)
            @sprites[y * tilesX + x + (tilesX * tilesY * z)].x = x * 32
            @sprites[y * tilesX + x + (tilesX * tilesY * z)].y = y * 32
            @sprites[y * tilesX + x + (tilesX * tilesY * z)].z = 32 + z
       */
    }

    Tilemap.prototype.expandAutotiles = function() {
      var i, k, ref, results;
      AutotileExpander.tileSize = this.tileSize;
      results = [];
      for (i = k = 0, ref = this.autotiles.length; 0 <= ref ? k < ref : k > ref; i = 0 <= ref ? ++k : --k) {
        if (this.autotiles[i]) {
          results.push(this.expandedAutotiles[i] = AutotileExpander.expand(this.autotiles[i]));
        } else {
          results.push(void 0);
        }
      }
      return results;
    };

    Tilemap.prototype.clearCache = function(indices) {
      var i, k, ref;
      for (i = k = 0, ref = indices.length; 0 <= ref ? k < ref : k > ref; i = 0 <= ref ? ++k : --k) {
        this.cache[indices[i]] = null;
      }
      return null;
    };

    Tilemap.prototype.draw = function() {};

    Tilemap.prototype.update = function() {
      var autotile, autotileOffset, autotileWidth, data, frameX, frameY, framesX, i, index, k, l, m, n, ox, oy, priority, pxHeight, pxWidth, ref, ref1, ref2, ref3, ref4, ref5, ref6, renderer, scrollX, scrollY, shiftX, shiftY, sprite, spriteIndex, tile, tilesX, tilesY, tx, ty, x, y, z;
      Tilemap.__super__.update.call(this);
      if (!this.visible || (this.tileset == null) || !this.tileset.loaded || (this.mapData == null)) {
        return;
      }
      renderer = Graphics.renderer;
      pxWidth = this.width * this.tileSize;
      pxHeight = this.height * this.tileSize;
      ox = Math.floor(this.ox + this.viewport.ox);
      oy = Math.floor(this.oy + this.viewport.oy);
      framesX = Math.ceil(this.tileset.width / this.tileSize);
      frameX = 0;
      frameY = 0;
      tilesX = Math.floor(Graphics.width / this.tileSize) + 2;
      tilesY = Math.floor(Graphics.height / this.tileSize) + 2;
      shiftX = 0;
      shiftY = 0;
      scrollX = Math.floor(ox / this.tileSize);
      scrollY = Math.floor(oy / this.tileSize);
      sprite = null;
      priority = 0;
      autotileOffset = 0;
      autotileWidth = Graphics.height === 480 ? 256 : 128;
      spriteIndex = 0;
      tx = 0;
      ty = 0;
      if (Math.floor(Graphics.frameCount % 16) === 0) {
        this.autotileOffset = (this.autotileOffset + 1) % 4;
        this.autotileOffset2 = (this.autotileOffset2 + 1) % 3;
      }
      for (x = k = ref = scrollX, ref1 = tilesX + scrollX; ref <= ref1 ? k < ref1 : k > ref1; x = ref <= ref1 ? ++k : --k) {
        tx = x < 0 ? this.width + x : (x >= this.width ? x - this.width : x);
        for (y = l = ref2 = scrollY, ref3 = tilesY + scrollY; ref2 <= ref3 ? l < ref3 : l > ref3; y = ref2 <= ref3 ? ++l : --l) {
          ty = y < 0 ? this.height + y : (y >= this.height ? y - this.height : y);
          for (z = m = 0, ref4 = this.depth; 0 <= ref4 ? m < ref4 : m > ref4; z = 0 <= ref4 ? ++m : --m) {
            index = (ty * this.width + tx) + (this.width * this.height) * z;
            data = this.cache[index];
            if (data != null) {
              if (data.visible) {
                sprite = this.sprites[spriteIndex];
                if (sprite == null) {
                  sprite = new TilemapSprite(this.viewport, this.tileSize);
                  this.sprites[spriteIndex] = sprite;
                }
                spriteIndex++;
                sprite.visible = true;
                sprite.z = data.z;
                sprite.bitmap = data.bitmap;
                sprite.x = x * this.tileSize;
                sprite.y = y * this.tileSize;
                sprite.opacity = 255;
                sprite.srcRect.x = data.frameX;
                sprite.srcRect.y = data.frameY;
                sprite.srcRect.width = this.tileSize;
                sprite.srcRect.height = this.tileSize;
                if (data.autotile && (sprite.bitmap != null) && sprite.bitmap.width > autotileWidth) {
                  if (sprite.bitmap.width > (autotileWidth * 3)) {
                    autotileOffset = this.autotileOffset * autotileWidth;
                  } else {
                    autotileOffset = this.autotileOffset2 * autotileWidth;
                  }
                  sprite.srcRect.x += autotileOffset;
                }
              }
            } else {
              tile = this.mapData[index];
              if (tile <= 0) {
                this.cache[index] = {
                  visible: false
                };
                continue;
              }
              sprite = this.sprites[spriteIndex];
              if (sprite == null) {
                sprite = new TilemapSprite(this.viewport, this.tileSize);
                this.sprites[spriteIndex] = sprite;
              }
              spriteIndex++;
              autotile = Math.floor(tile / 64);
              sprite.visible = true;
              priority = this.priorities[tile] || 0;
              sprite.z = (32 + priority * 32) + z;
              if (autotile <= 7) {
                tile -= autotile * 64;
                sprite.bitmap = this.expandedAutotiles[autotile];
                autotileOffset = 0;
                if ((sprite.bitmap != null) && sprite.bitmap.width > autotileWidth) {
                  if (sprite.bitmap.width > (autotileWidth * 3)) {
                    autotileOffset = this.autotileOffset * autotileWidth;
                  } else {
                    autotileOffset = this.autotileOffset2 * autotileWidth;
                  }
                }
                frameX = (tile % framesX) * this.tileSize;
                frameY = Math.floor(tile / framesX) * this.tileSize;
              } else {
                sprite.bitmap = this.tileset;
                frameX = ((tile - 512) % framesX) * this.tileSize;
                frameY = Math.floor((tile - 512) / framesX) * this.tileSize;
              }
              sprite.x = x * this.tileSize;
              sprite.y = y * this.tileSize;
              sprite.srcRect.x = frameX;
              sprite.srcRect.y = frameY;
              sprite.srcRect.width = this.tileSize;
              sprite.srcRect.height = this.tileSize;
              this.cache[index] = {
                x: sprite.x,
                y: sprite.y,
                frameX: frameX,
                frameY: frameY,
                bitmap: sprite.bitmap,
                z: sprite.z,
                visible: true,
                autotile: autotile <= 7
              };
            }
          }
        }
      }
      for (i = n = ref5 = spriteIndex, ref6 = this.sprites.length; ref5 <= ref6 ? n < ref6 : n > ref6; i = ref5 <= ref6 ? ++n : --n) {
        this.sprites[i].visible = false;
      }
      return null;
    };

    Tilemap.prototype.dispose = function() {
      var k, len, ref, results, sprite;
      Tilemap.__super__.dispose.apply(this, arguments);
      ref = this.sprites;
      results = [];
      for (k = 0, len = ref.length; k < len; k++) {
        sprite = ref[k];
        results.push(sprite.dispose());
      }
      return results;
    };

    Tilemap.prototype.hide = function(duration) {
      var i, k, ref, results;
      Tilemap.__super__.hide.call(this, duration);
      results = [];
      for (i = k = 0, ref = this.sprites.length; 0 <= ref ? k < ref : k > ref; i = 0 <= ref ? ++k : --k) {
        results.push(this.sprites[i].visible = false);
      }
      return results;
    };

    Tilemap.prototype.show = function(visible) {
      var i, k, l, ref, ref1;
      if (visible === this.visible || (this.hideCounter != null)) {
        return;
      }
      if (visible) {
        for (i = k = 0, ref = this.sprites.length; 0 <= ref ? k < ref : k > ref; i = 0 <= ref ? ++k : --k) {
          this.sprites[i].initialize();
        }
        this.visible = true;
      } else {
        for (i = l = 0, ref1 = this.sprites.length; 0 <= ref1 ? l < ref1 : l > ref1; i = 0 <= ref1 ? ++l : --l) {
          this.sprites[i].dispose();
        }
        this.visible = false;
      }
      return null;
    };

    return Tilemap;

  })(GraphicObject);

  TilesetConfiguration = (function() {

    /**
    * A tileset configuration is used for gs.Tilemap to describe the structure of the tileset. For more
    * info, check out the "Tilemaps" chapter in "Scripter's Guide" in help-file.
    *
    * @namespace gs
    * @class TilesetConfiguration
    * @memberof gs
    * @constructor
    * @param {number} tileSize - The size of a single tile on the tileset. A single tile needs to be quadratic so a value of 32
    * means 32x32 pixels. If omitted, a value of 32 is used.
     */
    function TilesetConfiguration(tileSize) {
      tileSize = tileSize != null ? tileSize : 32;

      /**
      * </p>Indicates if no internal tileset should be created and the original tileset image should be used as is. In that case,
      * the expandedSize, regularTiles and specialTiles properties have no effect. In addition, gs.Tilemap.autotiles
      * are ignored too and no autotile expansion will happen. You have to prepare the original tileset image yourself.</p>
      * <p>Notice that this is set to <b>true</b> by default.</p>
      * @property useOriginalTileset
      * @type boolean
      * @default true
       */
      this.useOriginalTileset = true;

      /**
      * The tile-id used for invisible tiles. Its recommended to use the invisible tile id for non-visible
      * tiles instead of choosing a random transparent tile on the tileset because it safes performance.
      *
      * @property invisibleTileId
      * @type number
      * @default 32767
       */
      this.invisibleTileId = 32767;

      /**
      * The internal tileset size. The original tileset will be copied into the internal tileset at the position
      * defined in regularTiles area. That only happens if useOriginalTileset is set to <b>false</b>.
      *
      * @property expandedSize
      * @type gs.Size
      * @default { width: 2048, height: 2048 }
       */
      this.expandedSize = {
        width: 2048,
        height: 2048
      };

      /**
      * The original tileset will be copied into the internal tileset at the position
      * defined here. That only happens if useOriginalTileset is set to <b>false</b>.
      *
      * @property expandedSize
      * @type gs.Rect
      * @default { 0, 0, tileSize * 8, tileSize * 64 }
       */
      this.regularTiles = new gs.Rect(0, 0, tileSize * 8, tileSize * 64);

      /**
      * <p>Defines the area on the tileset used for special tiles such as auto/terrain or animated tiles.
      * It is important that the y-coordinate of this area is always 0 and the height should match
      * the tileset height or should be dividable by the special tile height.</p>
      *
      * <p>Only the x-coordinate and width can be configured. The more the x-coordinate is moved to the right, the
      * the more space becomes available for regular tiles. But in exchange for that, the space for animation-frames
      * becomes less. So if you don't use animated tiles at all, you can move the x-coordinate completely to right.</p>
      *
      * <p>There is technically no difference between an auto/terrain or animated tile. Only one kind of special tiles
      * is supported per tileset. For Example: If you decide to use auto/terrain tile taking 256x192 pixels in expanded state, you
      * cannot have 32x32 animated tiles in addition. In that case, your animated tiles need to be auto/terrain tiles as
      * well with 256x192 offset per animation frame. That kind of limitation is necessary for optimization on mobile devices.</p>
      *
      * @property specialTiles
      * @type gs.Rect
      * @default { tileSize * 8, 0, tileSize * 8 * 4, tileSize * 6 * 10 }
       */
      this.specialTiles = new gs.Rect(tileSize * 8, 0, tileSize * 8 * 4, tileSize * 6 * 10);

      /**
      * <p>Defines the size of a special/animated tile. For normal animated tiles, this size is just the
      * regular tile size. But for auto or terrain tiles, this is the size of the expanded auto/terrain-tileset and not
      * of a single tile.</p>
      * <p>This setting is important to animate auto / terrain tiles correctly. The width is used as the animation-offset.</p>
      *
      *
      * @property @specialTileSize
      * @type gs.Rect
      * @default { width: tileSize*8, height: tileSize*6 }
       */
      this.specialTileSize = {
        width: tileSize * 8,
        height: tileSize * 6
      };

      /**
      * Number of animation frames for animated tiles.
      *
      * @property animationFrames
      * @type number
      * @default 4
       */
      this.animationFrames = 4;
    }

    return TilesetConfiguration;

  })();

  gs.TilesetConfiguration = TilesetConfiguration;

  Tilemap_OpenGL = (function(superClass) {
    extend(Tilemap_OpenGL, superClass);


    /**
    * A tilemap can be used to display the environment of a level/map of a game. For more information
    * about how to use tilemaps, features and limitations please read the "Scripter's Guide" in the help-file.
    *
    * @namespace gs
    * @class Tilemap
    * @memberof gs
    * @constructor
    * @param {gs.Viewport} viewport - The viewport to associate the tilemap with.
     */

    function Tilemap_OpenGL(viewport) {
      Tilemap_OpenGL.__super__.constructor.call(this, viewport);

      /**
      * The color tone.
      * @property tone
      * @type gs.Tone
       */
      this.tone = new Tone(0, 0, 0, 0);

      /**
      * The color. Can be used for flashing-effects, etc.
      * @property color
      * @type boolean
       */
      this.color = new Color(255, 255, 255, 0);

      /**
      * The mask used for masking-effects.
      * @property mask
      * @type gs.Mask
       */
      this.mask = {
        ox: 0,
        oy: 0,
        source: null,
        type: 0,
        value: 0,
        vague: 0
      };

      /**
      * An array of autotile-tiles in standard format which are automatically expanded and drawn on the
      * expanded tileset bitmap.
      *
      * @property autotiles
      * @type gs.Bitmap[]
       */
      this.autotiles = new Array(7);

      /**
      * An array of expanded autotile-tiles. They are automatically generated from autotiles-property.
      *
      * @property expandedAutotiles
      * @type gs.Bitmap[]
       */
      this.expandedAutotiles = new Array(7);

      /**
      * The tileset bitmap used to construct the tilemap.
      *
      * @property tileset
      * @type gs.Bitmap
       */
      this.tileset = null;
      this.tilesets = [];
      this.expandedTilesets = [];

      /**
      * An array storing numbers by tile-id to control which tiles are overlapping.
      *
      * @property priorities
      * @type number[]
       */
      this.priorities = null;

      /**
      * An array storing the map-data. Each number in the array refers to a tile on the tileset.
      *
      * @property mapData
      * @type number[]
       */
      this.mapData = null;

      /**
      * The size of a single tile. A tile needs to be quadratic so a value of 32
      * means 32x32 pixels.
      *
      * @property tileSize
      * @type number
      * @default 32
       */
      this.tileSize = 32;
      this.autotileOffset = 0;

      /**
      * The animation speed for animated tiles. It defines the duration in frames before the next animation frame
      * is displayed.
      *
      * @property animationSpeed
      * @type number
      * @default 16
       */
      this.animationSpeed = 16;

      /**
      * The width of the tilemap in tiles
      *
      * @property width
      * @type number
      * @default 50
       */
      this.width = 50;

      /**
      * The height of the tilemap in tiles
      *
      * @property height
      * @type number
      * @default 50
       */
      this.height = 50;

      /**
      * The count of tilemap layers.
      *
      * @property depth
      * @type number
      * @default 3
       */
      this.depth = 3;
      this.cache = [];

      /**
      * Array of layer objects used to render the layers of the tilemap.
      *
      * @property layers
      * @type gs.TilemapLayer[]
      * @readonly
       */
      this.layers = null;

      /**
      * The tileset configuration to configure tileset size, regular tiles, auto-tiles and animated tiles. If you want
      * change the configuration, you have to do that before you call the setup-method of the tilemap.
      *
      * @property tilesetConfig
      * @type gs.TilesetConfiguration
       */
      this.tilesetConfig = new gs.TilesetConfiguration(this.tileSize);
    }


    /**
    * Disposes the tilemap and frees the memory. Should be called
    * if the tilemap is no longer needed to free resources.
    *
    * @method dispose
     */

    Tilemap_OpenGL.prototype.dispose = function() {
      var autoTile, k, l, layer, len, len1, ref, ref1, ref2;
      if (!this.tilesetConfig.useOriginalTileset) {
        if ((ref = this.expandedTileset) != null) {
          ref.dispose();
        }
      }
      ref1 = this.expandedAutotiles;
      for (k = 0, len = ref1.length; k < len; k++) {
        autoTile = ref1[k];
        if (autoTile != null) {
          autoTile.dispose();
        }
      }
      ref2 = this.layers;
      for (l = 0, len1 = ref2.length; l < len1; l++) {
        layer = ref2[l];
        if (layer != null) {
          layer.dispose();
        }
      }
      return null;
    };


    /**
    * Initializes the tilemap. Must be called after the tilemap settings, map-data and tileset
    * data are set.
    *
    * @method setup
     */

    Tilemap_OpenGL.prototype.setup = function() {
      return this.expandAutotiles();
    };

    Tilemap_OpenGL.prototype.expandAutotiles = function() {
      var autoTilesArea, autotile, height, i, j, k, l, len, ref, ref1, tilesArea, tileset, width;
      AutotileExpander.tileSize = this.tileSize;
      AutotileExpander.tilesetConfig = this.tilesetConfig;
      height = this.tilesetConfig.expandedSize.height;
      width = this.tilesetConfig.expandedSize.width;
      tilesArea = this.tilesetConfig.regularTiles;
      autoTilesArea = this.tilesetConfig.specialTiles;
      if (this.tileset) {
        this.tilesets[0] = this.tileset;
      } else {
        this.tileset = this.tilesets[0];
      }
      this.expandedTilesets = new Array(this.tilesets.length);
      ref = this.tilesets;
      for (i = k = 0, len = ref.length; k < len; i = ++k) {
        tileset = ref[i];
        if (!this.tilesetConfig.useOriginalTileset) {
          this.expandedTilesets[i] = new Bitmap(width, height);
          this.expandedTilesets[i].blt(tilesArea.x, tilesArea.y, tileset, new Rect(0, 0, tileset.width, tileset.height));
          for (j = l = 0, ref1 = this.autotiles.length; 0 <= ref1 ? l < ref1 : l > ref1; j = 0 <= ref1 ? ++l : --l) {
            if (this.autotiles[j] != null) {
              autotile = AutotileExpander.expand(this.autotiles[j]);
              this.expandedTilesets[i].blt(autoTilesArea.x, autoTilesArea.y + this.tileSize * 6 * j, autotile, new Rect(0, 0, autotile.width, autotile.height));
            }
          }
        } else {
          this.expandedTilesets[i] = tileset;
        }
      }
      this.calculateMapData();
      return this.splitMapData();
    };

    Tilemap_OpenGL.prototype.calculateMapData = function() {};


    /**
    * If necessary, you can override or patch this method to convert tile-ids into another ID space. No conversion
    * is done by default, it returns the tile-id without any changes.
    *
    * @method convertTileId
    * @param {number} tile - The tile id to convert.
    * @param {index} index - The index of the tile id in the map-data array.
    * @param {number} framesX - The number of tiles on x-axis in the tileset.
    * @param {number} x - The x-coordinate of the tile on the tilemap.
    * @param {number} y - The y-coordinate of the tile on the tilemap.
    * @param {number} z - The tilemap layer the tile is placed on.
    * @return {number} The converted tile id.
     */

    Tilemap_OpenGL.prototype.convertTileId = function(tile, index, framesX, x, y, z) {
      return tile;

      /*
      tx = tile % framesX
      ty = Math.floor(tile / framesX)
      
      if ty < 64
        tx += 8
      else
        ty -= 64
      
      return ty * 40 + tx
       */
    };

    Tilemap_OpenGL.prototype.splitMapData = function() {
      var framesX, i, index, k, l, newTile, ref, ref1, results, tile, x, y, z;
      this.upperMapData = [];
      this.lowerMapData = [];
      framesX = 8;
      for (i = k = 0, ref = this.depth; 0 <= ref ? k < ref : k > ref; i = 0 <= ref ? ++k : --k) {
        this.upperMapData[i] = [];
        this.lowerMapData[i] = [];
      }
      results = [];
      for (x = l = 0, ref1 = this.width; 0 <= ref1 ? l < ref1 : l > ref1; x = 0 <= ref1 ? ++l : --l) {
        results.push((function() {
          var m, ref2, results1;
          results1 = [];
          for (y = m = 0, ref2 = this.height; 0 <= ref2 ? m < ref2 : m > ref2; y = 0 <= ref2 ? ++m : --m) {
            results1.push((function() {
              var n, ref3, results2;
              results2 = [];
              for (z = n = 0, ref3 = this.depth; 0 <= ref3 ? n < ref3 : n > ref3; z = 0 <= ref3 ? ++n : --n) {
                index = (y * this.width + x) + this.width * this.height * z;
                tile = this.mapData[index];
                newTile = this.convertTileId(tile, index, framesX, x, y, z);
                if ((this.priorities[tile] || 0) > 0) {
                  this.upperMapData[z][y * this.width + x] = newTile;
                  results2.push(this.lowerMapData[z][y * this.width + x] = this.tilesetConfig.invisibleTileId);
                } else {
                  this.upperMapData[z][y * this.width + x] = this.tilesetConfig.invisibleTileId;
                  results2.push(this.lowerMapData[z][y * this.width + x] = newTile);
                }
              }
              return results2;
            }).call(this));
          }
          return results1;
        }).call(this));
      }
      return results;
    };

    Tilemap_OpenGL.prototype.shrinkLayers = function(mapData) {
      var index, k, l, layer1, layer2, m, mix, n, ref, ref1, ref2, ref3, results, tile, tile1, tile2, x, y;
      layer1 = 0;
      layer2 = 1;
      results = [];
      while (layer2 < mapData.length) {
        mix = true;
        for (x = k = 0, ref = this.width; 0 <= ref ? k < ref : k > ref; x = 0 <= ref ? ++k : --k) {
          for (y = l = 0, ref1 = this.height; 0 <= ref1 ? l < ref1 : l > ref1; y = 0 <= ref1 ? ++l : --l) {
            index = y * this.width + x;
            tile1 = mapData[layer1][index];
            tile2 = mapData[layer2][index];
            if (tile1 !== this.tilesetConfig.invisibleTileId && tile2 !== this.tilesetConfig.invisibleTileId) {
              mix = false;
              break;
            }
          }
        }
        if (mix) {
          for (x = m = 0, ref2 = this.width; 0 <= ref2 ? m < ref2 : m > ref2; x = 0 <= ref2 ? ++m : --m) {
            for (y = n = 0, ref3 = this.height; 0 <= ref3 ? n < ref3 : n > ref3; y = 0 <= ref3 ? ++n : --n) {
              index = y * this.width + x;
              tile = mapData[layer1][index];
              if (tile === this.tilesetConfig.invisibleTileId) {
                mapData[layer1][index] = mapData[layer2][index];
              }
            }
          }
          results.push(mapData.splice(layer2, 1));
        } else {
          layer1++;
          results.push(layer2++);
        }
      }
      return results;
    };

    Tilemap_OpenGL.prototype.clearCache = function() {};

    Tilemap_OpenGL.prototype.createLayers = function() {
      var i, k, l, layer, ref, ref1, ref2, ref3, results;
      this.layers = [];
      for (i = k = 0, ref = this.lowerMapData.length; 0 <= ref ? k < ref : k > ref; i = 0 <= ref ? ++k : --k) {
        layer = new TilemapLayer_OpenGL(this.viewport, this, this.width, this.height, this.lowerMapData[i], this.tileSize, (ref1 = this.expandedTilesets[i]) != null ? ref1 : this.expandedTilesets[0], this.animationSpeed);
        layer.z = 0 + i;
        this.layers.push(layer);
      }
      results = [];
      for (i = l = 0, ref2 = this.upperMapData.length; 0 <= ref2 ? l < ref2 : l > ref2; i = 0 <= ref2 ? ++l : --l) {
        layer = new TilemapLayer_OpenGL(this.viewport, this, this.width, this.height, this.upperMapData[i], this.tileSize, (ref3 = this.expandedTilesets[i]) != null ? ref3 : this.expandedTilesets[0], this.animationSpeed);
        layer.z = 64 + i;
        results.push(this.layers.push(layer));
      }
      return results;
    };

    Tilemap_OpenGL.prototype.draw = function() {};


    /**
    * Updates the tilemap scrolling, etc. Needs to be called once per frame.
    *
    * @method update
     */

    Tilemap_OpenGL.prototype.update = function() {
      var i, k, ref, results;
      if (!this.visible || (this.tileset == null) || !this.tileset.loaded || (this.mapData == null)) {
        return;
      }
      if (this.layers == null) {
        this.createLayers();
      }
      results = [];
      for (i = k = 0, ref = this.layers.length; 0 <= ref ? k < ref : k > ref; i = 0 <= ref ? ++k : --k) {
        this.layers[i].ox = this.ox;
        this.layers[i].oy = this.oy;
        this.layers[i].tone = this.tone;
        this.layers[i].color = this.color;
        results.push(this.layers[i].mask = this.mask);
      }
      return results;
    };

    Tilemap_OpenGL.prototype.show = function(visible) {};

    Tilemap_OpenGL.prototype.hide = function(duration) {};

    return Tilemap_OpenGL;

  })(GraphicObject);

  TilemapLayer_OpenGL = (function(superClass) {
    extend(TilemapLayer_OpenGL, superClass);

    TilemapLayer_OpenGL.objectCodecBlackList = ["onIndexChange"];


    /**
    * A tilemap layer is used to display a single layer of a gs.Tilemap. It is not recommended to use this class directly. For more information
    * about how to use tilemaps, features and limitations please read the "Scripter's Guide" in the help-file.
    *
    * @namespace gs
    * @class TilemapLayer
    * @memberof gs
    * @constructor
    * @param {gs.Viewport} viewport - The viewport to associate the tilemap layer with.
    * @param {gs.Tilemap} tilemap - The tilemap to associate the tilemap layer with.
    * @param {number} width - The width of the layer in tiles.
    * @param {number} height - The height of the layer in tiles.
    * @param {number[]} mapData - The map-data for the layer.
    * @param {number} tileSize - The tile-size of this layer.
    * @param {number} tilesetBitmap - The tileset to use.
    * @param {number} animationSpeed - The animation speed for animated tiles. (The duration in frames before next pattern is displayed)
     */

    function TilemapLayer_OpenGL(viewport, tilemap, width, height, mapData, tileSize, tilesetBitmap, animationSpeed) {
      var data, framesX, imageData, invisibleTileId, k, l, ref, ref1, tile, x, y;
      TilemapLayer_OpenGL.__super__.constructor.call(this, viewport);

      /**
      * The color tone.
      * @property tone
      * @type gs.Tone
       */
      this.tone = new Tone(0, 0, 0, 0);

      /**
      * The color. Can be used for flashing-effects, etc.
      * @property color
      * @type boolean
       */
      this.color = new Color(255, 255, 255, 0);

      /**
      * The mask used for masking-effects.
      * @property mask
      * @type gs.Mask
       */
      this.mask = {
        ox: 0,
        oy: 0,
        source: null,
        type: 0,
        value: 0,
        vague: 0
      };

      /**
      * The tilemap associated with the layer.
      *
      * @property tilemap
      * @type gs.Tilemap
      * @readOnly
       */
      this.tilemap = tilemap;

      /**
      * The rotation angle.
      *
      * @property angle
      * @type number
       */
      this.angle = 0;

      /**
      * The zoom on x-axis.
      * @property zoomX
      * @type number
       */
      this.zoomX = 1;

      /**
      * The zoom on y-axis.
      * @property zoomY
      * @type number
       */
      this.zoomY = 1;

      /**
      * The size of a single tile. A single tile needs to be quadratic so a value of 32 means 32x32 pixels.
      * @property tileSize
      * @type number
       */
      this.tileSize = tileSize;

      /**
      * The tileset used for this layer.
      * @property tilesetBitmap
      * @type gs.Bitmap
       */
      this.tilesetBitmap = tilesetBitmap;
      this.dataBitmap = new Bitmap(width, height);
      this.autotileOffset = 0;

      /**
      * The animation speed for animated tiles. (The duration in frames before the next pattern is displayed)
      * @property animationSpeed
      * @type number
       */
      this.animationSpeed = 16;
      invisibleTileId = this.tilemap.tilesetConfig.invisibleTileId;
      imageData = this.dataBitmap.getImageData(0, 0, width, height);
      data = imageData.data;
      framesX = Math.ceil(this.tilesetBitmap.width / this.tileSize);
      for (x = k = 0, ref = width; 0 <= ref ? k < ref : k > ref; x = 0 <= ref ? ++k : --k) {
        for (y = l = 0, ref1 = height; 0 <= ref1 ? l < ref1 : l > ref1; y = 0 <= ref1 ? ++l : --l) {
          tile = mapData[y * width + x];
          data[(y * width + x) * 4] = tile === invisibleTileId ? 255 : tile % framesX;
          data[(y * width + x) * 4 + 1] = tile === invisibleTileId ? 255 : Math.floor(tile / framesX);
          data[(y * width + x) * 4 + 3] = 255;
        }
      }
      this.dataBitmap.setImageData(imageData, 0, 0);
    }


    /**
    * Disposes the tilemap layer and frees resources. Should be called if the tilemap layer
    * is no longer needed.
    *
    * @method dispose
     */

    TilemapLayer_OpenGL.prototype.dispose = function() {
      var ref;
      return (ref = this.dataBitmap) != null ? ref.dispose() : void 0;
    };

    TilemapLayer_OpenGL.prototype.draw = function() {
      var autoTileFrames, color, renderer, tone;
      TilemapLayer_OpenGL.__super__.draw.call(this);
      renderer = Graphics.renderer;
      color = renderer.color;
      tone = renderer.tone;
      renderer.color = this.color;
      renderer.tone = this.tone;
      renderer.mask = this.mask;
      this.updateTransform(0, 0, Graphics.width, Graphics.height);
      renderer.transform = this.transform;
      if (Math.floor(Graphics.frameCount % this.animationSpeed) === 0) {
        autoTileFrames = this.tilemap.tilesetConfig.animationFrames;
        this.autotileOffset = (this.autotileOffset + 1) % autoTileFrames;
      }
      renderer.drawTiles(this.ox + this.viewport.ox, this.oy + this.viewport.oy, this.dataBitmap, this.tilesetBitmap, this.autotileOffset * this.tilemap.tilesetConfig.specialTileSize.width, this.tileSize, this.tilemap.tilesetConfig);
      renderer.tone = tone;
      renderer.color = color;
      return renderer.mask = null;
    };


    /**
    * Updates the tilemap layer. Must be called once per frame.
    *
    * @method update
     */

    TilemapLayer_OpenGL.prototype.update = function() {};

    return TilemapLayer_OpenGL;

  })(GraphicObject);

  window.Tilemap = Tilemap_OpenGL;

  gs.Tilemap = Tilemap_OpenGL;

  gs.TilemapLayer = TilemapLayer_OpenGL;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var ThreePartImage,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  ThreePartImage = (function(superClass) {
    extend(ThreePartImage, superClass);

    ThreePartImage.objectCodecBlackList = ["onIndexChange"];


    /**
    * A three-part image is rendered using three
    * sub-images: start, middle and end. The start and end will be rendered
    * with fixed size while the middle-part will be stretched in a way like this:<br>
    * <br>
    * |Start-Part Fixed|<-----------Middle-Part Stretched----------->|End-Part Fixed<br>
    * <br>
    * A three-part image is automatically added to the graphics-system
    * and rendered every frame until it gets disposed. It can be horizontal or
    * vertical.
    *
    * @namespace gs
    * @class ThreePartImage
    * @extends gs.GraphicObject
    * @memberof gs
    * @constructor
     */

    function ThreePartImage(viewport) {
      ThreePartImage.__super__.constructor.call(this, viewport);

      /**
      * The source-rectangle. Only that area of the skin is used to construct the three-part image.
      * @property srcRect
      * @type gs.Rect
       */
      this.srcRect = new Rect(0, 0, 64, 64);

      /**
      * The size of the start-part. Default is 16px.
      * @property firstPartSize
      * @type number
       */
      this.firstPartSize = 16;

      /**
      * The size of the middle-part. Default is 1px.
      * @property middlePartSize
      * @type number
       */
      this.middlePartSize = 1;

      /**
      * The size of the last-part. Default is 16px.
      * @property lastPartSize
      * @type number
       */
      this.lastPartSize = 16;

      /**
      * The x-coordinate of the position.
      * @property x
      * @type number
       */
      this.x = 0;

      /**
      * The y-coordinate of the position.
      * @property y
      * @type number
       */
      this.y = 0;

      /**
      * The width of the three-part image. The middle-part will stretched if orientation is horizontal.
      * @property width
      * @type number
       */
      this.width = 100;

      /**
      * The height of the three-part image. The middle-part will stretched if orientation is vertical.
      * @property height
      * @type number
       */
      this.height = 100;

      /**
      * The skin-bitmap.
      * @property skin
      * @type gs.Bitmap
       */
      this.skin = null;

      /**
      * The opacity goes from 0 to 255. For example: A value of 0 means fully transparent, 255 means fully opaque and 128 means semi-transparent.
      * @property opacity
      * @type number
       */
      this.opacity = 255;

      /**
      * The color tone.
      * @property tone
      * @type gs.Tone
       */
      this.tone = new Tone(0, 0, 0, 0);

      /**
      * The color. Can be used for flashing-effects, etc.
      * @property color
      * @type boolean
       */
      this.color = new Color(255, 255, 255, 0);

      /**
      * The orientation. Default is gs.Orientation.VERTICAL.
      * @property orientation
      * @type gs.Orientation
       */
      this.orientation = 1;

      /**
      * The clip-rect. Everything outside the clip-rect is clipped and not visible.
      * @property clipRect
      * @type gs.Rect
       */
      this.clipRect = null;
      this.zoomX = 1.0;
      this.zoomY = 1.0;
      this.angle = 0;
    }


    /**
    * Disposes the three-part image and frees resources. After that the image is no longer rendered.
    * The associated skin-bitmap is <b>not</b> disposed.
    *
    * @method dispose
     */

    ThreePartImage.prototype.dispose = function() {
      return ThreePartImage.__super__.dispose.call(this);
    };


    /**
    * Updates the three-part image.
    *
    * @method update
     */

    ThreePartImage.prototype.update = function() {
      return ThreePartImage.__super__.update.call(this);
    };

    ThreePartImage.prototype.updateTransform = function(x, y, width, height) {
      mat2d.identity(this.transform);
      mat2d.mul(this.transform, this.transform, this.viewport.transform);
      if (this.angle !== 0) {
        this.position[0] = this.x - this.ox + this.width * this.anchor.x;
        this.position[1] = this.y - this.oy + this.height * this.anchor.y;
        mat2d.translate(this.transform, this.transform, this.position);
        mat2d.rotate(this.transform, this.transform, this.angle);
        this.position[0] = -this.position[0];
        this.position[1] = -this.position[1];
        mat2d.translate(this.transform, this.transform, this.position);
      }
      if (this.angle !== 0 || this.zoomX !== 1 || this.zoomY !== 1) {
        this.position[0] = x - (this.width * (this.zoomX - 1.0) * this.anchor.x);
        this.position[1] = y - (this.height * (this.zoomY - 1.0) * this.anchor.y);
        mat2d.translate(this.transform, this.transform, this.position);
        this.position[0] = width * this.zoomX;
        this.position[1] = height * this.zoomY;
        return mat2d.scale(this.transform, this.transform, this.position);
      } else {
        this.position[0] = x;
        this.position[1] = y;
        mat2d.translate(this.transform, this.transform, this.position);
        this.position[0] = width;
        this.position[1] = height;
        return mat2d.scale(this.transform, this.transform, this.position);
      }
    };

    ThreePartImage.prototype._drawParts = function() {
      var blendingMode, color, effects, height, renderer, tone, width, x, y;
      if ((this.skin == null) || !this.skin.loaded) {
        return;
      }
      this.width = Math.max(this.firstPartSize + this.middlePartSize + this.lastPartSize, this.width);
      this.height = Math.max(this.firstPartSize + this.middlePartSize + this.lastPartSize, this.height);
      x = ~~(this.x - this.ox);
      y = ~~(this.y - this.oy);
      renderer = Graphics.renderer;
      renderer.reset();
      color = renderer.color;
      tone = renderer.tone;
      blendingMode = renderer.blendType;
      effects = renderer.effects;
      renderer.tone = this.tone;
      renderer.color = this.color;
      renderer.blendType = this.blendingMode;
      renderer.effects = this.effects;
      renderer.opacity = this.opacity;
      if (this.orientation === 0) {
        width = this.width - this.firstPartSize - this.lastPartSize;
        this.updateTransform(x + this.firstPartSize, y, width, this.height);
        renderer.transform = this.transform;
        renderer.setBitmap(this.skin);
        renderer.setSourceRectangle(this.firstPartSize, 0, this.middlePartSize, this.skin.height);
        renderer.drawTransform();
        this.updateTransform(x, y, this.firstPartSize, this.height);
        renderer.setSourceRectangle(0, 0, this.firstPartSize, this.skin.height);
        renderer.drawTransform();
        this.updateTransform(x + this.firstPartSize + width, y, this.lastPartSize, this.height);
        renderer.setSourceRectangle(this.firstPartSize + this.middlePartSize, 0, this.lastPartSize, this.skin.height);
        renderer.drawTransform();
      } else {
        height = this.height - this.firstPartSize - this.lastPartSize;
        this.updateTransform(x, y + this.firstPartSize, this.width, height);
        renderer.transform = this.transform;
        renderer.setBitmap(this.skin);
        renderer.setSourceRectangle(0, this.firstPartSize, this.skin.width, this.middlePartSize);
        renderer.drawTransform();
        this.updateTransform(x, y, this.width, this.firstPartSize);
        renderer.setSourceRectangle(0, 0, this.skin.width, this.firstPartSize);
        renderer.drawTransform();
        this.updateTransform(x, y + this.firstPartSize + height, this.width, this.lastPartSize);
        renderer.setSourceRectangle(0, this.firstPartSize + this.middlePartSize, this.skin.width, this.lastPartSize);
        renderer.drawTransform();
      }
      renderer.tone = tone;
      renderer.color = color;
      renderer.blendType = blendingMode;
      renderer.effects = effects;
      renderer.opacity = 255;
      return null;
    };


    /**
    * Draws the three-part image. Shouldn't be called manually.
    *
    * @method draw
     */

    ThreePartImage.prototype.draw = function() {
      var renderer;
      renderer = Graphics.renderer;
      renderer.blendType = 0;
      renderer.reset();
      if (this.clipRect != null) {
        renderer.clipRect(this.clipRect);
      } else {
        renderer.clipRect(Graphics.viewport.rect);
      }
      if (this.opacity > 0) {
        return this._drawParts();
      }
    };

    return ThreePartImage;

  })(GraphicObject);

  window.gs.ThreePartImage = ThreePartImage;

}).call(this);

// Generated by CoffeeScript 1.12.2
(function() {
  var Frame,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  Frame = (function(superClass) {
    extend(Frame, superClass);

    Frame.objectCodecBlackList = ["onIndexChange"];


    /**
    * A frame useful to create in-game UI elements like buttons or windows. For more information
    * about the frame skin-graphic see help-file. A frame is automatically added to the graphics-system
    * and rendered every frame until it gets disposed.
    *
    * @namespace gs
    * @class Frame
    * @extends gs.GraphicObject
    * @memberof gs
    * @constructor
    * @param {gs.Viewport} [viewport=gs.Graphics.viewport] - The viewport.
     */

    function Frame(viewport) {
      Frame.__super__.constructor.call(this, viewport);

      /**
      * The source-rectangle. Only that area of the skin is used to construct the frame.
      * @property srcRect
      * @type gs.Rect
       */
      this.srcRect = new Rect(0, 0, 64, 64);

      /**
      * The corner-size of the frame. Default is 16. (16x16)
      * @property cornerSize
      * @type number
       */
      this.cornerSize = 16;

      /**
      * The thickness of the frame. Default is 16.
      * @property thickness
      * @type number
       */
      this.thickness = 16;

      /**
      * The x-coordinate of the frame's position.
      * @property x
      * @type number
       */
      this.x = 0;

      /**
      * The y-coordinate of the frame's position.
      * @property y
      * @type number
       */
      this.y = 0;

      /**
      * The frame's width.
      * @property width
      * @type number
       */
      this.width = 100;

      /**
      * The frame's height.
      * @property height
      * @type number
       */
      this.height = 100;

      /**
      * The skin-bitmap.
      * @property skin
      * @type gs.Bitmap
       */
      this.skin = null;

      /**
      * The frame's opacity goes from 0 to 255. For example: A value of 0 means fully transparent, 255 means fully opaque and 128 means semi-transparent.
      * @property opacity
      * @type number
       */
      this.opacity = 255;

      /**
      * The frame's color tone.
      * @property tone
      * @type gs.Tone
       */
      this.tone = new Tone(0, 0, 0, 0);

      /**
      * The color. Can be used for flashing-effects, etc.
      * @property color
      * @type boolean
       */
      this.color = new Color(255, 255, 255, 0);

      /**
      * The zoom on x-axis.
      * @property zoomX
      * @type number
       */
      this.zoomX = 1.0;

      /**
      * The zoom on y-axis.
      * @property zoomY
      * @type number
       */
      this.zoomY = 1.0;
      this.useTiling = false;
      this.angle = 0;
      this.tmpRect = new Rect(0, 0, 0, 0);
      this.scale = vec2.create();
      this.scaleTransform = mat2d.create();
    }


    /**
    * Disposes the frame and frees resources. After that the frame is no longer rendered. The
    * associated skin-bitmap is <b>not</b> disposed.
    *
    * @method dispose
     */

    Frame.prototype.dispose = function() {
      return Frame.__super__.dispose.call(this);
    };


    /**
    * Updates the frame.
    *
    * @method update
     */

    Frame.prototype.update = function() {
      return Frame.__super__.update.call(this);
    };

    Frame.prototype.updateTransform = function(x, y, width, height) {
      mat2d.identity(this.transform);
      mat2d.mul(this.transform, this.transform, this.viewport.transform);
      if (this.angle !== 0) {
        this.position[0] = this.x - this.ox + this.width * this.anchor.x;
        this.position[1] = this.y - this.oy + this.height * this.anchor.y;
        mat2d.translate(this.transform, this.transform, this.position);
        mat2d.rotate(this.transform, this.transform, Math.degreesToRadians(this.angle));
        this.position[0] = -this.position[0];
        this.position[1] = -this.position[1];
        mat2d.translate(this.transform, this.transform, this.position);
      }
      if (this.angle !== 0 || this.zoomX !== 1 || this.zoomY !== 1) {
        this.position[0] = x - (this.width * (this.zoomX - 1.0) * this.anchor.x);
        this.position[1] = y - (this.height * (this.zoomY - 1.0) * this.anchor.y);
        mat2d.translate(this.transform, this.transform, this.position);
        this.position[0] = width * this.zoomX;
        this.position[1] = height * this.zoomY;
        return mat2d.scale(this.transform, this.transform, this.position);
      } else {
        this.position[0] = x;
        this.position[1] = y;
        mat2d.translate(this.transform, this.transform, this.position);
        this.position[0] = width;
        this.position[1] = height;
        return mat2d.scale(this.transform, this.transform, this.position);
      }
    };

    Frame.prototype._drawFrame = function() {
      var blendingMode, color, cornerSizeX, cornerSizeY, cornerSpace, effects, h, renderer, tone, w, x, y;
      if ((this.skin == null) || !this.skin.loaded) {
        return;
      }
      this.srcRect.width = this.skin.width;
      this.srcRect.height = this.skin.height;
      w = this.width * this.zoomX;
      h = this.height * this.zoomY;
      x = ~~(this.x - this.ox);
      y = ~~(this.y - this.oy);
      cornerSizeX = this.cornerSize * this.zoomX;
      cornerSizeY = this.cornerSize * this.zoomY;
      renderer = Graphics.renderer;
      color = renderer.color;
      tone = renderer.tone;
      blendingMode = renderer.blendType;
      effects = renderer.effects;
      renderer.reset();
      renderer.tone = this.tone;
      renderer.color = this.color;
      renderer.blendType = this.blendingMode;
      renderer.effects = this.effects;
      renderer.opacity = this.opacity;
      if (this.clipRect != null) {
        renderer.clipRect(this.clipRect);
      } else {
        renderer.clipRect(Graphics.viewport.rect);
      }
      cornerSpace = this.cornerSize * 2;
      this.updateTransform(x, y + cornerSizeY, this.thickness, this.height - cornerSpace);
      renderer.transform = this.transform;
      renderer.setBitmap(this.skin);
      renderer.setSourceRectangle(this.srcRect.x, this.srcRect.y + this.cornerSize, this.thickness, this.srcRect.height - this.cornerSize * 2);
      renderer.drawTransform();
      this.updateTransform(x + cornerSizeX, y, this.width - cornerSpace, this.thickness);
      renderer.setSourceRectangle(this.srcRect.x + this.cornerSize, this.srcRect.y, this.srcRect.width - this.cornerSize * 2, this.thickness);
      renderer.drawTransform();
      this.updateTransform(x + w - cornerSizeX, y + cornerSizeY, this.thickness, this.height - cornerSpace);
      renderer.setSourceRectangle(this.srcRect.width - this.thickness, this.srcRect.y + this.cornerSize, this.thickness, this.srcRect.height - this.cornerSize * 2);
      renderer.drawTransform();
      this.updateTransform(x + cornerSizeX, y + h - cornerSizeY, this.width - cornerSpace, this.thickness);
      renderer.setSourceRectangle(this.srcRect.x + this.cornerSize, this.srcRect.height - this.thickness, this.srcRect.width - this.cornerSize * 2, this.thickness);
      renderer.drawTransform();
      this.updateTransform(x, y, this.cornerSize, this.cornerSize);
      renderer.setSourceRectangle(this.srcRect.x, this.srcRect.y, this.cornerSize, this.cornerSize);
      renderer.drawTransform();
      this.position[0] = (w - cornerSizeX) / cornerSizeX;
      this.position[1] = 0;
      mat2d.translate(this.transform, this.transform, this.position);
      renderer.setSourceRectangle(this.srcRect.width - this.cornerSize, this.srcRect.y, this.cornerSize, this.cornerSize);
      renderer.drawTransform();
      this.position[0] = 0 - this.position[0];
      this.position[1] = 0 + (h - cornerSizeY) / cornerSizeY - this.position[1];
      mat2d.translate(this.transform, this.transform, this.position);
      renderer.setSourceRectangle(this.srcRect.x, this.srcRect.height - this.cornerSize, this.cornerSize, this.cornerSize);
      renderer.drawTransform();
      this.position[0] = 0 - this.position[0];
      this.position[1] = 0 + (h - cornerSizeY) / cornerSizeY - this.position[1];
      mat2d.translate(this.transform, this.transform, this.position);
      renderer.setSourceRectangle(this.srcRect.width - this.cornerSize, this.srcRect.height - this.cornerSize, this.cornerSize, this.cornerSize);
      renderer.drawTransform();

      /*
      renderer.drawBitmapSimple(@skin, x, y, @cornerSize, @cornerSize,
        @srcRect.x, @srcRect.y, @cornerSize, @cornerSize)
      renderer.drawBitmapSimple(@skin, x + w - @cornerSize, y, @cornerSize, @cornerSize,
        @srcRect.width - @cornerSize, @srcRect.y, @cornerSize, @cornerSize)
      renderer.drawBitmapSimple(@skin, x, y + h - @cornerSize, @cornerSize, @cornerSize,
        @srcRect.x, @srcRect.height - @cornerSize, @cornerSize, @cornerSize)
      renderer.drawBitmapSimple(@skin, x + w - @cornerSize, y + h - @cornerSize, @cornerSize, @cornerSize,
        @srcRect.width - @cornerSize, @srcRect.height - @cornerSize, @cornerSize, @cornerSize)
       */

      /*
      renderer.drawBitmapSimple(@skin, x, y + @cornerSize, @thickness, h - cornerSpace,
                                       @srcRect.x, @srcRect.y + @cornerSize, @thickness, @srcRect.height - @cornerSize*2)
      
      renderer.drawBitmapSimple(@skin, x + @cornerSize, y, w - cornerSpace, @thickness,
                                       @srcRect.x + @cornerSize, @srcRect.y, @srcRect.width - @cornerSize*2, @thickness)
      
      renderer.drawBitmapSimple(@skin, x + w - @cornerSize, y + @cornerSize, @thickness, h - cornerSpace,
        @srcRect.width - @thickness, @srcRect.y + @cornerSize, @thickness, @srcRect.height - @cornerSize*2)
      
      renderer.drawBitmapSimple(@skin, x + @cornerSize, y + h - @cornerSize, w - cornerSpace, @thickness,
        @srcRect.x + @cornerSize, @srcRect.height - @thickness, @srcRect.width - @cornerSize*2, @thickness)
       */

      /*
      tiles = Math.ceil(height / tmpRect.height)
      for i in [0...tiles]
        if i+1 == tiles
          p = y + @cornerSize + i * tmpRect.height
          h = tmpRect.height-(p+tmpRect.height-(y+@cornerSize+height))
          renderer.drawBitmapSimple(@skin, x, p, tmpRect.width, h,
            tmpRect.x, tmpRect.y, tmpRect.width, h)
        else
          renderer.drawBitmapSimple(@skin, x, y + @cornerSize + i * tmpRect.height, tmpRect.width, tmpRect.height,
            tmpRect.x, tmpRect.y, tmpRect.width, tmpRect.height)
      
       * Draw top side
      width =  Math.max(@width - @cornerSize*2, 1)
      tmpRect.set(@srcRect.x + @cornerSize,
        @srcRect.y,
        @srcRect.width - @cornerSize*2,
        @thickness)
      
      tiles = Math.ceil(width / tmpRect.width)
      
      for i in [0...tiles]
        if i+1 == tiles
          p = x + @cornerSize + i * tmpRect.width
          w = tmpRect.width-(p+tmpRect.width-(x+@cornerSize+width))
          renderer.drawBitmapSimple(@skin, p, y, w, tmpRect.height,
            tmpRect.x, tmpRect.y, w, tmpRect.height)
        else
          renderer.drawBitmapSimple(@skin, x + @cornerSize + i * tmpRect.width, y, tmpRect.width, tmpRect.height,
            tmpRect.x, tmpRect.y, tmpRect.width, tmpRect.height)
      
       * Draw right side
      height =  Math.max(@height - @cornerSize*2, 1)
      tmpRect.set(@srcRect.width - @thickness,
        @srcRect.y + @cornerSize,
        @thickness,
        @srcRect.height - @cornerSize*2)
      
      tiles = Math.ceil(height / tmpRect.height)
      for i in [0...tiles]
        if i+1 == tiles
          p = y + @cornerSize + i * tmpRect.height
          h = tmpRect.height-(p+tmpRect.height-(y+@cornerSize+height))
          renderer.drawBitmapSimple(@skin, x + @width - @cornerSize, p, tmpRect.width, h,
            tmpRect.x, tmpRect.y, tmpRect.width, h)
        else
          renderer.drawBitmapSimple(@skin, x + @width - @cornerSize, y + @cornerSize + i * tmpRect.height, tmpRect.width, tmpRect.height,
            tmpRect.x, tmpRect.y, tmpRect.width, tmpRect.height)
      
       * Draw bottom side
      width =  Math.max(@width - @cornerSize*2, 1)
      tmpRect.set(@srcRect.x + @cornerSize,
        @srcRect.height - @thickness,
        @srcRect.width - @cornerSize*2,
        @thickness)
      
      tiles = Math.ceil(width / tmpRect.width)
      
      for i in [0...tiles]
        if i+1 == tiles
          p = x + @cornerSize + i * tmpRect.width
          w = tmpRect.width-(p+tmpRect.width-(x+@cornerSize+width))
          renderer.drawBitmapSimple(@skin, p, y + @height - @cornerSize, w, tmpRect.height,
            tmpRect.x, tmpRect.y, w, tmpRect.height)
        else
          renderer.drawBitmapSimple(@skin, x + @cornerSize + i * tmpRect.width, y + @height - @cornerSize, tmpRect.width, tmpRect.height,
            tmpRect.x, tmpRect.y, tmpRect.width, tmpRect.height)
       */
      renderer.tone = tone;
      renderer.color = color;
      renderer.blendType = blendingMode;
      renderer.effects = effects;
      renderer.opacity = 255;
      return null;
    };


    /**
    * Draws the frame. Shouldn't be called manually.
    *
    * @method draw
     */

    Frame.prototype.draw = function() {
      var renderer;
      renderer = Graphics.renderer;
      renderer.blendType = 0;
      renderer.reset();
      if (this.opacity > 0) {
        return this._drawFrame();
      }
    };

    return Frame;

  })(GraphicObject);

  window.gs.Frame = Frame;

}).call(this);

// Generated by CoffeeScript 1.6.3
(function() {
  var Window,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  Window = (function(_super) {
    __extends(Window, _super);

    Window.backRect = new Rect(0, 0, 64, 64);

    Window.backRect2 = new Rect(0, 64, 64, 64);

    Window.frameRect = new Rect(64, 0, 64, 64);

    Window.frameThickness = 16;

    Window.framePadding = 2;

    Window.leftArrowRect = new Rect(Window.frameRect.x + Window.frameThickness, Window.frameRect.y + 24, 16, 16);

    Window.topArrowRect = new Rect(Window.frameRect.x + 24, Window.frameRect.y + Window.frameThickness, 16, 16);

    Window.rightArrowRect = new Rect(40 + Window.frameRect.x, Window.frameRect.y + 24, 16, 16);

    Window.bottomArrowRect = new Rect(Window.frameRect.x + 24, 40 + Window.frameRect.y, 16, 16);

    Window.cornerSize = 16;

    Window.cursorRect = new Rect(64, 64, 32, 32);

    Window.cursorBorder = 2;

    Window.pauseRect = new Rect(96, 64, 32, 32);

    Window.padding = 12;

    Window.pauseIconSize = new Rect(0, 0, 16, 16);

    Window.textColorRect = new Rect(64, 96, 64, 32);

    Window.textColorSize = new Rect(0, 0, 8, 8);

    function Window(viewport) {
      Window.__super__.constructor.call(this, viewport);
      this.x = 0;
      this.y = 0;
      this.cx = 0;
      this.cy = 0;
      this.width = 100;
      this.height = 100;
      this.windowSkin = null;
      this.contents = null;
      this.window = null;
      this.cursorRect = new Rect(0, 0, 0, 0);
      this.active = true;
      this.arrowsVisible = true;
      this.pause = false;
      this.padding = Window.padding || 12;
      this.opacity = 255;
      this.backOpacity = 192;
      this.contentsOpacity = 255;
      this.openness = 255;
      this.tone = new Tone(0, 0, 0, 0);
      this.animationIncrement = 0.1;
      this.cursorOpacity = 1.0;
      this.pauseArrowPattern = 0;
      this.pauseAnimationCount = 0;
      this.renderer = null;
      this.oldWindowSkin = null;
    }

    Window.prototype.dispose = function() {
      Window.__super__.dispose.call(this);
      if (this.window != null) {
        this.window.dispose();
      }
      if (this.contents != null) {
        return this.contents.dispose();
      }
    };

    Window.prototype.update = function() {
      Window.__super__.update.call(this);
      if (this.active) {
        if (this.cursorOpacity >= 1.0) {
          this.animationIncrement = -0.025;
        }
        if (this.cursorOpacity <= 0.5) {
          this.animationIncrement = 0.025;
        }
        this.cursorOpacity += this.animationIncrement;
      }
      if (this.pause) {
        this.pauseAnimationCount++;
        if (this.pauseAnimationCount > 15) {
          this.pauseAnimationCount = 0;
          return this.pauseArrowPattern = (this.pauseArrowPattern + 1) % 4;
        }
      }
    };

    Window.prototype._createWindowBitmap = function() {
      /*
        if not @windowSkin? or not @windowSkin.loaded then return
        @oldWindowSkin = @windowSkin
      
        if not @window? or @width != @window.width or @height != @window.height
          if @window? then @window.dispose()
          @window = new Bitmap(@width, @height)
          @renderer = new Renderer_Canvas(@window.context)
        else
          @window.clear()
      
        @window.opacity = @opacity
        x = 0
        y = 0
      
        renderer = @renderer #Graphics.renderer
        ox = renderer.ox
        oy = renderer.oy
        renderer.ox = 0
        renderer.oy = 0
        renderer.reset()
        tmpRect = new Rect(0, 0, 0, 0)
      
        if @opacity < 255
          renderer.opacity = @opacity
        else
          renderer.opacity = @backOpacity
      
        renderer.size.width = @width - @constructor.framePadding*2
        renderer.size.height = @height - @constructor.framePadding*2
        renderer.tone = @tone
        renderer.drawBitmap(@windowSkin,
          x + @constructor.framePadding,
          y + @constructor.framePadding,
          new Rect(@constructor.backRect.x,
          @constructor.backRect.y,
          @constructor.backRect.width,
          @constructor.backRect.height))
        renderer.size.width = 0
        renderer.size.height = 0
      
        renderer.clipRect(new Rect(x + @constructor.framePadding, y + @constructor.framePadding,
          @width - @constructor.framePadding*2, @height - @constructor.framePadding*2))
        renderer.drawTiledBitmap(@windowSkin, x + @constructor.framePadding, y + @constructor.framePadding,
          renderer.clip.width, renderer.clip.height, @constructor.backRect2.x,
          @constructor.backRect2.y,
          @constructor.backRect2.width,
          @constructor.backRect2.height)
        renderer.clipRect(new Rect(0, 0, Graphics.width, Graphics.height))
      
        renderer.opacity = @opacity
        renderer.context.globalAlpha = @opacity * 0.00392156862745
        # Draw left side
        height =  @height - @constructor.cornerSize*2
        tmpRect.set(@constructor.frameRect.x,
          @constructor.frameRect.y + @constructor.cornerSize,
          @constructor.frameThickness,
          @constructor.frameRect.height - @constructor.cornerSize*2)
      
        tiles = Math.ceil(height / tmpRect.height)
        for i in [0...tiles]
          if i+1 == tiles
            p = y + @constructor.cornerSize + i * tmpRect.height
            h = tmpRect.height-(p+tmpRect.height-(y+@constructor.cornerSize+height))
            renderer.drawBitmapSimple(@windowSkin, x, p, tmpRect.width, h,
              tmpRect.x, tmpRect.y, tmpRect.width, h)
          else
            renderer.drawBitmapSimple(@windowSkin, x, y + @constructor.cornerSize + i * tmpRect.height, tmpRect.width, tmpRect.height,
              tmpRect.x, tmpRect.y, tmpRect.width, tmpRect.height)
      
        # Draw top side
        width =  @width - @constructor.cornerSize*2
        tmpRect.set(@constructor.frameRect.x + @constructor.cornerSize,
          @constructor.frameRect.y,
          @constructor.frameRect.width - @constructor.cornerSize*2,
          @constructor.frameThickness)
      
        tiles = Math.ceil(width / tmpRect.width)
      
        for i in [0...tiles]
          if i+1 == tiles
            p = x + @constructor.cornerSize + i * tmpRect.width
            w = tmpRect.width-(p+tmpRect.width-(x+@constructor.cornerSize+width))
            renderer.drawBitmapSimple(@windowSkin, p, y, w, tmpRect.height,
              tmpRect.x, tmpRect.y, w, tmpRect.height)
          else
            renderer.drawBitmapSimple(@windowSkin, x + @constructor.cornerSize + i * tmpRect.width, y, tmpRect.width, tmpRect.height,
              tmpRect.x, tmpRect.y, tmpRect.width, tmpRect.height)
      
        # Draw right side
        height =  @height - @constructor.cornerSize*2
        tmpRect.set(@constructor.frameRect.x + @constructor.cornerSize*3,
          @constructor.frameRect.y + @constructor.cornerSize,
          @constructor.frameThickness,
          @constructor.frameRect.height - @constructor.cornerSize*2)
      
        tiles = Math.ceil(height / tmpRect.height)
        for i in [0...tiles]
          if i+1 == tiles
            p = y + @constructor.cornerSize + i * tmpRect.height
            h = tmpRect.height-(p+tmpRect.height-(y+@constructor.cornerSize+height))
            renderer.drawBitmapSimple(@windowSkin, x + @width - @constructor.cornerSize, p, tmpRect.width, h,
              tmpRect.x, tmpRect.y, tmpRect.width, h)
          else
            renderer.drawBitmapSimple(@windowSkin, x + @width - @constructor.cornerSize, y + @constructor.cornerSize + i * tmpRect.height, tmpRect.width, tmpRect.height,
              tmpRect.x, tmpRect.y, tmpRect.width, tmpRect.height)
      
        # Draw bottom side
        width =  @width - @constructor.cornerSize*2
        tmpRect.set(@constructor.frameRect.x + @constructor.cornerSize,
          @constructor.frameRect.y + @constructor.cornerSize*3,
          @constructor.frameRect.width - @constructor.cornerSize*2,
          @constructor.frameThickness)
      
        tiles = Math.ceil(width / tmpRect.width)
      
        for i in [0...tiles]
          if i+1 == tiles
            p = x + @constructor.cornerSize + i * tmpRect.width
            w = tmpRect.width-(p+tmpRect.width-(x+@constructor.cornerSize+width))
            renderer.drawBitmapSimple(@windowSkin, p, y + @height - @constructor.cornerSize, w, tmpRect.height,
              tmpRect.x, tmpRect.y, w, tmpRect.height)
          else
            renderer.drawBitmapSimple(@windowSkin, x + @constructor.cornerSize + i * tmpRect.width, y + @height - @constructor.cornerSize, tmpRect.width, tmpRect.height,
              tmpRect.x, tmpRect.y, tmpRect.width, tmpRect.height)
      
      
        # Draw corners
        renderer.drawBitmapSimple(@windowSkin, x, y, @constructor.cornerSize, @constructor.cornerSize,
          @constructor.frameRect.x, @constructor.frameRect.y, @constructor.cornerSize, @constructor.cornerSize)
        renderer.drawBitmapSimple(@windowSkin, x + @width - @constructor.cornerSize, y, @constructor.cornerSize, @constructor.cornerSize,
          @constructor.frameRect.x + @constructor.cornerSize*3, @constructor.frameRect.y, @constructor.cornerSize, @constructor.cornerSize)
        renderer.drawBitmapSimple(@windowSkin, x, y + @height - @constructor.cornerSize, @constructor.cornerSize, @constructor.cornerSize,
          @constructor.frameRect.x, @constructor.frameRect.y + @constructor.cornerSize*3, @constructor.cornerSize, @constructor.cornerSize)
        renderer.drawBitmapSimple(@windowSkin, x + @width - @constructor.cornerSize, y + @height - @constructor.cornerSize, @constructor.cornerSize, @constructor.cornerSize,
          @constructor.frameRect.x + @constructor.cornerSize*3, @constructor.frameRect.y + @constructor.cornerSize*3, @constructor.cornerSize, @constructor.cornerSize)
      
        renderer.context.globalAlpha = 1.0
        renderer.ox = ox
        renderer.oy = oy
      
        return null
      */

    };

    Window.prototype._drawWindow = function() {
      var h, height, i, ox, oy, p, renderer, tiles, tmpRect, w, width, x, y, _i, _j, _k, _l;
      if ((this.windowSkin == null) || !this.windowSkin.loaded) {
        return;
      }
      this.oldWindowSkin = this.windowSkin;
      x = this.x;
      y = this.y;
      renderer = Graphics.renderer;
      ox = renderer.ox;
      oy = renderer.oy;
      renderer.ox = 0;
      renderer.oy = 0;
      renderer.reset();
      tmpRect = new Rect(0, 0, 0, 0);
      renderer.size.width = this.width - this.constructor.framePadding * 2;
      renderer.size.height = this.height - this.constructor.framePadding * 2;
      renderer.tone = this.tone;
      renderer.drawBitmapSimple(this.windowSkin, x + this.constructor.framePadding, y + this.constructor.framePadding, this.width - this.constructor.framePadding * 2, this.height - this.constructor.framePadding * 2, new Rect(this.constructor.backRect.x, this.constructor.backRect.y, this.constructor.backRect.width, this.constructor.backRect.height));
      renderer.size.width = 0;
      renderer.size.height = 0;
      renderer.tone = Tone.NORMAL;
      renderer.clip = new Rect(x + this.constructor.framePadding, y + this.constructor.framePadding, this.width - this.constructor.framePadding * 2, this.height - this.constructor.framePadding * 2);
      renderer.drawTiledBitmap(this.windowSkin, x + this.constructor.framePadding, y + this.constructor.framePadding, renderer.clip.width, renderer.clip.height, this.constructor.backRect2.x, this.constructor.backRect2.y, this.constructor.backRect2.width, this.constructor.backRect2.height);
      renderer.clip = new Rect(0, 0, Graphics.width, Graphics.height);
      renderer.opacity = this.opacity;
      renderer.context.globalAlpha = this.opacity * 0.00392156862745;
      height = this.height - this.constructor.cornerSize * 2;
      tmpRect.set(this.constructor.frameRect.x, this.constructor.frameRect.y + this.constructor.cornerSize, this.constructor.frameThickness, this.constructor.frameRect.height - this.constructor.cornerSize * 2);
      tiles = Math.ceil(height / tmpRect.height);
      for (i = _i = 0; 0 <= tiles ? _i < tiles : _i > tiles; i = 0 <= tiles ? ++_i : --_i) {
        if (i + 1 === tiles) {
          p = y + this.constructor.cornerSize + i * tmpRect.height;
          h = tmpRect.height - (p + tmpRect.height - (y + this.constructor.cornerSize + height));
          renderer.drawBitmapSimple(this.windowSkin, x, p, tmpRect.width, h, tmpRect.x, tmpRect.y, tmpRect.width, h);
        } else {
          renderer.drawBitmapSimple(this.windowSkin, x, y + this.constructor.cornerSize + i * tmpRect.height, tmpRect.width, tmpRect.height, tmpRect.x, tmpRect.y, tmpRect.width, tmpRect.height);
        }
      }
      width = this.width - this.constructor.cornerSize * 2;
      tmpRect.set(this.constructor.frameRect.x + this.constructor.cornerSize, this.constructor.frameRect.y, this.constructor.frameRect.width - this.constructor.cornerSize * 2, this.constructor.frameThickness);
      tiles = Math.ceil(width / tmpRect.width);
      for (i = _j = 0; 0 <= tiles ? _j < tiles : _j > tiles; i = 0 <= tiles ? ++_j : --_j) {
        if (i + 1 === tiles) {
          p = x + this.constructor.cornerSize + i * tmpRect.width;
          w = tmpRect.width - (p + tmpRect.width - (x + this.constructor.cornerSize + width));
          renderer.drawBitmapSimple(this.windowSkin, p, y, w, tmpRect.height, tmpRect.x, tmpRect.y, w, tmpRect.height);
        } else {
          renderer.drawBitmapSimple(this.windowSkin, x + this.constructor.cornerSize + i * tmpRect.width, y, tmpRect.width, tmpRect.height, tmpRect.x, tmpRect.y, tmpRect.width, tmpRect.height);
        }
      }
      height = this.height - this.constructor.cornerSize * 2;
      tmpRect.set(this.constructor.frameRect.x + this.constructor.cornerSize * 3, this.constructor.frameRect.y + this.constructor.cornerSize, this.constructor.frameThickness, this.constructor.frameRect.height - this.constructor.cornerSize * 2);
      tiles = Math.ceil(height / tmpRect.height);
      for (i = _k = 0; 0 <= tiles ? _k < tiles : _k > tiles; i = 0 <= tiles ? ++_k : --_k) {
        if (i + 1 === tiles) {
          p = y + this.constructor.cornerSize + i * tmpRect.height;
          h = tmpRect.height - (p + tmpRect.height - (y + this.constructor.cornerSize + height));
          renderer.drawBitmapSimple(this.windowSkin, x + this.width - this.constructor.cornerSize, p, tmpRect.width, h, tmpRect.x, tmpRect.y, tmpRect.width, h);
        } else {
          renderer.drawBitmapSimple(this.windowSkin, x + this.width - this.constructor.cornerSize, y + this.constructor.cornerSize + i * tmpRect.height, tmpRect.width, tmpRect.height, tmpRect.x, tmpRect.y, tmpRect.width, tmpRect.height);
        }
      }
      width = this.width - this.constructor.cornerSize * 2;
      tmpRect.set(this.constructor.frameRect.x + this.constructor.cornerSize, this.constructor.frameRect.y + this.constructor.cornerSize * 3, this.constructor.frameRect.width - this.constructor.cornerSize * 2, this.constructor.frameThickness);
      tiles = Math.ceil(width / tmpRect.width);
      for (i = _l = 0; 0 <= tiles ? _l < tiles : _l > tiles; i = 0 <= tiles ? ++_l : --_l) {
        if (i + 1 === tiles) {
          p = x + this.constructor.cornerSize + i * tmpRect.width;
          w = tmpRect.width - (p + tmpRect.width - (x + this.constructor.cornerSize + width));
          renderer.drawBitmapSimple(this.windowSkin, p, y + this.height - this.constructor.cornerSize, w, tmpRect.height, tmpRect.x, tmpRect.y, w, tmpRect.height);
        } else {
          renderer.drawBitmapSimple(this.windowSkin, x + this.constructor.cornerSize + i * tmpRect.width, y + this.height - this.constructor.cornerSize, tmpRect.width, tmpRect.height, tmpRect.x, tmpRect.y, tmpRect.width, tmpRect.height);
        }
      }
      renderer.drawBitmapSimple(this.windowSkin, x, y, this.constructor.cornerSize, this.constructor.cornerSize, this.constructor.frameRect.x, this.constructor.frameRect.y, this.constructor.cornerSize, this.constructor.cornerSize);
      renderer.drawBitmapSimple(this.windowSkin, x + this.width - this.constructor.cornerSize, y, this.constructor.cornerSize, this.constructor.cornerSize, this.constructor.frameRect.x + this.constructor.cornerSize * 3, this.constructor.frameRect.y, this.constructor.cornerSize, this.constructor.cornerSize);
      renderer.drawBitmapSimple(this.windowSkin, x, y + this.height - this.constructor.cornerSize, this.constructor.cornerSize, this.constructor.cornerSize, this.constructor.frameRect.x, this.constructor.frameRect.y + this.constructor.cornerSize * 3, this.constructor.cornerSize, this.constructor.cornerSize);
      renderer.drawBitmapSimple(this.windowSkin, x + this.width - this.constructor.cornerSize, y + this.height - this.constructor.cornerSize, this.constructor.cornerSize, this.constructor.cornerSize, this.constructor.frameRect.x + this.constructor.cornerSize * 3, this.constructor.frameRect.y + this.constructor.cornerSize * 3, this.constructor.cornerSize, this.constructor.cornerSize);
      renderer.opacity = 255;
      renderer.context.globalAlpha = 1.0;
      renderer.ox = ox;
      renderer.oy = oy;
      return null;
    };

    Window.prototype.draw = function() {
      var contentHeight, contentWidth, cursorBorder, cursorBorder2, cursorRect, flr, height, pauseIconSize, pauseRect, renderer, x, y;
      renderer = Graphics.renderer;
      renderer.blendType = 0;
      renderer.reset();
      x = Math.floor(this.x) - this.ox;
      y = Math.floor(this.y) - this.oy;
      if (this.openness === 0) {
        return null;
      }
      if (this.openness < 255 && this.opacity > 0) {
        height = this.height * (this.openness * 0.00392156862745);
        this._drawWindow();
        return null;
      }
      if (this.opacity > 0) {
        this._drawWindow();
      }
      cursorBorder = this.constructor.cursorBorder;
      cursorBorder2 = cursorBorder * 2;
      cursorRect = this.constructor.cursorRect;
      x = x + this.padding - this.cx;
      y = y + this.padding - this.cy;
      if (this.cursorRect.width > 0 && this.cursorRect.height > 0) {
        renderer.context.globalAlpha = this.cursorOpacity;
        renderer.opacity = this.cursorOpacity * 255;
        renderer.drawBitmapSimple(this.windowSkin, x + this.cursorRect.x + cursorBorder, y + this.cursorRect.y, this.cursorRect.width - cursorBorder2, cursorBorder, cursorRect.x + cursorBorder, cursorRect.y, cursorRect.width - cursorBorder2, cursorBorder);
        renderer.drawBitmapSimple(this.windowSkin, x + this.cursorRect.x, y + this.cursorRect.y + cursorBorder, cursorBorder, this.cursorRect.height - cursorBorder2, cursorRect.x, cursorRect.y + cursorBorder, cursorBorder, cursorRect.height - cursorBorder2);
        renderer.drawBitmapSimple(this.windowSkin, x + this.cursorRect.x + cursorBorder, y + this.cursorRect.y + this.cursorRect.height - cursorBorder, this.cursorRect.width - cursorBorder2, cursorBorder, cursorRect.x + cursorBorder, cursorRect.y + cursorRect.height - cursorBorder, cursorRect.width - cursorBorder2, cursorBorder);
        renderer.drawBitmapSimple(this.windowSkin, x + this.cursorRect.x + this.cursorRect.width - cursorBorder, y + this.cursorRect.y + cursorBorder, cursorBorder, this.cursorRect.height - cursorBorder2, cursorRect.x + cursorRect.width - cursorBorder, cursorRect.y + cursorBorder, cursorBorder, cursorRect.height - cursorBorder2);
        renderer.drawBitmapSimple(this.windowSkin, x + this.cursorRect.x, y + this.cursorRect.y, cursorBorder, cursorBorder, cursorRect.x, cursorRect.y, cursorBorder, cursorBorder);
        renderer.drawBitmapSimple(this.windowSkin, x + this.cursorRect.x + this.cursorRect.width - cursorBorder, y + this.cursorRect.y, cursorBorder, cursorBorder, cursorRect.x + cursorRect.width - cursorBorder, cursorRect.y, cursorBorder, cursorBorder);
        renderer.drawBitmapSimple(this.windowSkin, x + this.cursorRect.x + this.cursorRect.width - cursorBorder, y + this.cursorRect.y + this.cursorRect.height - cursorBorder, cursorBorder, cursorBorder, cursorRect.x + cursorRect.width - cursorBorder, cursorRect.y + cursorRect.height - cursorBorder, cursorBorder, cursorBorder);
        renderer.drawBitmapSimple(this.windowSkin, x + this.cursorRect.x, y + this.cursorRect.y + this.cursorRect.height - cursorBorder, cursorBorder, cursorBorder, cursorRect.x, cursorRect.y + cursorRect.height - cursorBorder, cursorBorder, cursorBorder);
        renderer.opacity = 255;
        renderer.context.globalAlpha = 1.0;
      }
      x = x - this.padding + this.cx;
      y = y - this.padding + this.cy;
      if (this.pause) {
        pauseIconSize = this.constructor.pauseIconSize;
        pauseRect = this.constructor.pauseRect;
        flr = Math.floor;
        switch (this.pauseArrowPattern) {
          case 0:
            renderer.drawBitmapSimple(this.windowSkin, flr(x + this.width * 0.5 - 8), flr(y + this.height - pauseIconSize.height), pauseIconSize.width, pauseIconSize.height, pauseRect.x, pauseRect.y, pauseIconSize.width, pauseIconSize.height);
            break;
          case 1:
            renderer.drawBitmapSimple(this.windowSkin, flr(x + this.width * 0.5 - 8), flr(y + this.height - pauseIconSize.height), pauseIconSize.width, pauseIconSize.height, pauseRect.x + pauseIconSize.width, pauseRect.y, pauseIconSize.width, pauseIconSize.height);
            break;
          case 2:
            renderer.drawBitmapSimple(this.windowSkin, flr(x + this.width * 0.5 - 8), flr(y + this.height - pauseIconSize.height), pauseIconSize.width, pauseIconSize.height, pauseRect.x, pauseRect.y + pauseIconSize.height, pauseIconSize.width, pauseIconSize.height);
            break;
          case 3:
            renderer.drawBitmapSimple(this.windowSkin, flr(x + this.width * 0.5 - 8), flr(y + this.height - pauseIconSize.height), pauseIconSize.width, pauseIconSize.height, pauseRect.x + pauseIconSize.width, pauseRect.y + pauseIconSize.height, pauseIconSize.width, pauseIconSize.height);
        }
      }
      if (this.arrowsVisible && (this.contents != null)) {
        if (this.cx > 0) {
          renderer.drawBitmapSimple(this.windowSkin, x + 4, y + this.height * 0.5 - 8, this.constructor.leftArrowRect.width, this.constructor.leftArrowRect.height, this.constructor.leftArrowRect.x, this.constructor.leftArrowRect.y, this.constructor.leftArrowRect.width, this.constructor.leftArrowRect.height);
        }
        if (this.cy > 0) {
          renderer.drawBitmapSimple(this.windowSkin, x + this.width * 0.5 - 8, y + 4, this.constructor.topArrowRect.width, this.constructor.topArrowRect.height, this.constructor.topArrowRect.x, this.constructor.topArrowRect.y, this.constructor.topArrowRect.width, this.constructor.topArrowRect.height);
        }
        if (this.contents.width - this.cx > this.width - this.padding) {
          renderer.drawBitmapSimple(this.windowSkin, x + this.width - 11, y + this.height * 0.5 - 8, this.constructor.rightArrowRect.width, this.constructor.rightArrowRect.height, this.constructor.rightArrowRect.x, this.constructor.rightArrowRect.y, this.constructor.rightArrowRect.width, this.constructor.rightArrowRect.height);
        }
        if (this.contents.height - this.cy > this.height - this.padding) {
          renderer.drawBitmapSimple(this.windowSkin, x + this.width * 0.5 - 8, y + this.height - 11, this.constructor.bottomArrowRect.width, this.constructor.bottomArrowRect.height, this.constructor.bottomArrowRect.x, this.constructor.bottomArrowRect.y, this.constructor.bottomArrowRect.width, this.constructor.bottomArrowRect.height);
        }
      }
      if (this.contents != null) {
        renderer.context.globalAlpha = this.contentsOpacity * 0.00392156862745;
        contentWidth = this.contents.width - this.cx > this.width - this.padding * 2 ? this.width - this.padding * 2 : this.contents.width - this.cx;
        contentHeight = this.contents.height - this.cy > this.height - this.padding * 2 ? this.height - this.padding * 2 : this.contents.height - this.cy;
        contentWidth = Math.min(contentWidth, this.contents.image.width);
        contentHeight = Math.min(contentHeight, this.contents.image.height);
        return renderer.drawBitmapSimple(this.contents, x + this.padding, y + this.padding, contentWidth, contentHeight, Math.max(this.cx, 0), Math.max(this.cy, 0), contentWidth, contentHeight);
        /*
        renderer.context.drawImage(@contents.image,
            x + @padding, y + @padding, contentWidth, contentHeight,
          Math.max(@ox, 0), Math.max(@oy, 0), contentWidth,
          contentHeight)
        */

      }
    };

    return Window;

  })(GraphicObject);

  window.Window = Window;

}).call(this);

/////////////////////////////////////////////////////////////////////
//
// 	File: Object.js
// 	Created on: 2013-01-27
//
// 	Copyright (C) 2013 André Radomski. All rights reserved.
// 
/////////////////////////////////////////////////////////////////////
//
// 	Class: Object
//
//   Description...
//
/////////////////////////////////////////////////////////////////////

Function.prototype.accessors = function(prop, desc) {
    Object.defineProperty(this.prototype, prop, desc)
}

Object.mixin = function(target, source, blacklist)
{
    for(k in source)
    {
        if(source.hasOwnProperty(k) && (!blacklist || blacklist.indexOf(k) == -1))
        {
            target[k] = source[k];
        }
    }

    return target;
}



Object.extend = function(object, base)
{
    object.prototype = Object.create(base.prototype)
    object.prototype.constructor = object

    object.base = base.prototype
    object.basec = base.prototype.constructor;
}

Object.getConstructorOf = function(c)
{
    return Object.getPrototypeOf(c.prototype).constructor
}

Object.getClassFromString = function(classPath)
{
    var result = null;
    var packages = classPath.split(".");
    var currentPackage = globalScope;
    for(var i = 0; i < packages.length; i++)
    {
        currentPackage = currentPackage[packages[i]];
    }

    return currentPackage;
}

Object.flatCopy = function(object) {
    var result = {};

    for(k in object) {
        result[k] = object[k];
    }

    return result;
}

Object.copy = function(object)
{
    return JSON.parse(JSON.stringify(object));
}

/* This file is part of OWL JavaScript Utilities.

 OWL JavaScript Utilities is free software: you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public License
 as published by the Free Software Foundation, either version 3 of
 the License, or (at your option) any later version.

 OWL JavaScript Utilities is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with OWL JavaScript Utilities.  If not, see
 <http://www.gnu.org/licenses/>.
 */

owl = (function() {

    // the re-usable constructor function used by clone().
    function Clone() {}

    // clone objects, skip other types.
    function clone(target) {
        if ( typeof target == 'object' ) {
            Clone.prototype = target;
            return new Clone();
        } else {
            return target;
        }
    }


    // Shallow Copy
    function copy(target) {
        if (typeof target !== 'object' ) {
            return target;  // non-object have value sematics, so target is already a copy.
        } else {
            var value = target.valueOf();
            if (target != value) {
                // the object is a standard object wrapper for a native type, say String.
                // we can make a copy by instantiating a new object around the value.
                return new target.constructor(value);
            } else {
                // ok, we have a normal object. If possible, we'll clone the original's prototype
                // (not the original) to get an empty object with the same prototype chain as
                // the original.  If just copy the instance properties.  Otherwise, we have to
                // copy the whole thing, property-by-property.
                if ( target instanceof target.constructor && target.constructor !== Object ) {
                    var c = clone(target.constructor.prototype);

                    // give the copy all the instance properties of target.  It has the same
                    // prototype as target, so inherited properties are already there.
                    for ( var property in target) {
                        if (target.hasOwnProperty(property)) {
                            c[property] = target[property];
                        }
                    }
                } else {
                    var c = {};
                    for ( var property in target ) c[property] = target[property];
                }

                return c;
            }
        }
    }

    // Deep Copy
    var deepCopiers = [];

    function DeepCopier(config) {
        for ( var key in config ) this[key] = config[key];
    }
    DeepCopier.prototype = {
        constructor: DeepCopier,

        // determines if this DeepCopier can handle the given object.
        canCopy: function(source) { return false; },

        // starts the deep copying process by creating the copy object.  You
        // can initialize any properties you want, but you can't call recursively
        // into the DeeopCopyAlgorithm.
        create: function(source) { },

        // Completes the deep copy of the source object by populating any properties
        // that need to be recursively deep copied.  You can do this by using the
        // provided deepCopyAlgorithm instance's deepCopy() method.  This will handle
        // cyclic references for objects already deepCopied, including the source object
        // itself.  The "result" passed in is the object returned from create().
        populate: function(deepCopyAlgorithm, source, result) {}
    };

    function DeepCopyAlgorithm() {
        // copiedObjects keeps track of objects already copied by this
        // deepCopy operation, so we can correctly handle cyclic references.
        this.copiedObjects = [];
        thisPass = this;
        this.recursiveDeepCopy = function(source) {
            return thisPass.deepCopy(source);
        }
        this.depth = 0;
    }
    DeepCopyAlgorithm.prototype = {
        constructor: DeepCopyAlgorithm,

        maxDepth: 256,

        // add an object to the cache.  No attempt is made to filter duplicates;
        // we always check getCachedResult() before calling it.
        cacheResult: function(source, result) {
            this.copiedObjects.push([source, result]);
        },

        // Returns the cached copy of a given object, or undefined if it's an
        // object we haven't seen before.
        getCachedResult: function(source) {
            var copiedObjects = this.copiedObjects;
            var length = copiedObjects.length;
            for ( var i=0; i<length; i++ ) {
                if ( copiedObjects[i][0] === source ) {
                    return copiedObjects[i][1];
                }
            }
            return undefined;
        },

        // deepCopy handles the simple cases itself: non-objects and object's we've seen before.
        // For complex cases, it first identifies an appropriate DeepCopier, then calls
        // applyDeepCopier() to delegate the details of copying the object to that DeepCopier.
        deepCopy: function(source) {
            // null is a special case: it's the only value of type 'object' without properties.
            if ( source === null ) return null;

            // All non-objects use value semantics and don't need explict copying.
            if ( typeof source !== 'object' ) return source;

            var cachedResult = this.getCachedResult(source);

            // we've already seen this object during this deep copy operation
            // so can immediately return the result.  This preserves the cyclic
            // reference structure and protects us from infinite recursion.
            if ( cachedResult ) return cachedResult;

            // objects may need special handling depending on their class.  There is
            // a class of handlers call "DeepCopiers"  that know how to copy certain
            // objects.  There is also a final, generic deep copier that can handle any object.
            for ( var i=0; i<deepCopiers.length; i++ ) {
                var deepCopier = deepCopiers[i];
                if ( deepCopier.canCopy(source) ) {
                    return this.applyDeepCopier(deepCopier, source);
                }
            }
            // the generic copier can handle anything, so we should never reach this line.
            throw new Error("no DeepCopier is able to copy " + source);
        },

        // once we've identified which DeepCopier to use, we need to call it in a very
        // particular order: create, cache, populate.  This is the key to detecting cycles.
        // We also keep track of recursion depth when calling the potentially recursive
        // populate(): this is a fail-fast to prevent an infinite loop from consuming all
        // available memory and crashing or slowing down the browser.
        applyDeepCopier: function(deepCopier, source) {
            // Start by creating a stub object that represents the copy.
            var result = deepCopier.create(source);

            // we now know the deep copy of source should always be result, so if we encounter
            // source again during this deep copy we can immediately use result instead of
            // descending into it recursively.
            this.cacheResult(source, result);

            // only DeepCopier::populate() can recursively deep copy.  So, to keep track
            // of recursion depth, we increment this shared counter before calling it,
            // and decrement it afterwards.
            this.depth++;
            if ( this.depth > this.maxDepth ) {
                throw new Error("Exceeded max recursion depth in deep copy.");
            }

            // It's now safe to let the deepCopier recursively deep copy its properties.
            deepCopier.populate(this.recursiveDeepCopy, source, result);

            this.depth--;

            return result;
        }
    };

    // entry point for deep copy.
    //   source is the object to be deep copied.
    //   maxDepth is an optional recursion limit. Defaults to 256.
    function deepCopy(source, maxDepth) {
        var deepCopyAlgorithm = new DeepCopyAlgorithm();
        if ( maxDepth ) deepCopyAlgorithm.maxDepth = maxDepth;
        return deepCopyAlgorithm.deepCopy(source);
    }

    // publicly expose the DeepCopier class.
    deepCopy.DeepCopier = DeepCopier;

    // publicly expose the list of deepCopiers.
    deepCopy.deepCopiers = deepCopiers;

    // make deepCopy() extensible by allowing others to
    // register their own custom DeepCopiers.
    deepCopy.register = function(deepCopier) {
        if ( !(deepCopier instanceof DeepCopier) ) {
            deepCopier = new DeepCopier(deepCopier);
        }
        deepCopiers.unshift(deepCopier);
    }

    // Generic Object copier
    // the ultimate fallback DeepCopier, which tries to handle the generic case.  This
    // should work for base Objects and many user-defined classes.
    deepCopy.register({
        canCopy: function(source) { return true; },

        create: function(source) {
            if ( source instanceof source.constructor ) {
                return clone(source.constructor.prototype);
            } else {
                return {};
            }
        },

        populate: function(deepCopy, source, result) {
            for ( var key in source ) {
                if ( source.hasOwnProperty(key) ) {
                    result[key] = deepCopy(source[key]);
                }
            }
            return result;
        }
    });

    // Array copier
    deepCopy.register({
        canCopy: function(source) {
            return ( source instanceof Array );
        },

        create: function(source) {
            return new source.constructor();
        },

        populate: function(deepCopy, source, result) {
            for ( var i=0; i<source.length; i++) {
                result.push( deepCopy(source[i]) );
            }
            return result;
        }
    });

    // Date copier
    deepCopy.register({
        canCopy: function(source) {
            return ( source instanceof Date );
        },

        create: function(source) {
            return new Date(source);
        }
    });

    // HTML DOM Node

    // utility function to detect Nodes.  In particular, we're looking
    // for the cloneNode method.  The global document is also defined to
    // be a Node, but is a special case in many ways.
    function isNode(source) {
        if ( window.Node ) {
            return source instanceof Node;
        } else {
            // the document is a special Node and doesn't have many of
            // the common properties so we use an identity check instead.
            if ( source === document ) return true;
            return (
                typeof source.nodeType === 'number' &&
                source.attributes &&
                source.childNodes &&
                source.cloneNode
            );
        }
    }

    // Node copier
    deepCopy.register({
        canCopy: function(source) { return isNode(source); },

        create: function(source) {
            // there can only be one (document).
            if ( source === document ) return document;

            // start with a shallow copy.  We'll handle the deep copy of
            // its children ourselves.
            return source.cloneNode(false);
        },

        populate: function(deepCopy, source, result) {
            // we're not copying the global document, so don't have to populate it either.
            if ( source === document ) return document;

            // if this Node has children, deep copy them one-by-one.
            if ( source.childNodes && source.childNodes.length ) {
                for ( var i=0; i<source.childNodes.length; i++ ) {
                    var childCopy = deepCopy(source.childNodes[i]);
                    result.appendChild(childCopy);
                }
            }
        }
    });

    return {
        DeepCopyAlgorithm: DeepCopyAlgorithm,
        copy: copy,
        clone: clone,
        deepCopy: deepCopy
    };
})();

if(!Object.values) {
    Object.values = function(o) {
        var values = []

        for(k in o) {
            values.push(o[k])
        }

        return values;
    }
}
Object.deepCopy = owl.deepCopy

function Utf8ArrayToStr(array) {
    var out, i, len, c;
    var char2, char3;

    out = "";
    len = array.length;
    i = 0;
    while(i < len) {
        c = array[i++];
        switch(c >> 4)
        {
            case 0: case 1: case 2: case 3: case 4: case 5: case 6: case 7:
            // 0xxxxxxx
            out += String.fromCharCode(c);
            break;
            case 12: case 13:
            // 110x xxxx   10xx xxxx
            char2 = array[i++];
            out += String.fromCharCode(((c & 0x1F) << 6) | (char2 & 0x3F));
            break;
            case 14:
                // 1110 xxxx  10xx xxxx  10xx xxxx
                char2 = array[i++];
                char3 = array[i++];
                out += String.fromCharCode(((c & 0x0F) << 12) |
                    ((char2 & 0x3F) << 6) |
                    ((char3 & 0x3F) << 0));
                break;
        }
    }

    return out;
}
gs.Utf8ArrayToStr = Utf8ArrayToStr;
/*Object.deepCopy = function(object){
    context = new gs.ObjectCodecContext()
    object = gs.ObjectCodec.encode(object, context)
    context = new gs.ObjectCodecContext(null, context.encodedObjectStore, null)
    object = gs.ObjectCodec.decode(object, context)

    return object
}*/
Math.degreesToRadians = function(degrees) { return degrees * 0.0174532925199432957692369076848; }
Math.nextPow2 = function(value)
{
    value--;
    value = (value >> 1) | value;
    value = (value >> 2) | value;
    value = (value >> 4) | value;
    value = (value >> 8) | value;
    value = (value >> 16) | value;
    value++;

    return value;
}

Math.toRadians = function(degrees)
{
    return degrees * 0.01745329251994
}

Math.mod = function(a, n) { return ((a%n)+n)%n; }

Math.linearTween = function (t, b, c, d) {
    return c*t/d + b;
};

Math.easeInQuad = function (t, b, c, d) {
    t /= d;
    return c*t*t + b;
};

Math.easeOutQuad = function (t, b, c, d) {
    t /= d;
    return -c * t*(t-2) + b;
};

Math.easeInOutQuad = function (t, b, c, d) {
    t /= d/2;
    if (t < 1) return c/2*t*t + b;
    t--;
    return -c/2 * (t*(t-2) - 1) + b;
};

Math.easeInCubic = function (t, b, c, d) {
    t /= d;
    return c*t*t*t + b;
};

Math.easeOutCubic = function (t, b, c, d) {
    t /= d;
    t--;
    return c*(t*t*t + 1) + b;
};

Math.easeInOutCubic = function (t, b, c, d) {
    t /= d/2;
    if (t < 1) return c/2*t*t*t + b;
    t -= 2;
    return c/2*(t*t*t + 2) + b;
};

Math.easeInQuart = function (t, b, c, d) {
    t /= d;
    return c*t*t*t*t + b;
};

Math.easeOutQuart = function (t, b, c, d) {
    t /= d;
    t--;
    return -c * (t*t*t*t - 1) + b;
};

Math.easeInOutQuart = function (t, b, c, d) {
    t /= d/2;
    if (t < 1) return c/2*t*t*t*t + b;
    t -= 2;
    return -c/2 * (t*t*t*t - 2) + b;
};

Math.easeInQuint = function (t, b, c, d) {
    t /= d;
    return c*t*t*t*t*t + b;
};

Math.easeOutQuint = function (t, b, c, d) {
    t /= d;
    t--;
    return c*(t*t*t*t*t + 1) + b;
};

Math.easeInOutQuint = function (t, b, c, d) {
    t /= d/2;
    if (t < 1) return c/2*t*t*t*t*t + b;
    t -= 2;
    return c/2*(t*t*t*t*t + 2) + b;
};

Math.easeInSine = function (t, b, c, d) {
    return -c * Math.cos(t/d * (Math.PI/2)) + c + b;
};

Math.easeOutSine = function (t, b, c, d) {
    return c * Math.sin(t/d * (Math.PI/2)) + b;
};

Math.easeInOutSine = function (t, b, c, d) {
    return -c/2 * (Math.cos(Math.PI*t/d) - 1) + b;
};

Math.easeInExpo = function (t, b, c, d) {
    return c * Math.pow( 2, 10 * (t/d - 1) ) + b;
};

Math.easeOutExpo = function (t, b, c, d) {
    return c * ( -Math.pow( 2, -10 * t/d ) + 1 ) + b;
};

Math.easeInOutExpo = function (t, b, c, d) {
    t /= d/2;
    if (t < 1) return c/2 * Math.pow( 2, 10 * (t - 1) ) + b;
    t--;
    return c/2 * ( -Math.pow( 2, -10 * t) + 2 ) + b;
};

Math.easeInCirc = function (t, b, c, d) {
    t /= d;
    return -c * (Math.sqrt(1 - t*t) - 1) + b;
};

Math.easeOutCirc = function (t, b, c, d) {
    t /= d;
    t--;
    return c * Math.sqrt(1 - t*t) + b;
};

Math.easeInOutCirc = function (t, b, c, d) {
    t /= d/2;
    if (t < 1) return -c/2 * (Math.sqrt(1 - t*t) - 1) + b;
    t -= 2;
    return c/2 * (Math.sqrt(1 - t*t) + 1) + b;
};

function MAX(a, b) { return Math.max(a, b); }
function MIN(a, b) { return Math.min(a, b); }
/////////////////////////////////////////////////////////////////////
//
// 	File: String.js
// 	Created on: 2013-01-27
//
// 	Copyright (C) 2013 André Radomski. All rights reserved.
// 
/////////////////////////////////////////////////////////////////////
//
// 	Class: String
//
//  Description...
//
/////////////////////////////////////////////////////////////////////

String.prototype.replaceAt = function(index, character)
{
    return this.substr(0, index) + character + this.substr(index+character.length);
}

String.prototype.fill = function(character, maximum, direction)
{
    var result = this;
    var difference = 0;
    var fill = "";

    if(this.length < maximum)
    {
        difference = maximum - this.length;
        for(var i = 0; i < difference; i++)
        {
            fill = fill + character;
        }

        result = direction == 0 ? fill + result : result + fill;
    }

    return result;
}

String.prototype.lfill = function(character, maximum)
{
    return this.fill(character, maximum, 0);
}

String.prototype.rfill = function(character, maximum)
{
    return this.fill(character, maximum, 1);
}

String.prototype.ltrim = function()
{
    return this.replace (/^\s+/, '');
}

String.prototype.rtrim = function()
{
    return this.replace (/\s+$/, '');
}

String.prototype.trim = function()
{
    return this.ltrim ().rtrim ();
};

String.prototype.format = function()
{
    var args = arguments

    return this.replace(/{(\d+)}/g, function(match, number)
    {
        return typeof args[number] != 'undefined' ? args[number] : match
    })
};

String.prototype.capitalize = function()
{
    return this.charAt(0).toUpperCase() + this.slice(1);
}

if(!String.prototype.startsWith) {
    String.prototype.startsWith = function (s) {
        return this.match('^' + s) == s;
    }
}

if(!String.prototype.endsWith) {
    String.prototype.endsWith = function (s) {
        return this.match(s + '$') == s;
    }
}

String.prototype.contains = function(s)
{
    return this.indexOf(s) != -1;
}


String.prototype.truncate = function(length) {
    if (this.length > length) {
        return this.substring(0, length);
    } else {
        return this;
    }
}
String.prototype.escapeHtml = function()
{
    var result = "";
    for(var i = 0; i < this.length; i++)
    {
        if(this.charAt(i) == "&")
        {
            result = result + "&amp;";
        }
        else if(this.charAt(i)== "<")
        {
            result = result + "&lt;";
        }
        else if(this.charAt(i)== ">"){
            result = result + "&gt;";
        }
        else
        {
            result = result + this.charAt(i);
        }
    }
    return result;
};

String.prototype.unescapeHtml = function()
{
    var result = this.replace("&amp;", "&");
    result = this.replace("&gt;", ">");
    result = this.replace("&lt;", "<");

    return result;
};

String.prototype.withoutUnit = function()
{
    var result = this;
    var lastCharacter = this.charAt(this.length - 1);

    if(lastCharacter > 57 || lastCharacter < 48)
    {
        result = this.substr(this.length - 2, 2);
    }

    return result;
}

/////////////////////////////////////////////////////////////////////
//
// 	File: Array.js
// 	Created on: 2013-01-27
//
// 	Copyright (C) 2013 André Radomski. All rights reserved.
// 
/////////////////////////////////////////////////////////////////////
//
// 	Class: Array
//
//  Description...
//
/////////////////////////////////////////////////////////////////////

Array.prototype.removeAll = function (f) {
    var item;
    var i = 0;
    var count = 0;
    while (i < this.length) {
        if(f(this[i])) {
            this.splice(i, 1);
            i--;
            count++;
        }

        i++;
    }

    return count;
};


if(!Array.prototype.indexOf)
{
Array.prototype.indexOf = function(searchElement, fromIndex)
{
    var result = -1;
    fromIndex = fromIndex || 0;

    for(var i = fromIndex; i < this.length; i++)
    {
        if(this[i] == searchElement)
        {
            result = i;
            break;
        }
    }

    return result;
}
}

Array.remove = function(array, element)
{
    var index = array.indexOf(element, 0);

    if(index != -1)
    {
        array.splice(index, 1);
    }
}
Array.prototype.remove = function(element)
{
    var index = Array.prototype.indexOf.call(this, element, 0);

    if(index != -1)
    {
        this.splice(index, 1);
    }
}

Array.prototype.toIndexArray = function(indexOffset)
{
    var result = [];
    indexOffset = indexOffset || 0

    for(var i = 0; i < this.length; i++)
    {
        result.push(indexOffset + i);
    }

    return result;
}

Array.prototype.toPropertyArray = function(propertyName)
{
    var result = [];

    for(var i = 0; i < this.length; i++)
    {
        result.push(this[i][propertyName]);
    }

    return result;
}

Array.prototype.pushArray = function(array)
{
    for(var i = 0; i < array.length; i++)
    {
        this.push(array[i]);
    }
}

Array.prototype.fill = function(value, start, length)
{
    var start = start || 0;
    var length = length || this.length;

    for(var i = start; i < length; i++)
    {
        this[i] = value;
    }

    return this;
}

